@@ -254,11 +254,11 @@ class Logger(args.ArgumentProcessor):
           f.flush()
     return
 
-  def logPrint(self, msg, debugLevel = -1, debugSection = None, indent = 1, comm = None):
+  def logPrint(self, msg, debugLevel = -1, debugSection = None, indent = 1, comm = None, forceScroll = 0):
     '''Write the message to the log streams with proper indentation and a newline'''
     if indent:
       self.logIndent(debugLevel, debugSection, comm)
-    self.logWrite(msg, debugLevel, debugSection)
+    self.logWrite(msg, debugLevel, debugSection, forceScroll = forceScroll)
     for writeAll, f in enumerate([self.out, self.log]):
       if self.checkWrite(f, debugLevel, debugSection, writeAll):
         if writeAll or self.linewidth < 0:
@@ -270,17 +270,21 @@ class Logger(args.ArgumentProcessor):
     '''Return the directory containing this module
        - This has the problem that when we reload a module of the same name, this gets screwed up
          Therefore, we call it in the initializer, and stash it'''
-    if not hasattr(self, '_root_'):
+    #print '      In getRoot'
+    #print hasattr(self, '__root')
+    #print '      done checking'
+    if not hasattr(self, '__root'):
       import os
       import sys
 
       # Work around a bug with pdb in 2.3
       if hasattr(sys.modules[self.__module__], '__file__') and not os.path.basename(sys.modules[self.__module__].__file__) == 'pdb.py':
-        self._root_ = os.path.abspath(os.path.dirname(sys.modules[self.__module__].__file__))
+        self.__root = os.path.abspath(os.path.dirname(sys.modules[self.__module__].__file__))
       else:
-        self._root_ = os.getcwd()
-    return self._root_
+        self.__root = os.getcwd()
+    #print '      Exiting getRoot'
+    return self.__root
   def setRoot(self, root):
-    self._root_ = root
+    self.__root = root
     return
   root = property(getRoot, setRoot, doc = 'The directory containing this module')
