@@ -1,14 +1,16 @@
 import PETSc.package
 
-class Configure(PETSc.package.Package):
+class Configure(PETSc.package.NewPackage):
   def __init__(self, framework):
-    PETSc.package.Package.__init__(self, framework)
-    self.download = ['ftp://ftp.mcs.anl.gov/pub/petsc/externalpackages/Generator.tar.gz']
+    PETSc.package.NewPackage.__init__(self, framework)
+    self.download = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/Generator.tar.gz']
     self.complex  = 1
+    self.worksonWindows    = 1
+    self.downloadonWindows = 1
     return
 
   def setupDependencies(self, framework):
-    PETSc.package.Package.setupDependencies(self, framework)
+    PETSc.package.NewPackage.setupDependencies(self, framework)
     self.deps = []
     return
 
