@@ -7,7 +7,7 @@ class Configure(PETSc.package.NewPackage):
     self.functions    = ['party_lib']
     self.includes     = ['party_lib.h']
     self.liblist      = [['libparty.a']]
-    self.license      = 'http://wwwcs.upb.de/fachbereich/AG/monien/RESEARCH/PART/party.html'
+    self.license      = 'http://www2.cs.uni-paderborn.de/cs/robsy/party.html'
     return
 
   def Install(self):
@@ -22,7 +22,7 @@ class Configure(PETSc.package.NewPackage):
     if self.installNeeded('make.inc'):
       try:
         self.logPrintBox('Compiling party; this may take several minutes')
-        output,err,ret  = PETSc.package.NewPackage.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+'; PARTY_INSTALL_DIR='+self.installDir+';export PARTY_INSTALL_DIR; make clean; make all; cd ..; mv -f *.a '+os.path.join(self.installDir,self.libdir,'/,')+'; cp -f party_lib.h '+os.path.join(self.installDir,self.includedir,'.'), timeout=2500, log = self.framework.log)
+        output,err,ret  = PETSc.package.NewPackage.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+' && make clean && make all && cd .. && mv -f *.a '+os.path.join(self.installDir,self.libdir,'')+' && cp -f party_lib.h '+os.path.join(self.installDir,self.includedir,''), timeout=2500, log = self.framework.log)
       except RuntimeError, e:
         raise RuntimeError('Error running make on PARTY: '+str(e))
       self.postInstall(output+err,'make.inc')
