@@ -3,7 +3,7 @@ import PETSc.package
 class Configure(PETSc.package.NewPackage):
   def __init__(self, framework):
     PETSc.package.NewPackage.__init__(self, framework)
-    self.download          = ['http://www.columbia.edu/~ma2325/Prometheus-1.8.8.tar.gz']
+    self.download          = ['http://www.columbia.edu/~ma2325/Prometheus-1.8.9.tar.gz']
     self.functions         = []
     self.includes          = []
     self.liblist           = [['libpromfei.a','libprometheus.a']]
@@ -83,7 +83,7 @@ class Configure(PETSc.package.NewPackage):
     if self.framework.argDB['with-'+self.package]:
       # Prometheus requires LAPACK routine dorgqr()
       if not self.blasLapack.checkForRoutine('dorgqr'):
-        raise RuntimeError('Prometheus requires the LAPACK routine dorgqr(), the current Lapack libraries '+str(self.blasLapack.lib)+' does not have it\nIf you are using the IBM ESSL library, it does not contain this function. After installing a complete copy of lapack\n You can run config/configure.py with --with-blas-lib=libessl.a --with-lapack-lib=/usr/local/lib/liblapack.a')
+        raise RuntimeError('Prometheus requires the LAPACK routine dorgqr(), the current Lapack libraries '+str(self.blasLapack.lib)+' does not have it\nIf you are using the IBM ESSL library, it does not contain this function. After installing a complete copy of lapack\n You can run ./configure with --with-blas-lib=libessl.a --with-lapack-lib=/usr/local/lib/liblapack.a')
       self.framework.log.write('Found dorgqr() in Lapack library as needed by Prometheus\n')
     return
 
@@ -91,7 +91,7 @@ class Configure(PETSc.package.NewPackage):
     if self.compilePrometheus:
       try:
         self.logPrintBox('Compiling Prometheus; this may take several minutes')
-        output,err,ret  = PETSc.package.NewPackage.executeShellCommand('cd '+self.packageDir+'; make clean cleanlib; make prom minstall',timeout=1000, log = self.framework.log)
+        output,err,ret  = PETSc.package.NewPackage.executeShellCommand('cd '+self.packageDir+' && make clean cleanlib && make prom minstall',timeout=1000, log = self.framework.log)
         self.framework.log.write(output)
       except RuntimeError, e:
         raise RuntimeError('Error running make on Prometheus: '+str(e))
