@@ -3,13 +3,17 @@ import PETSc.package
 class Configure(PETSc.package.NewPackage):
   def __init__(self, framework):
     PETSc.package.NewPackage.__init__(self, framework)
-    self.download     = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/superlu_4.0-March_7_2010.tar.gz']
+    self.download     = ['http://crd.lbl.gov/~xiaoye/SuperLU/superlu_4.2.tar.gz']
     self.functions    = ['set_default_options']
     self.includes     = ['slu_ddefs.h']
-    self.liblist      = [['libsuperlu_4.0.a']]
+    self.liblist      = [['libsuperlu_4.2.a']]
     self.complex      = 1
     # SuperLU has NO support for 64 bit integers, use SuperLU_Dist if you need that
     self.excludedDirs = ['SuperLU_DIST']
+    # SuperLU does not work with --download-f-blas-lapack with Compaqf90 compiler on windows.
+    # However it should work with intel ifort.
+    self.worksonWindows   = 1
+    self.downloadonWindows= 1
     return
 
   def setupDependencies(self, framework):
@@ -24,11 +28,11 @@ class Configure(PETSc.package.NewPackage):
 
     g = open(os.path.join(self.packageDir,'make.inc'),'w')
     g.write('SuperLUroot  = '+self.packageDir+'\n')
-    g.write('TMGLIB       = tmglib.a\n')
-    g.write('SUPERLULIB   = $(SuperLUroot)/lib/libsuperlu_4.0.a\n')
+    g.write('TMGLIB       = tmglib.'+self.setCompilers.AR_LIB_SUFFIX+'\n')
+    g.write('SUPERLULIB   = $(SuperLUroot)/lib/libsuperlu_4.2.'+self.setCompilers.AR_LIB_SUFFIX+'\n')
     g.write('BLASLIB      = '+self.libraries.toString(self.blasLapack.dlib)+'\n')
     g.write('BLASDEF      = -DUSE_VENDOR_BLAS\n')
-    g.write('LIBS	  = $(SUPERLULIB) $(BLASLIB)\n')
+    g.write('LIBS         = $(SUPERLULIB) $(BLASLIB)\n')
     g.write('ARCH         = '+self.setCompilers.AR+'\n')
     g.write('ARCHFLAGS    = '+self.setCompilers.AR_FLAGS+'\n')
     g.write('RANLIB       = '+self.setCompilers.RANLIB+'\n')
@@ -46,25 +50,17 @@ class Configure(PETSc.package.NewPackage):
       g.write('CDEFS   = -DUpCase')
     else:
       g.write('CDEFS   = -DNoChange')
-    if self.framework.argDB['with-64-bit-indices']:
-      g.write(' -D_LONGINT')
     g.write('\n')
 
-    if hasattr(self.compilers, 'FC'):
-      self.setCompilers.pushLanguage('FC')
-      g.write('FORTRAN      = '+self.setCompilers.getCompiler()+'\n')
-      g.write('FFLAGS       = '+self.setCompilers.getCompilerFlags().replace('-Mfree','')+'\n')
-      self.setCompilers.popLanguage()
-    else:
-      g.write('FORTRAN    = \n')
-      g.write('FFLAGS     = \n')
     g.write('MATLAB       =\n')
     g.write('NOOPTS       = '+self.blasLapack.getSharedFlag(self.setCompilers.getCompilerFlags())+' '+self.blasLapack.getPrecisionFlag(self.setCompilers.getCompilerFlags())+' '+self.blasLapack.getWindowsNonOptFlags(self.setCompilers.getCompilerFlags())+'\n')
     g.close()
     if self.installNeeded('make.inc'):
       try:
         self.logPrintBox('Compiling superlu; this may take several minutes')
-        output,err,ret = PETSc.package.NewPackage.executeShellCommand('cd '+self.packageDir+'; SUPERLU_INSTALL_DIR='+self.installDir+'/lib; export SUPERLU_INSTALL_DIR; make clean; make lib LAAUX="" SLASRC="" DLASRC="" CLASRC="" ZLASRC="" SCLAUX="" DZLAUX=""; cp -f lib/*.a '+os.path.join(self.installDir,'lib')+';  cp -f SRC/*.h '+os.path.join(self.installDir,self.includedir)+'/.', timeout=2500, log = self.framework.log)
+        if not os.path.exists(os.path.join(self.packageDir,'lib')):
+          os.makedirs(os.path.join(self.packageDir,'lib'))
+        output,err,ret = PETSc.package.NewPackage.executeShellCommand('cd '+self.packageDir+' && make clean && make superlulib LAAUX="" SLASRC="" DLASRC="" CLASRC="" ZLASRC="" SCLAUX="" DZLAUX="" && cp -f lib/*.'+self.setCompilers.AR_LIB_SUFFIX+' '+os.path.join(self.installDir,self.libdir,'')+' &&  cp -f SRC/*.h '+os.path.join(self.installDir,self.includedir,''), timeout=2500, log = self.framework.log)
       except RuntimeError, e:
         raise RuntimeError('Error running make on SUPERLU: '+str(e))
       self.postInstall(output+err,'make.inc')
