@@ -3,16 +3,18 @@ import PETSc.package
 class Configure(PETSc.package.NewPackage):
   def __init__(self, framework):
     PETSc.package.NewPackage.__init__(self, framework)
-    self.download   = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/SuperLU_DIST_2.3-hg.tar.gz']
+    self.download   = ['http://crd.lbl.gov/~xiaoye/SuperLU/superlu_dist_2.5.tar.gz']
     self.functions  = ['set_default_options_dist']
     self.includes   = ['superlu_ddefs.h']
-    self.liblist    = [['libsuperlu_dist_2.3.a']]
-    #
-    #  SuperLU_dist supports 64 bit integers but uses ParMetis which does not, it has
-    #  a hack that uses the 32 bit parmetis
-    #  SuperLU_dist's support for 64 bit integers is nonsense! (Fortran code -qintsize=8 compile options)
-    self.requires32bitint = 1;
-    self.complex    = 1
+    self.liblist    = [['libsuperlu_dist_2.5.a']]
+    #  SuperLU_dist supports 64 bit integers but uses ParMetis which does not, see the comment in ParMetis.py
+    #  in the method configureLibrary()
+    self.requires32bitint = 0
+    self.complex          = 1
+    # SuperLU_Dist does not work with --download-f-blas-lapack with Compaqf90 compiler on windows.
+    # However it should work with intel ifort.
+    self.worksonWindows   = 1
+    self.downloadonWindows= 1
     return
 
   def setupDependencies(self, framework):
@@ -27,7 +29,7 @@ class Configure(PETSc.package.NewPackage):
 
     g = open(os.path.join(self.packageDir,'make.inc'),'w')
     g.write('DSuperLUroot = '+self.packageDir+'\n')
-    g.write('DSUPERLULIB  = $(DSuperLUroot)/libsuperlu_dist_2.3.a\n')
+    g.write('DSUPERLULIB  = $(DSuperLUroot)/libsuperlu_dist_2.5.'+self.setCompilers.AR_LIB_SUFFIX+'\n')
     g.write('BLASDEF      = -DUSE_VENDOR_BLAS\n')
     g.write('BLASLIB      = '+self.libraries.toString(self.blasLapack.dlib)+'\n')
     g.write('IMPI         = '+self.headers.toString(self.mpi.include)+'\n')
@@ -43,6 +45,7 @@ class Configure(PETSc.package.NewPackage):
     g.write('LOADER       = '+self.setCompilers.getLinker()+'\n') 
     g.write('LOADOPTS     = \n')
     self.setCompilers.popLanguage()
+    # set blas/lapack name mangling
     if self.blasLapack.mangling == 'underscore':
       g.write('CDEFS   = -DAdd_')
     elif self.blasLapack.mangling == 'caps':
@@ -52,20 +55,16 @@ class Configure(PETSc.package.NewPackage):
     if self.framework.argDB['with-64-bit-indices']:
       g.write(' -D_LONGINT')
     g.write('\n')
-    if hasattr(self.compilers, 'FC'):
-      self.setCompilers.pushLanguage('FC')
-      g.write('FORTRAN      = '+self.setCompilers.getCompiler()+'\n')
-      g.write('FFLAGS       = '+self.setCompilers.getCompilerFlags().replace('-Mfree','')+'\n')
-      # set fortran name mangling
-      # this mangling information is for both BLAS and the Fortran compiler so cannot use the BlasLapack mangling flag      
-      self.setCompilers.popLanguage()
+    # not sure what this is for
     g.write('NOOPTS       = '+self.blasLapack.getSharedFlag(self.setCompilers.getCompilerFlags())+' '+self.blasLapack.getPrecisionFlag(self.setCompilers.getCompilerFlags())+' '+self.blasLapack.getWindowsNonOptFlags(self.setCompilers.getCompilerFlags())+'\n')
     g.close()
 
     if self.installNeeded('make.inc'):
       try:
         self.logPrintBox('Compiling superlu_dist; this may take several minutes')
-        output,err,ret  = PETSc.package.NewPackage.executeShellCommand('cd '+self.packageDir+';SUPERLU_DIST_INSTALL_DIR='+self.installDir+'/lib;export SUPERLU_DIST_INSTALL_DIR; make clean; make lib LAAUX=""; mv -f *.a '+os.path.join(self.installDir,'lib')+'; cp -f SRC/*.h '+os.path.join(self.installDir,'include')+'/.', timeout=2500, log = self.framework.log)
+        if not os.path.exists(os.path.join(self.packageDir,'lib')):
+          os.makedirs(os.path.join(self.packageDir,'lib'))
+        output,err,ret  = PETSc.package.NewPackage.executeShellCommand('cd '+self.packageDir+' && make clean && make lib LAAUX="" && mv -f *.'+self.setCompilers.AR_LIB_SUFFIX+' '+os.path.join(self.installDir,self.libdir,'')+' && cp -f SRC/*.h '+os.path.join(self.installDir,self.includedir,''), timeout=2500, log = self.framework.log)
       except RuntimeError, e:
         raise RuntimeError('Error running make on SUPERLU_DIST: '+str(e))
       self.postInstall(output+err,'make.inc')
