@@ -35,8 +35,8 @@ class Configure(PETSc.package.NewPackage):
 
     if self.installNeeded('Makefile.in'):
       self.logPrintBox('Configuring and compiling Spai; this may take several minutes')
-      output1,err1,ret1  = PETSc.package.NewPackage.executeShellCommand('cd '+os.path.join(self.packageDir,'lib')+'; make clean; make ; mv -f libspai.a '+os.path.join(self.installDir,'lib','libspai.a'),timeout=250, log = self.framework.log)
-      output2,err2,ret2  = PETSc.package.NewPackage.executeShellCommand('cd '+os.path.join(self.packageDir,'lib')+'; cp -f *.h '+os.path.join(self.installDir,'include'),timeout=250, log = self.framework.log)
+      output1,err1,ret1  = PETSc.package.NewPackage.executeShellCommand('cd '+os.path.join(self.packageDir,'lib')+' && make clean && make && mv -f libspai.a '+os.path.join(self.installDir,'lib','libspai.a'),timeout=250, log = self.framework.log)
+      output2,err2,ret2  = PETSc.package.NewPackage.executeShellCommand('cd '+os.path.join(self.packageDir,'lib')+' && cp -f *.h '+os.path.join(self.installDir,'include'),timeout=250, log = self.framework.log)
       try:
         output3,err3,ret3  = PETSc.package.NewPackage.executeShellCommand(self.setCompilers.RANLIB+' '+os.path.join(self.installDir,'lib')+'/libspai.a', timeout=250, log = self.framework.log)
       except RuntimeError, e:
