@@ -2,7 +2,7 @@
 
 configure_options = [
   '--with-mpi-dir=/home/balay/soft/linux64/mpich2-1.1-sun',
-  '--with-shared=1',
+  '--with-shared-libraries=1',
   '--with-debugging=0',
   #'CFLAGS=-g',                 # workarround for optimzier bug with gltr.c
   #'LIBS=/usr/lib/libm.a'       # workarround to configure convering '/usr/lib/libm.a' to '-lm'
