@@ -85,7 +85,7 @@ extern int ad_grad_size_shadow;
 #   define ad_AD_IncrementTotalGradSize(x) \
     { \
          if (x + ad_grad_size_shadow > ad_GRAD_MAX) {\
-                SETERRQ2(1, "The number of independent variables exceeds the maximum compiled for!\n Edit your program and change Process adiC(%d) to Process adiC(%d)", x + ad_grad_size_shadow,ad_GRAD_MAX);\
+                SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_USER, "The number of independent variables exceeds the maximum compiled for!\n Edit your program and change Process adiC(%d) to Process adiC(%d)", x + ad_grad_size_shadow,ad_GRAD_MAX);\
         }\
         ad_grad_size_shadow += x;\
     }
@@ -115,7 +115,7 @@ extern int ad_grad_size_shadow;
     { \
         int pOsItIoN = ad_AD_IncrShadowVar(); \
         if (pOsItIoN > ad_GRAD_MAX) {\
-          SETERRQ2(1, "The number of independent variables exceeds the maximum compiled for!\n Edit your program and change Process adiC(%d) to Process adiC(%d)",pOsItIoN ,ad_GRAD_MAX);\
+          SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_USER, "The number of independent variables exceeds the maximum compiled for!\n Edit your program and change Process adiC(%d) to Process adiC(%d)",pOsItIoN ,ad_GRAD_MAX);\
         }\
         ad_AD_ClearGrad(DERIV_grad(var)); \
         DERIV_grad(var)[pOsItIoN] = 1; \
