@@ -8,9 +8,10 @@
 #include "finclude/petscisdef.h"
 #include "finclude/petscvecdef.h"
 #include "finclude/petscmatdef.h"
+#include "finclude/petscdmdef.h"
+#include "finclude/petscdmdadef.h"
+#include "finclude/petscdmmeshdef.h"
 #include "finclude/petscpcdef.h"
 #include "finclude/petsckspdef.h"
-#include "finclude/petscmeshdef.h"
 #include "finclude/petscsnesdef.h"
 #include "finclude/petsctsdef.h"
-#include "finclude/petscdadef.h"
