@@ -52,12 +52,12 @@
 !
 !  Possible arguments to KSPSetNormType()
 !
-      PetscEnum KSP_NORM_NO
+      PetscEnum KSP_NORM_NONE
       PetscEnum KSP_NORM_PRECONDITIONED
       PetscEnum KSP_NORM_UNPRECONDITIONED
       PetscEnum KSP_NORM_NATURAL
       
-      parameter (KSP_NORM_NO=0)
+      parameter (KSP_NORM_NONE=0)
       parameter (KSP_NORM_PRECONDITIONED=1)
       parameter (KSP_NORM_UNPRECONDITIONED=2)
       parameter (KSP_NORM_NATURAL=3) 
