@@ -10,6 +10,7 @@
 #if !defined(PETSC_USE_FORTRAN_DATATYPES)
 #define Vec PetscFortranAddr
 #define VecScatter PetscFortranAddr
+#define PetscSection PetscFortranAddr
 #endif
 
 #define NormType PetscEnum
@@ -21,7 +22,6 @@
 
 #define VECSEQ 'seq'
 #define VECMPI 'mpi'
-#define VECFETI 'feti'
 #define VECSHARED 'shared'
 #define VECESI 'esi'
 #define VECPETSCESI 'petscesi'
