@@ -1,6 +1,5 @@
-/*$Id: MPI_Wtime.c,v 1.18 2001/08/29 20:59:42 balay Exp $*/
 
-#include "petsc.h"
+#include <petscsys.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -34,6 +33,6 @@ int main(int argc,char **argv)
   y = MPI_Wtime();
   fprintf(stdout,"%-15s : %e sec - Slept for 10 sec \n","MPI_Wtime",(y-x));
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   PetscFunctionReturn(0);
 }
