@@ -1,5 +1,5 @@
 
-#include "petscsys.h"
+#include <petscsys.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -8,7 +8,7 @@ int main(int argc,char **argv)
   PetscLogDouble x,y;
   PetscLogEvent  e1;
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscInitialize(&argc,&argv,0,0);
   PetscLogEventRegister("*DummyEvent",0,&e1);
@@ -50,6 +50,6 @@ int main(int argc,char **argv)
   
   fprintf(stderr,"\n");
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   PetscFunctionReturn(0);
 }
