@@ -1,6 +1,5 @@
-/*$Id: PetscGetCPUTime.c,v 1.12 2001/08/29 20:59:41 balay Exp $*/
 
-#include "petsc.h"
+#include <petscsys.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -28,6 +27,6 @@ int main(int argc,char **argv)
     fprintf(stdout,"%-15s : %e sec\n","PetscGetCPUTime",(y-x)/10.0);
   }
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   PetscFunctionReturn(0);
 }
