@@ -1,5 +1,5 @@
 
-#include "petscsys.h"
+#include <petscsys.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -28,6 +28,6 @@ int main(int argc,char **argv)
     fprintf(stdout,"%-15s : %e sec\n","PetscGetTime",(y-x)/10.0);
   }
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   PetscFunctionReturn(0);
 }
