@@ -1,4 +1,5 @@
-#include "petscsys.h" 
+#include <petscsys.h> 
+#include <private/fortranimpl.h>
 
 #define max_colors  200
 #define max_nbtran 20
