@@ -4,12 +4,12 @@
 */
 #if !defined(__BCGSL)
 #define __BCGSL
-#include "petscsys.h"
+#include <petscsys.h>
 
 typedef struct {
   PetscInt        ell;		/* Number of search directions. */
   PetscReal	delta;		/* Threshold for recomputing exact residual norm */
-  PetscTruth	bConvex;	/* Compute Enhanced BiCGstab polynomial when set to PETSC_TRUE */
+  PetscBool 	bConvex;	/* Compute Enhanced BiCGstab polynomial when set to PETSC_TRUE */
   
   /* Workspace Vectors */
   Vec	vB;
