@@ -1,6 +1,5 @@
-#define PETSCKSP_DLL
 
-#include "include/private/kspimpl.h"
+#include <private/kspimpl.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "KSPSetUp_CR"
@@ -9,8 +8,8 @@ static PetscErrorCode KSPSetUp_CR(KSP ksp)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (ksp->pc_side == PC_RIGHT) {SETERRQ(PETSC_ERR_SUP,"no right preconditioning for KSPCR");}
-  else if (ksp->pc_side == PC_SYMMETRIC) {SETERRQ(PETSC_ERR_SUP,"no symmetric preconditioning for KSPCR");}
+  if (ksp->pc_side == PC_RIGHT) SETERRQ(((PetscObject)ksp)->comm,PETSC_ERR_SUP,"no right preconditioning for KSPCR");
+  else if (ksp->pc_side == PC_SYMMETRIC) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"no symmetric preconditioning for KSPCR");
   ierr = KSPDefaultGetWork(ksp,6);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -50,22 +49,28 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
   ierr = KSP_MatMult(ksp,Amat,P,AP);CHKERRQ(ierr);      /*   AP  <- A*P         */
   ierr = VecCopy(P,RT);CHKERRQ(ierr);                   /*   RT  <- P           */
   ierr = VecCopy(AP,ART);CHKERRQ(ierr);                 /*   ART <- AP          */
-  ierr = VecDot(RT,ART,&btop);CHKERRQ(ierr);          /*   (RT,ART)           */
+  ierr = VecDotBegin(RT,ART,&btop);CHKERRQ(ierr);          /*   (RT,ART)           */
+    
+  if (ksp->normtype == KSP_NORM_PRECONDITIONED) {
+    ierr = VecNormBegin(RT,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
+    ierr = VecDotEnd   (RT,ART,&btop) ;CHKERRQ(ierr);          /*   (RT,ART)           */
+    ierr = VecNormEnd  (RT,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
+  } else if (ksp->normtype == KSP_NORM_UNPRECONDITIONED) {
+    ierr = VecNormBegin(R,NORM_2,&dp);CHKERRQ(ierr);         /*   dp <- R'*R         */
+    ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);          /*   (RT,ART)           */
+    ierr = VecNormEnd  (R,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
+  } else if (ksp->normtype == KSP_NORM_NATURAL) {
+    ierr = VecDotEnd   (RT,ART,&btop) ;CHKERRQ(ierr);          /*   (RT,ART)           */
+    dp = PetscSqrtReal(PetscAbsScalar(btop));                    /* dp = sqrt(R,AR)      */
+  }
   if (PetscAbsScalar(btop) < 0.0) {
     ksp->reason = KSP_DIVERGED_INDEFINITE_MAT;
     ierr = PetscInfo(ksp,"diverging due to indefinite or negative definite matrix\n");CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-    
-  if (ksp->normtype == KSP_NORM_PRECONDITIONED) {
-    ierr = VecNorm(RT,NORM_2,&dp);CHKERRQ(ierr);        /*   dp <- RT'*RT       */
-  } else if (ksp->normtype == KSP_NORM_UNPRECONDITIONED) {
-    ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);         /*   dp <- R'*R         */
-  } else if (ksp->normtype == KSP_NORM_NATURAL) {
-    dp = sqrt(PetscAbsScalar(btop));                    /* dp = sqrt(R,AR)      */
-  }
+
   ksp->its = 0;
-  KSPMonitor(ksp,0,dp);
+  ierr = KSPMonitor(ksp,0,dp);CHKERRQ(ierr);
   ierr = PetscObjectTakeAccess(ksp);CHKERRQ(ierr);
   ksp->rnorm              = dp;
   ierr = PetscObjectGrantAccess(ksp);CHKERRQ(ierr);
@@ -80,7 +85,7 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
     ierr   = VecDot(AP,Q,&apq);CHKERRQ(ierr);  
     if (PetscAbsScalar(apq) <= 0.0) {
       ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
-      ierr = PetscInfo(ksp,"diverging due to indefinite or negative definite PC\n");CHKERRQ(ierr);
+      ierr = PetscInfo(ksp,"KSPSolve_CR:diverging due to indefinite or negative definite PC\n");CHKERRQ(ierr);
       break;
     }
     ai = btop/apq;                                      /* ai = (RT,ART)/(AP,Q)  */
@@ -89,24 +94,30 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
     ierr   = VecAXPY(RT,-ai,Q);CHKERRQ(ierr);           /*   RT  <- RT - ai*Q    */
     ierr   = KSP_MatMult(ksp,Amat,RT,ART);CHKERRQ(ierr);/*   ART <-   A*RT       */
     bbot = btop;
-    ierr   = VecDot(RT,ART,&btop);CHKERRQ(ierr);
-    if (PetscAbsScalar(btop) < 0.0) {
-      ksp->reason = KSP_DIVERGED_INDEFINITE_MAT;
-      ierr = PetscInfo(ksp,"diverging due to indefinite or negative definite matrix\n");CHKERRQ(ierr);
-      break;
-    }
+    ierr   = VecDotBegin(RT,ART,&btop);CHKERRQ(ierr);
 
     if (ksp->normtype == KSP_NORM_PRECONDITIONED) {
-      ierr = VecNorm(RT,NORM_2,&dp);CHKERRQ(ierr);      /*   dp <- || RT ||      */
+      ierr = VecNormBegin(RT,NORM_2,&dp);CHKERRQ(ierr);      /*   dp <- || RT ||      */
+      ierr = VecDotEnd   (RT,ART,&btop) ;CHKERRQ(ierr);
+      ierr = VecNormEnd  (RT,NORM_2,&dp);CHKERRQ(ierr);      /*   dp <- || RT ||      */
     } else if (ksp->normtype == KSP_NORM_NATURAL) {
-      dp = sqrt(PetscAbsScalar(btop));                  /* dp = sqrt(R,AR)       */
-    } else if (ksp->normtype == KSP_NORM_NO) {
+      ierr = VecDotEnd(RT,ART,&btop);CHKERRQ(ierr);
+      dp = PetscSqrtReal(PetscAbsScalar(btop));                  /* dp = sqrt(R,AR)       */
+    } else if (ksp->normtype == KSP_NORM_NONE) {
+      ierr = VecDotEnd(RT,ART,&btop);CHKERRQ(ierr);
       dp = 0.0; 
     } else if (ksp->normtype == KSP_NORM_UNPRECONDITIONED) {
-      ierr = VecAXPY(R,-ai,AP);CHKERRQ(ierr);           /*   R   <- R - ai*AP    */
-      ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);       /*   dp <- R'*R          */
+      ierr = VecAXPY(R,ai,AP);CHKERRQ(ierr);           /*   R   <- R - ai*AP    */
+      ierr = VecNormBegin(R,NORM_2,&dp);CHKERRQ(ierr);       /*   dp <- R'*R          */
+      ierr = VecDotEnd   (RT,ART,&btop);CHKERRQ(ierr);
+      ierr = VecNormEnd  (R,NORM_2,&dp);CHKERRQ(ierr);       /*   dp <- R'*R          */
     } else {
-      SETERRQ1(PETSC_ERR_SUP,"KSPNormType of %d not supported",(int)ksp->normtype);
+      SETERRQ1(((PetscObject)ksp)->comm,PETSC_ERR_SUP,"KSPNormType of %d not supported",(int)ksp->normtype);
+    }
+    if (PetscAbsScalar(btop) < 0.0) {
+      ksp->reason = KSP_DIVERGED_INDEFINITE_MAT;
+      ierr = PetscInfo(ksp,"diverging due to indefinite or negative definite PC\n");CHKERRQ(ierr);
+      break;
     }
 
     ierr = PetscObjectTakeAccess(ksp);CHKERRQ(ierr);
@@ -115,7 +126,7 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
     ierr = PetscObjectGrantAccess(ksp);CHKERRQ(ierr);
 
     KSPLogResidualHistory(ksp,dp);
-    KSPMonitor(ksp,i+1,dp);
+    ierr = KSPMonitor(ksp,i+1,dp);CHKERRQ(ierr);
     ierr = (*ksp->converged)(ksp,i+1,dp,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
     if (ksp->reason) break;
 
@@ -141,16 +152,28 @@ static PetscErrorCode  KSPSolve_CR(KSP ksp)
 
    Notes: The operator and the preconditioner must be symmetric for this method. The 
           preconditioner must be POSITIVE-DEFINITE and the operator POSITIVE-SEMIDEFINITE
+          Support only for left preconditioning.
+
+   References:
+   Methods of Conjugate Gradients for Solving Linear Systems, Magnus R. Hestenes and Eduard Stiefel,
+   Journal of Research of the National Bureau of Standards Vol. 49, No. 6, December 1952 Research Paper 2379
+   pp. 409--436.
+
 
 .seealso: KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPCG
 M*/
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "KSPCreate_CR"
-PetscErrorCode PETSCKSP_DLLEXPORT KSPCreate_CR(KSP ksp)
+PetscErrorCode  KSPCreate_CR(KSP ksp)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  ksp->pc_side                   = PC_LEFT;
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,1);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NATURAL,PC_LEFT,1);CHKERRQ(ierr);
+
   ksp->ops->setup                = KSPSetUp_CR;
   ksp->ops->solve                = KSPSolve_CR;
   ksp->ops->destroy              = KSPDefaultDestroy;
