@@ -2,7 +2,7 @@
 #define PCEUCLID      "euclid"
 #define PCSPBASICC    "spbasicc"
 
-PetscErrorCode PETSCKSP_DLLEXPORT KSPLSQRGetArnorm( KSP ksp,
+PetscErrorCode  KSPLSQRGetArnorm( KSP ksp,
      PetscReal *arnorm, PetscReal *rhs_norm , PetscReal *anorm);
 PetscErrorCode precond_lsqr_monitor(
                   KSP       solksp, /* Krylov Subspace method context */
