@@ -1,4 +1,4 @@
-#include "private/kspimpl.h"          /*I "petscksp.h" I*/
+#include <private/kspimpl.h>          /*I "petscksp.h" I*/
 
 #undef __FUNCT__
 #define __FUNCT__ "KSPPythonSetType"
@@ -20,15 +20,13 @@
 
 .seealso: KSPCreate(), KSPSetType(), KSPPYTHON, PetscPythonInitialize()
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT KSPPythonSetType(KSP ksp,const char pyname[])
+PetscErrorCode  KSPPythonSetType(KSP ksp,const char pyname[])
 {
-  PetscErrorCode (*f)(KSP, const char[]) = 0;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ksp,KSP_COOKIE,1);
+  PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
   PetscValidCharPointer(pyname,2);
-  ierr = PetscObjectQueryFunction((PetscObject)ksp,"KSPPythonSetType_C",(PetscVoidFunction*)&f);CHKERRQ(ierr);
-  if (f) {ierr = (*f)(ksp,pyname);CHKERRQ(ierr);}
+  ierr = PetscTryMethod(ksp,"KSPPythonSetType_C",(KSP, const char[]),(ksp,pyname));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
