@@ -1,6 +1,6 @@
 
-#include "petscksp.h"
-#include "src/sys/f90/f90impl.h"
+#include <petscksp.h>
+#include <../src/sys/f90-src/f90impl.h>
 
 #ifdef PETSC_HAVE_FORTRAN_CAPS
 #define kspgetresidualhistoryf90_     KSPGETRESIDUALHISTORYF90
@@ -9,7 +9,7 @@
 #endif
 
 EXTERN_C_BEGIN
-void PETSC_STDCALL kspgetresidualhistoryf90_(KSP *ksp,F90Array1d *indices,int *n,int *ierr PETSC_F90_2PTR_PROTO(ptrd))
+void PETSC_STDCALL kspgetresidualhistoryf90_(KSP *ksp,F90Array1d *indices,PetscInt *n,int *ierr PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscReal *hist;
   *ierr = KSPGetResidualHistory(*ksp,&hist,n); if (*ierr) return;
