@@ -1,5 +1,5 @@
-#include "private/fortranimpl.h"
-#include "petscpc.h"
+#include <private/fortranimpl.h>
+#include <petscpc.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define pccompositeaddpc_          PCCOMPOSITEADDPC
