@@ -1,10 +1,9 @@
-#define PETSCKSP_DLL
 
 /*
       Defines a preconditioner defined by R^T S R 
 */
-#include "private/pcimpl.h"   /*I "petscpc.h" I*/
-#include "petscksp.h"         /*I "petscksp.h" I*/
+#include <private/pcimpl.h>   /*I "petscpc.h" I*/
+#include <petscksp.h>         /*I "petscksp.h" I*/
 
 typedef struct {
   KSP  ksp;
@@ -34,13 +33,13 @@ static PetscErrorCode PCSetUp_Galerkin(PC pc)
 {
   PetscErrorCode  ierr;
   PC_Galerkin     *jac = (PC_Galerkin*)pc->data;
-  PetscTruth      a;
+  PetscBool       a;
   Vec             *xx,*yy;
 
   PetscFunctionBegin;
   if (!jac->x) {
     ierr = KSPGetOperatorsSet(jac->ksp,&a,PETSC_NULL);CHKERRQ(ierr);
-    if (!a) SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"Must set operator of PCGALERKIN KSP with PCGalerkinGetKSP()/KSPSetOperators()");
+    if (!a) SETERRQ(((PetscObject)pc)->comm,PETSC_ERR_ARG_WRONGSTATE,"Must set operator of PCGALERKIN KSP with PCGalerkinGetKSP()/KSPSetOperators()");
     ierr   = KSPGetVecs(jac->ksp,1,&xx,1,&yy);CHKERRQ(ierr);    
     jac->x = *xx;
     jac->b = *yy;
@@ -48,12 +47,28 @@ static PetscErrorCode PCSetUp_Galerkin(PC pc)
     ierr   = PetscFree(yy);CHKERRQ(ierr);
   }
   if (!jac->R && !jac->P) {
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"Must set restriction or interpolation of PCGALERKIN with PCGalerkinSetRestriction/Interpolation()");
+    SETERRQ(((PetscObject)pc)->comm,PETSC_ERR_ARG_WRONGSTATE,"Must set restriction or interpolation of PCGALERKIN with PCGalerkinSetRestriction/Interpolation()");
   }
   /* should check here that sizes of R/P match size of a */
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "PCReset_Galerkin"
+static PetscErrorCode PCReset_Galerkin(PC pc)
+{
+  PC_Galerkin     *jac = (PC_Galerkin*)pc->data;
+  PetscErrorCode   ierr;
+
+  PetscFunctionBegin;
+  ierr = MatDestroy(&jac->R);CHKERRQ(ierr);
+  ierr = MatDestroy(&jac->P);CHKERRQ(ierr);
+  ierr = VecDestroy(&jac->x);CHKERRQ(ierr);
+  ierr = VecDestroy(&jac->b);CHKERRQ(ierr);
+  ierr = KSPReset(jac->ksp);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "PCDestroy_Galerkin"
 static PetscErrorCode PCDestroy_Galerkin(PC pc)
@@ -62,12 +77,9 @@ static PetscErrorCode PCDestroy_Galerkin(PC pc)
   PetscErrorCode   ierr;
 
   PetscFunctionBegin;
-  if (jac->R) {ierr = MatDestroy(jac->R);CHKERRQ(ierr);}
-  if (jac->P) {ierr = MatDestroy(jac->P);CHKERRQ(ierr);}
-  if (jac->x) {ierr = VecDestroy(jac->x);CHKERRQ(ierr);}
-  if (jac->b) {ierr = VecDestroy(jac->b);CHKERRQ(ierr);}
-  ierr = KSPDestroy(jac->ksp);CHKERRQ(ierr);
-  ierr = PetscFree(jac);CHKERRQ(ierr);
+  ierr = PCReset_Galerkin(pc);CHKERRQ(ierr);
+  ierr = KSPDestroy(&jac->ksp);CHKERRQ(ierr);
+  ierr = PetscFree(pc->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -77,16 +89,16 @@ static PetscErrorCode PCView_Galerkin(PC pc,PetscViewer viewer)
 {
   PC_Galerkin     *jac = (PC_Galerkin*)pc->data;
   PetscErrorCode   ierr;
-  PetscTruth       iascii;
+  PetscBool        iascii;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"Galerkin PC\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"KSP on Galerkin follow\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"---------------------------------\n");CHKERRQ(ierr);
   } else {
-    SETERRQ1(PETSC_ERR_SUP,"Viewer type %s not supported for PCGalerkin",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)pc)->comm,PETSC_ERR_SUP,"Viewer type %s not supported for PCGalerkin",((PetscObject)viewer)->type_name);
   }
   ierr = KSPView(jac->ksp,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -95,7 +107,7 @@ static PetscErrorCode PCView_Galerkin(PC pc,PetscViewer viewer)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCGalerkinGetKSP_Galerkin"
-PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinGetKSP_Galerkin(PC pc,KSP *ksp)
+PetscErrorCode  PCGalerkinGetKSP_Galerkin(PC pc,KSP *ksp)
 {
   PC_Galerkin     *jac = (PC_Galerkin*)pc->data;
 
@@ -108,14 +120,14 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCGalerkinSetRestriction_Galerkin"
-PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetRestriction_Galerkin(PC pc,Mat R)
+PetscErrorCode  PCGalerkinSetRestriction_Galerkin(PC pc,Mat R)
 {
   PC_Galerkin     *jac = (PC_Galerkin*)pc->data;
   PetscErrorCode   ierr;
 
   PetscFunctionBegin;
   ierr = PetscObjectReference((PetscObject)R);CHKERRQ(ierr);
-  if (jac->R) {ierr = MatDestroy(jac->R);CHKERRQ(ierr);}
+  ierr = MatDestroy(&jac->R);CHKERRQ(ierr);
   jac->R = R;
   PetscFunctionReturn(0);
 }
@@ -124,14 +136,14 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCGalerkinSetInterpolation_Galerkin"
-PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetInterpolation_Galerkin(PC pc,Mat P)
+PetscErrorCode  PCGalerkinSetInterpolation_Galerkin(PC pc,Mat P)
 {
   PC_Galerkin     *jac = (PC_Galerkin*)pc->data;
   PetscErrorCode   ierr;
 
   PetscFunctionBegin;
   ierr = PetscObjectReference((PetscObject)P);CHKERRQ(ierr);
-  if (jac->P) {ierr = MatDestroy(jac->P);CHKERRQ(ierr);}
+  ierr = MatDestroy(&jac->P);CHKERRQ(ierr);
   jac->P = P;
   PetscFunctionReturn(0);
 }
@@ -143,7 +155,7 @@ EXTERN_C_END
 /*@
    PCGalerkinSetRestriction - Sets the restriction operator for the "Galerkin-type" preconditioner
    
-   Collective on PC
+   Logically Collective on PC
 
    Input Parameter:
 +  pc - the preconditioner context
@@ -159,16 +171,13 @@ EXTERN_C_END
            PCGalerkinSetInterpolation(), PCGalerkinGetKSP()
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetRestriction(PC pc,Mat R)
+PetscErrorCode  PCGalerkinSetRestriction(PC pc,Mat R)
 {
-  PetscErrorCode ierr,(*f)(PC,Mat);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCGalerkinSetRestriction_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc,R);CHKERRQ(ierr);
-  } 
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscTryMethod(pc,"PCGalerkinSetRestriction_C",(PC,Mat),(pc,R));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -177,7 +186,7 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetRestriction(PC pc,Mat R)
 /*@
    PCGalerkinSetInterpolation - Sets the interpolation operator for the "Galerkin-type" preconditioner
    
-   Collective on PC
+   Logically Collective on PC
 
    Input Parameter:
 +  pc - the preconditioner context
@@ -193,16 +202,13 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetRestriction(PC pc,Mat R)
            PCGalerkinSetRestriction(), PCGalerkinGetKSP()
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetInterpolation(PC pc,Mat P)
+PetscErrorCode  PCGalerkinSetInterpolation(PC pc,Mat P)
 {
-  PetscErrorCode ierr,(*f)(PC,Mat);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCGalerkinSetInterpolation_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc,P);CHKERRQ(ierr);
-  } 
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscTryMethod(pc,"PCGalerkinSetInterpolation_C",(PC,Mat),(pc,P));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -227,19 +233,14 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinSetInterpolation(PC pc,Mat P)
            PCGalerkinSetRestriction(), PCGalerkinSetInterpolation()
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCGalerkinGetKSP(PC pc,KSP *ksp)
+PetscErrorCode  PCGalerkinGetKSP(PC pc,KSP *ksp)
 {
-  PetscErrorCode ierr,(*f)(PC,KSP *);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   PetscValidPointer(ksp,2);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCGalerkinGetKSP_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc,ksp);CHKERRQ(ierr);
-  } else {
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Cannot get KSP, not Galerkin type");
-  }
+  ierr = PetscUseMethod(pc,"PCGalerkinGetKSP_C",(PC,KSP *),(pc,ksp));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -262,7 +263,7 @@ M*/
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCCreate_Galerkin"
-PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_Galerkin(PC pc)
+PetscErrorCode  PCCreate_Galerkin(PC pc)
 {
   PetscErrorCode ierr;
   PC_Galerkin    *jac;
@@ -271,6 +272,7 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_Galerkin(PC pc)
   ierr = PetscNewLog(pc,PC_Galerkin,&jac);CHKERRQ(ierr);
   pc->ops->apply              = PCApply_Galerkin;
   pc->ops->setup              = PCSetUp_Galerkin;
+  pc->ops->reset              = PCReset_Galerkin;
   pc->ops->destroy            = PCDestroy_Galerkin;
   pc->ops->view               = PCView_Galerkin;
   pc->ops->applyrichardson    = 0;
