@@ -1,10 +1,9 @@
-#define PETSCKSP_DLL
 
-#include "private/pcimpl.h"   /*I "petscpc.h" I*/
-#include "petscksp.h"            /*I "petscksp.h" I*/
+#include <private/pcimpl.h>   /*I "petscpc.h" I*/
+#include <petscksp.h>            /*I "petscksp.h" I*/
 
 typedef struct {
-  PetscTruth use_true_matrix;       /* use mat rather than pmat in inner linear solve */
+  PetscBool  use_true_matrix;       /* use mat rather than pmat in inner linear solve */
   KSP        ksp; 
   PetscInt   its;                   /* total number of iterations KSP uses */
 } PC_KSP;
@@ -12,7 +11,7 @@ typedef struct {
 
 #undef __FUNCT__  
 #define __FUNCT__ "PCKSPCreateKSP_KSP"
-static PetscErrorCode PETSCKSP_DLLEXPORT PCKSPCreateKSP_KSP(PC pc)
+static PetscErrorCode  PCKSPCreateKSP_KSP(PC pc)
 {
   PetscErrorCode ierr;
   const char     *prefix;
@@ -65,7 +64,7 @@ static PetscErrorCode PCSetUp_KSP(PC pc)
   PetscErrorCode ierr;
   PC_KSP         *jac = (PC_KSP*)pc->data;
   Mat            mat;
-  PetscTruth     A;
+  PetscBool      A;
 
   PetscFunctionBegin;
   if (!jac->ksp) {ierr = PCKSPCreateKSP_KSP(pc);CHKERRQ(ierr);}
@@ -82,6 +81,18 @@ static PetscErrorCode PCSetUp_KSP(PC pc)
 }
 
 /* Default destroy, if it has never been setup */
+#undef __FUNCT__  
+#define __FUNCT__ "PCReset_KSP"
+static PetscErrorCode PCReset_KSP(PC pc)
+{
+  PC_KSP         *jac = (PC_KSP*)pc->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (jac->ksp) {ierr = KSPReset(jac->ksp);CHKERRQ(ierr);}
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "PCDestroy_KSP"
 static PetscErrorCode PCDestroy_KSP(PC pc)
@@ -90,8 +101,9 @@ static PetscErrorCode PCDestroy_KSP(PC pc)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (jac->ksp) {ierr = KSPDestroy(jac->ksp);CHKERRQ(ierr);}
-  ierr = PetscFree(jac);CHKERRQ(ierr);
+  ierr = PCReset_KSP(pc);CHKERRQ(ierr);
+  ierr = KSPDestroy(&jac->ksp);CHKERRQ(ierr);
+  ierr = PetscFree(pc->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -101,11 +113,11 @@ static PetscErrorCode PCView_KSP(PC pc,PetscViewer viewer)
 {
   PC_KSP         *jac = (PC_KSP*)pc->data;
   PetscErrorCode ierr;
-  PetscTruth     iascii;
+  PetscBool      iascii;
 
   PetscFunctionBegin;
   if (!jac->ksp) {ierr = PCKSPCreateKSP_KSP(pc);CHKERRQ(ierr);}
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     if (jac->use_true_matrix) {
       ierr = PetscViewerASCIIPrintf(viewer,"Using true matrix (not preconditioner matrix) on inner solve\n");CHKERRQ(ierr);
@@ -113,7 +125,7 @@ static PetscErrorCode PCView_KSP(PC pc,PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"KSP and PC on KSP preconditioner follow\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"---------------------------------\n");CHKERRQ(ierr);
   } else {
-    SETERRQ1(PETSC_ERR_SUP,"Viewer type %s not supported for this object",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)pc)->comm,PETSC_ERR_SUP,"Viewer type %s not supported for this object",((PetscObject)viewer)->type_name);
   }
   ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
   ierr = KSPView(jac->ksp,viewer);CHKERRQ(ierr);
@@ -129,11 +141,11 @@ static PetscErrorCode PCView_KSP(PC pc,PetscViewer viewer)
 static PetscErrorCode PCSetFromOptions_KSP(PC pc)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg = PETSC_FALSE;
+  PetscBool      flg = PETSC_FALSE;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead("KSP preconditioner options");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-pc_ksp_true","Use true matrix to define inner linear system, not preconditioner matrix","PCKSPSetUseTrue",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-pc_ksp_true","Use true matrix to define inner linear system, not preconditioner matrix","PCKSPSetUseTrue",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
     if (flg) {
       ierr = PCKSPSetUseTrue(pc);CHKERRQ(ierr);
     }
@@ -146,7 +158,7 @@ static PetscErrorCode PCSetFromOptions_KSP(PC pc)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCKSPSetUseTrue_KSP"
-PetscErrorCode PETSCKSP_DLLEXPORT PCKSPSetUseTrue_KSP(PC pc)
+PetscErrorCode  PCKSPSetUseTrue_KSP(PC pc)
 {
   PC_KSP   *jac;
 
@@ -160,7 +172,7 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCKSPGetKSP_KSP"
-PetscErrorCode PETSCKSP_DLLEXPORT PCKSPGetKSP_KSP(PC pc,KSP *ksp)
+PetscErrorCode  PCKSPGetKSP_KSP(PC pc,KSP *ksp)
 {
   PC_KSP         *jac = (PC_KSP*)pc->data;
   PetscErrorCode ierr;
@@ -179,7 +191,7 @@ EXTERN_C_END
    the matrix used to define the preconditioner) is used to compute the
    residual inside the inner solve.
 
-   Collective on PC
+   Logically Collective on PC
 
    Input Parameters:
 .  pc - the preconditioner context
@@ -197,16 +209,13 @@ EXTERN_C_END
 
 .seealso: PCSetOperators(), PCBJacobiSetUseTrueLocal()
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCKSPSetUseTrue(PC pc)
+PetscErrorCode  PCKSPSetUseTrue(PC pc)
 {
-  PetscErrorCode ierr,(*f)(PC);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCKSPSetUseTrue_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscTryMethod(pc,"PCKSPSetUseTrue_C",(PC),(pc));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -230,17 +239,14 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCKSPSetUseTrue(PC pc)
 
 .keywords:  PC, KSP, get, context
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCKSPGetKSP(PC pc,KSP *ksp)
+PetscErrorCode  PCKSPGetKSP(PC pc,KSP *ksp)
 {
-  PetscErrorCode ierr,(*f)(PC,KSP*);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   PetscValidPointer(ksp,2);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCKSPGetKSP_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc,ksp);CHKERRQ(ierr);
-  }
+  ierr = PetscTryMethod(pc,"PCKSPGetKSP_C",(PC,KSP*),(pc,ksp));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -262,6 +268,9 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCKSPGetKSP(PC pc,KSP *ksp)
    Notes: Using a Krylov method inside another Krylov method can be dangerous (you get divergence or
           the incorrect answer) unless you use KSPFGMRES as the other Krylov method
 
+   Developer Notes: PCApply_KSP() uses the flag set by PCSetInitialGuessNonzero(), I think this is totally wrong, because it is then not
+     using this inner KSP as a preconditioner (that is a linear operator applied to some vector), it is actually just using 
+     the inner KSP just like the outer KSP.
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC,
            PCSHELL, PCCOMPOSITE, PCKSPUseTrue(), PCKSPGetKSP()
@@ -271,7 +280,7 @@ M*/
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCCreate_KSP"
-PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_KSP(PC pc)
+PetscErrorCode  PCCreate_KSP(PC pc)
 {
   PetscErrorCode ierr;
   PC_KSP         *jac;
@@ -281,6 +290,7 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_KSP(PC pc)
   pc->ops->apply              = PCApply_KSP;
   pc->ops->applytranspose     = PCApplyTranspose_KSP;
   pc->ops->setup              = PCSetUp_KSP;
+  pc->ops->reset              = PCReset_KSP;
   pc->ops->destroy            = PCDestroy_KSP;
   pc->ops->setfromoptions     = PCSetFromOptions_KSP;
   pc->ops->view               = PCView_KSP;
