@@ -1,4 +1,3 @@
-#define PETSCKSP_DLL
 
 /**********************************ivec.c**************************************
 
@@ -15,7 +14,7 @@ Last Modification:
 6.21.97
 ***********************************ivec.c*************************************/
 
-#include "../src/ksp/pc/impls/tfs/tfs.h"
+#include <../src/ksp/pc/impls/tfs/tfs.h>
 
 /* sorting args ivec.c ivec.c ... */
 #define   SORT_OPT	6     
@@ -188,7 +187,7 @@ PetscErrorCode ivec_non_uniform(PetscInt *arg1, PetscInt *arg2,  PetscInt n,  Pe
       else if (type == GL_L_AND)   
 	{ivec_land(arg1,arg2,j);}
       else
-	{SETERRQ(PETSC_ERR_PLIB,"unrecognized type passed to ivec_non_uniform()!");}
+	SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"unrecognized type passed to ivec_non_uniform()!");
 
       arg1+=j; arg2+=j; i+=j;
     }
@@ -279,8 +278,7 @@ PetscErrorCode ivec_sort( PetscInt *ar,  PetscInt size)
 	  SWAP(*ar,*pj)
 
 	  /* test stack_size to see if we've exhausted our stack */
-	  if (top_s-bottom_s >= SORT_STACK)
-	    {SETERRQ(PETSC_ERR_PLIB,"ivec_sort() :: STACK EXHAUSTED!!!");}
+	  if (top_s-bottom_s >= SORT_STACK) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"ivec_sort() :: STACK EXHAUSTED!!!");
 
 	  /* push right hand child iff length > 1 */
 	  if ((*top_s = size-((PetscInt) (pi-ar))))
@@ -389,8 +387,7 @@ PetscErrorCode ivec_sort_companion( PetscInt *ar,  PetscInt *ar2,  PetscInt size
 	  SWAP(*ar2,*pj2)
 
 	  /* test stack_size to see if we've exhausted our stack */
-	  if (top_s-bottom_s >= SORT_STACK)
-	    {SETERRQ(PETSC_ERR_PLIB,"ivec_sort_companion() :: STACK EXHAUSTED!!!");}
+	  if (top_s-bottom_s >= SORT_STACK) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"ivec_sort_companion() :: STACK EXHAUSTED!!!");
 
 	  /* push right hand child iff length > 1 */
 	  if ((*top_s = size-((PetscInt) (pi-ar))))
@@ -505,8 +502,7 @@ PetscErrorCode ivec_sort_companion_hack( PetscInt *ar,  PetscInt **ar2, PetscInt
 	  P_SWAP(*ar2,*pj2)
 
 	  /* test stack_size to see if we've exhausted our stack */
-	  if (top_s-bottom_s >= SORT_STACK)
-         {SETERRQ(PETSC_ERR_PLIB,"ivec_sort_companion_hack() :: STACK EXHAUSTED!!!");}
+	  if (top_s-bottom_s >= SORT_STACK) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"ivec_sort_companion_hack() :: STACK EXHAUSTED!!!");
 
 	  /* push right hand child iff length > 1 */
 	  if ((*top_s = size-((PetscInt) (pi-ar))))
@@ -562,32 +558,20 @@ PetscErrorCode ivec_sort_companion_hack( PetscInt *ar,  PetscInt **ar2, PetscInt
 PetscErrorCode SMI_sort(void *ar1, void *ar2, PetscInt size, PetscInt type)
 {
   PetscFunctionBegin;
-  if (type == SORT_INTEGER)
-    {
-      if (ar2)
-	{ivec_sort_companion((PetscInt*)ar1,(PetscInt*)ar2,size);}
-      else
-	{ivec_sort((PetscInt*)ar1,size);}
-    }
-  else if (type == SORT_INT_PTR)
-    {
-      if (ar2)
-	{ivec_sort_companion_hack((PetscInt*)ar1,(PetscInt **)ar2,size);}
-      else
-	{ivec_sort((PetscInt*)ar1,size);}
-    }
-
-  else
-    {
-      SETERRQ(PETSC_ERR_PLIB,"SMI_sort only does SORT_INTEGER!");
-    }
+  if (type == SORT_INTEGER) {
+    if (ar2) ivec_sort_companion((PetscInt*)ar1,(PetscInt*)ar2,size);
+    else ivec_sort((PetscInt*)ar1,size);
+  } else if (type == SORT_INT_PTR) {
+    if (ar2) ivec_sort_companion_hack((PetscInt*)ar1,(PetscInt **)ar2,size);
+    else ivec_sort((PetscInt*)ar1,size);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"SMI_sort only does SORT_INTEGER!");
   PetscFunctionReturn(0);
 }
 
 /***********************************ivec.c*************************************/
 PetscInt ivec_linear_search( PetscInt item,  PetscInt *list,  PetscInt n)
 {
-   PetscInt tmp = n-1;
+  PetscInt tmp = n-1;
   PetscFunctionBegin;
   while (n--)  {if (*list++ == item) {return(tmp-n);}}
   return(-1);
@@ -741,8 +725,7 @@ PetscErrorCode rvec_non_uniform(PetscScalar *arg1, PetscScalar *arg2,  PetscInt
 	{rvec_min_abs(arg1,arg2,j);}
       else if (type == GL_EXISTS)
 	{rvec_exists(arg1,arg2,j);}
-      else
-	{SETERRQ(PETSC_ERR_PLIB,"unrecognized type passed to rvec_non_uniform()!");}
+      else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"unrecognized type passed to rvec_non_uniform()!");
 
       arg1+=j; arg2+=j; i+=j;
     }
