@@ -1,16 +1,12 @@
-#define PETSCKSP_DLL
 
-
-#include "petscpc.h"   /*I "petscpc.h" I*/
-#include "petscmg.h"   /*I "petscmg.h" I*/
-#include "petscda.h"   /*I "petscda.h" I*/
-#include "../src/ksp/pc/impls/mg/mgimpl.h"
+#include <petscpcmg.h>   /*I "petscpcmg.h" I*/
+#include <petscdmda.h>   /*I "petscdmda.h" I*/
+#include <../src/ksp/pc/impls/mg/mgimpl.h>
 
 typedef struct {
-  DA           da;
   PCExoticType type;
   Mat          P;            /* the constructed interpolation matrix */
-  PetscTruth   directSolve;  /* use direct LU factorization to construct interpolation */
+  PetscBool    directSolve;  /* use direct LU factorization to construct interpolation */
   KSP          ksp;
 } PC_Exotic;
 
@@ -18,14 +14,13 @@ const char *PCExoticTypes[] = {"face","wirebasket","PCExoticType","PC_Exotic",0}
 
 
 #undef __FUNCT__  
-#define __FUNCT__ "DAGetWireBasketInterpolation"
+#define __FUNCT__ "DMDAGetWireBasketInterpolation"
 /*
-      DAGetWireBasketInterpolation - Gets the interpolation for a wirebasket based coarse space
+      DMDAGetWireBasketInterpolation - Gets the interpolation for a wirebasket based coarse space
 
 */
-PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reuse,Mat *P)
+PetscErrorCode DMDAGetWireBasketInterpolation(DM da,PC_Exotic *exotic,Mat Aglobal,MatReuse reuse,Mat *P)
 {
-  DA                     da = exotic->da;
   PetscErrorCode         ierr;
   PetscInt               dim,i,j,k,m,n,p,dof,Nint,Nface,Nwire,Nsurf,*Iint,*Isurf,cint = 0,csurf = 0,istart,jstart,kstart,*II,N,c = 0;
   PetscInt               mwidth,nwidth,pwidth,cnt,mp,np,pp,Ntotal,gl[26],*globals,Ng,*IIint,*IIsurf;
@@ -42,11 +37,11 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
   PetscTable             ht;
 
   PetscFunctionBegin;
-  ierr = DAGetInfo(da,&dim,0,0,0,&mp,&np,&pp,&dof,0,0,0);CHKERRQ(ierr);
-  if (dof != 1) SETERRQ(PETSC_ERR_SUP,"Only for single field problems");
-  if (dim != 3) SETERRQ(PETSC_ERR_SUP,"Only coded for 3d problems");
-  ierr = DAGetCorners(da,0,0,0,&m,&n,&p);CHKERRQ(ierr);
-  ierr = DAGetGhostCorners(da,&istart,&jstart,&kstart,&mwidth,&nwidth,&pwidth);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(da,&dim,0,0,0,&mp,&np,&pp,&dof,0,0,0,0,0);CHKERRQ(ierr);
+  if (dof != 1) SETERRQ(((PetscObject)da)->comm,PETSC_ERR_SUP,"Only for single field problems");
+  if (dim != 3) SETERRQ(((PetscObject)da)->comm,PETSC_ERR_SUP,"Only coded for 3d problems");
+  ierr = DMDAGetCorners(da,0,0,0,&m,&n,&p);CHKERRQ(ierr);
+  ierr = DMDAGetGhostCorners(da,&istart,&jstart,&kstart,&mwidth,&nwidth,&pwidth);CHKERRQ(ierr);
   istart = istart ? -1 : 0;
   jstart = jstart ? -1 : 0;
   kstart = kstart ? -1 : 0;
@@ -77,9 +72,9 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
      Require that all 12 edges and 6 faces have at least one grid point. Otherwise some of the columns of 
      Xsurf will be all zero (thus making the coarse matrix singular). 
   */
-  if (m-istart < 3) SETERRQ(PETSC_ERR_SUP,"Number of grid points per process in X direction must be at least 3");
-  if (n-jstart < 3) SETERRQ(PETSC_ERR_SUP,"Number of grid points per process in Y direction must be at least 3");
-  if (p-kstart < 3) SETERRQ(PETSC_ERR_SUP,"Number of grid points per process in Z direction must be at least 3");
+  if (m-istart < 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Number of grid points per process in X direction must be at least 3");
+  if (n-jstart < 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Number of grid points per process in Y direction must be at least 3");
+  if (p-kstart < 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Number of grid points per process in Z direction must be at least 3");
 
   cnt = 0;
   xsurf[cnt++] = 1; for (i=1; i<m-istart-1; i++) { xsurf[cnt++ + Nsurf] = 1;} xsurf[cnt++ + 2*Nsurf] = 1;
@@ -101,7 +96,7 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
     for (j=0; j<26; j++) {
       tmp += xsurf[i+j*Nsurf];
     }
-    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_ERR_PLIB,"Wrong Xsurf interpolation at i %D value %G",i,PetscAbsScalar(tmp));
+    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Wrong Xsurf interpolation at i %D value %G",i,PetscAbsScalar(tmp));
   }
 #endif
   ierr = MatRestoreArray(Xsurf,&xsurf);CHKERRQ(ierr);
@@ -112,7 +107,7 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
        I are the indices for all the needed vertices (in global numbering)
        Iint are the indices for the interior values, I surf for the surface values
             (This is just for the part of the global matrix obtained with MatGetSubMatrix(), it 
-             is NOT the local DA ordering.)
+             is NOT the local DMDA ordering.)
        IIint and IIsurf are the same as the Iint, Isurf except they are in the global numbering
   */
 #define Endpoint(a,start,b) (a == 0 || a == (b-1-start))
@@ -133,17 +128,17 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
       }
     }
   }
-  if (c != N) SETERRQ(PETSC_ERR_PLIB,"c != N");
-  if (cint != Nint) SETERRQ(PETSC_ERR_PLIB,"cint != Nint");
-  if (csurf != Nsurf) SETERRQ(PETSC_ERR_PLIB,"csurf != Nsurf");
-  ierr = DAGetISLocalToGlobalMapping(da,&ltg);CHKERRQ(ierr);
+  if (c != N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"c != N");
+  if (cint != Nint) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"cint != Nint");
+  if (csurf != Nsurf) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"csurf != Nsurf");
+  ierr = DMGetLocalToGlobalMapping(da,&ltg);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingApply(ltg,N,II,II);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingApply(ltg,Nint,IIint,IIint);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingApply(ltg,Nsurf,IIsurf,IIsurf);CHKERRQ(ierr);
   ierr = PetscObjectGetComm((PetscObject)da,&comm);CHKERRQ(ierr);
-  ierr = ISCreateGeneral(comm,N,II,&is);CHKERRQ(ierr);
-  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nint,Iint,&isint);CHKERRQ(ierr);
-  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nsurf,Isurf,&issurf);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(comm,N,II,PETSC_COPY_VALUES,&is);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nint,Iint,PETSC_COPY_VALUES,&isint);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nsurf,Isurf,PETSC_COPY_VALUES,&issurf);CHKERRQ(ierr);
   ierr = PetscFree3(II,Iint,Isurf);CHKERRQ(ierr);
 
   ierr = MatGetSubMatrices(Aglobal,1,&is,&is,MAT_INITIAL_MATRIX,&Aholder);CHKERRQ(ierr);
@@ -161,15 +156,15 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
   ierr = MatMatMult(Ais,Xsurf,MAT_REUSE_MATRIX,PETSC_DETERMINE,&Xint_tmp);CHKERRQ(ierr);
   ierr = MatScale(Xint_tmp,-1.0);CHKERRQ(ierr);
   if (exotic->directSolve) {
-    ierr = MatGetFactor(Aii,MAT_SOLVER_PETSC,MAT_FACTOR_LU,&iAii);CHKERRQ(ierr);
+    ierr = MatGetFactor(Aii,MATSOLVERPETSC,MAT_FACTOR_LU,&iAii);CHKERRQ(ierr);
     ierr = MatFactorInfoInitialize(&info);CHKERRQ(ierr);
-    ierr = MatGetOrdering(Aii,MATORDERING_ND,&row,&col);CHKERRQ(ierr);
+    ierr = MatGetOrdering(Aii,MATORDERINGND,&row,&col);CHKERRQ(ierr);
     ierr = MatLUFactorSymbolic(iAii,Aii,row,col,&info);CHKERRQ(ierr);
-    ierr = ISDestroy(row);CHKERRQ(ierr);
-    ierr = ISDestroy(col);CHKERRQ(ierr);
+    ierr = ISDestroy(&row);CHKERRQ(ierr);
+    ierr = ISDestroy(&col);CHKERRQ(ierr);
     ierr = MatLUFactorNumeric(iAii,Aii,&info);CHKERRQ(ierr);
     ierr = MatMatSolve(iAii,Xint_tmp,Xint);CHKERRQ(ierr);
-    ierr = MatDestroy(iAii);CHKERRQ(ierr);
+    ierr = MatDestroy(&iAii);CHKERRQ(ierr);
   } else {
     Vec         b,x;
     PetscScalar *xint_tmp;
@@ -188,10 +183,10 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
     }
     ierr = MatRestoreArray(Xint,&xint);CHKERRQ(ierr);
     ierr = MatRestoreArray(Xint_tmp,&xint_tmp);CHKERRQ(ierr);
-    ierr = VecDestroy(x);CHKERRQ(ierr);
-    ierr = VecDestroy(b);CHKERRQ(ierr);
+    ierr = VecDestroy(&x);CHKERRQ(ierr);
+    ierr = VecDestroy(&b);CHKERRQ(ierr);
   }
-  ierr = MatDestroy(Xint_tmp);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xint_tmp);CHKERRQ(ierr);
 
 #if defined(PETSC_USE_DEBUG_foo)
   ierr = MatGetArray(Xint,&xint);CHKERRQ(ierr);
@@ -200,7 +195,7 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
     for (j=0; j<26; j++) {
       tmp += xint[i+j*Nint];
     }
-    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_ERR_PLIB,"Wrong Xint interpolation at i %D value %G",i,PetscAbsScalar(tmp));
+    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Wrong Xint interpolation at i %D value %G",i,PetscAbsScalar(tmp));
   }
   ierr = MatRestoreArray(Xint,&xint);CHKERRQ(ierr);
   /* ierr =MatView(Xint,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr); */
@@ -239,13 +234,13 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
     ierr = PetscTableAddCount(ht,globals[i]+1);CHKERRQ(ierr);
   }
   ierr = PetscTableGetCount(ht,&cnt);CHKERRQ(ierr);
-  if (cnt != Ntotal) SETERRQ2(PETSC_ERR_PLIB,"Hash table size %D not equal to total number coarse grid points %D",cnt,Ntotal);
+  if (cnt != Ntotal) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Hash table size %D not equal to total number coarse grid points %D",cnt,Ntotal);
   ierr = PetscFree(globals);CHKERRQ(ierr);
   for (i=0; i<26; i++) {
     ierr = PetscTableFind(ht,gl[i]+1,&gl[i]);CHKERRQ(ierr);
     gl[i]--;
   }
-  ierr = PetscTableDestroy(ht);CHKERRQ(ierr);
+  ierr = PetscTableDestroy(&ht);CHKERRQ(ierr);
   /* PetscIntView(26,gl,PETSC_VIEWER_STDOUT_WORLD); */
 
   /* construct global interpolation matrix */
@@ -276,7 +271,7 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
     ierr = MatMult(*P,x,y);CHKERRQ(ierr);
     ierr = VecGetArray(y,&yy);CHKERRQ(ierr);
     for (i=0; i<Ng; i++) {
-      if (PetscAbsScalar(yy[i]-1.0) > 1.e-10) SETERRQ2(PETSC_ERR_PLIB,"Wrong p interpolation at i %D value %G",i,PetscAbsScalar(yy[i]));
+      if (PetscAbsScalar(yy[i]-1.0) > 1.e-10) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Wrong p interpolation at i %D value %G",i,PetscAbsScalar(yy[i]));
     }
     ierr = VecRestoreArray(y,&yy);CHKERRQ(ierr);
     ierr = VecDestroy(x);CHKERRQ(ierr);
@@ -284,27 +279,26 @@ PetscErrorCode DAGetWireBasketInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReu
   }
 #endif
     
-  ierr = MatDestroy(Aii);CHKERRQ(ierr);
-  ierr = MatDestroy(Ais);CHKERRQ(ierr);
-  ierr = MatDestroy(Asi);CHKERRQ(ierr);
-  ierr = MatDestroy(A);CHKERRQ(ierr);
-  ierr = ISDestroy(is);CHKERRQ(ierr);
-  ierr = ISDestroy(isint);CHKERRQ(ierr);
-  ierr = ISDestroy(issurf);CHKERRQ(ierr);
-  ierr = MatDestroy(Xint);CHKERRQ(ierr);
-  ierr = MatDestroy(Xsurf);CHKERRQ(ierr);
+  ierr = MatDestroy(&Aii);CHKERRQ(ierr);
+  ierr = MatDestroy(&Ais);CHKERRQ(ierr);
+  ierr = MatDestroy(&Asi);CHKERRQ(ierr);
+  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  ierr = ISDestroy(&is);CHKERRQ(ierr);
+  ierr = ISDestroy(&isint);CHKERRQ(ierr);
+  ierr = ISDestroy(&issurf);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xint);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xsurf);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "DAGetFaceInterpolation"
+#define __FUNCT__ "DMDAGetFaceInterpolation"
 /*
-      DAGetFaceInterpolation - Gets the interpolation for a face based coarse space
+      DMDAGetFaceInterpolation - Gets the interpolation for a face based coarse space
 
 */
-PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reuse,Mat *P)
+PetscErrorCode DMDAGetFaceInterpolation(DM da,PC_Exotic *exotic,Mat Aglobal,MatReuse reuse,Mat *P)
 {
-  DA                     da = exotic->da;
   PetscErrorCode         ierr;
   PetscInt               dim,i,j,k,m,n,p,dof,Nint,Nface,Nwire,Nsurf,*Iint,*Isurf,cint = 0,csurf = 0,istart,jstart,kstart,*II,N,c = 0;
   PetscInt               mwidth,nwidth,pwidth,cnt,mp,np,pp,Ntotal,gl[6],*globals,Ng,*IIint,*IIsurf;
@@ -321,11 +315,11 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
   PetscTable             ht;
 
   PetscFunctionBegin;
-  ierr = DAGetInfo(da,&dim,0,0,0,&mp,&np,&pp,&dof,0,0,0);CHKERRQ(ierr);
-  if (dof != 1) SETERRQ(PETSC_ERR_SUP,"Only for single field problems");
-  if (dim != 3) SETERRQ(PETSC_ERR_SUP,"Only coded for 3d problems");
-  ierr = DAGetCorners(da,0,0,0,&m,&n,&p);CHKERRQ(ierr);
-  ierr = DAGetGhostCorners(da,&istart,&jstart,&kstart,&mwidth,&nwidth,&pwidth);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(da,&dim,0,0,0,&mp,&np,&pp,&dof,0,0,0,0,0);CHKERRQ(ierr);
+  if (dof != 1) SETERRQ(((PetscObject)da)->comm,PETSC_ERR_SUP,"Only for single field problems");
+  if (dim != 3) SETERRQ(((PetscObject)da)->comm,PETSC_ERR_SUP,"Only coded for 3d problems");
+  ierr = DMDAGetCorners(da,0,0,0,&m,&n,&p);CHKERRQ(ierr);
+  ierr = DMDAGetGhostCorners(da,&istart,&jstart,&kstart,&mwidth,&nwidth,&pwidth);CHKERRQ(ierr);
   istart = istart ? -1 : 0;
   jstart = jstart ? -1 : 0;
   kstart = kstart ? -1 : 0;
@@ -356,9 +350,9 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
      Require that all 12 edges and 6 faces have at least one grid point. Otherwise some of the columns of 
      Xsurf will be all zero (thus making the coarse matrix singular). 
   */
-  if (m-istart < 3) SETERRQ(PETSC_ERR_SUP,"Number of grid points per process in X direction must be at least 3");
-  if (n-jstart < 3) SETERRQ(PETSC_ERR_SUP,"Number of grid points per process in Y direction must be at least 3");
-  if (p-kstart < 3) SETERRQ(PETSC_ERR_SUP,"Number of grid points per process in Z direction must be at least 3");
+  if (m-istart < 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Number of grid points per process in X direction must be at least 3");
+  if (n-jstart < 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Number of grid points per process in Y direction must be at least 3");
+  if (p-kstart < 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Number of grid points per process in Z direction must be at least 3");
 
   cnt = 0;
   for (j=1;j<n-1-jstart;j++) {  for (i=1; i<m-istart-1; i++) { xsurf[cnt++ + 0*Nsurf] = 1;} }
@@ -375,7 +369,7 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
     for (j=0; j<6; j++) {
       tmp += xsurf[i+j*Nsurf];
     }
-    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_ERR_PLIB,"Wrong Xsurf interpolation at i %D value %G",i,PetscAbsScalar(tmp));
+    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Wrong Xsurf interpolation at i %D value %G",i,PetscAbsScalar(tmp));
   }
 #endif
   ierr = MatRestoreArray(Xsurf,&xsurf);CHKERRQ(ierr);
@@ -386,7 +380,7 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
        I are the indices for all the needed vertices (in global numbering)
        Iint are the indices for the interior values, I surf for the surface values
             (This is just for the part of the global matrix obtained with MatGetSubMatrix(), it 
-             is NOT the local DA ordering.)
+             is NOT the local DMDA ordering.)
        IIint and IIsurf are the same as the Iint, Isurf except they are in the global numbering
   */
 #define Endpoint(a,start,b) (a == 0 || a == (b-1-start))
@@ -407,17 +401,17 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
       }
     }
   }
-  if (c != N) SETERRQ(PETSC_ERR_PLIB,"c != N");
-  if (cint != Nint) SETERRQ(PETSC_ERR_PLIB,"cint != Nint");
-  if (csurf != Nsurf) SETERRQ(PETSC_ERR_PLIB,"csurf != Nsurf");
-  ierr = DAGetISLocalToGlobalMapping(da,&ltg);CHKERRQ(ierr);
+  if (c != N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"c != N");
+  if (cint != Nint) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"cint != Nint");
+  if (csurf != Nsurf) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"csurf != Nsurf");
+  ierr = DMGetLocalToGlobalMapping(da,&ltg);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingApply(ltg,N,II,II);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingApply(ltg,Nint,IIint,IIint);CHKERRQ(ierr);
   ierr = ISLocalToGlobalMappingApply(ltg,Nsurf,IIsurf,IIsurf);CHKERRQ(ierr);
   ierr = PetscObjectGetComm((PetscObject)da,&comm);CHKERRQ(ierr);
-  ierr = ISCreateGeneral(comm,N,II,&is);CHKERRQ(ierr);
-  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nint,Iint,&isint);CHKERRQ(ierr);
-  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nsurf,Isurf,&issurf);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(comm,N,II,PETSC_COPY_VALUES,&is);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nint,Iint,PETSC_COPY_VALUES,&isint);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF,Nsurf,Isurf,PETSC_COPY_VALUES,&issurf);CHKERRQ(ierr);
   ierr = PetscFree3(II,Iint,Isurf);CHKERRQ(ierr);
 
   ierr = MatGetSubMatrices(Aglobal,1,&is,&is,MAT_INITIAL_MATRIX,&Aholder);CHKERRQ(ierr);
@@ -436,15 +430,15 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
   ierr = MatScale(Xint_tmp,-1.0);CHKERRQ(ierr);
 
   if (exotic->directSolve) {
-    ierr = MatGetFactor(Aii,MAT_SOLVER_PETSC,MAT_FACTOR_LU,&iAii);CHKERRQ(ierr);
+    ierr = MatGetFactor(Aii,MATSOLVERPETSC,MAT_FACTOR_LU,&iAii);CHKERRQ(ierr);
     ierr = MatFactorInfoInitialize(&info);CHKERRQ(ierr);
-    ierr = MatGetOrdering(Aii,MATORDERING_ND,&row,&col);CHKERRQ(ierr);
+    ierr = MatGetOrdering(Aii,MATORDERINGND,&row,&col);CHKERRQ(ierr);
     ierr = MatLUFactorSymbolic(iAii,Aii,row,col,&info);CHKERRQ(ierr);
-    ierr = ISDestroy(row);CHKERRQ(ierr);
-    ierr = ISDestroy(col);CHKERRQ(ierr);
+    ierr = ISDestroy(&row);CHKERRQ(ierr);
+    ierr = ISDestroy(&col);CHKERRQ(ierr);
     ierr = MatLUFactorNumeric(iAii,Aii,&info);CHKERRQ(ierr);
     ierr = MatMatSolve(iAii,Xint_tmp,Xint);CHKERRQ(ierr);
-    ierr = MatDestroy(iAii);CHKERRQ(ierr);
+    ierr = MatDestroy(&iAii);CHKERRQ(ierr);
   } else {
     Vec         b,x;
     PetscScalar *xint_tmp;
@@ -463,10 +457,10 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
     }
     ierr = MatRestoreArray(Xint,&xint);CHKERRQ(ierr);
     ierr = MatRestoreArray(Xint_tmp,&xint_tmp);CHKERRQ(ierr);
-    ierr = VecDestroy(x);CHKERRQ(ierr);
-    ierr = VecDestroy(b);CHKERRQ(ierr);
+    ierr = VecDestroy(&x);CHKERRQ(ierr);
+    ierr = VecDestroy(&b);CHKERRQ(ierr);
   }
-  ierr = MatDestroy(Xint_tmp);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xint_tmp);CHKERRQ(ierr);
 
 #if defined(PETSC_USE_DEBUG_foo)
   ierr = MatGetArray(Xint,&xint);CHKERRQ(ierr);
@@ -475,7 +469,7 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
     for (j=0; j<6; j++) {
       tmp += xint[i+j*Nint];
     }
-    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_ERR_PLIB,"Wrong Xint interpolation at i %D value %G",i,PetscAbsScalar(tmp));
+    if (PetscAbsScalar(tmp-1.0) > 1.e-10) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Wrong Xint interpolation at i %D value %G",i,PetscAbsScalar(tmp));
   }
   ierr = MatRestoreArray(Xint,&xint);CHKERRQ(ierr);
   /* ierr =MatView(Xint,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr); */
@@ -510,13 +504,13 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
     ierr = PetscTableAddCount(ht,globals[i]+1);CHKERRQ(ierr);
   }
   ierr = PetscTableGetCount(ht,&cnt);CHKERRQ(ierr);
-  if (cnt != Ntotal) SETERRQ2(PETSC_ERR_PLIB,"Hash table size %D not equal to total number coarse grid points %D",cnt,Ntotal); 
+  if (cnt != Ntotal) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Hash table size %D not equal to total number coarse grid points %D",cnt,Ntotal); 
   ierr = PetscFree(globals);CHKERRQ(ierr);
   for (i=0; i<6; i++) {
     ierr = PetscTableFind(ht,gl[i]+1,&gl[i]);CHKERRQ(ierr);
     gl[i]--;
   }
-  ierr = PetscTableDestroy(ht);CHKERRQ(ierr);
+  ierr = PetscTableDestroy(&ht);CHKERRQ(ierr);
   /* PetscIntView(6,gl,PETSC_VIEWER_STDOUT_WORLD); */
 
   /* construct global interpolation matrix */
@@ -548,7 +542,7 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
     ierr = MatMult(*P,x,y);CHKERRQ(ierr);
     ierr = VecGetArray(y,&yy);CHKERRQ(ierr);
     for (i=0; i<Ng; i++) {
-      if (PetscAbsScalar(yy[i]-1.0) > 1.e-10) SETERRQ2(PETSC_ERR_PLIB,"Wrong p interpolation at i %D value %G",i,PetscAbsScalar(yy[i]));
+      if (PetscAbsScalar(yy[i]-1.0) > 1.e-10) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Wrong p interpolation at i %D value %G",i,PetscAbsScalar(yy[i]));
     }
     ierr = VecRestoreArray(y,&yy);CHKERRQ(ierr);
     ierr = VecDestroy(x);CHKERRQ(ierr);
@@ -556,15 +550,15 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
   }
 #endif
     
-  ierr = MatDestroy(Aii);CHKERRQ(ierr);
-  ierr = MatDestroy(Ais);CHKERRQ(ierr);
-  ierr = MatDestroy(Asi);CHKERRQ(ierr);
-  ierr = MatDestroy(A);CHKERRQ(ierr);
-  ierr = ISDestroy(is);CHKERRQ(ierr);
-  ierr = ISDestroy(isint);CHKERRQ(ierr);
-  ierr = ISDestroy(issurf);CHKERRQ(ierr);
-  ierr = MatDestroy(Xint);CHKERRQ(ierr);
-  ierr = MatDestroy(Xsurf);CHKERRQ(ierr);
+  ierr = MatDestroy(&Aii);CHKERRQ(ierr);
+  ierr = MatDestroy(&Ais);CHKERRQ(ierr);
+  ierr = MatDestroy(&Asi);CHKERRQ(ierr);
+  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  ierr = ISDestroy(&is);CHKERRQ(ierr);
+  ierr = ISDestroy(&isint);CHKERRQ(ierr);
+  ierr = ISDestroy(&issurf);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xint);CHKERRQ(ierr);
+  ierr = MatDestroy(&Xsurf);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -574,7 +568,7 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
 /*@
    PCExoticSetType - Sets the type of coarse grid interpolation to use
 
-   Collective on PC
+   Logically Collective on PC
 
    Input Parameters:
 +  pc - the preconditioner context
@@ -598,22 +592,20 @@ PetscErrorCode DAGetFaceInterpolation(PC_Exotic *exotic,Mat Aglobal,MatReuse reu
 
 .seealso: PCEXOTIC, PCExoticType()
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCExoticSetType(PC pc,PCExoticType type)
+PetscErrorCode  PCExoticSetType(PC pc,PCExoticType type)
 {
-  PetscErrorCode ierr,(*f)(PC,PCExoticType);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCExoticSetType_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc,type);CHKERRQ(ierr);
-  } 
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(pc,type,2);
+  ierr = PetscTryMethod(pc,"PCExoticSetType_C",(PC,PCExoticType),(pc,type));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "PCExoticSetType_Exotic"
-PetscErrorCode PETSCKSP_DLLEXPORT PCExoticSetType_Exotic(PC pc,PCExoticType type)
+PetscErrorCode  PCExoticSetType_Exotic(PC pc,PCExoticType type)
 {
   PC_MG     *mg = (PC_MG*)pc->data;
   PC_Exotic *ctx = (PC_Exotic*) mg->innerctx;
@@ -634,13 +626,16 @@ PetscErrorCode PCSetUp_Exotic(PC pc)
   MatReuse       reuse = (ex->P) ? MAT_REUSE_MATRIX : MAT_INITIAL_MATRIX;
 
   PetscFunctionBegin;
+  if (!pc->dm) SETERRQ(((PetscObject)pc)->comm,PETSC_ERR_ARG_WRONGSTATE,"Need to call PCSetDM() before using this PC");
   ierr = PCGetOperators(pc,PETSC_NULL,&A,PETSC_NULL);CHKERRQ(ierr);
   if (ex->type == PC_EXOTIC_FACE) {
-    ierr = DAGetFaceInterpolation(ex,A,reuse,&ex->P);CHKERRQ(ierr);
+    ierr = DMDAGetFaceInterpolation(pc->dm,ex,A,reuse,&ex->P);CHKERRQ(ierr);
   } else if (ex->type == PC_EXOTIC_WIREBASKET) {
-    ierr = DAGetWireBasketInterpolation(ex,A,reuse,&ex->P);CHKERRQ(ierr);
-  } else SETERRQ1(PETSC_ERR_PLIB,"Unknown exotic coarse space %d",ex->type);
+    ierr = DMDAGetWireBasketInterpolation(pc->dm,ex,A,reuse,&ex->P);CHKERRQ(ierr);
+  } else SETERRQ1(((PetscObject)pc)->comm,PETSC_ERR_PLIB,"Unknown exotic coarse space %d",ex->type);
   ierr = PCMGSetInterpolation(pc,1,ex->P);CHKERRQ(ierr);
+  /* if PC has attached DM we must remove it or the PCMG will use it to compute incorrect sized vectors and interpolations */
+  ierr = PCSetDM(pc,PETSC_NULL);CHKERRQ(ierr);
   ierr = PCSetUp_MG(pc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -654,80 +649,24 @@ PetscErrorCode PCDestroy_Exotic(PC pc)
   PC_Exotic      *ctx = (PC_Exotic*) mg->innerctx;
 
   PetscFunctionBegin;
-  if (ctx->da) {ierr = DADestroy(ctx->da);CHKERRQ(ierr);}
-  if (ctx->P) {ierr = MatDestroy(ctx->P);CHKERRQ(ierr);}
-  if (ctx->ksp) {ierr = KSPDestroy(ctx->ksp);CHKERRQ(ierr);}
+  ierr = MatDestroy(&ctx->P);CHKERRQ(ierr);
+  ierr = KSPDestroy(&ctx->ksp);CHKERRQ(ierr);
   ierr = PetscFree(ctx);CHKERRQ(ierr);
   ierr = PCDestroy_MG(pc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_Exotic_Error"
-PetscErrorCode PCSetUp_Exotic_Error(PC pc)
-{
-  PetscFunctionBegin;
-  SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"You are using the Exotic preconditioner but never called PCSetDA()");
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "PCSetDA"
-/*@
-   PCSetDA - Sets the DA that is to be used by the PCEXOTIC or certain other preconditioners
-
-   Collective on PC
-
-   Input Parameters:
-+  pc - the preconditioner context
--  da - the da
-
-   Level: intermediate
-
-
-.seealso: PCEXOTIC, PCExoticType()
-@*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCSetDA(PC pc,DA da)
-{
-  PetscErrorCode ierr,(*f)(PC,DA);
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
-  PetscValidHeaderSpecific(da,DM_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCSetDA_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(pc,da);CHKERRQ(ierr);
-  } 
-  PetscFunctionReturn(0);
-}
-
-
-#undef __FUNCT__  
-#define __FUNCT__ "PCSetDA_Exotic"
-PetscErrorCode PETSCKSP_DLLEXPORT PCSetDA_Exotic(PC pc,DA da)
-{
-  PetscErrorCode ierr;
-  PC_MG          *mg = (PC_MG*)pc->data;
-  PC_Exotic      *ctx = (PC_Exotic*) mg->innerctx;
-
-  PetscFunctionBegin;
-  ctx->da = da;
-  pc->ops->setup = PCSetUp_Exotic;
-  ierr   = PetscObjectReference((PetscObject)da);CHKERRQ(ierr); 
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "PCView_Exotic"
 PetscErrorCode PCView_Exotic(PC pc,PetscViewer viewer)
 {
   PC_MG          *mg = (PC_MG*)pc->data;
   PetscErrorCode ierr;
-  PetscTruth     iascii;
+  PetscBool      iascii;
   PC_Exotic      *ctx = (PC_Exotic*) mg->innerctx;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"    Exotic type = %s\n",PCExoticTypes[ctx->type]);CHKERRQ(ierr);
     if (ctx->directSolve) {
@@ -758,7 +697,7 @@ PetscErrorCode PCView_Exotic(PC pc,PetscViewer viewer)
 PetscErrorCode PCSetFromOptions_Exotic(PC pc)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
   PC_MG          *mg = (PC_MG*)pc->data;
   PCExoticType   mgctype;
   PC_Exotic      *ctx = (PC_Exotic*) mg->innerctx;
@@ -769,7 +708,7 @@ PetscErrorCode PCSetFromOptions_Exotic(PC pc)
     if (flg) {
       ierr = PCExoticSetType(pc,mgctype);CHKERRQ(ierr);
     }
-    ierr = PetscOptionsTruth("-pc_exotic_direct_solver","use direct solver to construct interpolation","None",ctx->directSolve,&ctx->directSolve,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-pc_exotic_direct_solver","use direct solver to construct interpolation","None",ctx->directSolve,&ctx->directSolve,PETSC_NULL);CHKERRQ(ierr);
     if (!ctx->directSolve) {
       if (!ctx->ksp) {
         const char *prefix;
@@ -791,7 +730,11 @@ PetscErrorCode PCSetFromOptions_Exotic(PC pc)
      PCEXOTIC - Two level overlapping Schwarz preconditioner with exotic (non-standard) coarse grid spaces
 
      This uses the PCMG infrastructure restricted to two levels and the face and wirebasket based coarse
-   grid spaces. These coarse grid spaces originate in the work of Bramble, Pasciak  and Schatz, "The Construction
+   grid spaces. 
+
+   Notes: By default this uses GMRES on the fine grid smoother so this should be used with KSPFGMRES or the smoother changed to not use GMRES
+
+   References: These coarse grid spaces originate in the work of Bramble, Pasciak  and Schatz, "The Construction
    of Preconditioners for Elliptic Problems by Substructing IV", Mathematics of Computation, volume 53 pages 1--24, 1989.
    They were generalized slightly in "Domain Decomposition Method for Linear Elasticity", Ph. D. thesis, Barry Smith,
    New York University, 1990. They were then explored in great detail in Dryja, Smith, Widlund, "Schwarz Analysis
@@ -806,7 +749,7 @@ PetscErrorCode PCSetFromOptions_Exotic(PC pc)
    Springer-Verlag, Lecture Notes in Computational Science and Engineering, pages 255-261, 2007.
    Clark R. Dohrmann, Axel Klawonn, and Olof B. Widlund. A family of energy min-
    imizing coarse spaces for overlapping Schwarz preconditioners. In Ulrich Langer,
-   Marco Discacciati, David Keyes, OlofWidlund, andWalter Zulehner, editors, Proceedings
+   Marco Discacciati, David Keyes, Olof Widlund, and Walter Zulehner, editors, Proceedings
    of the 17th International Conference on Domain Decomposition Methods
    in Science and Engineering, held in Strobl, Austria, July 3-7, 2006, number 60 in
    Springer-Verlag, Lecture Notes in Computational Science and Engineering, pages 247-254, 2007
@@ -824,13 +767,13 @@ PetscErrorCode PCSetFromOptions_Exotic(PC pc)
 
    Level: advanced
 
-.seealso:  PCMG, PCSetDA(), PCExoticType, PCExoticSetType()
+.seealso:  PCMG, PCSetDM(), PCExoticType, PCExoticSetType()
 M*/
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PCCreate_Exotic"
-PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_Exotic(PC pc)
+PetscErrorCode  PCCreate_Exotic(PC pc)
 {
   PetscErrorCode ierr;
   PC_Exotic      *ex;
@@ -839,12 +782,12 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_Exotic(PC pc)
   PetscFunctionBegin;
   /* if type was previously mg; must manually destroy it because call to PCSetType(pc,PCMG) will not destroy it */
   if (pc->ops->destroy) { ierr =  (*pc->ops->destroy)(pc);CHKERRQ(ierr); pc->data = 0;}
-  ierr = PetscStrfree(((PetscObject)pc)->type_name);CHKERRQ(ierr);
+  ierr = PetscFree(((PetscObject)pc)->type_name);CHKERRQ(ierr);
   ((PetscObject)pc)->type_name = 0;
 
   ierr = PCSetType(pc,PCMG);CHKERRQ(ierr);
   ierr = PCMGSetLevels(pc,2,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PCMGSetGalerkin(pc);CHKERRQ(ierr);
+  ierr = PCMGSetGalerkin(pc,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscNew(PC_Exotic,&ex);CHKERRQ(ierr);\
   ex->type = PC_EXOTIC_FACE;
   mg = (PC_MG*) pc->data;
@@ -854,9 +797,8 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCCreate_Exotic(PC pc)
   pc->ops->setfromoptions = PCSetFromOptions_Exotic;
   pc->ops->view           = PCView_Exotic;
   pc->ops->destroy        = PCDestroy_Exotic;
-  pc->ops->setup          = PCSetUp_Exotic_Error;
+  pc->ops->setup          = PCSetUp_Exotic;
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)pc,"PCExoticSetType_C","PCExoticSetType_Exotic",PCExoticSetType_Exotic);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)pc,"PCSetDA_C","PCSetDA_Exotic",PCSetDA_Exotic);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
