@@ -1,11 +1,11 @@
-#define PETSCKSP_DLL
+
 /*
     Routines to set PC methods and options.
 */
 
-#include "private/pcimpl.h"      /*I "petscpc.h" I*/
+#include <private/pcimpl.h>      /*I "petscpc.h" I*/
 
-PetscTruth PCRegisterAllCalled = PETSC_FALSE;
+PetscBool  PCRegisterAllCalled = PETSC_FALSE;
 /*
    Contains the list of registered KSP routines
 */
@@ -51,20 +51,20 @@ PetscFList PCList = 0;
 .seealso: KSPSetType(), PCType
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCSetType(PC pc,const PCType type)
+PetscErrorCode  PCSetType(PC pc,const PCType type)
 {
   PetscErrorCode ierr,(*r)(PC);
-  PetscTruth     match;
+  PetscBool      match;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   PetscValidCharPointer(type,2);
 
   ierr = PetscTypeCompare((PetscObject)pc,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  ierr =  PetscFListFind(PCList,((PetscObject)pc)->comm,type,(void (**)(void)) &r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested PC type %s",type);
+  ierr =  PetscFListFind(PCList,((PetscObject)pc)->comm,type,PETSC_TRUE,(void (**)(void)) &r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(((PetscObject)pc)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested PC type %s",type);
   /* Destroy the previous private PC context */
   if (pc->ops->destroy) { ierr =  (*pc->ops->destroy)(pc);CHKERRQ(ierr); pc->data = 0;}
   ierr = PetscFListDestroy(&((PetscObject)pc)->qlist);CHKERRQ(ierr);
@@ -75,8 +75,13 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCSetType(PC pc,const PCType type)
   pc->modifysubmatricesP       = 0;
   /* Call the PCCreate_XXX routine for this particular preconditioner */
   pc->setupcalled = 0;
-  ierr = (*r)(pc);CHKERRQ(ierr);
   ierr = PetscObjectChangeTypeName((PetscObject)pc,type);CHKERRQ(ierr);
+  ierr = (*r)(pc);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_AMS)
+  if (PetscAMSPublishAll) {
+    ierr = PetscObjectAMSPublish((PetscObject)pc);CHKERRQ(ierr);
+  }
+#endif
   PetscFunctionReturn(0);
 }
 
@@ -95,7 +100,7 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCSetType(PC pc,const PCType type)
 .seealso: PCRegisterAll(), PCRegisterAll()
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCRegisterDestroy(void)
+PetscErrorCode  PCRegisterDestroy(void)
 {
   PetscErrorCode ierr;
 
@@ -126,16 +131,16 @@ PetscErrorCode PETSCKSP_DLLEXPORT PCRegisterDestroy(void)
 .seealso: PCSetType()
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCGetType(PC pc,const PCType *type)
+PetscErrorCode  PCGetType(PC pc,const PCType *type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   PetscValidPointer(type,2);
   *type = ((PetscObject)pc)->type_name;
   PetscFunctionReturn(0);
 }
 
-EXTERN PetscErrorCode PCGetDefaultType_Private(PC,const char*[]);
+extern PetscErrorCode PCGetDefaultType_Private(PC,const char*[]);
 
 #undef __FUNCT__  
 #define __FUNCT__ "PCSetFromOptions"
@@ -156,35 +161,147 @@ EXTERN PetscErrorCode PCGetDefaultType_Private(PC,const char*[]);
 .seealso: 
 
 @*/
-PetscErrorCode PETSCKSP_DLLEXPORT PCSetFromOptions(PC pc)
+PetscErrorCode  PCSetFromOptions(PC pc)
 {
   PetscErrorCode ierr;
   char           type[256];
   const char     *def;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_COOKIE,1);
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
 
   if (!PCRegisterAllCalled) {ierr = PCRegisterAll(PETSC_NULL);CHKERRQ(ierr);}
-  ierr = PetscOptionsBegin(((PetscObject)pc)->comm,((PetscObject)pc)->prefix,"Preconditioner (PC) Options","PC");CHKERRQ(ierr);
-    if (!((PetscObject)pc)->type_name) {
-      ierr = PCGetDefaultType_Private(pc,&def);CHKERRQ(ierr);
-    } else {
-      def = ((PetscObject)pc)->type_name;
-    }
-
-    ierr = PetscOptionsList("-pc_type","Preconditioner","PCSetType",PCList,def,type,256,&flg);CHKERRQ(ierr);
-    if (flg) {
-      ierr = PCSetType(pc,type);CHKERRQ(ierr);
-    } else if (!((PetscObject)pc)->type_name){
-      ierr = PCSetType(pc,def);CHKERRQ(ierr);
-    } 
-
-    if (pc->ops->setfromoptions) {
-      ierr = (*pc->ops->setfromoptions)(pc);CHKERRQ(ierr);
-    }
+ierr = PetscObjectOptionsBegin((PetscObject)pc);CHKERRQ(ierr);
+  if (!((PetscObject)pc)->type_name) {
+    ierr = PCGetDefaultType_Private(pc,&def);CHKERRQ(ierr);
+  } else {
+    def = ((PetscObject)pc)->type_name;
+  }
+  
+  ierr = PetscOptionsList("-pc_type","Preconditioner","PCSetType",PCList,def,type,256,&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = PCSetType(pc,type);CHKERRQ(ierr);
+  } else if (!((PetscObject)pc)->type_name){
+    ierr = PCSetType(pc,def);CHKERRQ(ierr);
+  } 
+  
+  ierr = PetscOptionsGetInt(((PetscObject)pc)->prefix,"-pc_reuse",&pc->reuse,PETSC_NULL);CHKERRQ(ierr);
+  
+  if (pc->ops->setfromoptions) {
+    ierr = (*pc->ops->setfromoptions)(pc);CHKERRQ(ierr);
+  }
+  
+  /* process any options handlers added with PetscObjectAddOptionsHandler() */
+  ierr = PetscObjectProcessOptionsHandlers((PetscObject)pc);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   pc->setfromoptionscalled++;
   PetscFunctionReturn(0);
 }
+
+#undef __FUNCT__  
+#define __FUNCT__ "PCSetDM"
+/*@
+   PCSetDM - Sets the DM that may be used by some preconditioners
+
+   Logically Collective on PC
+
+   Input Parameters:
++  pc - the preconditioner context
+-  dm - the dm
+
+   Level: intermediate
+
+
+.seealso: PCGetDM(), KSPSetDM(), KSPGetDM()
+@*/
+PetscErrorCode  PCSetDM(PC pc,DM dm)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  if (dm) {ierr = PetscObjectReference((PetscObject)dm);CHKERRQ(ierr);}
+  ierr = DMDestroy(&pc->dm);CHKERRQ(ierr);
+  pc->dm = dm;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PCGetDM"
+/*@
+   PCGetDM - Gets the DM that may be used by some preconditioners
+
+   Not Collective
+
+   Input Parameter:
+. pc - the preconditioner context
+
+   Output Parameter:
+.  dm - the dm
+
+   Level: intermediate
+
+
+.seealso: PCSetDM(), KSPSetDM(), KSPGetDM()
+@*/
+PetscErrorCode  PCGetDM(PC pc,DM *dm)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  *dm = pc->dm;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PCSetApplicationContext"
+/*@
+   PCSetApplicationContext - Sets the optional user-defined context for the linear solver.
+
+   Logically Collective on PC
+
+   Input Parameters:
++  pc - the PC context
+-  usrP - optional user context
+
+   Level: intermediate
+
+.keywords: PC, set, application, context
+
+.seealso: PCGetApplicationContext()
+@*/
+PetscErrorCode  PCSetApplicationContext(PC pc,void *usrP)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  pc->user = usrP;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PCGetApplicationContext"
+/*@
+   PCGetApplicationContext - Gets the user-defined context for the linear solver.
+
+   Not Collective
+
+   Input Parameter:
+.  pc - PC context
+
+   Output Parameter:
+.  usrP - user context
+
+   Level: intermediate
+
+.keywords: PC, get, application, context
+
+.seealso: PCSetApplicationContext()
+@*/
+PetscErrorCode  PCGetApplicationContext(PC pc,void *usrP)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  *(void**)usrP = pc->user;
+  PetscFunctionReturn(0);
+}
+
