@@ -1,11 +1,10 @@
-#define PETSCMAT_DLL
 
 /*
      Routines that call the kernel minpack coloring subroutines
 */
 
-#include "private/matimpl.h"
-#include "../src/mat/color/color.h"
+#include <private/matimpl.h>
+#include <../src/mat/color/color.h>
 
 /*
     MatFDColoringDegreeSequence_Minpack - Calls the MINPACK routine seqr() that
@@ -50,45 +49,85 @@ PetscErrorCode MatFDColoringMinimumNumberofColors_Private(PetscInt m,PetscInt *i
 EXTERN_C_BEGIN
 /* ----------------------------------------------------------------------------*/
 /*
-    MatFDColoringSL_Minpack - Uses the smallest-last (SL) coloring of minpack
+    MatGetColoring_SL_Minpack - Uses the smallest-last (SL) coloring of minpack
 */
 #undef __FUNCT__  
-#define __FUNCT__ "MatFDColoringSL_Minpack" 
-PetscErrorCode PETSCMAT_DLLEXPORT MatFDColoringSL_Minpack(Mat mat,MatColoringType name,ISColoring *iscoloring)
+#define __FUNCT__ "MatGetColoring_SL_Minpack" 
+PetscErrorCode  MatGetColoring_SL_Minpack(Mat mat,MatColoringType name,ISColoring *iscoloring)
 {
-  PetscErrorCode ierr;
+  PetscErrorCode  ierr;
   PetscInt        *list,*work,clique,*ria,*rja,*cia,*cja,*seq,*coloring,n;
   PetscInt        ncolors,i;
-  PetscTruth      done;
+  PetscBool       done;
+  Mat             mat_seq = mat;
+  PetscMPIInt     size;
+  MPI_Comm        comm;
+  ISColoring      iscoloring_seq;
+  PetscInt        bs = 1,rstart,rend,N_loc,nc;
+  ISColoringValue *colors_loc;
+  PetscBool       flg1,flg2;
 
   PetscFunctionBegin;
-  ierr = MatGetRowIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
-  ierr = MatGetColumnIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
-  if (!done) SETERRQ(PETSC_ERR_SUP,"Ordering requires IJ");
+  /* this is ugly way to get blocksize but cannot call MatGetBlockSize() because AIJ can have bs > 1 */
+  ierr = PetscTypeCompare((PetscObject)mat,MATSEQBAIJ,&flg1);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)mat,MATMPIBAIJ,&flg2);CHKERRQ(ierr);
+  if (flg1 || flg2) {
+    ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
+  }
+
+  ierr = PetscObjectGetComm((PetscObject)mat,&comm);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  if (size > 1){
+    /* create a sequential iscoloring on all processors */
+    ierr = MatGetSeqNonzeroStructure(mat,&mat_seq);CHKERRQ(ierr);
+  }
+
+  ierr = MatGetRowIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
+  ierr = MatGetColumnIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
+  if (!done) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Ordering requires IJ");
 
   ierr = MatFDColoringDegreeSequence_Minpack(n,cja,cia,rja,ria,&seq);CHKERRQ(ierr);
 
-  ierr = PetscMalloc(5*n*sizeof(PetscInt),&list);CHKERRQ(ierr);
-  work = list + n;
+  ierr = PetscMalloc2(n,PetscInt,&list,4*n,PetscInt,&work);CHKERRQ(ierr);
 
   MINPACKslo(&n,cja,cia,rja,ria,seq,list,&clique,work,work+n,work+2*n,work+3*n);
 
   ierr = PetscMalloc(n*sizeof(PetscInt),&coloring);CHKERRQ(ierr);
   MINPACKseq(&n,cja,cia,rja,ria,list,coloring,&ncolors,work);
 
-  ierr = PetscFree(list);CHKERRQ(ierr);
+  ierr = PetscFree2(list,work);CHKERRQ(ierr);
   ierr = PetscFree(seq);CHKERRQ(ierr);
-  ierr = MatRestoreRowIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
-  ierr = MatRestoreColumnIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
+  ierr = MatRestoreRowIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
+  ierr = MatRestoreColumnIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
 
   /* shift coloring numbers to start at zero and shorten */
-  if (ncolors > IS_COLORING_MAX-1) SETERRQ(PETSC_ERR_SUP,"Maximum color size exceeded");
+  if (ncolors > IS_COLORING_MAX-1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Maximum color size exceeded");
   { 
     ISColoringValue *s = (ISColoringValue*) coloring;
     for (i=0; i<n; i++) {
       s[i] = (ISColoringValue) (coloring[i]-1);
     }
-    ierr = MatColoringPatch(mat,ncolors,n,s,iscoloring);CHKERRQ(ierr);
+    ierr = MatColoringPatch(mat_seq,ncolors,n,s,iscoloring);CHKERRQ(ierr);
+  }
+
+  if (size > 1) {
+    ierr = MatDestroySeqNonzeroStructure(&mat_seq);CHKERRQ(ierr);
+
+    /* convert iscoloring_seq to a parallel iscoloring */  
+    iscoloring_seq = *iscoloring;
+    rstart = mat->rmap->rstart/bs;
+    rend   = mat->rmap->rend/bs;
+    N_loc  = rend - rstart; /* number of local nodes */
+
+    /* get local colors for each local node */
+    ierr = PetscMalloc((N_loc+1)*sizeof(ISColoringValue),&colors_loc);CHKERRQ(ierr);
+    for (i=rstart; i<rend; i++){
+      colors_loc[i-rstart] = iscoloring_seq->colors[i];
+    }
+    /* create a parallel iscoloring */ 
+    nc=iscoloring_seq->n;
+    ierr = ISColoringCreate(comm,nc,N_loc,colors_loc,iscoloring);CHKERRQ(ierr); 
+    ierr = ISColoringDestroy(&iscoloring_seq);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -97,26 +136,46 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 /* ----------------------------------------------------------------------------*/
 /*
-    MatFDColoringLF_Minpack - 
+    MatGetColoring_LF_Minpack - 
 */
 #undef __FUNCT__  
-#define __FUNCT__ "MatFDColoringLF_Minpack" 
-PetscErrorCode PETSCMAT_DLLEXPORT MatFDColoringLF_Minpack(Mat mat,MatColoringType name,ISColoring *iscoloring)
+#define __FUNCT__ "MatGetColoring_LF_Minpack" 
+PetscErrorCode  MatGetColoring_LF_Minpack(Mat mat,MatColoringType name,ISColoring *iscoloring)
 {
-  PetscErrorCode ierr;
-  PetscInt       *list,*work,*ria,*rja,*cia,*cja,*seq,*coloring,n;
-  PetscInt       n1, none,ncolors,i;
-  PetscTruth     done;
+  PetscErrorCode  ierr;
+  PetscInt        *list,*work,*ria,*rja,*cia,*cja,*seq,*coloring,n;
+  PetscInt        n1, none,ncolors,i;
+  PetscBool       done;
+  Mat             mat_seq = mat;
+  PetscMPIInt     size;
+  MPI_Comm        comm;
+  ISColoring      iscoloring_seq;
+  PetscInt        bs = 1,rstart,rend,N_loc,nc;
+  ISColoringValue *colors_loc;
+  PetscBool       flg1,flg2;
 
   PetscFunctionBegin;
-  ierr = MatGetRowIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
-  ierr = MatGetColumnIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
-  if (!done) SETERRQ(PETSC_ERR_SUP,"Ordering requires IJ");
+  /* this is ugly way to get blocksize but cannot call MatGetBlockSize() because AIJ can have bs > 1 */
+  ierr = PetscTypeCompare((PetscObject)mat,MATSEQBAIJ,&flg1);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)mat,MATMPIBAIJ,&flg2);CHKERRQ(ierr);
+  if (flg1 || flg2) {
+    ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
+  }
+
+  ierr = PetscObjectGetComm((PetscObject)mat,&comm);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  if (size > 1){
+    /* create a sequential iscoloring on all processors */
+    ierr = MatGetSeqNonzeroStructure(mat,&mat_seq);CHKERRQ(ierr);
+  }
+
+  ierr = MatGetRowIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
+  ierr = MatGetColumnIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
+  if (!done) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Ordering requires IJ");
 
   ierr = MatFDColoringDegreeSequence_Minpack(n,cja,cia,rja,ria,&seq);CHKERRQ(ierr);
 
-  ierr = PetscMalloc(5*n*sizeof(PetscInt),&list);CHKERRQ(ierr);
-  work = list + n;
+  ierr = PetscMalloc2(n,PetscInt,&list,4*n,PetscInt,&work);CHKERRQ(ierr);
 
   n1   = n - 1;
   none = -1;
@@ -124,20 +183,40 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatFDColoringLF_Minpack(Mat mat,MatColoringTyp
   ierr = PetscMalloc(n*sizeof(PetscInt),&coloring);CHKERRQ(ierr);
   MINPACKseq(&n,cja,cia,rja,ria,list,coloring,&ncolors,work);
 
-  ierr = PetscFree(list);CHKERRQ(ierr);
+  ierr = PetscFree2(list,work);CHKERRQ(ierr);
   ierr = PetscFree(seq);CHKERRQ(ierr);
 
-  ierr = MatRestoreRowIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
-  ierr = MatRestoreColumnIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
+  ierr = MatRestoreRowIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
+  ierr = MatRestoreColumnIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
 
   /* shift coloring numbers to start at zero and shorten */
-  if (ncolors > IS_COLORING_MAX-1) SETERRQ(PETSC_ERR_SUP,"Maximum color size exceeded");
+  if (ncolors > IS_COLORING_MAX-1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Maximum color size exceeded");
   { 
     ISColoringValue *s = (ISColoringValue*) coloring;
     for (i=0; i<n; i++) {
       s[i] = (ISColoringValue) (coloring[i]-1);
     }
-    ierr = MatColoringPatch(mat,ncolors,n,s,iscoloring);CHKERRQ(ierr);
+    ierr = MatColoringPatch(mat_seq,ncolors,n,s,iscoloring);CHKERRQ(ierr);
+  }
+
+  if (size > 1) {
+    ierr = MatDestroySeqNonzeroStructure(&mat_seq);CHKERRQ(ierr);
+
+    /* convert iscoloring_seq to a parallel iscoloring */  
+    iscoloring_seq = *iscoloring;
+    rstart = mat->rmap->rstart/bs;
+    rend   = mat->rmap->rend/bs;
+    N_loc  = rend - rstart; /* number of local nodes */
+
+    /* get local colors for each local node */
+    ierr = PetscMalloc((N_loc+1)*sizeof(ISColoringValue),&colors_loc);CHKERRQ(ierr);
+    for (i=rstart; i<rend; i++){
+      colors_loc[i-rstart] = iscoloring_seq->colors[i];
+    }
+    /* create a parallel iscoloring */ 
+    nc=iscoloring_seq->n;
+    ierr = ISColoringCreate(comm,nc,N_loc,colors_loc,iscoloring);CHKERRQ(ierr); 
+    ierr = ISColoringDestroy(&iscoloring_seq);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -146,46 +225,86 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 /* ----------------------------------------------------------------------------*/
 /*
-    MatFDColoringID_Minpack - 
+    MatGetColoring_ID_Minpack - 
 */
 #undef __FUNCT__  
-#define __FUNCT__ "MatFDColoringID_Minpack" 
-PetscErrorCode PETSCMAT_DLLEXPORT MatFDColoringID_Minpack(Mat mat,MatColoringType name,ISColoring *iscoloring)
+#define __FUNCT__ "MatGetColoring_ID_Minpack" 
+PetscErrorCode  MatGetColoring_ID_Minpack(Mat mat,MatColoringType name,ISColoring *iscoloring)
 {
-  PetscErrorCode ierr;
-  PetscInt       *list,*work,clique,*ria,*rja,*cia,*cja,*seq,*coloring,n;
-  PetscInt       ncolors,i;
-  PetscTruth     done;
+  PetscErrorCode  ierr;
+  PetscInt        *list,*work,clique,*ria,*rja,*cia,*cja,*seq,*coloring,n;
+  PetscInt        ncolors,i;
+  PetscBool       done;
+  Mat             mat_seq = mat;
+  PetscMPIInt     size;
+  MPI_Comm        comm;
+  ISColoring      iscoloring_seq;
+  PetscInt        bs = 1,rstart,rend,N_loc,nc;
+  ISColoringValue *colors_loc;
+  PetscBool       flg1,flg2;
 
   PetscFunctionBegin;
-  ierr = MatGetRowIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
-  ierr = MatGetColumnIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
-  if (!done) SETERRQ(PETSC_ERR_SUP,"Ordering requires IJ");
+  /* this is ugly way to get blocksize but cannot call MatGetBlockSize() because AIJ can have bs > 1 */
+  ierr = PetscTypeCompare((PetscObject)mat,MATSEQBAIJ,&flg1);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)mat,MATMPIBAIJ,&flg2);CHKERRQ(ierr);
+  if (flg1 || flg2) {
+    ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
+  }
+
+  ierr = PetscObjectGetComm((PetscObject)mat,&comm);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  if (size > 1){
+    /* create a sequential iscoloring on all processors */
+    ierr = MatGetSeqNonzeroStructure(mat,&mat_seq);CHKERRQ(ierr);
+  }
+
+  ierr = MatGetRowIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
+  ierr = MatGetColumnIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
+  if (!done) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Ordering requires IJ");
 
   ierr = MatFDColoringDegreeSequence_Minpack(n,cja,cia,rja,ria,&seq);CHKERRQ(ierr);
 
-  ierr = PetscMalloc(5*n*sizeof(PetscInt),&list);CHKERRQ(ierr);
-  work = list + n;
+  ierr = PetscMalloc2(n,PetscInt,&list,4*n,PetscInt,&work);CHKERRQ(ierr);
 
   MINPACKido(&n,&n,cja,cia,rja,ria,seq,list,&clique,work,work+n,work+2*n,work+3*n);
 
   ierr = PetscMalloc(n*sizeof(PetscInt),&coloring);CHKERRQ(ierr);
   MINPACKseq(&n,cja,cia,rja,ria,list,coloring,&ncolors,work);
 
-  ierr = PetscFree(list);CHKERRQ(ierr);
+  ierr = PetscFree2(list,work);CHKERRQ(ierr);
   ierr = PetscFree(seq);CHKERRQ(ierr);
 
-  ierr = MatRestoreRowIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
-  ierr = MatRestoreColumnIJ(mat,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
+  ierr = MatRestoreRowIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&ria,&rja,&done);CHKERRQ(ierr);
+  ierr = MatRestoreColumnIJ(mat_seq,1,PETSC_FALSE,PETSC_TRUE,&n,&cia,&cja,&done);CHKERRQ(ierr);
 
   /* shift coloring numbers to start at zero and shorten */
-  if (ncolors > IS_COLORING_MAX-1) SETERRQ(PETSC_ERR_SUP,"Maximum color size exceeded");
+  if (ncolors > IS_COLORING_MAX-1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Maximum color size exceeded");
   { 
     ISColoringValue *s = (ISColoringValue*) coloring;
     for (i=0; i<n; i++) {
       s[i] = (ISColoringValue) (coloring[i]-1);
     }
-    ierr = MatColoringPatch(mat,ncolors,n,s,iscoloring);CHKERRQ(ierr);
+    ierr = MatColoringPatch(mat_seq,ncolors,n,s,iscoloring);CHKERRQ(ierr);
+  }
+
+  if (size > 1) {
+    ierr = MatDestroySeqNonzeroStructure(&mat_seq);CHKERRQ(ierr);
+
+    /* convert iscoloring_seq to a parallel iscoloring */  
+    iscoloring_seq = *iscoloring;
+    rstart = mat->rmap->rstart/bs;
+    rend   = mat->rmap->rend/bs;
+    N_loc  = rend - rstart; /* number of local nodes */
+
+    /* get local colors for each local node */
+    ierr = PetscMalloc((N_loc+1)*sizeof(ISColoringValue),&colors_loc);CHKERRQ(ierr);
+    for (i=rstart; i<rend; i++){
+      colors_loc[i-rstart] = iscoloring_seq->colors[i];
+    }
+    /* create a parallel iscoloring */ 
+    nc=iscoloring_seq->n;
+    ierr = ISColoringCreate(comm,nc,N_loc,colors_loc,iscoloring);CHKERRQ(ierr); 
+    ierr = ISColoringDestroy(&iscoloring_seq);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -196,37 +315,79 @@ EXTERN_C_BEGIN
    Simplest coloring, each column of the matrix gets its own unique color.
 */
 #undef __FUNCT__  
-#define __FUNCT__ "MatColoring_Natural" 
-PetscErrorCode PETSCMAT_DLLEXPORT MatColoring_Natural(Mat mat,MatColoringType color, ISColoring *iscoloring)
+#define __FUNCT__ "MatGetColoring_Natural" 
+PetscErrorCode  MatGetColoring_Natural(Mat mat,MatColoringType color, ISColoring *iscoloring)
 {
   PetscErrorCode  ierr;
-  PetscInt        start,end,i;
+  PetscInt        start,end,i,bs = 1,n;
   ISColoringValue *colors;
   MPI_Comm        comm;
+  PetscBool       flg1,flg2;
+  Mat             mat_seq = mat;
+  PetscMPIInt     size;
+  ISColoring      iscoloring_seq;
+  ISColoringValue *colors_loc;
+  PetscInt        rstart,rend,N_loc,nc;
 
   PetscFunctionBegin;
-  ierr = MatGetOwnershipRange(mat,&start,&end);CHKERRQ(ierr);
-  ierr = PetscObjectGetComm((PetscObject)mat,&comm);CHKERRQ(ierr);
-  ierr = PetscMalloc((end-start+1)*sizeof(PetscInt),&colors);CHKERRQ(ierr);
-  for (i=start; i<end; i++) {
-    colors[i-start] = i;
+  /* this is ugly way to get blocksize but cannot call MatGetBlockSize() because AIJ can have bs > 1 */
+  ierr = PetscTypeCompare((PetscObject)mat,MATSEQBAIJ,&flg1);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)mat,MATMPIBAIJ,&flg2);CHKERRQ(ierr);
+  if (flg1 || flg2) {
+    ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
   }
-  ierr = ISColoringCreate(comm,mat->cmap->N,end-start,colors,iscoloring);CHKERRQ(ierr);
 
+  ierr = PetscObjectGetComm((PetscObject)mat,&comm);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  if (size > 1){
+    /* create a sequential iscoloring on all processors */
+    ierr = MatGetSeqNonzeroStructure(mat,&mat_seq);CHKERRQ(ierr);
+  }
+
+  ierr  = MatGetSize(mat_seq,PETSC_NULL,&n);CHKERRQ(ierr);
+  ierr  = MatGetOwnershipRange(mat_seq,&start,&end);CHKERRQ(ierr);
+  n     = n/bs;
+  if (n > IS_COLORING_MAX-1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Maximum color size exceeded");
+
+  start = start/bs;
+  end   = end/bs;
+  ierr  = PetscMalloc((end-start+1)*sizeof(PetscInt),&colors);CHKERRQ(ierr);
+  for (i=start; i<end; i++) {
+    colors[i-start] = (ISColoringValue)i;
+  }
+  ierr = ISColoringCreate(comm,n,end-start,colors,iscoloring);CHKERRQ(ierr);
+
+  if (size > 1) {
+    ierr = MatDestroySeqNonzeroStructure(&mat_seq);CHKERRQ(ierr);
+
+    /* convert iscoloring_seq to a parallel iscoloring */  
+    iscoloring_seq = *iscoloring;
+    rstart = mat->rmap->rstart/bs;
+    rend   = mat->rmap->rend/bs;
+    N_loc  = rend - rstart; /* number of local nodes */
+
+    /* get local colors for each local node */
+    ierr = PetscMalloc((N_loc+1)*sizeof(ISColoringValue),&colors_loc);CHKERRQ(ierr);
+    for (i=rstart; i<rend; i++){
+      colors_loc[i-rstart] = iscoloring_seq->colors[i];
+    }
+    /* create a parallel iscoloring */ 
+    nc=iscoloring_seq->n;
+    ierr = ISColoringCreate(comm,nc,N_loc,colors_loc,iscoloring);CHKERRQ(ierr); 
+    ierr = ISColoringDestroy(&iscoloring_seq);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
   
 /* ===========================================================================================*/
 
-#include "petscsys.h"
-
 PetscFList MatColoringList = 0;
-PetscTruth MatColoringRegisterAllCalled = PETSC_FALSE;
+PetscBool  MatColoringRegisterAllCalled = PETSC_FALSE;
 
 #undef __FUNCT__  
 #define __FUNCT__ "MatColoringRegister" 
-PetscErrorCode PETSCMAT_DLLEXPORT MatColoringRegister(const char sname[],const char path[],const char name[],PetscErrorCode (*function)(Mat,MatColoringType,ISColoring*))
+PetscErrorCode  MatColoringRegister(const char sname[],const char path[],const char name[],PetscErrorCode (*function)(Mat,MatColoringType,ISColoring*))
 {
   PetscErrorCode ierr;
   char           fullname[PETSC_MAX_PATH_LEN];
@@ -250,7 +411,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatColoringRegister(const char sname[],const c
 
 .seealso: MatColoringRegisterDynamic(), MatColoringRegisterAll()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatColoringRegisterDestroy(void)
+PetscErrorCode  MatColoringRegisterDestroy(void)
 {
   PetscErrorCode ierr;
 
@@ -271,10 +432,10 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatColoringRegisterDestroy(void)
    Input Parameters:
 .  mat - the matrix
 .  type - type of coloring, one of the following:
-$      MATCOLORING_NATURAL - natural (one color for each column, very slow)
-$      MATCOLORING_SL - smallest-last
-$      MATCOLORING_LF - largest-first
-$      MATCOLORING_ID - incidence-degree
+$      MATCOLORINGNATURAL - natural (one color for each column, very slow)
+$      MATCOLORINGSL - smallest-last
+$      MATCOLORINGLF - largest-first
+$      MATCOLORINGID - incidence-degree
 
    Output Parameters:
 .   iscoloring - the coloring
@@ -299,27 +460,35 @@ $    -mat_coloring_view
    For parallel matrices currently converts to sequential matrix and uses the sequential coloring
    on that.
 
-   The sequential colorings SL, LF, and ID are obtained via the Minpack software that was
+   The colorings SL, LF, and ID are obtained via the Minpack software that was
    converted to C using f2c.
 
+   For BAIJ matrices this colors the blocks. The true number of colors would be block size times the number of colors
+   returned here.
+
+   References:
+$     Thomas F. Coleman and Jorge J. More, Estimation of Sparse {J}acobian Matrices and Graph Coloring Problems,
+$         SIAM Journal on Numerical Analysis, 1983, pages 187-209, volume 20
+$     Jorge J. Mor\'{e} and Danny C. Sorenson and  Burton S. Garbow and Kenneth E. Hillstrom, The {MINPACK} Project,
+$         Sources and Development of Mathematical Software, Wayne R. Cowell editor, 1984, pages 88-111
+
 .keywords: matrix, get, coloring
 
 .seealso:  MatGetColoringTypeFromOptions(), MatColoringRegisterDynamic(), MatFDColoringCreate(),
            SNESDefaultComputeJacobianColor()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatGetColoring(Mat mat,const MatColoringType type,ISColoring *iscoloring)
+PetscErrorCode  MatGetColoring(Mat mat,const MatColoringType type,ISColoring *iscoloring)
 {
-  PetscTruth     flag;
+  PetscBool      flag;
   PetscErrorCode ierr,(*r)(Mat,const MatColoringType,ISColoring *);
   char           tname[PETSC_MAX_PATH_LEN];
   MPI_Comm       comm;
-  PetscMPIInt    size;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   PetscValidPointer(iscoloring,3);
-  if (!mat->assembled) SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled matrix");
-  if (mat->factor) SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"Not for factored matrix"); 
+  if (!mat->assembled) SETERRQ(((PetscObject)mat)->comm,PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled matrix");
+  if (mat->factortype) SETERRQ(((PetscObject)mat)->comm,PETSC_ERR_ARG_WRONGSTATE,"Not for factored matrix"); 
 
   /* look for type on command line */
   if (!MatColoringRegisterAllCalled) {ierr = MatColoringRegisterAll(PETSC_NULL);CHKERRQ(ierr);}
@@ -327,44 +496,16 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatGetColoring(Mat mat,const MatColoringType t
   if (flag) { type = tname; }
 
   ierr = PetscObjectGetComm((PetscObject)mat,&comm);CHKERRQ(ierr); 
-  ierr = PetscFListFind(MatColoringList,comm, type,(void (**)(void)) &r);CHKERRQ(ierr);
-  if (!r) {SETERRQ1(PETSC_ERR_ARG_OUTOFRANGE,"Unknown or unregistered type: %s",type);}
+  ierr = PetscFListFind(MatColoringList,comm, type,PETSC_TRUE,(void (**)(void)) &r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(((PetscObject)mat)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Unknown or unregistered type: %s",type);
 
   ierr = PetscLogEventBegin(MAT_GetColoring,mat,0,0,0);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  if (size == 1){
-    ierr = (*r)(mat,type,iscoloring);CHKERRQ(ierr);
-  } else { /* for parallel matrix */
-    Mat             *mat_seq;
-    ISColoring      iscoloring_seq;
-    ISColoringValue *colors_loc;
-    PetscInt        i,rstart,rend,N_loc,nc;
-      
-    /* create a sequential iscoloring on all processors */
-    ierr = MatGetSeqNonzeroStructure(mat,&mat_seq);CHKERRQ(ierr);
-    ierr = (*r)(*mat_seq,type,&iscoloring_seq);CHKERRQ(ierr);
-    ierr = MatDestroySeqNonzeroStructure(&mat_seq);CHKERRQ(ierr);
-
-    /* convert iscoloring_seq to a parallel iscoloring */  
-    rstart = mat->rmap->rstart;
-    rend   = mat->rmap->rend;
-    N_loc  = rend - rstart; /* number of local nodes */
-
-    /* get local colors for each local node */
-    ierr = PetscMalloc((N_loc+1)*sizeof(ISColoringValue),&colors_loc);CHKERRQ(ierr);
-    for (i=rstart; i<rend; i++){
-      colors_loc[i-rstart] = iscoloring_seq->colors[i];
-    }
-    /* create a parallel iscoloring */ 
-    nc=iscoloring_seq->n;
-    ierr = ISColoringCreate(comm,nc,N_loc,colors_loc,iscoloring);CHKERRQ(ierr); 
-    ierr = ISColoringDestroy(iscoloring_seq);CHKERRQ(ierr);
-    /* ierr = ISColoringView(iscoloring_mpi,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr); */  
-  }
+  ierr = (*r)(mat,type,iscoloring);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(MAT_GetColoring,mat,0,0,0);CHKERRQ(ierr);
 
   ierr = PetscInfo1(mat,"Number of colors %d\n",(*iscoloring)->n);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(PETSC_NULL,"-mat_coloring_view",&flag);CHKERRQ(ierr);
+  flag = PETSC_FALSE;
+  ierr = PetscOptionsGetBool(PETSC_NULL,"-mat_coloring_view",&flag,PETSC_NULL);CHKERRQ(ierr);
   if (flag) {
     PetscViewer viewer;
     ierr = PetscViewerASCIIGetStdout((*iscoloring)->comm,&viewer);CHKERRQ(ierr);
