@@ -1,13 +1,12 @@
-#define PETSCMAT_DLL
  
-#include "petscmat.h"
-#include "../src/mat/color/color.h"
+#include <petscmat.h>
+#include <../src/mat/color/color.h>
 
 EXTERN_C_BEGIN
-EXTERN PetscErrorCode PETSCMAT_DLLEXPORT MatGetColoring_Natural(Mat,MatColoringType,ISColoring*);
-EXTERN PetscErrorCode PETSCMAT_DLLEXPORT MatGetColoring_SL_Minpack(Mat,MatColoringType,ISColoring*);
-EXTERN PetscErrorCode PETSCMAT_DLLEXPORT MatGetColoring_LF_Minpack(Mat,MatColoringType,ISColoring*);
-EXTERN PetscErrorCode PETSCMAT_DLLEXPORT MatGetColoring_ID_Minpack(Mat,MatColoringType,ISColoring*);
+extern PetscErrorCode  MatGetColoring_Natural(Mat,MatColoringType,ISColoring*);
+extern PetscErrorCode  MatGetColoring_SL_Minpack(Mat,MatColoringType,ISColoring*);
+extern PetscErrorCode  MatGetColoring_LF_Minpack(Mat,MatColoringType,ISColoring*);
+extern PetscErrorCode  MatGetColoring_ID_Minpack(Mat,MatColoringType,ISColoring*);
 EXTERN_C_END
 
 #undef __FUNCT__  
@@ -40,10 +39,10 @@ PetscErrorCode MatColoringRegisterAll(const char path[])
 
   PetscFunctionBegin;
   MatColoringRegisterAllCalled = PETSC_TRUE;  
-  ierr = MatColoringRegisterDynamic(MATCOLORING_NATURAL,path,"MatGetColoring_Natural",   MatGetColoring_Natural);CHKERRQ(ierr);
-  ierr = MatColoringRegisterDynamic(MATCOLORING_SL,     path,"MatGetColoring_SL_Minpack",MatGetColoring_SL_Minpack);CHKERRQ(ierr);
-  ierr = MatColoringRegisterDynamic(MATCOLORING_LF,     path,"MatGetColoring_LF_Minpack",MatGetColoring_LF_Minpack);CHKERRQ(ierr);
-  ierr = MatColoringRegisterDynamic(MATCOLORING_ID,     path,"MatGetColoring_ID_Minpack",MatGetColoring_ID_Minpack);CHKERRQ(ierr);
+  ierr = MatColoringRegisterDynamic(MATCOLORINGNATURAL,path,"MatGetColoring_Natural",   MatGetColoring_Natural);CHKERRQ(ierr);
+  ierr = MatColoringRegisterDynamic(MATCOLORINGSL,     path,"MatGetColoring_SL_Minpack",MatGetColoring_SL_Minpack);CHKERRQ(ierr);
+  ierr = MatColoringRegisterDynamic(MATCOLORINGLF,     path,"MatGetColoring_LF_Minpack",MatGetColoring_LF_Minpack);CHKERRQ(ierr);
+  ierr = MatColoringRegisterDynamic(MATCOLORINGID,     path,"MatGetColoring_ID_Minpack",MatGetColoring_ID_Minpack);CHKERRQ(ierr);
 
   PetscFunctionReturn(0);
 }
