@@ -1,8 +1,7 @@
-#define PETSCMAT_DLL
 
 /* setr.f -- translated by f2c (version of 25 March 1992  12:58:56). */
 
-#include "../src/mat/color/color.h"
+#include <../src/mat/color/color.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "MINPACKsetr" 
