@@ -1,6 +1,6 @@
 static char help[] = "Test MatMatMult() for AIJ and Dense matrices.\n\n";
 
-#include "petscmat.h"
+#include <petscmat.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -11,13 +11,11 @@ int main(int argc,char **argv)
   PetscScalar    *array,*a;
   PetscErrorCode ierr;
   PetscRandom    r;
-  PetscTruth     equal=PETSC_FALSE;
+  PetscBool      equal=PETSC_FALSE;
   PetscReal      fill = 1.0;
-  PetscMPIInt    size;
   PetscInt       rstart,rend,nza,col,am,an,bm,bn;
 
   PetscInitialize(&argc,&argv,(char *)0,help);
-  ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   ierr = PetscOptionsGetInt(PETSC_NULL,"-M",&M,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscOptionsGetInt(PETSC_NULL,"-N",&N,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscOptionsGetReal(PETSC_NULL,"-fill",&fill,PETSC_NULL);CHKERRQ(ierr);
@@ -28,11 +26,7 @@ int main(int argc,char **argv)
   /* create a aij matrix A */
   ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
   ierr = MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,N,M);CHKERRQ(ierr);
-  if (size == 1){
-    ierr = MatSetType(A,MATSEQAIJ);CHKERRQ(ierr);
-  } else {
-    ierr = MatSetType(A,MATMPIAIJ);CHKERRQ(ierr);
-  }
+  ierr = MatSetType(A,MATAIJ);CHKERRQ(ierr);
   nza  = (PetscInt)(.3*M); /* num of nozeros in each row of A */
   ierr = MatSeqAIJSetPreallocation(A,nza,PETSC_NULL);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(A,nza,PETSC_NULL,nza,PETSC_NULL);CHKERRQ(ierr);  
@@ -52,11 +46,7 @@ int main(int argc,char **argv)
   ierr = MatGetLocalSize(A,&am,&an);CHKERRQ(ierr);
   ierr = MatCreate(PETSC_COMM_WORLD,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,an,PETSC_DECIDE,PETSC_DECIDE,N);CHKERRQ(ierr);
-  if (size == 1){
-    ierr = MatSetType(B,MATSEQDENSE);CHKERRQ(ierr);
-  } else {
-    ierr = MatSetType(B,MATMPIDENSE);CHKERRQ(ierr);
-  }
+  ierr = MatSetType(B,MATDENSE);CHKERRQ(ierr);
   ierr = MatSeqDenseSetPreallocation(B,PETSC_NULL);CHKERRQ(ierr);
   ierr = MatMPIDenseSetPreallocation(B,PETSC_NULL);CHKERRQ(ierr);
   ierr = MatGetLocalSize(B,&bm,&bn);CHKERRQ(ierr);
@@ -68,7 +58,7 @@ int main(int argc,char **argv)
     }
   }
   ierr = MatRestoreArray(B,&array);CHKERRQ(ierr);
-  ierr = PetscRandomDestroy(r);CHKERRQ(ierr);
+  ierr = PetscRandomDestroy(&r);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
 
@@ -80,12 +70,12 @@ int main(int argc,char **argv)
     ierr = MatMatMultNumeric(A,B,D);CHKERRQ(ierr);
   }  
   ierr = MatEqual(C,D,&equal);CHKERRQ(ierr);
-  if (!equal) SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"C != D");
+  if (!equal) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"C != D");
 
-  ierr = MatDestroy(D);CHKERRQ(ierr); 
-  ierr = MatDestroy(C);CHKERRQ(ierr);
-  ierr = MatDestroy(B);CHKERRQ(ierr);
-  ierr = MatDestroy(A);CHKERRQ(ierr);
+  ierr = MatDestroy(&D);CHKERRQ(ierr); 
+  ierr = MatDestroy(&C);CHKERRQ(ierr);
+  ierr = MatDestroy(&B);CHKERRQ(ierr);
+  ierr = MatDestroy(&A);CHKERRQ(ierr);
   ierr = PetscFree(a);CHKERRQ(ierr);
   PetscFinalize();
   return(0);
