@@ -4,7 +4,7 @@ Input arguments are:\n\
   -f <input_file>  : file to load\n\n";
 /* e.g., mpiexec -n 3 ./ex113 -f <file> */
 
-#include "petscmat.h"
+#include <petscmat.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -15,17 +15,18 @@ int main(int argc,char **args)
   PetscErrorCode ierr;
   PetscReal      fill=4.0;
   char           file[128]; 
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscInitialize(&argc,&args,(char *)0,help);
   
   /*  Load the matrix A */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-f",file,127,&flg);CHKERRQ(ierr);
-  if (!flg) SETERRQ(1,"Must indicate a file name for matrix A with the -f option.");
+  ierr = PetscOptionsGetString(PETSC_NULL,"-f",file,128,&flg);CHKERRQ(ierr);
+  if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate a file name for matrix A with the -f option.");
 
   ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&viewer);CHKERRQ(ierr);
-  ierr = MatLoad(viewer,MATAIJ,&A);CHKERRQ(ierr); 
-  ierr = PetscViewerDestroy(viewer);CHKERRQ(ierr);             
+  ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
+  ierr = MatLoad(A,viewer);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);             
 
   ierr = MatDuplicate(A,MAT_COPY_VALUES,&A1);CHKERRQ(ierr);
   ierr = MatDuplicate(A,MAT_COPY_VALUES,&A2);CHKERRQ(ierr);
@@ -33,26 +34,26 @@ int main(int argc,char **args)
   /* dstMat = A*A1*A2 */
   ierr = MatMatMult(A1,A2,MAT_INITIAL_MATRIX,fill,&Mtmp);CHKERRQ(ierr);
   ierr = MatMatMult(A,Mtmp,MAT_INITIAL_MATRIX,fill,&dstMat);CHKERRQ(ierr); 
-  ierr = MatDestroy(Mtmp);CHKERRQ(ierr);
+  ierr = MatDestroy(&Mtmp);CHKERRQ(ierr);
 
   /* dstMat += A1*A2 */
   ierr = MatMatMult(A1,A2,MAT_INITIAL_MATRIX,fill,&Mtmp);CHKERRQ(ierr);
   ierr = MatAXPY(dstMat,1.0,Mtmp,SUBSET_NONZERO_PATTERN);CHKERRQ(ierr); 
-  ierr = MatDestroy(Mtmp);CHKERRQ(ierr); 
+  ierr = MatDestroy(&Mtmp);CHKERRQ(ierr); 
 
   /* dstMat += A*A1 */
   ierr = MatMatMult(A,A1,MAT_INITIAL_MATRIX,fill,&Mtmp);CHKERRQ(ierr);
   ierr = MatAXPY(dstMat, 1.0, Mtmp,SUBSET_NONZERO_PATTERN);CHKERRQ(ierr); 
-  ierr = MatDestroy(Mtmp);CHKERRQ(ierr);
+  ierr = MatDestroy(&Mtmp);CHKERRQ(ierr);
 
   /* dstMat += A */
   ierr = MatAXPY(dstMat, 1.0, A,SUBSET_NONZERO_PATTERN);CHKERRQ(ierr); 
 
-  ierr = MatDestroy(A);CHKERRQ(ierr); 
-  ierr = MatDestroy(A1);CHKERRQ(ierr);
-  ierr = MatDestroy(A2);CHKERRQ(ierr);  
-  ierr = MatDestroy(dstMat);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = MatDestroy(&A);CHKERRQ(ierr); 
+  ierr = MatDestroy(&A1);CHKERRQ(ierr);
+  ierr = MatDestroy(&A2);CHKERRQ(ierr);  
+  ierr = MatDestroy(&dstMat);CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 
