@@ -1,7 +1,7 @@
 
 static char help[] = "Creates a matrix, inserts some values, and tests MatGetSubMatrices() and MatZeroEntries().\n\n";
 
-#include "petscmat.h"
+#include <petscmat.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -50,11 +50,11 @@ int main(int argc,char **argv)
   ierr = MatZeroEntries(mat);CHKERRQ(ierr);
   ierr = MatView(mat,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
 
-  ierr = ISDestroy(irkeep);CHKERRQ(ierr);
-  ierr = ISDestroy(ickeep);CHKERRQ(ierr);
-  ierr = MatDestroy(submat);CHKERRQ(ierr);
-  ierr = MatDestroy(mat);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = ISDestroy(&irkeep);CHKERRQ(ierr);
+  ierr = ISDestroy(&ickeep);CHKERRQ(ierr);
+  ierr = MatDestroy(&submat);CHKERRQ(ierr);
+  ierr = MatDestroy(&mat);CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
  
