@@ -1,8 +1,8 @@
 
 static char help[] = "Tests converting a matrix to another format with MatConvert().\n\n";
 
-#include "petscmat.h"
-/* Usage: mpiexec -n <np> ex55 -display <0 or 1> */
+#include <petscmat.h>
+/* Usage: mpiexec -n <np> ex55 -verbose <0 or 1> */
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -10,18 +10,17 @@ int main(int argc,char **args)
 {
   Mat            C,A,B,D; 
   PetscErrorCode ierr;
-  PetscInt       i,j,ntypes,bs,mbs,m,block,d_nz=6, o_nz=3,col[3],row,displ=0;
+  PetscInt       i,j,ntypes,bs,mbs,m,block,d_nz=6, o_nz=3,col[3],row,verbose=0;
   PetscMPIInt    size,rank;
-  /* const MatType  type[9] = {MATMPIAIJ,MATMPIBAIJ};*/
   const MatType  type[9]; 
   char           file[PETSC_MAX_PATH_LEN];
   PetscViewer    fd;
-  PetscTruth     equal,flg_loadmat,flg;
+  PetscBool      equal,flg_loadmat,flg;
   PetscScalar    value[3];
 
   PetscInitialize(&argc,&args,(char *)0,help);
-  ierr = PetscOptionsGetInt(PETSC_NULL,"-displ",&displ,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetString(PETSC_NULL,"-f",file,PETSC_MAX_PATH_LEN-1,&flg_loadmat);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(PETSC_NULL,"-verbose",&verbose,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg_loadmat);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
 
@@ -40,9 +39,9 @@ int main(int argc,char **args)
     type[1] = MATSEQBAIJ;
     type[2] = MATSEQSBAIJ;
   } else {
-    ntypes = 2; 
+    ntypes = 3; 
     type[1] = MATMPIBAIJ;
-    type[2] = MATMPISBAIJ; /* Matconvert from mpisbaij mat to other formats are not supported */
+    type[2] = MATMPISBAIJ;
   }
 
   /* input matrix C */
@@ -51,17 +50,20 @@ int main(int argc,char **args)
     ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd);CHKERRQ(ierr);
 
     /* Load a baij matrix, then destroy the viewer. */
+    ierr = MatCreate(PETSC_COMM_WORLD,&C);CHKERRQ(ierr);
     if (size == 1){
-      ierr = MatLoad(fd,MATSEQBAIJ,&C);CHKERRQ(ierr);
+      ierr = MatSetType(C,MATSEQBAIJ);CHKERRQ(ierr);
+      ierr = MatLoad(C,fd);CHKERRQ(ierr);
     } else {
-      ierr = MatLoad(fd,MATMPIBAIJ,&C);CHKERRQ(ierr);
+      ierr = MatSetType(C,MATMPIBAIJ);CHKERRQ(ierr);
+      ierr = MatLoad(C,fd);CHKERRQ(ierr);
     }
-    ierr = PetscViewerDestroy(fd);CHKERRQ(ierr);
+    ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
   } else { /* Create a baij mat with bs>1  */
     bs = 2; mbs=8;
     ierr = PetscOptionsGetInt(PETSC_NULL,"-mbs",&mbs,PETSC_NULL);CHKERRQ(ierr);
     ierr = PetscOptionsGetInt(PETSC_NULL,"-bs",&bs,PETSC_NULL);CHKERRQ(ierr);
-    if (bs <= 1) SETERRQ(PETSC_ERR_ARG_WRONG," bs must be >1 in this case");
+    if (bs <= 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG," bs must be >1 in this case");
     m = mbs*bs;
     if (size == 1){
       ierr = MatCreateSeqBAIJ(PETSC_COMM_SELF,bs,m,m,d_nz,PETSC_NULL,&C);CHKERRQ(ierr); 
@@ -94,52 +96,68 @@ int main(int argc,char **args)
     ierr = MatAssemblyBegin(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
     ierr = MatAssemblyEnd(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   }
+
+  {
+    /* Check the symmetry of C because it will be converted to a sbaij matrix */
+    Mat Ctrans;
+    ierr = MatTranspose(C, MAT_INITIAL_MATRIX,&Ctrans);
+    ierr = MatEqual(C, Ctrans, &flg);CHKERRQ(ierr);
+    if (flg) {
+      ierr = MatSetOption(C,MAT_SYMMETRIC,PETSC_TRUE);CHKERRQ(ierr);
+    } else {
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"C must be symmetric for this example");
+    }
+    ierr = MatDestroy(&Ctrans);CHKERRQ(ierr);
+  }
+  //ierr = MatView(C,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
  
   /* convert C to other formats */
   for (i=0; i<ntypes; i++) {
     ierr = MatConvert(C,type[i],MAT_INITIAL_MATRIX,&A);CHKERRQ(ierr);
     ierr = MatMultEqual(A,C,10,&equal);CHKERRQ(ierr);
-    if (!equal) SETERRQ1(PETSC_ERR_ARG_NOTSAMETYPE,"Error in conversion from BAIJ to %s",type[i]);
+    if (!equal) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_NOTSAMETYPE,"Error in conversion from BAIJ to %s",type[i]);
     for (j=i+1; j<ntypes; j++) { 
-      if (displ>0) {
-        ierr = PetscPrintf(PETSC_COMM_WORLD," [%d] test conversion between %s and %s\n",rank,type[i],type[j]);CHKERRQ(ierr);
+      if (verbose>0) {
+        ierr = PetscPrintf(PETSC_COMM_WORLD," \n[%d] test conversion between %s and %s\n",rank,type[i],type[j]);CHKERRQ(ierr);
       }
-
+      
+      if (!rank && verbose) printf("Convert %s A to %s B\n",type[i],type[j]);
       ierr = MatConvert(A,type[j],MAT_INITIAL_MATRIX,&B);CHKERRQ(ierr);
-      ierr = MatConvert(B,type[i],MAT_INITIAL_MATRIX,&D);CHKERRQ(ierr); 
-
-      if (bs == 1){
-        ierr = MatEqual(A,D,&equal);CHKERRQ(ierr);
-        if (!equal){
-          ierr = PetscPrintf(PETSC_COMM_SELF," A: %s\n",type[i]);
-          MatView(A,PETSC_VIEWER_STDOUT_WORLD);
-          ierr = PetscPrintf(PETSC_COMM_SELF," B: %s\n",type[j]);
-          MatView(B,PETSC_VIEWER_STDOUT_WORLD);
-          ierr = PetscPrintf(PETSC_COMM_SELF," D: %s\n",type[i]);
-          MatView(D,PETSC_VIEWER_STDOUT_WORLD);
-          SETERRQ2(1,"Error in conversion from %s to %s",type[i],type[j]);
-        }
-      } else { /* bs > 1 */
-        ierr = MatMultEqual(A,B,10,&equal);CHKERRQ(ierr);
-        if (!equal) SETERRQ2(PETSC_ERR_PLIB,"Error in conversion from %s to %s",type[i],type[j]);
+      /*
+      if (j == 2){
+        ierr = PetscPrintf(PETSC_COMM_SELF," A: %s\n",type[i]);CHKERRQ(ierr);
+        ierr = MatView(A,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
+        ierr = PetscPrintf(PETSC_COMM_SELF," B: %s\n",type[j]);CHKERRQ(ierr);
+        ierr = MatView(B,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
       }
-      ierr = MatDestroy(B);CHKERRQ(ierr);
-      ierr = MatDestroy(D);CHKERRQ(ierr);
-      B = PETSC_NULL;
-      D = PETSC_NULL;
+       */
+      ierr = MatMultEqual(A,B,10,&equal);CHKERRQ(ierr);
+      if (!equal) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_NOTSAMETYPE,"Error in conversion from %s to %s",type[i],type[j]);
+
+      if (size == 1 || j != 2){ /* Matconvert from mpisbaij mat to other formats are not supported */
+        if (!rank && verbose) printf("Convert %s B to %s D\n",type[j],type[i]);
+        ierr = MatConvert(B,type[i],MAT_INITIAL_MATRIX,&D);CHKERRQ(ierr); 
+        ierr = MatMultEqual(B,D,10,&equal);CHKERRQ(ierr);
+        if (!equal) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_NOTSAMETYPE,"Error in conversion from %s to %s",type[j],type[i]);
+
+        ierr = MatDestroy(&D);CHKERRQ(ierr);
+      }
+      ierr = MatDestroy(&B);CHKERRQ(ierr);
+      ierr = MatDestroy(&D);CHKERRQ(ierr);
     }
+
     /* Test in-place convert */
     if (size == 1){ /* size > 1 is not working yet! */
-    j = (i+1)%ntypes;
-    /* printf("[%d] i: %d, j: %d\n",rank,i,j); */
-    ierr = MatConvert(A,type[j],MAT_REUSE_MATRIX,&A);CHKERRQ(ierr);
+      j = (i+1)%ntypes;
+      /* printf("[%d] i: %d, j: %d\n",rank,i,j); */
+      ierr = MatConvert(A,type[j],MAT_REUSE_MATRIX,&A);CHKERRQ(ierr);
     }
 
-    ierr = MatDestroy(A);CHKERRQ(ierr);
+    ierr = MatDestroy(&A);CHKERRQ(ierr);
   }
-  ierr = MatDestroy(C);CHKERRQ(ierr);
+  ierr = MatDestroy(&C);CHKERRQ(ierr);
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 
