@@ -1,5 +1,5 @@
 
-static char help[] = "Reads in a matrix in ASCII Matlab format (I,J,A), read in vectors rhs and exact_solu in ASCII format.\n\
+static char help[] = "Reads in a matrix in ASCII MATLAB format (I,J,A), read in vectors rhs and exact_solu in ASCII format.\n\
 Writes them using the PETSc sparse format.\n\
 Note: I and J start at 1, not 0, use -noshift if indices in file start with zero!\n\
 Input parameters are:\n\
@@ -33,7 +33,7 @@ solu
 0 1.0
 */
 
-#include "petscmat.h"
+#include <petscmat.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -49,15 +49,15 @@ int main(int argc,char **args)
   PetscReal      res_norm;
   FILE           *Afile,*bfile,*ufile;
   PetscViewer    view;
-  PetscTruth     flg_A,flg_b,flg_u,flg;
+  PetscBool      flg_A,flg_b,flg_u,flg;
   PetscMPIInt    size;
 
   PetscInitialize(&argc,&args,(char *)0,help);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
-  if (size != 1) SETERRQ(PETSC_ERR_SUP,"This is a uniprocessor example only!");
+  if (size != 1) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"This is a uniprocessor example only!");
 
   /* Read in matrix, rhs and exact solution from ascii files */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-Ain",Ain,PETSC_MAX_PATH_LEN-1,&flg_A);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-Ain",Ain,PETSC_MAX_PATH_LEN,&flg_A);CHKERRQ(ierr);
   ierr = PetscOptionsHasName(PETSC_NULL,"-noshift",&flg);CHKERRQ(ierr);
   if (flg) shift = 0;
   if (flg_A){
@@ -66,15 +66,15 @@ int main(int argc,char **args)
     nsizes = 3;
     ierr = PetscOptionsGetIntArray(PETSC_NULL,"-nosizesinfile",sizes,&nsizes,&flg);CHKERRQ(ierr);
     if (flg) {
-      if (nsizes != 3) SETERRQ(1,"Must pass in three m,n,nz as arguments for -nosizesinfile");
+      if (nsizes != 3) SETERRQ(PETSC_COMM_WORLD,1,"Must pass in three m,n,nz as arguments for -nosizesinfile");
       m = sizes[0];
       n = sizes[1];
       nz = sizes[2];
     } else {
       fscanf(Afile,"%d %d %d\n",&m,&n,&nz);
     }
-    printf("m: %d, n: %d, nz: %d \n", m,n,nz);
-    if (m != n) SETERRQ(PETSC_ERR_ARG_SIZ, "Number of rows, cols must be same for this example\n");
+    ierr = PetscPrintf(PETSC_COMM_SELF,"m: %d, n: %d, nz: %d \n", m,n,nz);CHKERRQ(ierr);
+    if (m != n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ, "Number of rows, cols must be same for this example\n");
     ierr = MatCreate(PETSC_COMM_SELF,&A);CHKERRQ(ierr);
     ierr = MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,m,n);CHKERRQ(ierr);
     ierr = MatSetFromOptions(A);CHKERRQ(ierr);
@@ -91,7 +91,7 @@ int main(int argc,char **args)
     fclose(Afile);  
   }
 
-  ierr = PetscOptionsGetString(PETSC_NULL,"-rhs",rhs,PETSC_MAX_PATH_LEN-1,&flg_b);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-rhs",rhs,PETSC_MAX_PATH_LEN,&flg_b);CHKERRQ(ierr);
   if (flg_b){
     ierr = VecCreate(PETSC_COMM_SELF,&b);CHKERRQ(ierr);
     ierr = VecSetSizes(b,PETSC_DECIDE,n);CHKERRQ(ierr);
@@ -108,7 +108,7 @@ int main(int argc,char **args)
     fclose(bfile);
   }
 
-  ierr = PetscOptionsGetString(PETSC_NULL,"-solu",solu,PETSC_MAX_PATH_LEN-1,&flg_u);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-solu",solu,PETSC_MAX_PATH_LEN,&flg_u);CHKERRQ(ierr);
   if (flg_u){
     ierr = VecCreate(PETSC_COMM_SELF,&u);CHKERRQ(ierr);
     ierr = VecSetSizes(u,PETSC_DECIDE,n);CHKERRQ(ierr);
@@ -125,28 +125,35 @@ int main(int argc,char **args)
     fclose(ufile);
   }
  
+  /* Write matrix, rhs and exact solution in Petsc binary file */
+  ierr = PetscPrintf(PETSC_COMM_SELF,"\n Write matrix in binary to 'matrix.dat' ...\n");CHKERRQ(ierr);
+  ierr = PetscViewerBinaryOpen(PETSC_COMM_SELF,"matrix.dat",FILE_MODE_WRITE,&view);CHKERRQ(ierr);
+  ierr = MatView(A,view);CHKERRQ(ierr);
+
+  if (flg_b){ /* Write rhs in Petsc binary file */
+    ierr = PetscPrintf(PETSC_COMM_SELF,"\n Write rhs in binary to 'matrix.dat' ...\n");CHKERRQ(ierr);
+    ierr = VecView(b,view);CHKERRQ(ierr);
+  }
+  if (flg_u){
+    ierr = PetscPrintf(PETSC_COMM_SELF,"\n Write exact solution in binary to 'matrix.dat' ...\n");CHKERRQ(ierr);
+    ierr = VecView(u,view);CHKERRQ(ierr);
+  }
+
   /* Check accuracy of the data */
   if (flg_A & flg_b & flg_u){
     ierr = VecDuplicate(u,&u_tmp);CHKERRQ(ierr); 
     ierr = MatMult(A,u,u_tmp);CHKERRQ(ierr);
     ierr = VecAXPY(u_tmp,-1.0,b);CHKERRQ(ierr);
     ierr = VecNorm(u_tmp,NORM_2,&res_norm);CHKERRQ(ierr);
-    printf("\n Accuracy of the reading data: | b - A*u |_2 : %g \n",res_norm); 
-
-    /* Write the matrix, rhs and exact solution in Petsc binary file */
-    ierr = PetscPrintf(PETSC_COMM_SELF,"\n Write matrix and rhs in binary to 'matrix.dat' ...\n");CHKERRQ(ierr);
-    ierr = PetscViewerBinaryOpen(PETSC_COMM_SELF,"matrix.dat",FILE_MODE_WRITE,&view);CHKERRQ(ierr);
-    ierr = MatView(A,view);CHKERRQ(ierr);
-    ierr = VecView(b,view);CHKERRQ(ierr);
-    ierr = VecView(u,view);CHKERRQ(ierr);
-    ierr = PetscViewerDestroy(view);CHKERRQ(ierr);
-
-    ierr = VecDestroy(b);CHKERRQ(ierr);
-    ierr = VecDestroy(u);CHKERRQ(ierr);
-    ierr = VecDestroy(u_tmp);CHKERRQ(ierr);
-    ierr = MatDestroy(A);CHKERRQ(ierr);
+    ierr = PetscPrintf(PETSC_COMM_SELF,"\n Accuracy of the reading data: | b - A*u |_2 : %g \n",res_norm);CHKERRQ(ierr);
+    ierr = VecDestroy(&u_tmp);CHKERRQ(ierr);
   }
-  ierr = PetscFinalize();CHKERRQ(ierr);
+
+  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  if (flg_b) {ierr = VecDestroy(&b);CHKERRQ(ierr);}
+  if (flg_u) {ierr = VecDestroy(&u);CHKERRQ(ierr);}
+  ierr = PetscViewerDestroy(&view);CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 
