@@ -1,4 +1,3 @@
-/*$Id: ex80.c,v 1.10 2001/04/10 19:35:44 bsmith Exp $*/
 
 static char help[] = "Partition tiny grid.\n\n";
 
@@ -10,30 +9,32 @@ T*/
 /* 
   Include "petscmat.h" so that we can use matrices.  Note that this file
   automatically includes:
-     petsc.h       - base PETSc routines   petscvec.h - vectors
-     petscsys.h    - system routines       petscmat.h - matrices
+     petscsys.h       - base PETSc routines   petscvec.h - vectors
+     petscmat.h - matrices
      petscis.h     - index sets            
      petscviewer.h - viewers               
 */
-#include "petscksp.h"
+#include <petscksp.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat             A;
-  int             ierr,rank,*ia,*ja,size;
+  PetscErrorCode  ierr;
+  PetscMPIInt     rank,size;
+  PetscInt        *ia,*ja;
   MatPartitioning part;
   IS              is,isn;
 
   PetscInitialize(&argc,&args,(char *)0,help);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
-  if (size != 4) SETERRQ(1,"Must run with 4 processors");
+  if (size != 4) SETERRQ(PETSC_COMM_WORLD,1,"Must run with 4 processors");
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
 
-  ierr = PetscMalloc(5*sizeof(int),&ia);CHKERRQ(ierr);
-  ierr = PetscMalloc(16*sizeof(int),&ja);CHKERRQ(ierr);
-  if (rank == 0) {
+  ierr = PetscMalloc(5*sizeof(PetscInt),&ia);CHKERRQ(ierr);
+  ierr = PetscMalloc(16*sizeof(PetscInt),&ja);CHKERRQ(ierr);
+  if (!rank) {
     ja[0] = 1; ja[1] = 4; ja[2] = 0; ja[3] = 2; ja[4] = 5; ja[5] = 1; ja[6] = 3; ja[7] = 6;
     ja[8] = 2; ja[9] = 7;
     ia[0] = 0; ia[1] = 2; ia[2] = 5; ia[3] = 8; ia[4] = 10;
@@ -65,19 +66,19 @@ int main(int argc,char **args)
   /* get new global number of each old global number */
   ierr = ISPartitioningToNumbering(is,&isn);CHKERRQ(ierr);
   ierr = ISView(isn,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
-  ierr = ISDestroy(is);CHKERRQ(ierr);
+  ierr = ISDestroy(&is);CHKERRQ(ierr);
 
-  ierr = ISDestroy(isn);CHKERRQ(ierr);
-  ierr = MatPartitioningDestroy(part);CHKERRQ(ierr);
+  ierr = ISDestroy(&isn);CHKERRQ(ierr);
+  ierr = MatPartitioningDestroy(&part);CHKERRQ(ierr);
 
   /*
        Free work space.  All PETSc objects should be destroyed when they
        are no longer needed.
   */
-  ierr = MatDestroy(A);CHKERRQ(ierr); 
+  ierr = MatDestroy(&A);CHKERRQ(ierr); 
 
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 
