@@ -9,9 +9,9 @@ Input parameters include\n\
 e.g. ./ex99 -f0 $D/small -fA $D/Eigdftb/dftb_bin/diamond_xxs_A -fB $D/Eigdftb/dftb_bin/diamond_xxs_B -mat_getrow_uppertriangular,\n\
      where $D = /home/petsc/datafiles/matrices/Eigdftb/dftb_bin\n\n";
 
-#include "petscmat.h"
-#include "../src/mat/impls/sbaij/seq/sbaij.h"
-#include "petscblaslapack.h"
+#include <petscmat.h>
+#include <../src/mat/impls/sbaij/seq/sbaij.h>
+#include <petscblaslapack.h>
 
 extern PetscErrorCode CkEigenSolutions(PetscInt*,Mat*,PetscReal*,Vec*,PetscInt*,PetscInt*,PetscReal*);
 
@@ -23,9 +23,9 @@ PetscInt main(PetscInt argc,char **args)
   Vec            *evecs;
   PetscViewer    fd;                /* viewer */
   char           file[3][PETSC_MAX_PATH_LEN];     /* input file name */
-  PetscTruth     flg,flgA=PETSC_FALSE,flgB=PETSC_FALSE,TestSYGVX=PETSC_TRUE;
+  PetscBool      flg,flgA=PETSC_FALSE,flgB=PETSC_FALSE,TestSYGVX=PETSC_TRUE;
   PetscErrorCode ierr;
-  PetscTruth     preload=PETSC_TRUE,isSymmetric;
+  PetscBool      preload=PETSC_TRUE,isSymmetric;
   PetscScalar    sigma,one=1.0,*arrayA,*arrayB,*evecs_array,*work,*evals;
   PetscMPIInt    size;
   PetscInt       m,n,i,j,nevs,il,iu;
@@ -42,33 +42,37 @@ PetscInt main(PetscInt argc,char **args)
   
   PetscInitialize(&argc,&args,(char *)0,help);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
-  if (size != 1) SETERRQ(PETSC_ERR_SUP,"This is a uniprocessor example only!");
+  if (size != 1) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"This is a uniprocessor example only!");
   ierr = PetscLogStageRegister("EigSolve",&stages[0]);
   ierr = PetscLogStageRegister("EigCheck",&stages[1]);
 
   /* Determine files from which we read the two matrices */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-f0",file[0],PETSC_MAX_PATH_LEN-1,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-f0",file[0],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (!flg) {
-    ierr = PetscOptionsGetString(PETSC_NULL,"-fA",file[0],PETSC_MAX_PATH_LEN-1,&flgA);CHKERRQ(ierr);
-    if (!flgA) SETERRQ(PETSC_ERR_USER,"Must indicate binary file with the -fA or -fB options");
-    ierr = PetscOptionsGetString(PETSC_NULL,"-fB",file[1],PETSC_MAX_PATH_LEN-1,&flgB);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(PETSC_NULL,"-fA",file[0],PETSC_MAX_PATH_LEN,&flgA);CHKERRQ(ierr);
+    if (!flgA) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_USER,"Must indicate binary file with the -fA or -fB options");
+    ierr = PetscOptionsGetString(PETSC_NULL,"-fB",file[1],PETSC_MAX_PATH_LEN,&flgB);CHKERRQ(ierr);
     preload = PETSC_FALSE;
   } else {
-    ierr = PetscOptionsGetString(PETSC_NULL,"-fA",file[1],PETSC_MAX_PATH_LEN-1,&flgA);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(PETSC_NULL,"-fA",file[1],PETSC_MAX_PATH_LEN,&flgA);CHKERRQ(ierr);
     if (!flgA) {preload = PETSC_FALSE;} /* don't bother with second system */
-    ierr = PetscOptionsGetString(PETSC_NULL,"-fB",file[2],PETSC_MAX_PATH_LEN-1,&flgB);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(PETSC_NULL,"-fB",file[2],PETSC_MAX_PATH_LEN,&flgB);CHKERRQ(ierr);
   }
 
-  PreLoadBegin(preload,"Load system");
+  PetscPreLoadBegin(preload,"Load system");
     /* Load matrices */
-    ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[PreLoadIt],FILE_MODE_READ,&fd);CHKERRQ(ierr);
-    ierr  = MatLoad(fd,MATSBAIJ,&A);CHKERRQ(ierr);
-    ierr = PetscViewerDestroy(fd);CHKERRQ(ierr); 
+    ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[PetscPreLoadIt],FILE_MODE_READ,&fd);CHKERRQ(ierr);
+    ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
+    ierr = MatSetType(A,MATSBAIJ);CHKERRQ(ierr);
+    ierr = MatLoad(A,fd);CHKERRQ(ierr); 
+    ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr); 
     ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
-    if ((flgB && PreLoadIt) || (flgB && !preload)){
-      ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[PreLoadIt+1],FILE_MODE_READ,&fd);CHKERRQ(ierr);
-      ierr  = MatLoad(fd,MATSBAIJ,&B);CHKERRQ(ierr);
-      ierr = PetscViewerDestroy(fd);CHKERRQ(ierr);
+    if ((flgB && PetscPreLoadIt) || (flgB && !preload)){
+      ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[PetscPreLoadIt+1],FILE_MODE_READ,&fd);CHKERRQ(ierr);
+      ierr = MatCreate(PETSC_COMM_WORLD,&B);CHKERRQ(ierr);
+      ierr = MatSetType(B,MATSBAIJ);CHKERRQ(ierr);
+      ierr = MatLoad(B,fd);CHKERRQ(ierr);
+      ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
     } else { /* create B=I */
       ierr = MatCreate(PETSC_COMM_WORLD,&B);CHKERRQ(ierr);
       ierr = MatSetSizes(B,PETSC_DECIDE,PETSC_DECIDE,m,n);CHKERRQ(ierr);
@@ -93,13 +97,13 @@ PetscInt main(PetscInt argc,char **args)
       Mat Trans;
       ierr = MatTranspose(A,MAT_INITIAL_MATRIX, &Trans);
       ierr = MatEqual(A, Trans, &isSymmetric);
-      if (!isSymmetric) SETERRQ(PETSC_ERR_USER,"A must be symmetric");
-      ierr = MatDestroy(Trans);CHKERRQ(ierr);
-      if (flgB && PreLoadIt){
+      if (!isSymmetric) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_USER,"A must be symmetric");
+      ierr = MatDestroy(&Trans);CHKERRQ(ierr);
+      if (flgB && PetscPreLoadIt){
         ierr = MatTranspose(B,MAT_INITIAL_MATRIX, &Trans);
         ierr = MatEqual(B, Trans, &isSymmetric);
-        if (!isSymmetric) SETERRQ(PETSC_ERR_USER,"B must be symmetric");
-        ierr = MatDestroy(Trans);CHKERRQ(ierr);
+        if (!isSymmetric) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_USER,"B must be symmetric");
+        ierr = MatDestroy(&Trans);CHKERRQ(ierr);
       }
     }
 
@@ -130,7 +134,7 @@ PetscInt main(PetscInt argc,char **args)
       ierr = MatAssemblyBegin(A_sp,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
       ierr = MatAssemblyEnd(A_sp,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr); 
 
-      ierr = MatDestroy(A_sp);CHKERRQ(ierr);
+      ierr = MatDestroy(&A_sp);CHKERRQ(ierr);
 
       ratio = (PetscReal)nzeros[0]/sbaij->nz;
       ierr = PetscPrintf(PETSC_COMM_SELF," %d matrix entries < %e, ratio %G of %d nonzeros\n",nzeros[0],tols[0],ratio,sbaij->nz);CHKERRQ(ierr);
@@ -164,16 +168,16 @@ PetscInt main(PetscInt argc,char **args)
       ierr = PetscMalloc((m*n+1)*sizeof(PetscScalar),&evecs_array);CHKERRQ(ierr);
       ierr = PetscMalloc((6*n+1)*sizeof(PetscBLASInt),&iwork);CHKERRQ(ierr);
       ifail = iwork + 5*n;
-      if(PreLoadIt){ierr = PetscLogStagePush(stages[0]);CHKERRQ(ierr);}
+      if(PetscPreLoadIt){ierr = PetscLogStagePush(stages[0]);CHKERRQ(ierr);}
       /* in the case "I", vl and vu are not referenced */
       LAPACKsygvx_(&lone,"V","I","U",&bn,arrayA,&bn,arrayB,&bn,&vl,&vu,&il,&iu,&abstol,&nevs,evals,evecs_array,&n,work,&lwork,iwork,ifail,&lierr);
-      if(PreLoadIt){ierr = PetscLogStagePop();}
+      if(PetscPreLoadIt){ierr = PetscLogStagePop();}
       ierr = PetscFree(iwork);CHKERRQ(ierr);
     }
     ierr = MatRestoreArray(A,&arrayA);CHKERRQ(ierr);
     ierr = MatRestoreArray(B,&arrayB);CHKERRQ(ierr);
 
-    if (nevs <= 0 ) SETERRQ1(PETSC_ERR_CONV_FAILED, "nev=%d, no eigensolution has found", nevs);
+    if (nevs <= 0 ) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_CONV_FAILED, "nev=%d, no eigensolution has found", nevs);
     /* View evals */
     ierr = PetscOptionsHasName(PETSC_NULL, "-eig_view", &flg);CHKERRQ(ierr);
     if (flg){
@@ -182,7 +186,7 @@ PetscInt main(PetscInt argc,char **args)
     }
 
     /* Check residuals and orthogonality */
-    if(PreLoadIt){
+    if(PetscPreLoadIt){
       mats[0] = A; mats[1] = B;
       one = (PetscInt)one;
       ierr = PetscMalloc((nevs+1)*sizeof(Vec),&evecs);CHKERRQ(ierr);
@@ -198,7 +202,7 @@ PetscInt main(PetscInt argc,char **args)
       ierr = PetscLogStagePush(stages[1]);CHKERRQ(ierr);
       ierr = CkEigenSolutions(&cklvl,mats,evals,evecs,ievbd_loc,&offset,tols);CHKERRQ(ierr);
       ierr = PetscLogStagePop();CHKERRQ(ierr);
-      for (i=0; i<nevs; i++){ ierr = VecDestroy(evecs[i]);CHKERRQ(ierr);}
+      for (i=0; i<nevs; i++){ ierr = VecDestroy(&evecs[i]);CHKERRQ(ierr);}
       ierr = PetscFree(evecs);CHKERRQ(ierr);
     }
     
@@ -208,13 +212,13 @@ PetscInt main(PetscInt argc,char **args)
     ierr = PetscFree(evals);CHKERRQ(ierr);
     ierr = PetscFree(work);CHKERRQ(ierr);
 
-    ierr = MatDestroy(A_dense);CHKERRQ(ierr); 
-    ierr = MatDestroy(B_dense);CHKERRQ(ierr); 
-    ierr = MatDestroy(B);CHKERRQ(ierr);
-    ierr = MatDestroy(A);CHKERRQ(ierr);
+    ierr = MatDestroy(&A_dense);CHKERRQ(ierr); 
+    ierr = MatDestroy(&B_dense);CHKERRQ(ierr); 
+    ierr = MatDestroy(&B);CHKERRQ(ierr);
+    ierr = MatDestroy(&A);CHKERRQ(ierr);
 
-  PreLoadEnd();
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  PetscPreLoadEnd();
+  ierr = PetscFinalize();
   return 0;
 }
 /*------------------------------------------------
@@ -299,7 +303,7 @@ PetscErrorCode CkEigenSolutions(PetscInt *fcklvl,Mat *mats,
   default:
     ierr = PetscPrintf(PETSC_COMM_SELF,"Error: cklvl=%d is not supported \n",cklvl);
   }
-  ierr = VecDestroy(vt2); 
-  ierr = VecDestroy(vt1);
+  ierr = VecDestroy(&vt2); 
+  ierr = VecDestroy(&vt1);
   PetscFunctionReturn(0);
 }
