@@ -1,6 +1,6 @@
-#include "../src/mat/impls/adj/mpi/mpiadj.h"
-#include "private/fortranimpl.h"
-#include "petscmat.h"
+#include <../src/mat/impls/adj/mpi/mpiadj.h>
+#include <private/fortranimpl.h>
+#include <petscmat.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define matcreatempiadj_                 MATCREATEMPIADJ
