@@ -1,21 +1,26 @@
-#define PETSCMAT_DLL
 
 /* 
     Provides an interface to the PaStiX sparse solver
 */
-#include "../src/mat/impls/aij/seq/aij.h"
-#include "../src/mat/impls/aij/mpi/mpiaij.h"
-#include "../src/mat/impls/sbaij/seq/sbaij.h"
-#include "../src/mat/impls/sbaij/mpi/mpisbaij.h"
+#include <../src/mat/impls/aij/seq/aij.h>
+#include <../src/mat/impls/aij/mpi/mpiaij.h>
+#include <../src/mat/impls/sbaij/seq/sbaij.h>
+#include <../src/mat/impls/sbaij/mpi/mpisbaij.h>
+
+#if defined(PETSC_HAVE_STDLIB_H)
+#include <stdlib.h>
+#endif
+#if defined(PETSC_HAVE_STRING_H)
+#include <string.h> 
+#endif
 
 EXTERN_C_BEGIN 
-#include "mpi.h"
-#include "pastix.h"
+#include <pastix.h>
 EXTERN_C_END  
 
 typedef struct Mat_Pastix_ {
   pastix_data_t *pastix_data;              /* Pastix data storage structure                        */
-  MatStructure   matstruc;                 				                           
+  MatStructure   matstruc;
   PetscInt       n;                        /* Number of columns in the matrix                      */
   PetscInt       *colptr;                  /* Index of first element of each column in row and val */
   PetscInt       *row;                     /* Row of each element of the matrix                    */
@@ -29,17 +34,18 @@ typedef struct Mat_Pastix_ {
   MPI_Comm       pastix_comm;              /* PaStiX MPI communicator                              */
   PetscMPIInt    commRank;                 /* MPI rank                                             */
   PetscMPIInt    commSize;                 /* MPI communicator size                                */
-  PetscTruth     CleanUpPastix;            /* Boolean indicating if we call PaStiX clean step      */
+  PetscBool      CleanUpPastix;            /* Boolean indicating if we call PaStiX clean step      */
   VecScatter     scat_rhs;
   VecScatter     scat_sol;
-  Vec            b_seq,x_seq;
-  PetscTruth     isAIJ;
-  PetscInt       nSolve;                   /* Number of consecutive solve                          */
-  PetscErrorCode (*MatDestroy)(Mat);
+  Vec            b_seq;
+  PetscBool      isAIJ;
+  PetscErrorCode (*Destroy)(Mat);
 } Mat_Pastix;
 
-EXTERN PetscErrorCode MatDuplicate_Pastix(Mat,MatDuplicateOption,Mat*);
+extern PetscErrorCode MatDuplicate_Pastix(Mat,MatDuplicateOption,Mat*);
 
+#undef __FUNCT__  
+#define __FUNCT__ "MatConvertToCSC"
 /* 
    convert Petsc seqaij matrix to CSC: colptr[n], row[nz], val[nz] 
 
@@ -53,7 +59,7 @@ EXTERN PetscErrorCode MatDuplicate_Pastix(Mat,MatDuplicateOption,Mat*);
     row     - Row of each element of the matrix                   
     values  - Value of each element of the matrix                 
  */
-PetscErrorCode MatConvertToCSC(Mat A,PetscTruth valOnly,PetscInt *n,PetscInt **colptr,PetscInt **row,PetscScalar **values) 
+PetscErrorCode MatConvertToCSC(Mat A,PetscBool  valOnly,PetscInt *n,PetscInt **colptr,PetscInt **row,PetscScalar **values) 
 {
   Mat_SeqAIJ     *aa      = (Mat_SeqAIJ*)A->data;
   PetscInt       *rowptr  = aa->i;
@@ -67,15 +73,12 @@ PetscErrorCode MatConvertToCSC(Mat A,PetscTruth valOnly,PetscInt *n,PetscInt **c
   PetscErrorCode  ierr;
   PetscInt        colidx;
   PetscInt       *colcount; 
-  PetscTruth      isSBAIJ;
-  PetscTruth      isSeqSBAIJ;
-  PetscTruth      isMpiSBAIJ;
-  PetscTruth      isSym;
-  
+  PetscBool       isSBAIJ;
+  PetscBool       isSeqSBAIJ;
+  PetscBool       isMpiSBAIJ;
+  PetscBool       isSym;
 
   PetscFunctionBegin;
-  /* Allocate the CSC */
-
   ierr = MatIsSymmetric(A,0.0,&isSym);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)A,MATSBAIJ,&isSBAIJ);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)A,MATSEQSBAIJ,&isSeqSBAIJ);CHKERRQ(ierr);
@@ -98,7 +101,6 @@ PetscErrorCode MatConvertToCSC(Mat A,PetscTruth valOnly,PetscInt *n,PetscInt **c
     ierr = PetscMalloc( nnz     *sizeof(PetscScalar),values);CHKERRQ(ierr);
 
     if (isSBAIJ || isSeqSBAIJ || isMpiSBAIJ) {
-	fprintf(stdout, "prout\n");
 	ierr = PetscMemcpy (*colptr, rowptr, ((*n)+1)*sizeof(PetscInt));CHKERRQ(ierr);
 	for (i = 0; i < *n+1; i++)
 	  (*colptr)[i] += base;
@@ -109,13 +111,13 @@ PetscErrorCode MatConvertToCSC(Mat A,PetscTruth valOnly,PetscInt *n,PetscInt **c
     } else {
       ierr = PetscMalloc((*n)*sizeof(PetscInt)   ,&colcount);CHKERRQ(ierr); 
 
-      for (i = 0; i < m; i++)
-	colcount[i] = 0;
+      for (i = 0; i < m; i++) colcount[i] = 0;
       /* Fill-in colptr */
-      for (i = 0; i < m; i++)
-	for (j = rowptr[i]; j < rowptr[i+1]; j++) 	
-	  if (!isSym || col[j] <= i)
-	    colcount[col[j]]++;
+      for (i = 0; i < m; i++) {
+	for (j = rowptr[i]; j < rowptr[i+1]; j++) { 	
+	  if (!isSym || col[j] <= i)  colcount[col[j]]++;
+        }
+      }
       
       (*colptr)[0] = base;
       for (j = 0; j < *n; j++) {
@@ -146,27 +148,49 @@ PetscErrorCode MatConvertToCSC(Mat A,PetscTruth valOnly,PetscInt *n,PetscInt **c
 	if ((isSBAIJ || isSeqSBAIJ || isMpiSBAIJ) ||!isSym || col[j] <= i)
 	  {
 	    /* look for the value to fill */
-	    for (k = (*colptr)[colidx] - base; 
-		 k < (*colptr)[colidx + 1] - base; 
-		 k++) {
-	      if ((*row)[k] == i) {
+	    for (k = (*colptr)[colidx] - base; k < (*colptr)[colidx + 1] - base; k++) {
+	      if (((*row)[k]-base) == i) {
 		(*values)[k] = rvalues[j];
 		break;
 	      }
 	    }
-	    /* shouldn't happen, overflow */
-	    if (k == (*colptr)[colidx + 1] - base)
-	      PetscFunctionReturn(1);
+	    /* data structure of sparse matrix has changed */
+	    if (k == (*colptr)[colidx + 1] - base) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"overflow on k %D",k);
 	  }
       }
     }
   }  
+  {
+    PetscScalar *tmpvalues;
+    PetscInt    *tmprows,*tmpcolptr;
+    tmpvalues = (PetscScalar*)malloc(nnz*sizeof(PetscScalar)); if (!tmpvalues) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM,"Unable to allocate memory");
+    tmprows   = (PetscInt*)malloc(nnz*sizeof(PetscInt));if (!tmprows) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM,"Unable to allocate memory");
+    tmpcolptr = (PetscInt*)malloc((*n+1)*sizeof(PetscInt));if (!tmpcolptr) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM,"Unable to allocate memory");
+
+    ierr = PetscMemcpy(tmpcolptr,*colptr,(*n+1)*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscMemcpy(tmprows,*row,nnz*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscMemcpy(tmpvalues,*values,nnz*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscFree(*row);CHKERRQ(ierr);
+    ierr = PetscFree(*values);CHKERRQ(ierr);
+
+    pastix_checkMatrix(MPI_COMM_WORLD,API_VERBOSE_NO,((isSym != 0) ? API_SYM_YES : API_SYM_NO),API_YES,*n,&tmpcolptr,&tmprows,&tmpvalues,NULL,1);
+    
+    ierr = PetscMemcpy(*colptr,tmpcolptr,(*n+1)*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscMalloc(((*colptr)[*n]-1)*sizeof(PetscInt),row);CHKERRQ(ierr);
+    ierr = PetscMemcpy(*row,tmprows,((*colptr)[*n]-1)*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscMalloc(((*colptr)[*n]-1)*sizeof(PetscScalar),values);CHKERRQ(ierr);
+    ierr = PetscMemcpy(*values,tmpvalues,((*colptr)[*n]-1)*sizeof(PetscScalar));CHKERRQ(ierr);
+    free(tmpvalues);
+    free(tmprows);
+    free(tmpcolptr);
+
+  }
   PetscFunctionReturn(0);
 }
 
 
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatDestroy_Pastix"
 /*
   Call clean step of PaStiX if lu->CleanUpPastix == true.
@@ -174,45 +198,46 @@ PetscErrorCode MatConvertToCSC(Mat A,PetscTruth valOnly,PetscInt *n,PetscInt **c
  */
 PetscErrorCode MatDestroy_Pastix(Mat A)
 {
-  Mat_Pastix      *lu=(Mat_Pastix*)A->spptr; 
+  Mat_Pastix      *lu=(Mat_Pastix*)A->spptr;
   PetscErrorCode   ierr;
   PetscMPIInt      size=lu->commSize;
 
   PetscFunctionBegin;
-  if (lu->CleanUpPastix) {    
+  if (lu && lu->CleanUpPastix) {
     /* Terminate instance, deallocate memories */
     if (size > 1){
-      ierr = VecScatterDestroy(lu->scat_rhs);CHKERRQ(ierr);
-      ierr = VecDestroy(lu->b_seq);CHKERRQ(ierr);
-      if (lu->nSolve && lu->scat_sol){ierr = VecScatterDestroy(lu->scat_sol);CHKERRQ(ierr);}
-      if (lu->nSolve && lu->x_seq){ierr = VecDestroy(lu->x_seq);CHKERRQ(ierr);}
+      ierr = VecScatterDestroy(&lu->scat_rhs);CHKERRQ(ierr);
+      ierr = VecDestroy(&lu->b_seq);CHKERRQ(ierr);
+      ierr = VecScatterDestroy(&lu->scat_sol);CHKERRQ(ierr);
     }
-    
-    lu->iparm[IPARM_START_TASK]=API_TASK_CLEAN; 
-    lu->iparm[IPARM_END_TASK]=API_TASK_CLEAN; 
+
+    lu->iparm[IPARM_START_TASK]=API_TASK_CLEAN;
+    lu->iparm[IPARM_END_TASK]=API_TASK_CLEAN;
 
     pastix((pastix_data_t **)&(lu->pastix_data),
-	                      lu->pastix_comm,
-	   (pastix_int_t)     lu->n,
-	   (pastix_int_t*)    lu->colptr,
-	   (pastix_int_t*)    lu->row,
-	   (pastix_float_t*)  lu->val,
-	   (pastix_int_t*)    lu->perm,
-	   (pastix_int_t*)    lu->invp,
-	   (pastix_float_t*)  lu->rhs,
-	   (pastix_int_t)     lu->rhsnbr,
-	   (pastix_int_t*)    lu->iparm,
-	                      lu->dparm);
+                              lu->pastix_comm,
+           (pastix_int_t)     lu->n,
+           (pastix_int_t*)    lu->colptr,
+           (pastix_int_t*)    lu->row,
+           (pastix_float_t*)  lu->val,
+           (pastix_int_t*)    lu->perm,
+           (pastix_int_t*)    lu->invp,
+           (pastix_float_t*)  lu->rhs,
+           (pastix_int_t)     lu->rhsnbr,
+           (pastix_int_t*)    lu->iparm,
+                              lu->dparm);
 
     ierr = PetscFree(lu->colptr);CHKERRQ(ierr);
-    ierr = PetscFree(lu->row);   CHKERRQ(ierr);    
-    ierr = PetscFree(lu->val);   CHKERRQ(ierr);
-    ierr = PetscFree(lu->perm);  CHKERRQ(ierr);
-    ierr = PetscFree(lu->invp);  CHKERRQ(ierr); 
-/*     ierr = PetscFree(lu->rhs);   CHKERRQ(ierr); */
+    ierr = PetscFree(lu->row);  CHKERRQ(ierr);
+    ierr = PetscFree(lu->val);  CHKERRQ(ierr);
+    ierr = PetscFree(lu->perm); CHKERRQ(ierr);
+    ierr = PetscFree(lu->invp); CHKERRQ(ierr);
     ierr = MPI_Comm_free(&(lu->pastix_comm));CHKERRQ(ierr);
   }
-  ierr = (lu->MatDestroy)(A);CHKERRQ(ierr);
+  if (lu && lu->Destroy) {
+    ierr = (lu->Destroy)(A);CHKERRQ(ierr);
+  }
+  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -268,9 +293,7 @@ PetscErrorCode MatSolve_PaStiX(Mat A,Vec b,Vec x)
 	 (pastix_int_t*)    lu->iparm, 
 	 (double*)          lu->dparm);
   
-  if (lu->iparm[IPARM_ERROR_NUMBER] < 0) {   
-    SETERRQ1(PETSC_ERR_LIB,"Error reported by PaStiX in solve phase: lu->iparm[IPARM_ERROR_NUMBER] = %d\n",lu->iparm[IPARM_ERROR_NUMBER] );
-  }
+  if (lu->iparm[IPARM_ERROR_NUMBER] < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error reported by PaStiX in solve phase: lu->iparm[IPARM_ERROR_NUMBER] = %d\n",lu->iparm[IPARM_ERROR_NUMBER] );
 
   if (lu->commSize == 1){
     ierr = VecRestoreArray(x,&(lu->rhs));CHKERRQ(ierr);
@@ -282,54 +305,15 @@ PetscErrorCode MatSolve_PaStiX(Mat A,Vec b,Vec x)
     ierr = VecScatterBegin(lu->scat_sol,x_seq,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     ierr = VecScatterEnd(lu->scat_sol,x_seq,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   }
-  lu->nSolve++; 
   PetscFunctionReturn(0);
 }
 
-#if !defined(PETSC_USE_COMPLEX)
-  /* 
-     TODO: Fill this function 
-     I didn't fill this function 
-     because I didn't understood its goal.
-  */
-
-/*
-  input:
-   F:        numeric factor
-  output:
-   nneg:     total number of pivots
-   nzero:    0
-   npos:     (global dimension of F) - nneg
-*/
-
-#undef __FUNCT__
-#define __FUNCT__ "MatGetInertia_SBAIJPASTIX"
-PetscErrorCode MatGetInertia_SBAIJPASTIX(Mat F,int *nneg,int *nzero,int *npos)
-{
-  PetscFunctionBegin;
-/*   ierr = MPI_Comm_size(((PetscObject)F)->comm,&size);CHKERRQ(ierr); */
-/*   /\* PASTIX 4.3.1 calls ScaLAPACK when ICNTL(13)=0 (default), which does not offer the possibility to compute the inertia of a dense matrix. Set ICNTL(13)=1 to skip ScaLAPACK *\/ */
-/*   if (size > 1 && lu->id.ICNTL(13) != 1){ */
-/*     SETERRQ1(PETSC_ERR_ARG_WRONG,"ICNTL(13)=%d. -mat_pastix_icntl_13 must be set as 1 for correct global matrix inertia\n",lu->id.INFOG(13)); */
-/*   } */
-/*   if (nneg){ */
-/*     if (!lu->commSize){ */
-/*       *nneg = lu->id.INFOG(12); */
-/*     } */
-/*     ierr = MPI_Bcast(nneg,1,MPI_INT,0,lu->comm_pastix);CHKERRQ(ierr); */
-/*   } */
-/*   if (nzero) *nzero = lu->iparm[IPARM_NNZEROS]; */
-/*   if (npos)  *npos  = F->rmap->N - (*nneg); */
-  PetscFunctionReturn(0);
-}
-#endif /* !defined(PETSC_USE_COMPLEX) */
-
 /*
   Numeric factorisation using PaStiX solver.
 
  */
 #undef __FUNCT__   
-#define __FUNCT__ "MatFactorNumeric_PASTIX"
+#define __FUNCT__ "MatFactorNumeric_PaStiX"
 PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info) 
 {
   Mat_Pastix    *lu =(Mat_Pastix*)(F)->spptr; 
@@ -337,24 +321,25 @@ PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info)
   PetscErrorCode ierr = 0;
   PetscInt       icntl;
   PetscInt       M=A->rmap->N;
-  PetscTruth     valOnly,flg, isSym;
+  PetscBool      valOnly,flg, isSym;
   Mat            F_diag; 
   IS             is_iden;
   Vec            b;
   IS             isrow;
-  PetscTruth     isSeqAIJ,isSeqSBAIJ;
+  PetscBool      isSeqAIJ,isSeqSBAIJ,isMPIAIJ;
 
   PetscFunctionBegin; 	
   
   ierr = PetscTypeCompare((PetscObject)A,MATSEQAIJ,&isSeqAIJ);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)A,MATMPIAIJ,&isMPIAIJ);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)A,MATSEQSBAIJ,&isSeqSBAIJ);CHKERRQ(ierr);
   if (lu->matstruc == DIFFERENT_NONZERO_PATTERN){ 
     (F)->ops->solve   = MatSolve_PaStiX;
 
     /* Initialize a PASTIX instance */
     ierr = MPI_Comm_dup(((PetscObject)A)->comm,&(lu->pastix_comm));CHKERRQ(ierr);
-    ierr = MPI_Comm_rank(lu->pastix_comm, &lu->commRank);          CHKERRQ(ierr);
-    ierr = MPI_Comm_size(lu->pastix_comm, &lu->commSize);          CHKERRQ(ierr);
+    ierr = MPI_Comm_rank(lu->pastix_comm, &lu->commRank);         CHKERRQ(ierr);
+    ierr = MPI_Comm_size(lu->pastix_comm, &lu->commSize);         CHKERRQ(ierr);
 
     /* Set pastix options */
     lu->iparm[IPARM_MODIFY_PARAMETER] = API_NO;
@@ -392,7 +377,12 @@ PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info)
     PetscOptionsEnd();
     valOnly = PETSC_FALSE; 
   }  else {
-    valOnly = PETSC_TRUE; 
+    if (isSeqAIJ || isMPIAIJ)  {
+      ierr = PetscFree(lu->colptr);CHKERRQ(ierr);
+      ierr = PetscFree(lu->row);CHKERRQ(ierr);
+      ierr = PetscFree(lu->val);CHKERRQ(ierr);
+      valOnly = PETSC_FALSE;
+    } else valOnly = PETSC_TRUE; 
   }
 
   lu->iparm[IPARM_MATRIX_VERIFICATION] = API_YES;
@@ -400,22 +390,22 @@ PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info)
   /* convert mpi A to seq mat A */
   ierr = ISCreateStride(PETSC_COMM_SELF,M,0,1,&isrow);CHKERRQ(ierr);  
   ierr = MatGetSubMatrices(A,1,&isrow,&isrow,MAT_INITIAL_MATRIX,&tseq);CHKERRQ(ierr);
-  ierr = ISDestroy(isrow);CHKERRQ(ierr);
+  ierr = ISDestroy(&isrow);CHKERRQ(ierr);
 
-  ierr = MatConvertToCSC(*tseq,valOnly, &lu->n, &lu->colptr, &lu->row, &lu->val); CHKERRQ(ierr);
+  ierr = MatConvertToCSC(*tseq,valOnly, &lu->n, &lu->colptr, &lu->row, &lu->val);CHKERRQ(ierr);
   ierr = MatIsSymmetric(*tseq,0.0,&isSym);CHKERRQ(ierr);
   ierr = MatDestroyMatrices(1,&tseq);CHKERRQ(ierr);
 
-  ierr = PetscMalloc((lu->n)*sizeof(PetscInt)   ,&(lu->perm));CHKERRQ(ierr);
-  ierr = PetscMalloc((lu->n)*sizeof(PetscInt)   ,&(lu->invp));CHKERRQ(ierr);
-
+  if (!lu->perm) {
+    ierr = PetscMalloc((lu->n)*sizeof(PetscInt)   ,&(lu->perm));CHKERRQ(ierr);
+    ierr = PetscMalloc((lu->n)*sizeof(PetscInt)   ,&(lu->invp));CHKERRQ(ierr);
+  }
 
   if (isSym) {
     /* On symmetric matrix, LLT */
     lu->iparm[IPARM_SYM] = API_SYM_YES;
     lu->iparm[IPARM_FACTORIZATION] = API_FACT_LDLT;
-  }
-  else {
+  } else {
     /* On unsymmetric matrix, LU */
     lu->iparm[IPARM_SYM] = API_SYM_NO;
     lu->iparm[IPARM_FACTORIZATION] = API_FACT_LU; 
@@ -433,8 +423,8 @@ PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info)
 	
 	ierr = VecScatterCreate(b,is_iden,lu->b_seq,is_iden,&lu->scat_rhs);CHKERRQ(ierr);
 	ierr = VecScatterCreate(lu->b_seq,is_iden,b,is_iden,&lu->scat_sol);CHKERRQ(ierr);
-	ierr = ISDestroy(is_iden);CHKERRQ(ierr);
-	ierr = VecDestroy(b);CHKERRQ(ierr);    
+	ierr = ISDestroy(&is_iden);CHKERRQ(ierr);
+	ierr = VecDestroy(&b);CHKERRQ(ierr);    
     }    
     lu->iparm[IPARM_START_TASK] = API_TASK_ORDERING;
     lu->iparm[IPARM_END_TASK]   = API_TASK_NUMFACT;
@@ -451,9 +441,7 @@ PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info)
 	   (pastix_int_t)     lu->rhsnbr, 
 	   (pastix_int_t*)    lu->iparm, 
 	   (double*)          lu->dparm);
-    if (lu->iparm[IPARM_ERROR_NUMBER] < 0) { 
-      SETERRQ1(PETSC_ERR_LIB,"Error reported by PaStiX in analysis phase: iparm(IPARM_ERROR_NUMBER)=%d\n",lu->iparm[IPARM_ERROR_NUMBER]); 
-    }
+    if (lu->iparm[IPARM_ERROR_NUMBER] < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error reported by PaStiX in analysis phase: iparm(IPARM_ERROR_NUMBER)=%d\n",lu->iparm[IPARM_ERROR_NUMBER]); 
   } else {
     lu->iparm[IPARM_START_TASK] = API_TASK_NUMFACT;
     lu->iparm[IPARM_END_TASK]   = API_TASK_NUMFACT;
@@ -470,31 +458,23 @@ PetscErrorCode MatFactorNumeric_PaStiX(Mat F,Mat A,const MatFactorInfo *info)
 	   (pastix_int_t*)    lu->iparm, 
 	   (double*)          lu->dparm);
 
-    if (lu->iparm[IPARM_ERROR_NUMBER] < 0) { 
-      SETERRQ1(PETSC_ERR_LIB,"Error reported by PaStiX in analysis phase: iparm(IPARM_ERROR_NUMBER)=%d\n",lu->iparm[IPARM_ERROR_NUMBER]);
-    }
+    if (lu->iparm[IPARM_ERROR_NUMBER] < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error reported by PaStiX in analysis phase: iparm(IPARM_ERROR_NUMBER)=%d\n",lu->iparm[IPARM_ERROR_NUMBER]);
   }
 
   if (lu->commSize > 1){
-    if ((F)->factor == MAT_FACTOR_LU){
+    if ((F)->factortype == MAT_FACTOR_LU){
       F_diag = ((Mat_MPIAIJ *)(F)->data)->A;
     } else {
       F_diag = ((Mat_MPISBAIJ *)(F)->data)->A;
     }
     F_diag->assembled = PETSC_TRUE;
-    if (lu->nSolve){
-      ierr = VecScatterDestroy(lu->scat_sol);CHKERRQ(ierr);  
-      ierr = VecDestroy(lu->x_seq);CHKERRQ(ierr);
-    }
   }
   (F)->assembled     = PETSC_TRUE;
   lu->matstruc       = SAME_NONZERO_PATTERN;
   lu->CleanUpPastix  = PETSC_TRUE;
-  lu->nSolve         = 0;
   PetscFunctionReturn(0);
 }
 
-
 /* Note the Petsc r and c permutations are ignored */
 #undef __FUNCT__  
 #define __FUNCT__ "MatLUFactorSymbolic_AIJPASTIX"
@@ -523,9 +503,6 @@ PetscErrorCode MatCholeskyFactorSymbolic_SBAIJPASTIX(Mat F,Mat A,IS r,const MatF
   lu->iparm[IPARM_SYM]            = API_SYM_NO;
   lu->matstruc                    = DIFFERENT_NONZERO_PATTERN;
   (F)->ops->choleskyfactornumeric = MatFactorNumeric_PaStiX;
-#if !defined(PETSC_USE_COMPLEX)
-  (F)->ops->getinertia            = MatGetInertia_SBAIJPASTIX;
-#endif
   PetscFunctionReturn(0);
 }
 
@@ -534,11 +511,11 @@ PetscErrorCode MatCholeskyFactorSymbolic_SBAIJPASTIX(Mat F,Mat A,IS r,const MatF
 PetscErrorCode MatView_PaStiX(Mat A,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
-  PetscTruth        iascii;
+  PetscBool         iascii;
   PetscViewerFormat format;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_INFO){
@@ -556,10 +533,10 @@ PetscErrorCode MatView_PaStiX(Mat A,PetscViewer viewer)
 
 
 /*MC
-     MAT_SOLVER_PASTIX  - A solver package providing direct solvers (LU) for distributed
+     MATSOLVERPASTIX  - A solver package providing direct solvers (LU) for distributed
   and sequential matrices via the external package PaStiX.
 
-  Use config/configure.py --download-pastix to have PETSc installed with PaStiX
+  Use ./configure --download-pastix to have PETSc installed with PaStiX
 
   Options Database Keys:
 + -mat_pastix_verbose   <0,1,2>   - print level
@@ -598,7 +575,7 @@ EXTERN_C_BEGIN
 PetscErrorCode MatFactorGetSolverPackage_pastix(Mat A,const MatSolverPackage *type)
 {
   PetscFunctionBegin;
-  *type = MAT_SOLVER_PASTIX;
+  *type = MATSOLVERPASTIX;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -616,9 +593,7 @@ PetscErrorCode MatGetFactor_seqaij_pastix(Mat A,MatFactorType ftype,Mat *F)
   Mat_Pastix    *pastix;
 
   PetscFunctionBegin;
-  if (ftype != MAT_FACTOR_LU) {
-    SETERRQ(PETSC_ERR_SUP,"Cannot use PETSc AIJ matrices with PaStiX Cholesky, use SBAIJ matrix");
-  }
+  if (ftype != MAT_FACTOR_LU) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot use PETSc AIJ matrices with PaStiX Cholesky, use SBAIJ matrix");
   /* Create the factorization matrix */
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,A->rmap->N,A->cmap->N);CHKERRQ(ierr);
@@ -628,18 +603,15 @@ PetscErrorCode MatGetFactor_seqaij_pastix(Mat A,MatFactorType ftype,Mat *F)
   B->ops->lufactorsymbolic = MatLUFactorSymbolic_AIJPASTIX;
   B->ops->view             = MatView_PaStiX;
   B->ops->getinfo          = MatGetInfo_PaStiX;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C",
-					   "MatFactorGetSolverPackage_pastix",
-					   MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
-  B->factor                = MAT_FACTOR_LU;  
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_pastix", MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
+  B->factortype            = MAT_FACTOR_LU;  
 
   ierr = PetscNewLog(B,Mat_Pastix,&pastix);CHKERRQ(ierr);
   pastix->CleanUpPastix             = PETSC_FALSE;
   pastix->isAIJ                     = PETSC_TRUE;
   pastix->scat_rhs                  = PETSC_NULL;
   pastix->scat_sol                  = PETSC_NULL;
-  pastix->nSolve                    = 0;
-  pastix->MatDestroy                = B->ops->destroy;
+  pastix->Destroy                   = B->ops->destroy;
   B->ops->destroy                   = MatDestroy_Pastix;
   B->spptr                          = (void*)pastix;
 
@@ -659,7 +631,7 @@ PetscErrorCode MatGetFactor_mpiaij_pastix(Mat A,MatFactorType ftype,Mat *F)
   Mat_Pastix    *pastix;
 
   PetscFunctionBegin;
-  if (ftype != MAT_FACTOR_LU) SETERRQ(PETSC_ERR_SUP,"Cannot use PETSc AIJ matrices with PaStiX Cholesky, use SBAIJ matrix");
+  if (ftype != MAT_FACTOR_LU) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot use PETSc AIJ matrices with PaStiX Cholesky, use SBAIJ matrix");
   /* Create the factorization matrix */
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,A->rmap->N,A->cmap->N);CHKERRQ(ierr);
@@ -669,21 +641,17 @@ PetscErrorCode MatGetFactor_mpiaij_pastix(Mat A,MatFactorType ftype,Mat *F)
 
   B->ops->lufactorsymbolic = MatLUFactorSymbolic_AIJPASTIX;
   B->ops->view             = MatView_PaStiX;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,
-					   "MatFactorGetSolverPackage_C",
-					   "MatFactorGetSolverPackage_pastix",
-					   MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
-  B->factor                = MAT_FACTOR_LU;  
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_pastix",MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
+  B->factortype            = MAT_FACTOR_LU;  
 
   ierr = PetscNewLog(B,Mat_Pastix,&pastix);CHKERRQ(ierr);
   pastix->CleanUpPastix             = PETSC_FALSE;
   pastix->isAIJ                     = PETSC_TRUE;
   pastix->scat_rhs                  = PETSC_NULL;
   pastix->scat_sol                  = PETSC_NULL;
-  pastix->nSolve                    = 0;
-  pastix->MatDestroy                = B->ops->destroy;
-  B->ops->destroy                  = MatDestroy_Pastix;
-  B->spptr                         = (void*)pastix;
+  pastix->Destroy                   = B->ops->destroy;
+  B->ops->destroy                   = MatDestroy_Pastix;
+  B->spptr                          = (void*)pastix;
 
   *F = B;
   PetscFunctionReturn(0); 
@@ -700,10 +668,8 @@ PetscErrorCode MatGetFactor_seqsbaij_pastix(Mat A,MatFactorType ftype,Mat *F)
   Mat_Pastix    *pastix;
 
   PetscFunctionBegin;
-  if (ftype != MAT_FACTOR_CHOLESKY) {
-    SETERRQ(PETSC_ERR_SUP,"Cannot use PETSc SBAIJ matrices with PaStiX LU, use AIJ matrix");
-  }
-  /* Create the factorization matrix */ 
+  if (ftype != MAT_FACTOR_CHOLESKY) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot use PETSc SBAIJ matrices with PaStiX LU, use AIJ matrix");
+  /* Create the factorization matrix */
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,A->rmap->N,A->cmap->N);CHKERRQ(ierr);
   ierr = MatSetType(B,((PetscObject)A)->type_name);CHKERRQ(ierr);
@@ -712,22 +678,17 @@ PetscErrorCode MatGetFactor_seqsbaij_pastix(Mat A,MatFactorType ftype,Mat *F)
 
   B->ops->choleskyfactorsymbolic = MatCholeskyFactorSymbolic_SBAIJPASTIX;
   B->ops->view                   = MatView_PaStiX;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,
-					   "MatFactorGetSolverPackage_C",
-					   "MatFactorGetSolverPackage_pastix",
-					   MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
-
-  B->factor                      = MAT_FACTOR_CHOLESKY;
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_pastix",MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
+  B->factortype                  = MAT_FACTOR_CHOLESKY;
 
   ierr = PetscNewLog(B,Mat_Pastix,&pastix);CHKERRQ(ierr);
   pastix->CleanUpPastix             = PETSC_FALSE;
   pastix->isAIJ                     = PETSC_TRUE;
   pastix->scat_rhs                  = PETSC_NULL;
   pastix->scat_sol                  = PETSC_NULL;
-  pastix->nSolve                    = 0;
-  pastix->MatDestroy                = B->ops->destroy;
-  B->ops->destroy                  = MatDestroy_Pastix;
-  B->spptr                         = (void*)pastix;
+  pastix->Destroy                   = B->ops->destroy;
+  B->ops->destroy                   = MatDestroy_Pastix;
+  B->spptr                          = (void*)pastix;
 
   *F = B;
   PetscFunctionReturn(0);
@@ -744,7 +705,7 @@ PetscErrorCode MatGetFactor_mpisbaij_pastix(Mat A,MatFactorType ftype,Mat *F)
   Mat_Pastix    *pastix;
  
   PetscFunctionBegin;
-  if (ftype != MAT_FACTOR_CHOLESKY) SETERRQ(PETSC_ERR_SUP,"Cannot use PETSc SBAIJ matrices with PaStiX LU, use AIJ matrix");
+  if (ftype != MAT_FACTOR_CHOLESKY) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot use PETSc SBAIJ matrices with PaStiX LU, use AIJ matrix");
 
   /* Create the factorization matrix */ 
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
@@ -755,19 +716,15 @@ PetscErrorCode MatGetFactor_mpisbaij_pastix(Mat A,MatFactorType ftype,Mat *F)
 
   B->ops->choleskyfactorsymbolic = MatCholeskyFactorSymbolic_SBAIJPASTIX;
   B->ops->view                   = MatView_PaStiX;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,
-					   "MatFactorGetSolverPackage_C",
-					   "MatFactorGetSolverPackage_pastix",
-					   MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
-  B->factor                      = MAT_FACTOR_CHOLESKY;
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_pastix",MatFactorGetSolverPackage_pastix);CHKERRQ(ierr);
+  B->factortype                  = MAT_FACTOR_CHOLESKY;
 
   ierr = PetscNewLog(B,Mat_Pastix,&pastix);CHKERRQ(ierr);
   pastix->CleanUpPastix             = PETSC_FALSE;
   pastix->isAIJ                     = PETSC_TRUE;
   pastix->scat_rhs                  = PETSC_NULL;
   pastix->scat_sol                  = PETSC_NULL;
-  pastix->nSolve                    = 0;
-  pastix->MatDestroy                = B->ops->destroy;
+  pastix->Destroy                   = B->ops->destroy;
   B->ops->destroy                   = MatDestroy_Pastix;
   B->spptr                          = (void*)pastix;
 
