@@ -1,48 +1,45 @@
-#define PETSCMAT_DLL
 
 /* 
    Provides an interface to the Spooles parallel sparse solver (MPI SPOOLES)
 */
 
-#include "../src/mat/impls/aij/seq/aij.h"
-#include "../src/mat/impls/sbaij/seq/sbaij.h"
-#include "../src/mat/impls/baij/seq/baij.h"
-#include "../src/mat/impls/aij/mpi/mpiaij.h"
-#include "../src/mat/impls/sbaij/mpi/mpisbaij.h"
-#include "../src/mat/impls/aij/seq/spooles/spooles.h"
+#include <../src/mat/impls/aij/seq/aij.h>
+#include <../src/mat/impls/sbaij/seq/sbaij.h>
+#include <../src/mat/impls/baij/seq/baij.h>
+#include <../src/mat/impls/aij/mpi/mpiaij.h>
+#include <../src/mat/impls/sbaij/mpi/mpisbaij.h>
+#include <../src/mat/impls/aij/seq/spooles/spooles.h>
 
-EXTERN int SetSpoolesOptions(Mat, Spooles_options *);
-EXTERN PetscErrorCode MatDestroy_MPIAIJ(Mat);
+extern int SetSpoolesOptions(Mat, Spooles_options *);
+extern PetscErrorCode MatDestroy_MPIAIJ(Mat);
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatDestroy_MPIAIJSpooles"
 PetscErrorCode MatDestroy_MPIAIJSpooles(Mat A)
 {
-  Mat_Spooles   *lu = (Mat_Spooles*)A->spptr; 
+  Mat_Spooles   *lu = (Mat_Spooles*)A->spptr;
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
-  if (lu->CleanUpSpooles) {
-    FrontMtx_free(lu->frontmtx);        
-    IV_free(lu->newToOldIV);            
-    IV_free(lu->oldToNewIV); 
+  if (lu && lu->CleanUpSpooles) {
+    FrontMtx_free(lu->frontmtx);
+    IV_free(lu->newToOldIV);
+    IV_free(lu->oldToNewIV);
     IV_free(lu->vtxmapIV);
-    InpMtx_free(lu->mtxA);             
-    ETree_free(lu->frontETree);          
-    IVL_free(lu->symbfacIVL);         
-    SubMtxManager_free(lu->mtxmanager);    
+    InpMtx_free(lu->mtxA);
+    ETree_free(lu->frontETree);
+    IVL_free(lu->symbfacIVL);
+    SubMtxManager_free(lu->mtxmanager);
     DenseMtx_free(lu->mtxX);
     DenseMtx_free(lu->mtxY);
     ierr = MPI_Comm_free(&(lu->comm_spooles));CHKERRQ(ierr);
-    if ( lu->scat ){
-      ierr = VecDestroy(lu->vec_spooles);CHKERRQ(ierr); 
-      ierr = ISDestroy(lu->iden);CHKERRQ(ierr); 
-      ierr = ISDestroy(lu->is_petsc);CHKERRQ(ierr);
-      ierr = VecScatterDestroy(lu->scat);CHKERRQ(ierr);
-    }
+    ierr = VecDestroy(&lu->vec_spooles);CHKERRQ(ierr);
+    ierr = ISDestroy(&lu->iden);CHKERRQ(ierr);
+    ierr = ISDestroy(&lu->is_petsc);CHKERRQ(ierr);
+    ierr = VecScatterDestroy(&lu->scat);CHKERRQ(ierr);
   }
+  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
   ierr = MatDestroy_MPIAIJ(A);CHKERRQ(ierr);
-
   PetscFunctionReturn(0);
 }
 
@@ -85,7 +82,7 @@ PetscErrorCode MatSolve_MPISpooles(Mat A,Vec b,Vec x)
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n 1 matrix in original ordering");CHKERRQ(ierr);
     DenseMtx_writeForHumanEye(lu->mtxY, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
   
   /* permute and redistribute Y if necessary */
@@ -95,7 +92,7 @@ PetscErrorCode MatSolve_MPISpooles(Mat A,Vec b,Vec x)
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n rhs matrix in new ordering");CHKERRQ(ierr);
     DenseMtx_writeForHumanEye(lu->mtxY, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   MPI_Barrier(((PetscObject)A)->comm); /* for initializing firsttag, because the num. of tags used
@@ -111,7 +108,7 @@ PetscErrorCode MatSolve_MPISpooles(Mat A,Vec b,Vec x)
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n split DenseMtx Y");CHKERRQ(ierr);
     DenseMtx_writeForHumanEye(lu->mtxY, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   if ( FRONTMTX_IS_PIVOTING(lu->frontmtx) ) {
@@ -132,7 +129,7 @@ PetscErrorCode MatSolve_MPISpooles(Mat A,Vec b,Vec x)
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n rhs matrix after split");CHKERRQ(ierr);
     DenseMtx_writeForHumanEye(lu->mtxY, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   if ( lu->nmycol > 0 ) IVcopy(lu->nmycol,lu->rowindX,IV_entries(lu->ownedColumnsIV)); /* must do for each solve */
@@ -155,7 +152,7 @@ PetscErrorCode MatSolve_MPISpooles(Mat A,Vec b,Vec x)
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n solution in old ordering");CHKERRQ(ierr);
     DenseMtx_writeForHumanEye(lu->mtxX, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
   
   /* scatter local solution mtxX into mpi vector x */ 
@@ -167,7 +164,7 @@ PetscErrorCode MatSolve_MPISpooles(Mat A,Vec b,Vec x)
     ierr = VecCreateSeq(PETSC_COMM_SELF,lu->nmycol,&lu->vec_spooles);CHKERRQ(ierr);
 #endif 
     ierr = ISCreateStride(PETSC_COMM_SELF,lu->nmycol,0,1,&lu->iden);CHKERRQ(ierr);
-    ierr = ISCreateGeneral(PETSC_COMM_SELF,lu->nmycol,lu->rowindX,&lu->is_petsc);CHKERRQ(ierr);  
+    ierr = ISCreateGeneral(PETSC_COMM_SELF,lu->nmycol,lu->rowindX,PETSC_COPY_VALUES,&lu->is_petsc);CHKERRQ(ierr);  
     ierr = VecScatterCreate(lu->vec_spooles,lu->iden,x,lu->is_petsc,&lu->scat);CHKERRQ(ierr); 
   }
 #if defined(PETSC_USE_COMPLEX)
@@ -215,7 +212,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     ierr = SetSpoolesOptions(A, &lu->options);CHKERRQ(ierr);
 
     (F)->assembled    = PETSC_TRUE;
-    if ((F)->factor == MAT_FACTOR_LU){
+    if ((F)->factortype == MAT_FACTOR_LU){
       F_diag = ((Mat_MPIAIJ *)(F)->data)->A;
     } else {
       F_diag = ((Mat_MPISBAIJ *)(F)->data)->A;
@@ -320,7 +317,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n [%d] input matrix\n",rank);CHKERRQ(ierr);
     InpMtx_writeForHumanEye(lu->mtxA, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   if ( lu->flg == DIFFERENT_NONZERO_PATTERN){ /* first numeric factorization */
@@ -341,7 +338,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       err = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n graph of the input matrix");CHKERRQ(ierr);
       Graph_writeForHumanEye(graph, lu->options.msgFile);
       fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file"); 
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file"); 
     }
 
     switch (lu->options.ordering) {
@@ -358,7 +355,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       lu->frontETree = orderViaND(graph, lu->options.maxdomainsize, 
                      lu->options.seed + rank,lu->options.msglvl,lu->options.msgFile); break;
     default:
-      SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown Spooles's ordering");
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown Spooles's ordering");
     }
 
     Graph_free(graph);
@@ -367,7 +364,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n front tree from ordering");CHKERRQ(ierr);
       ETree_writeForHumanEye(lu->frontETree, lu->options.msgFile);
       err = fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
     }
 
     opcounts = DVinit(size, 0.0);
@@ -384,7 +381,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n best front tree");CHKERRQ(ierr);
       ETree_writeForHumanEye(lu->frontETree, lu->options.msgFile);
       err = fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
     }
   
     /* get the permutations, permute the front tree, permute the matrix */
@@ -419,7 +416,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n map from vertices to owning processes");CHKERRQ(ierr);
       IV_writeForHumanEye(lu->vtxmapIV, lu->options.msgFile);
       err = fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
     }
 
     /* redistribute the matrix */
@@ -436,7 +433,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n split InpMtx");CHKERRQ(ierr);
       InpMtx_writeForHumanEye(lu->mtxA, lu->options.msgFile);
       err = fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
     }
  
     /* compute the symbolic factorization */
@@ -448,7 +445,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n local symbolic factorization");CHKERRQ(ierr);
       IVL_writeForHumanEye(lu->symbfacIVL, lu->options.msgFile);
       err = fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
     }
 
     lu->mtxmanager = SubMtxManager_new();
@@ -456,10 +453,9 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     lu->frontmtx = FrontMtx_new();
 
   } else { /* new num factorization using previously computed symbolic factor */
-    if (lu->options.pivotingflag) {                  /* different FrontMtx is required */
-      FrontMtx_free(lu->frontmtx);   
-      lu->frontmtx   = FrontMtx_new();
-    }
+    /* different FrontMtx is required */ 
+    FrontMtx_free(lu->frontmtx);   
+    lu->frontmtx   = FrontMtx_new();
 
     SubMtxManager_free(lu->mtxmanager);  
     lu->mtxmanager = SubMtxManager_new();
@@ -487,7 +483,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n split InpMtx");CHKERRQ(ierr);
       InpMtx_writeForHumanEye(lu->mtxA, lu->options.msgFile);
       err = fflush(lu->options.msgFile);
-      if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+      if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
     }
   } /* end of if ( lu->flg == DIFFERENT_NONZERO_PATTERN) */
 
@@ -495,29 +491,22 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
               FRONTMTX_DENSE_FRONTS, lu->options.pivotingflag, NO_LOCK, rank,
               lu->ownersIV, lu->mtxmanager, lu->options.msglvl, lu->options.msgFile);
 
-    if ( lu->options.symflag == SPOOLES_SYMMETRIC ) {
-    if ( lu->options.patchAndGoFlag == 1 ) {
+  if ( lu->options.symflag == SPOOLES_SYMMETRIC ) {
+    if (lu->options.patchAndGoFlag == 1 || lu->options.patchAndGoFlag == 2 ){
+      if (lu->frontmtx->patchinfo) PatchAndGoInfo_free(lu->frontmtx->patchinfo);
       lu->frontmtx->patchinfo = PatchAndGoInfo_new();
-      PatchAndGoInfo_init(lu->frontmtx->patchinfo, 1, lu->options.toosmall, lu->options.fudge,
+      PatchAndGoInfo_init(lu->frontmtx->patchinfo, lu->options.patchAndGoFlag, lu->options.toosmall, lu->options.fudge,
                        lu->options.storeids, lu->options.storevalues);
-    } else if ( lu->options.patchAndGoFlag == 2 ) {
-      lu->frontmtx->patchinfo = PatchAndGoInfo_new();
-      PatchAndGoInfo_init(lu->frontmtx->patchinfo, 2, lu->options.toosmall, lu->options.fudge,
-                       lu->options.storeids, lu->options.storevalues);
-    }   
+    }
   }
 
   /* numerical factorization */
-  chvmanager = ChvManager_new();
+  chvmanager = ChvManager_new();  
   ChvManager_init(chvmanager, NO_LOCK, 0);  
 
   tagbound = maxTagMPI(lu->comm_spooles);
   lasttag  = lu->firsttag + 3*lu->frontETree->nfront + 2;
-  /* if(!rank) PetscPrintf(PETSC_COMM_SELF,"\n firsttag: %d, nfront: %d\n",lu->firsttag, lu->frontETree->nfront);*/
-  if ( lasttag > tagbound ) {
-      SETERRQ3(PETSC_ERR_LIB,"fatal error in FrontMtx_MPI_factorInpMtx(), tag range is [%d,%d], tag_bound = %d",\
-               lu->firsttag, lasttag, tagbound); 
-  }
+  if ( lasttag > tagbound ) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_LIB,"fatal error in FrontMtx_MPI_factorInpMtx(), tag range is [%d,%d], tag_bound = %d",lu->firsttag, lasttag, tagbound); 
   rootchv = FrontMtx_MPI_factorInpMtx(lu->frontmtx, lu->mtxA, lu->options.tau, droptol,
                      chvmanager, lu->ownersIV, lookahead, &sierr, lu->cpus, 
                      lu->stats, lu->options.msglvl, lu->options.msgFile, lu->firsttag,lu->comm_spooles);
@@ -528,7 +517,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n numeric factorization");CHKERRQ(ierr);
     FrontMtx_writeForHumanEye(lu->frontmtx, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   if ( lu->options.symflag == SPOOLES_SYMMETRIC ) {
@@ -554,15 +543,12 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
       PatchAndGoInfo_free(lu->frontmtx->patchinfo);
     }
   }
-  if ( sierr >= 0 ) SETERRQ2(PETSC_ERR_LIB,"\n proc %d : factorization error at front %d", rank, sierr);
+  if ( sierr >= 0 ) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_LIB,"\n proc %d : factorization error at front %d", rank, sierr);
  
   /*  post-process the factorization and split 
       the factor matrices into submatrices */
   lasttag  = lu->firsttag + 5*size;
-  if ( lasttag > tagbound ) {
-      SETERRQ3(PETSC_ERR_LIB,"fatal error in FrontMtx_MPI_postProcess(), tag range is [%d,%d], tag_bound = %d",\
-               lu->firsttag, lasttag, tagbound); 
-  }
+  if ( lasttag > tagbound ) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_LIB,"fatal error in FrontMtx_MPI_postProcess(), tag range is [%d,%d], tag_bound = %d",lu->firsttag, lasttag, tagbound); 
   FrontMtx_MPI_postProcess(lu->frontmtx, lu->ownersIV, lu->stats, lu->options.msglvl,
                          lu->options.msgFile, lu->firsttag, lu->comm_spooles);
   lu->firsttag += 5*size ;
@@ -571,10 +557,11 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n numeric factorization after post-processing");CHKERRQ(ierr);
     FrontMtx_writeForHumanEye(lu->frontmtx, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
   
   /* create the solve map object */
+  if (lu->solvemap) SolveMap_free(lu->solvemap);
   lu->solvemap = SolveMap_new();
   SolveMap_ddMap(lu->solvemap, lu->frontmtx->symmetryflag, 
                FrontMtx_upperBlockIVL(lu->frontmtx),
@@ -585,7 +572,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     int err;
     SolveMap_writeForHumanEye(lu->solvemap, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   /* redistribute the submatrices of the factors */
@@ -596,7 +583,7 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     ierr = PetscFPrintf(PETSC_COMM_SELF,lu->options.msgFile, "\n\n numeric factorization after split");CHKERRQ(ierr);
     FrontMtx_writeForHumanEye(lu->frontmtx, lu->options.msgFile);
     err = fflush(lu->options.msgFile);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");    
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");    
   }
 
   /* create a solution DenseMtx object */  
@@ -613,57 +600,16 @@ PetscErrorCode MatFactorNumeric_MPISpooles(Mat F,Mat A,const MatFactorInfo *info
     lu->rowindX = 0;
   }
 
-  if ( lu->scat ){
-    ierr = VecDestroy(lu->vec_spooles);CHKERRQ(ierr); 
-    ierr = ISDestroy(lu->iden);CHKERRQ(ierr); 
-    ierr = ISDestroy(lu->is_petsc);CHKERRQ(ierr);
-    ierr = VecScatterDestroy(lu->scat);CHKERRQ(ierr);
-  }
-  lu->scat = PETSC_NULL;  
-  lu->flg = SAME_NONZERO_PATTERN;
-  F->ops->solve            = MatSolve_MPISpooles;
+  ierr = VecDestroy(&lu->vec_spooles);CHKERRQ(ierr);
+  ierr = ISDestroy(&lu->iden);CHKERRQ(ierr);
+  ierr = ISDestroy(&lu->is_petsc);CHKERRQ(ierr);
+  ierr = VecScatterDestroy(&lu->scat);CHKERRQ(ierr);
+  lu->scat      = PETSC_NULL;  
+  lu->flg       = SAME_NONZERO_PATTERN;
+  F->ops->solve = MatSolve_MPISpooles;
 
   lu->CleanUpSpooles = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
-/*MC
-  MATMPIAIJSPOOLES - MATMPIAIJSPOOLES = "mpiaijspooles" - A matrix type providing direct solvers (LU) for distributed matrices 
-  via the external package Spooles.
-
-  If MPIAIJSPOOLES is installed (see the manual for
-  instructions on how to declare the existence of external packages),
-  a matrix type can be constructed which invokes SPOOLES solvers.
-  After calling MatCreate(...,A), simply call MatSetType(A,MATMPIAIJSPOOLES), then 
-  optionally call MatMPIAIJSetPreallocation() etc DO NOT
-  call MatCreateMPIAIJ() directly or the preallocation information will be LOST!
-
-  This matrix inherits from MATMPIAIJ.  As a result, MatMPIAIJSetPreallocation() is 
-  supported for this matrix type.  One can also call MatConvert() for an inplace conversion to or from 
-  the MATMPIAIJ type without data copy AFTER the matrix values have been set.
-
-  Consult Spooles documentation for more information about the options database keys below.
-
-  Options Database Keys:
-+ -mat_type mpiaijspooles - sets the matrix type to "mpiaijspooles" during a call to MatSetFromOptions()
-. -mat_spooles_tau <tau> - upper bound on the magnitude of the largest element in L or U
-. -mat_spooles_seed <seed> - random number seed used for ordering
-. -mat_spooles_msglvl <msglvl> - message output level
-. -mat_spooles_ordering <BestOfNDandMS,MMD,MS,ND> - ordering used
-. -mat_spooles_maxdomainsize <n> - maximum subgraph size used by Spooles orderings
-. -mat_spooles_maxzeros <n> - maximum number of zeros inside a supernode
-. -mat_spooles_maxsize <n> - maximum size of a supernode
-. -mat_spooles_FrontMtxInfo <true,fase> - print Spooles information about the computed factorization
-. -mat_spooles_symmetryflag <0,1,2> - 0: SPOOLES_SYMMETRIC, 1: SPOOLES_HERMITIAN, 2: SPOOLES_NONSYMMETRIC
-. -mat_spooles_patchAndGoFlag <0,1,2> - 0: no patch, 1: use PatchAndGo strategy 1, 2: use PatchAndGo strategy 2
-. -mat_spooles_toosmall <dt> - drop tolerance for PatchAndGo strategy 1
-. -mat_spooles_storeids <bool integer> - if nonzero, stores row and col numbers where patches were applied in an IV object
-. -mat_spooles_fudge <delta> - fudge factor for rescaling diagonals with PatchAndGo strategy 2
-- -mat_spooles_storevalues <bool integer> - if nonzero and PatchAndGo strategy 2 is used, store change in diagonal value in a DV object
-
-   Level: beginner
-
-.seealso: PCLU
-M*/
-
 
