@@ -1,13 +1,12 @@
-#define PETSCMAT_DLL
 
 /* 
         Provides an interface to the SuperLU_DIST_2.2 sparse solver
 */
 
-#include "../src/mat/impls/aij/seq/aij.h"
-#include "../src/mat/impls/aij/mpi/mpiaij.h"
+#include <../src/mat/impls/aij/seq/aij.h>
+#include <../src/mat/impls/aij/mpi/mpiaij.h>
 #if defined(PETSC_HAVE_STDLIB_H) /* This is to get around weird problem with SuperLU on cray */
-#include "stdlib.h"
+#include <stdlib.h>
 #endif
 
 #if defined(PETSC_USE_64BIT_INDICES)
@@ -17,9 +16,9 @@
 
 EXTERN_C_BEGIN 
 #if defined(PETSC_USE_COMPLEX)
-#include "superlu_zdefs.h"
+#include <superlu_zdefs.h>
 #else
-#include "superlu_ddefs.h"
+#include <superlu_ddefs.h>
 #endif
 EXTERN_C_END 
 
@@ -43,8 +42,8 @@ typedef struct {
 #else
   double                  *val;
 #endif
-  PetscTruth              matsolve_iscalled,matmatsolve_iscalled;
-  PetscTruth CleanUpSuperLU_Dist; /* Flag to clean up (non-global) SuperLU objects during Destroy */
+  PetscBool               matsolve_iscalled,matmatsolve_iscalled;
+  PetscBool  CleanUpSuperLU_Dist; /* Flag to clean up (non-global) SuperLU objects during Destroy */
 } Mat_SuperLU_DIST;
 
 extern PetscErrorCode MatFactorInfo_SuperLU_DIST(Mat,PetscViewer);
@@ -55,21 +54,21 @@ extern PetscErrorCode MatSolve_SuperLU_DIST(Mat,Vec,Vec);
 extern PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat,Mat,IS,IS,const MatFactorInfo *);
 extern PetscErrorCode MatDestroy_MPIAIJ(Mat);
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatDestroy_SuperLU_DIST"
 PetscErrorCode MatDestroy_SuperLU_DIST(Mat A)
 {
   PetscErrorCode   ierr;
-  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->spptr; 
-  PetscTruth       flg;
-    
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->spptr;
+  PetscBool        flg;
+
   PetscFunctionBegin;
-  if (lu->CleanUpSuperLU_Dist) {
+  if (lu && lu->CleanUpSuperLU_Dist) {
     /* Deallocate SuperLU_DIST storage */
-    if (lu->MatInputMode == GLOBAL) { 
+    if (lu->MatInputMode == GLOBAL) {
       Destroy_CompCol_Matrix_dist(&lu->A_sup);
-    } else {     
-      Destroy_CompRowLoc_Matrix_dist(&lu->A_sup);  
+    } else {
+      Destroy_CompRowLoc_Matrix_dist(&lu->A_sup);
       if ( lu->options.SolveInitialized ) {
 #if defined(PETSC_USE_COMPLEX)
         zSolveFinalize(&lu->options, &lu->SOLVEstruct);
@@ -81,12 +80,13 @@ PetscErrorCode MatDestroy_SuperLU_DIST(Mat A)
     Destroy_LU(A->cmap->N, &lu->grid, &lu->LUstruct);
     ScalePermstructFree(&lu->ScalePermstruct);
     LUstructFree(&lu->LUstruct);
-    
+
     /* Release the SuperLU_DIST process grid. */
     superlu_gridexit(&lu->grid);
-    
+
     ierr = MPI_Comm_free(&(lu->comm_superlu));CHKERRQ(ierr);
   }
+  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
 
   ierr = PetscTypeCompare((PetscObject)A,MATSEQAIJ,&flg);CHKERRQ(ierr);
   if (flg) {
@@ -121,7 +121,7 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
     ierr = VecCreateSeq(PETSC_COMM_SELF,N,&x_seq);CHKERRQ(ierr);
     ierr = ISCreateStride(PETSC_COMM_SELF,N,0,1,&iden);CHKERRQ(ierr);
     ierr = VecScatterCreate(b_mpi,iden,x_seq,iden,&scat);CHKERRQ(ierr);
-    ierr = ISDestroy(iden);CHKERRQ(ierr);
+    ierr = ISDestroy(&iden);CHKERRQ(ierr);
 
     ierr = VecScatterBegin(scat,b_mpi,x_seq,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     ierr = VecScatterEnd(scat,b_mpi,x_seq,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
@@ -140,7 +140,7 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
     ierr = VecGetArray(x,&bptr);CHKERRQ(ierr);
   }
  
-  if (lu->options.Fact != FACTORED) SETERRQ(PETSC_ERR_ARG_WRONG,"SuperLU_DIST options.Fact mush equal FACTORED");
+  if (lu->options.Fact != FACTORED) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"SuperLU_DIST options.Fact mush equal FACTORED");
 
   PStatInit(&stat);        /* Initialize the statistics variables. */
   if (lu->MatInputMode == GLOBAL) { 
@@ -160,7 +160,7 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
 	    &lu->LUstruct,&lu->SOLVEstruct,berr,&stat,&info);
 #endif
   }
-  if (info) SETERRQ1(PETSC_ERR_LIB,"pdgssvx fails, info: %d\n",info);
+  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"pdgssvx fails, info: %d\n",info);
 
   if (lu->options.PrintStat) {
      PStatPrint(&lu->options, &stat, &lu->grid);     /* Print the statistics. */
@@ -172,8 +172,8 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
     ierr = VecRestoreArray(x_seq,&bptr);CHKERRQ(ierr);
     ierr = VecScatterBegin(scat,x_seq,x,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
     ierr = VecScatterEnd(scat,x_seq,x,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-    ierr = VecScatterDestroy(scat);CHKERRQ(ierr);
-    ierr = VecDestroy(x_seq);CHKERRQ(ierr);
+    ierr = VecScatterDestroy(&scat);CHKERRQ(ierr);
+    ierr = VecDestroy(&x_seq);CHKERRQ(ierr);
   } else {
     ierr = VecRestoreArray(x,&bptr);CHKERRQ(ierr);
     lu->matsolve_iscalled    = PETSC_TRUE;
@@ -194,14 +194,17 @@ PetscErrorCode MatMatSolve_SuperLU_DIST(Mat A,Mat B_mpi,Mat X)
   double           berr[1];
   PetscScalar      *bptr;  
   int              info; /* SuperLU_Dist info code is ALWAYS an int, even with long long indices */
+  PetscBool        flg;
 
   PetscFunctionBegin;
-  if (lu->options.Fact != FACTORED) SETERRQ(PETSC_ERR_ARG_WRONG,"SuperLU_DIST options.Fact mush equal FACTORED");
+  if (lu->options.Fact != FACTORED) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"SuperLU_DIST options.Fact mush equal FACTORED");
+  ierr = PetscTypeCompareAny((PetscObject)B_mpi,&flg,MATSEQDENSE,MATMPIDENSE,PETSC_NULL);CHKERRQ(ierr);
+  if (!flg) SETERRQ(((PetscObject)A)->comm,PETSC_ERR_ARG_WRONG,"Matrix B must be MATDENSE matrix");
+  ierr = PetscTypeCompareAny((PetscObject)X,&flg,MATSEQDENSE,MATMPIDENSE,PETSC_NULL);CHKERRQ(ierr);
+  if (!flg) SETERRQ(((PetscObject)A)->comm,PETSC_ERR_ARG_WRONG,"Matrix X must be MATDENSE matrix");
   
   ierr = MPI_Comm_size(((PetscObject)A)->comm,&size);CHKERRQ(ierr);
-  if (size > 1 && lu->MatInputMode == GLOBAL) {  
-    SETERRQ1(PETSC_ERR_SUP,"MatInputMode=GLOBAL for nproc %d>1 is not supported",size);
-  } 
+  if (size > 1 && lu->MatInputMode == GLOBAL) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"MatInputMode=GLOBAL for nproc %d>1 is not supported",size);
   /* size==1 or distributed mat input */
   if (lu->options.SolveInitialized && !lu->matmatsolve_iscalled){
     /* communication pattern of SOLVEstruct is unlikely created for matmatsolve, 
@@ -238,7 +241,7 @@ PetscErrorCode MatMatSolve_SuperLU_DIST(Mat A,Mat B_mpi,Mat X)
 	    &lu->LUstruct,&lu->SOLVEstruct,berr,&stat,&info);
 #endif
   }
-  if (info) SETERRQ1(PETSC_ERR_LIB,"pdgssvx fails, info: %d\n",info);
+  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"pdgssvx fails, info: %d\n",info);
   ierr = MatRestoreArray(X,&bptr);CHKERRQ(ierr);
 
   if (lu->options.PrintStat) {
@@ -260,7 +263,7 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
   Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)(F)->spptr;
   PetscErrorCode   ierr;
   PetscInt         M=A->rmap->N,N=A->cmap->N,i,*ai,*aj,*bi,*bj,nz,rstart,*garray,
-                   m=A->rmap->n, irow,colA_start,j,jcol,jB,countA,countB,*bjj,*ajj;
+                   m=A->rmap->n, colA_start,j,jcol,jB,countA,countB,*bjj,*ajj;
   int              sinfo; /* SuperLU_Dist info flag is always an int even with long long indices */
   PetscMPIInt      size,rank;
   SuperLUStat_t    stat;
@@ -287,13 +290,13 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
     if (size > 1) { /* convert mpi A to seq mat A */
       ierr = ISCreateStride(PETSC_COMM_SELF,M,0,1,&isrow);CHKERRQ(ierr);  
       ierr = MatGetSubMatrices(A,1,&isrow,&isrow,MAT_INITIAL_MATRIX,&tseq);CHKERRQ(ierr);
-      ierr = ISDestroy(isrow);CHKERRQ(ierr);
+      ierr = ISDestroy(&isrow);CHKERRQ(ierr);
    
       A_seq = *tseq;
       ierr = PetscFree(tseq);CHKERRQ(ierr);
       aa =  (Mat_SeqAIJ*)A_seq->data;
     } else {
-      PetscTruth flg;
+      PetscBool  flg;
       ierr = PetscTypeCompare((PetscObject)A,MATMPIAIJ,&flg);CHKERRQ(ierr);
       if (flg) {
         Mat_MPIAIJ *At = (Mat_MPIAIJ*)A->data;
@@ -305,14 +308,12 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
     /* Convert Petsc NR matrix to SuperLU_DIST NC. 
        Note: memories of lu->val, col and row are allocated by CompRow_to_CompCol_dist()! */
     if (lu->options.Fact != DOFACT) {/* successive numeric factorization, sparsity pattern is reused. */
+      Destroy_CompCol_Matrix_dist(&lu->A_sup);
       if (lu->FactPattern == SamePattern_SameRowPerm){
-        Destroy_CompCol_Matrix_dist(&lu->A_sup);
-        /* Destroy_LU(N, &lu->grid, &lu->LUstruct); Crash! Comment it out does not lead to mem leak. */
         lu->options.Fact = SamePattern_SameRowPerm; /* matrix has similar numerical values */
-      } else {
-        Destroy_CompCol_Matrix_dist(&lu->A_sup); 
+      } else { /* lu->FactPattern == SamePattern */
         Destroy_LU(N, &lu->grid, &lu->LUstruct); 
-        lu->options.Fact = SamePattern; 
+        lu->options.Fact = SamePattern;    
       }
     }
 #if defined(PETSC_USE_COMPLEX)
@@ -351,15 +352,15 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
       dallocateA_dist(m, nz, &lu->val, &lu->col, &lu->row);
 #endif
     } else { /* successive numeric factorization, sparsity pattern and perm_c are reused. */
+      /* Destroy_CompRowLoc_Matrix_dist(&lu->A_sup); */ /* this leads to crash! However, see SuperLU_DIST_2.5/EXAMPLE/pzdrive2.c */
       if (lu->FactPattern == SamePattern_SameRowPerm){
-        /* Destroy_LU(N, &lu->grid, &lu->LUstruct); Crash! Comment it out does not lead to mem leak. */
         lu->options.Fact = SamePattern_SameRowPerm; /* matrix has similar numerical values */
       } else {
         Destroy_LU(N, &lu->grid, &lu->LUstruct); /* Deallocate storage associated with the L and U matrices. */
         lu->options.Fact = SamePattern;
       }
     }
-    nz = 0; irow = rstart;   
+    nz = 0;
     for ( i=0; i<m; i++ ) {
       lu->row[i] = nz;
       countA = ai[i+1] - ai[i];
@@ -421,16 +422,16 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
 #if defined(PETSC_USE_COMPLEX)
     pzgssvx(&lu->options, &lu->A_sup, &lu->ScalePermstruct, 0, m, 0, &lu->grid,
 	    &lu->LUstruct, &lu->SOLVEstruct, berr, &stat, &sinfo);
-    if (sinfo) SETERRQ1(PETSC_ERR_LIB,"pzgssvx fails, info: %d\n",sinfo);
+    if (sinfo) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"pzgssvx fails, info: %d\n",sinfo);
 #else
     pdgssvx(&lu->options, &lu->A_sup, &lu->ScalePermstruct, 0, m, 0, &lu->grid,
 	    &lu->LUstruct, &lu->SOLVEstruct, berr, &stat, &sinfo);
-    if (sinfo) SETERRQ1(PETSC_ERR_LIB,"pdgssvx fails, info: %d\n",sinfo);
+    if (sinfo) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"pdgssvx fails, info: %d\n",sinfo);
 #endif
   }
 
   if (lu->MatInputMode == GLOBAL && size > 1){
-    ierr = MatDestroy(A_seq);CHKERRQ(ierr);
+    ierr = MatDestroy(&A_seq);CHKERRQ(ierr);
   }
 
   if (lu->options.PrintStat) {
@@ -463,7 +464,7 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
 #define __FUNCT__ "MatLUFactorSymbolic_SuperLU_DIST"
 PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
 {
-  Mat_SuperLU_DIST  *lu = (Mat_SuperLU_DIST*) (F)->spptr;   
+  Mat_SuperLU_DIST  *lu = (Mat_SuperLU_DIST*)F->spptr;   
   PetscInt          M=A->rmap->N,N=A->cmap->N;
 
   PetscFunctionBegin;
@@ -473,9 +474,10 @@ PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat F,Mat A,IS r,IS c,const MatF
   /* Initialize ScalePermstruct and LUstruct. */
   ScalePermstructInit(M, N, &lu->ScalePermstruct);
   LUstructInit(M, N, &lu->LUstruct); 
-  (F)->ops->lufactornumeric = MatLUFactorNumeric_SuperLU_DIST;
-  (F)->ops->solve           = MatSolve_SuperLU_DIST;
-  (F)->ops->matsolve        = MatMatSolve_SuperLU_DIST;
+  F->ops->lufactornumeric = MatLUFactorNumeric_SuperLU_DIST;
+  F->ops->solve           = MatSolve_SuperLU_DIST;
+  F->ops->matsolve        = MatMatSolve_SuperLU_DIST;
+  lu->CleanUpSuperLU_Dist = PETSC_TRUE;
   PetscFunctionReturn(0); 
 }
 
@@ -485,7 +487,7 @@ EXTERN_C_BEGIN
 PetscErrorCode MatFactorGetSolverPackage_aij_superlu_dist(Mat A,const MatSolverPackage *type)
 {
   PetscFunctionBegin;
-  *type = MAT_SOLVER_SUPERLU_DIST;
+  *type = MATSOLVERSUPERLU_DIST;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -500,9 +502,9 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
   PetscInt          M=A->rmap->N,N=A->cmap->N,indx;
   PetscMPIInt       size;
   superlu_options_t options;
-  PetscTruth        flg;
-  const char        *pctype[] = {"MMD_AT_PLUS_A","NATURAL","MMD_ATA","PARMETIS"}; 
-  const char        *prtype[] = {"LargeDiag","NATURAL"}; 
+  PetscBool         flg;
+  const char        *colperm[] = {"NATURAL","MMD_AT_PLUS_A","MMD_ATA","METIS_AT_PLUS_A","PARMETIS"}; 
+  const char        *rowperm[] = {"LargeDiag","NATURAL"}; 
   const char        *factPattern[] = {"SamePattern","SamePattern_SameRowPerm"};
 
   PetscFunctionBegin;
@@ -517,7 +519,7 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
   B->ops->view             = MatView_SuperLU_DIST;
   B->ops->destroy          = MatDestroy_SuperLU_DIST;
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_aij_superlu_dist",MatFactorGetSolverPackage_aij_superlu_dist);CHKERRQ(ierr);
-  B->factor                = MAT_FACTOR_LU;  
+  B->factortype            = MAT_FACTOR_LU;  
 
   ierr = PetscNewLog(B,Mat_SuperLU_DIST,&lu);CHKERRQ(ierr);
   B->spptr = lu;
@@ -526,7 +528,7 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
      options.Fact              = DOFACT;
      options.Equil             = YES;
      options.ParSymbFact       = NO;
-     options.ColPerm           = MMD_AT_PLUS_A;
+     options.ColPerm           = METIS_AT_PLUS_A; 
      options.RowPerm           = LargeDiag;
      options.ReplaceTinyPivot  = YES;
      options.IterRefine        = DOUBLE;
@@ -540,7 +542,7 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
   ierr = MPI_Comm_dup(((PetscObject)A)->comm,&(lu->comm_superlu));CHKERRQ(ierr);
   ierr = MPI_Comm_size(((PetscObject)A)->comm,&size);CHKERRQ(ierr);
   /* Default num of process columns and rows */
-  lu->npcol = PetscMPIIntCast((PetscInt)(0.5 + sqrt((PetscReal)size))); 
+  lu->npcol = PetscMPIIntCast((PetscInt)(0.5 + PetscSqrtReal((PetscReal)size))); 
   if (!lu->npcol) lu->npcol = 1;
   while (lu->npcol > 0) {
     lu->nprow = PetscMPIIntCast(size/lu->npcol);  
@@ -551,19 +553,18 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
   ierr = PetscOptionsBegin(((PetscObject)A)->comm,((PetscObject)A)->prefix,"SuperLU_Dist Options","Mat");CHKERRQ(ierr);
     ierr = PetscOptionsInt("-mat_superlu_dist_r","Number rows in processor partition","None",lu->nprow,&lu->nprow,PETSC_NULL);CHKERRQ(ierr);
     ierr = PetscOptionsInt("-mat_superlu_dist_c","Number columns in processor partition","None",lu->npcol,&lu->npcol,PETSC_NULL);CHKERRQ(ierr);
-    if (size != lu->nprow * lu->npcol) 
-      SETERRQ3(PETSC_ERR_ARG_SIZ,"Number of processes %d must equal to nprow %d * npcol %d",size,lu->nprow,lu->npcol);
+    if (size != lu->nprow * lu->npcol) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Number of processes %d must equal to nprow %d * npcol %d",size,lu->nprow,lu->npcol);
   
     lu->MatInputMode = DISTRIBUTED;
     ierr = PetscOptionsEnum("-mat_superlu_dist_matinput","Matrix input mode (global or distributed)","None",SuperLU_MatInputModes,(PetscEnum)lu->MatInputMode,(PetscEnum*)&lu->MatInputMode,PETSC_NULL);CHKERRQ(ierr);
     if(lu->MatInputMode == DISTRIBUTED && size == 1) lu->MatInputMode = GLOBAL;
 
-    ierr = PetscOptionsTruth("-mat_superlu_dist_equil","Equilibrate matrix","None",PETSC_TRUE,&flg,0);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-mat_superlu_dist_equil","Equilibrate matrix","None",PETSC_TRUE,&flg,0);CHKERRQ(ierr); 
     if (!flg) {
       options.Equil = NO;
     }
 
-    ierr = PetscOptionsEList("-mat_superlu_dist_rowperm","Row permutation","None",prtype,2,prtype[0],&indx,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsEList("-mat_superlu_dist_rowperm","Row permutation","None",rowperm,2,rowperm[0],&indx,&flg);CHKERRQ(ierr);
     if (flg) {
       switch (indx) {
       case 0:
@@ -575,31 +576,36 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
       }
     } 
 
-    ierr = PetscOptionsEList("-mat_superlu_dist_colperm","Column permutation","None",pctype,4,pctype[0],&indx,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsEList("-mat_superlu_dist_colperm","Column permutation","None",colperm,5,colperm[3],&indx,&flg);CHKERRQ(ierr);
     if (flg) {
       switch (indx) {
       case 0:
-        options.ColPerm = MMD_AT_PLUS_A;
+        options.ColPerm = NATURAL;
         break;
       case 1:
-        options.ColPerm = NATURAL;
+        options.ColPerm = MMD_AT_PLUS_A;
         break;
       case 2:
         options.ColPerm = MMD_ATA;
         break;
       case 3:
-        options.ColPerm = PARMETIS;
+        options.ColPerm = METIS_AT_PLUS_A;
         break;
+      case 4:
+        options.ColPerm = PARMETIS; /* only works for np>1 */
+        break;
+      default:
+        SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown column permutation");
       }
     }
 
-    ierr = PetscOptionsTruth("-mat_superlu_dist_replacetinypivot","Replace tiny pivots","None",PETSC_TRUE,&flg,0);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-mat_superlu_dist_replacetinypivot","Replace tiny pivots","None",PETSC_TRUE,&flg,0);CHKERRQ(ierr); 
     if (!flg) {
       options.ReplaceTinyPivot = NO;
     }
 
     options.ParSymbFact = NO;
-    ierr = PetscOptionsTruth("-mat_superlu_dist_parsymbfact","Parallel symbolic factorization","None",PETSC_FALSE,&flg,0);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-mat_superlu_dist_parsymbfact","Parallel symbolic factorization","None",PETSC_FALSE,&flg,0);CHKERRQ(ierr); 
     if (flg){
 #ifdef PETSC_HAVE_PARMETIS
       options.ParSymbFact = YES;
@@ -609,8 +615,8 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
 #endif
     }
 
-    lu->FactPattern = SamePattern;
-    ierr = PetscOptionsEList("-mat_superlu_dist_fact","Sparsity pattern for repeated matrix factorization","None",factPattern,2,factPattern[0],&indx,&flg);CHKERRQ(ierr);
+    lu->FactPattern = SamePattern_SameRowPerm;
+    ierr = PetscOptionsEList("-mat_superlu_dist_fact","Sparsity pattern for repeated matrix factorization","None",factPattern,2,factPattern[1],&indx,&flg);CHKERRQ(ierr);
     if (flg) {
       switch (indx) {
       case 0:
@@ -623,7 +629,7 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
     } 
     
     options.IterRefine = NOREFINE;
-    ierr = PetscOptionsTruth("-mat_superlu_dist_iterrefine","Use iterative refinement","None",PETSC_FALSE,&flg,0);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-mat_superlu_dist_iterrefine","Use iterative refinement","None",PETSC_FALSE,&flg,0);CHKERRQ(ierr);
     if (flg) {
       options.IterRefine = DOUBLE;    
     }
@@ -633,13 +639,12 @@ PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
     } else {
       options.PrintStat = NO;
     }
-    ierr = PetscOptionsTruth("-mat_superlu_dist_statprint","Print factorization information","None",
-                              (PetscTruth)options.PrintStat,(PetscTruth*)&options.PrintStat,0);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-mat_superlu_dist_statprint","Print factorization information","None",
+                              (PetscBool)options.PrintStat,(PetscBool*)&options.PrintStat,0);CHKERRQ(ierr); 
   PetscOptionsEnd();
 
-  lu->options             = options; 
-  lu->options.Fact        = DOFACT;
-  lu->CleanUpSuperLU_Dist = PETSC_TRUE;
+  lu->options              = options; 
+  lu->options.Fact         = DOFACT;
   lu->matsolve_iscalled    = PETSC_FALSE;
   lu->matmatsolve_iscalled = PETSC_FALSE;
   *F = B;
@@ -687,25 +692,34 @@ PetscErrorCode MatFactorInfo_SuperLU_DIST(Mat A,PetscViewer viewer)
   options = lu->options;
   ierr = PetscViewerASCIIPrintf(viewer,"SuperLU_DIST run parameters:\n");CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"  Process grid nprow %D x npcol %D \n",lu->nprow,lu->npcol);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"  Equilibrate matrix %s \n",PetscTruths[options.Equil != NO]);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Equilibrate matrix %s \n",PetscBools[options.Equil != NO]);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"  Matrix input mode %d \n",lu->MatInputMode);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"  Replace tiny pivots %s \n",PetscTruths[options.ReplaceTinyPivot != NO]);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"  Use iterative refinement %s \n",PetscTruths[options.IterRefine == DOUBLE]);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Replace tiny pivots %s \n",PetscBools[options.ReplaceTinyPivot != NO]);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Use iterative refinement %s \n",PetscBools[options.IterRefine == DOUBLE]);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"  Processors in row %d col partition %d \n",lu->nprow,lu->npcol);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"  Row permutation %s \n",(options.RowPerm == NOROWPERM) ? "NATURAL": "LargeDiag");CHKERRQ(ierr);
-  if (options.ColPerm == NATURAL) {
+ 
+  switch(options.ColPerm){
+  case NATURAL:
     ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation NATURAL\n");CHKERRQ(ierr);
-  } else if (options.ColPerm == MMD_AT_PLUS_A) {
+    break;
+  case MMD_AT_PLUS_A:
     ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation MMD_AT_PLUS_A\n");CHKERRQ(ierr);
-  } else if (options.ColPerm == MMD_ATA) {
+    break;
+  case MMD_ATA:
     ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation MMD_ATA\n");CHKERRQ(ierr);
-  } else if (options.ColPerm == PARMETIS) {
+    break;
+  case METIS_AT_PLUS_A:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation METIS_AT_PLUS_A\n");CHKERRQ(ierr);
+    break;
+  case PARMETIS:
     ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation PARMETIS\n");CHKERRQ(ierr);
-  } else {
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown column permutation");
+    break;
+  default:
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown column permutation");
   }
 
-  ierr = PetscViewerASCIIPrintf(viewer,"  Parallel symbolic factorization %s \n",PetscTruths[options.ParSymbFact != NO]);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Parallel symbolic factorization %s \n",PetscBools[options.ParSymbFact != NO]);CHKERRQ(ierr);
   
   if (lu->FactPattern == SamePattern){
     ierr = PetscViewerASCIIPrintf(viewer,"  Repeated factorization SamePattern\n");CHKERRQ(ierr);
@@ -720,11 +734,11 @@ PetscErrorCode MatFactorInfo_SuperLU_DIST(Mat A,PetscViewer viewer)
 PetscErrorCode MatView_SuperLU_DIST(Mat A,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
-  PetscTruth        iascii;
+  PetscBool         iascii;
   PetscViewerFormat format;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_INFO) {
@@ -736,7 +750,7 @@ PetscErrorCode MatView_SuperLU_DIST(Mat A,PetscViewer viewer)
 
 
 /*MC
-  MAT_SOLVER_SUPERLU_DIST - Parallel direct solver package for LU factorization
+  MATSOLVERSUPERLU_DIST - Parallel direct solver package for LU factorization
 
    Works with AIJ matrices  
 
