@@ -1,7 +1,6 @@
-#define PETSCMAT_DLL
 
 /*
-  Defines basic operations for the MATSEQCSRPERM matrix class.
+  Defines basic operations for the MATSEQAIJPERM matrix class.
   This class is derived from the MATSEQAIJ class and retains the 
   compressed row storage (aka Yale sparse matrix format) but augments 
   it with some permutation information that enables some operations 
@@ -11,7 +10,7 @@
   Eventually a variety of permutations may be supported.
 */
 
-#include "../src/mat/impls/aij/seq/aij.h"
+#include <../src/mat/impls/aij/seq/aij.h>
 
 #define NDIM 512
     /* NDIM specifies how many rows at a time we should work with when 
@@ -32,7 +31,7 @@ typedef struct {
 
   /* Flag that indicates whether we need to clean up permutation 
    * information during the MatDestroy. */
-  PetscTruth CleanUpCSRPERM;
+  PetscBool  CleanUpAIJPERM;
 
   /* Some of this stuff is for Ed's recursive triangular solve.
    * I'm not sure what I need yet. */
@@ -46,20 +45,20 @@ typedef struct {
   PetscInt **xgroup_list;
   PetscInt **nzgroup_list;
   PetscInt **iperm_list;
-} Mat_SeqCSRPERM;
+} Mat_SeqAIJPERM;
 
 extern PetscErrorCode MatAssemblyEnd_SeqAIJ(Mat,MatAssemblyType);
 
 EXTERN_C_BEGIN
 #undef __FUNCT__
-#define __FUNCT__ "MatConvert_SeqCSRPERM_SeqAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqCSRPERM_SeqAIJ(Mat A,MatType type,MatReuse reuse,Mat *newmat)
+#define __FUNCT__ "MatConvert_SeqAIJPERM_SeqAIJ"
+PetscErrorCode  MatConvert_SeqAIJPERM_SeqAIJ(Mat A,MatType type,MatReuse reuse,Mat *newmat)
 {
-  /* This routine is only called to convert a MATCSRPERM to its base PETSc type, */
+  /* This routine is only called to convert a MATAIJPERM to its base PETSc type, */
   /* so we will ignore 'MatType type'. */
   PetscErrorCode ierr;
   Mat            B = *newmat;
-  Mat_SeqCSRPERM *csrperm=(Mat_SeqCSRPERM*)A->spptr;
+  Mat_SeqAIJPERM *aijperm=(Mat_SeqAIJPERM*)A->spptr;
 
   PetscFunctionBegin;
   if (reuse == MAT_INITIAL_MATRIX) {
@@ -71,13 +70,13 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqCSRPERM_SeqAIJ(Mat A,MatType typ
   B->ops->destroy     = MatDestroy_SeqAIJ;
   B->ops->duplicate   = MatDuplicate_SeqAIJ;
 
-  /* Free everything in the Mat_SeqCSRPERM data structure. 
-   * We don't free the Mat_SeqCSRPERM struct itself, as this will 
+  /* Free everything in the Mat_SeqAIJPERM data structure. 
+   * We don't free the Mat_SeqAIJPERM struct itself, as this will 
    * cause problems later when MatDestroy() tries to free it. */
-  if(csrperm->CleanUpCSRPERM) {
-    ierr = PetscFree(csrperm->xgroup);CHKERRQ(ierr);
-    ierr = PetscFree(csrperm->nzgroup);CHKERRQ(ierr);
-    ierr = PetscFree(csrperm->iperm);CHKERRQ(ierr);
+  if(aijperm->CleanUpAIJPERM) {
+    ierr = PetscFree(aijperm->xgroup);CHKERRQ(ierr);
+    ierr = PetscFree(aijperm->nzgroup);CHKERRQ(ierr);
+    ierr = PetscFree(aijperm->iperm);CHKERRQ(ierr);
   }
 
   /* Change the type of B to MATSEQAIJ. */
@@ -89,59 +88,60 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqCSRPERM_SeqAIJ(Mat A,MatType typ
 EXTERN_C_END
 
 #undef __FUNCT__
-#define __FUNCT__ "MatDestroy_SeqCSRPERM"
-PetscErrorCode MatDestroy_SeqCSRPERM(Mat A)
+#define __FUNCT__ "MatDestroy_SeqAIJPERM"
+PetscErrorCode MatDestroy_SeqAIJPERM(Mat A)
 {
   PetscErrorCode ierr;
-  Mat_SeqCSRPERM *csrperm = (Mat_SeqCSRPERM *) A->spptr;
+  Mat_SeqAIJPERM *aijperm = (Mat_SeqAIJPERM *) A->spptr;
 
   PetscFunctionBegin;
-  /* Free everything in the Mat_SeqCSRPERM data structure. 
-   * Note that we don't need to free the Mat_SeqCSRPERM struct 
+  /* Free everything in the Mat_SeqAIJPERM data structure.
+   * Note that we don't need to free the Mat_SeqAIJPERM struct
    * itself, as MatDestroy() will do so. */
-  if(csrperm->CleanUpCSRPERM) {
-    ierr = PetscFree(csrperm->xgroup);CHKERRQ(ierr);
-    ierr = PetscFree(csrperm->nzgroup);CHKERRQ(ierr);
-    ierr = PetscFree(csrperm->iperm);CHKERRQ(ierr);
+  if(aijperm && aijperm->CleanUpAIJPERM) {
+    ierr = PetscFree(aijperm->xgroup);CHKERRQ(ierr);
+    ierr = PetscFree(aijperm->nzgroup);CHKERRQ(ierr);
+    ierr = PetscFree(aijperm->iperm);CHKERRQ(ierr);
   }
+  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
 
-  /* Change the type of A back to SEQAIJ and use MatDestroy_SeqAIJ() 
+  /* Change the type of A back to SEQAIJ and use MatDestroy_SeqAIJ()
    * to destroy everything that remains. */
   ierr = PetscObjectChangeTypeName( (PetscObject)A, MATSEQAIJ);CHKERRQ(ierr);
-  /* Note that I don't call MatSetType().  I believe this is because that 
-   * is only to be called when *building* a matrix.  I could be wrong, but 
+  /* Note that I don't call MatSetType().  I believe this is because that
+   * is only to be called when *building* a matrix.  I could be wrong, but
    * that is how things work for the SuperLU matrix class. */
   ierr = MatDestroy_SeqAIJ(A);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode MatDuplicate_SeqCSRPERM(Mat A, MatDuplicateOption op, Mat *M) 
+PetscErrorCode MatDuplicate_SeqAIJPERM(Mat A, MatDuplicateOption op, Mat *M) 
 {
   PetscErrorCode ierr;
-  Mat_SeqCSRPERM *csrperm = (Mat_SeqCSRPERM *) A->spptr;
-  Mat_SeqCSRPERM *csrperm_dest = (Mat_SeqCSRPERM *) (*M)->spptr;
+  Mat_SeqAIJPERM *aijperm = (Mat_SeqAIJPERM *) A->spptr;
+  Mat_SeqAIJPERM *aijperm_dest = (Mat_SeqAIJPERM *) (*M)->spptr;
 
   PetscFunctionBegin;
   ierr = MatDuplicate_SeqAIJ(A,op,M);CHKERRQ(ierr);
-  ierr = PetscMemcpy((*M)->spptr,csrperm,sizeof(Mat_SeqCSRPERM));CHKERRQ(ierr);
+  ierr = PetscMemcpy((*M)->spptr,aijperm,sizeof(Mat_SeqAIJPERM));CHKERRQ(ierr);
   /* Allocate space for, and copy the grouping and permutation info. 
    * I note that when the groups are initially determined in 
-   * SeqCSRPERM_create_perm, xgroup and nzgroup may be sized larger than 
+   * SeqAIJPERM_create_perm, xgroup and nzgroup may be sized larger than 
    * necessary.  But at this point, we know how large they need to be, and 
    * allocate only the necessary amount of memory.  So the duplicated matrix 
    * may actually use slightly less storage than the original! */
-  ierr = PetscMalloc(A->rmap->n*sizeof(PetscInt), csrperm_dest->iperm);CHKERRQ(ierr);
-  ierr = PetscMalloc((csrperm->ngroup+1)*sizeof(PetscInt), csrperm_dest->xgroup);CHKERRQ(ierr);
-  ierr = PetscMalloc((csrperm->ngroup)*sizeof(PetscInt), csrperm_dest->nzgroup);CHKERRQ(ierr);
-  ierr = PetscMemcpy(csrperm_dest->iperm,csrperm->iperm,sizeof(PetscInt)*A->rmap->n);CHKERRQ(ierr);
-  ierr = PetscMemcpy(csrperm_dest->xgroup,csrperm->xgroup,sizeof(PetscInt)*(csrperm->ngroup+1));CHKERRQ(ierr);
-  ierr = PetscMemcpy(csrperm_dest->nzgroup,csrperm->nzgroup,sizeof(PetscInt)*csrperm->ngroup);CHKERRQ(ierr);
+  ierr = PetscMalloc(A->rmap->n*sizeof(PetscInt), aijperm_dest->iperm);CHKERRQ(ierr);
+  ierr = PetscMalloc((aijperm->ngroup+1)*sizeof(PetscInt), aijperm_dest->xgroup);CHKERRQ(ierr);
+  ierr = PetscMalloc((aijperm->ngroup)*sizeof(PetscInt), aijperm_dest->nzgroup);CHKERRQ(ierr);
+  ierr = PetscMemcpy(aijperm_dest->iperm,aijperm->iperm,sizeof(PetscInt)*A->rmap->n);CHKERRQ(ierr);
+  ierr = PetscMemcpy(aijperm_dest->xgroup,aijperm->xgroup,sizeof(PetscInt)*(aijperm->ngroup+1));CHKERRQ(ierr);
+  ierr = PetscMemcpy(aijperm_dest->nzgroup,aijperm->nzgroup,sizeof(PetscInt)*aijperm->ngroup);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "SeqCSRPERM_create_perm"
-PetscErrorCode SeqCSRPERM_create_perm(Mat A)
+#define __FUNCT__ "SeqAIJPERM_create_perm"
+PetscErrorCode SeqAIJPERM_create_perm(Mat A)
 {
   PetscInt        m;  /* Number of rows in the matrix. */
   Mat_SeqAIJ      *a = (Mat_SeqAIJ *)(A)->data;
@@ -156,20 +156,20 @@ PetscErrorCode SeqCSRPERM_create_perm(Mat A)
     /* When constructing the iperm permutation vector, 
      * ipnz[nz] is used to point to the next place in the permutation vector 
      * that a row with nz nonzero elements should be placed.*/
-  Mat_SeqCSRPERM *csrperm = (Mat_SeqCSRPERM*) A->spptr;
-    /* Points to the MATSEQCSRPERM-specific data in the matrix B. */
+  Mat_SeqAIJPERM *aijperm = (Mat_SeqAIJPERM*) A->spptr;
+    /* Points to the MATSEQAIJPERM-specific data in the matrix B. */
   PetscErrorCode ierr;
   PetscInt       i, ngroup, istart, ipos;
 
   /* I really ought to put something in here to check if B is of 
-   * type MATSEQCSRPERM and return an error code if it is not.
+   * type MATSEQAIJPERM and return an error code if it is not.
    * Come back and do this! */
   PetscFunctionBegin;
   m  = A->rmap->n;
   ia = a->i;
    
   /* Allocate the arrays that will hold the permutation vector. */
-  ierr = PetscMalloc(m*sizeof(PetscInt), &csrperm->iperm);CHKERRQ(ierr);
+  ierr = PetscMalloc(m*sizeof(PetscInt), &aijperm->iperm);CHKERRQ(ierr);
 
   /* Allocate some temporary work arrays that will be used in 
    * calculating the permuation vector and groupings. */
@@ -204,8 +204,8 @@ PetscErrorCode SeqCSRPERM_create_perm(Mat A)
    * We allocate space for the maximum number of groups; 
    * that is potentially a little wasteful, but not too much so.  
    * Perhaps I should fix it later. */
-  ierr = PetscMalloc((maxnz+2)*sizeof(PetscInt), &csrperm->xgroup);CHKERRQ(ierr);
-  ierr = PetscMalloc((maxnz+1)*sizeof(PetscInt), &csrperm->nzgroup);CHKERRQ(ierr);
+  ierr = PetscMalloc((maxnz+2)*sizeof(PetscInt), &aijperm->xgroup);CHKERRQ(ierr);
+  ierr = PetscMalloc((maxnz+1)*sizeof(PetscInt), &aijperm->nzgroup);CHKERRQ(ierr);
 
   /* Second pass.  Look at what is in the buckets and create the groupings.
    * Note that it is OK to have a group of rows with no non-zero values. */
@@ -213,15 +213,15 @@ PetscErrorCode SeqCSRPERM_create_perm(Mat A)
   istart = 0;
   for (i=0; i<=maxnz; i++) {
     if (rows_in_bucket[i] > 0) {
-      csrperm->nzgroup[ngroup] = i;
-      csrperm->xgroup[ngroup] = istart;
+      aijperm->nzgroup[ngroup] = i;
+      aijperm->xgroup[ngroup] = istart;
       ngroup++;
       istart += rows_in_bucket[i];
     }
   }
 
-  csrperm->xgroup[ngroup] = istart;
-  csrperm->ngroup = ngroup;
+  aijperm->xgroup[ngroup] = istart;
+  aijperm->ngroup = ngroup;
 
   /* Now fill in the permutation vector iperm. */
   ipnz[0] = 0;
@@ -232,7 +232,7 @@ PetscErrorCode SeqCSRPERM_create_perm(Mat A)
   for (i=0; i<m; i++) {
     nz = nz_in_row[i];
     ipos = ipnz[nz];
-    csrperm->iperm[ipos] = i;
+    aijperm->iperm[ipos] = i;
     ipnz[nz]++;
   }
 
@@ -242,15 +242,15 @@ PetscErrorCode SeqCSRPERM_create_perm(Mat A)
   ierr = PetscFree(nz_in_row);CHKERRQ(ierr);
 
   /* Since we've allocated some memory to hold permutation info, 
-   * flip the CleanUpCSRPERM flag to true. */
-  csrperm->CleanUpCSRPERM = PETSC_TRUE;
+   * flip the CleanUpAIJPERM flag to true. */
+  aijperm->CleanUpAIJPERM = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
 
 #undef __FUNCT__
-#define __FUNCT__ "MatAssemblyEnd_SeqCSRPERM"
-PetscErrorCode MatAssemblyEnd_SeqCSRPERM(Mat A, MatAssemblyType mode)
+#define __FUNCT__ "MatAssemblyEnd_SeqAIJPERM"
+PetscErrorCode MatAssemblyEnd_SeqAIJPERM(Mat A, MatAssemblyType mode)
 {
   PetscErrorCode ierr;
   Mat_SeqAIJ     *a = (Mat_SeqAIJ*)A->data;
@@ -258,37 +258,38 @@ PetscErrorCode MatAssemblyEnd_SeqCSRPERM(Mat A, MatAssemblyType mode)
   PetscFunctionBegin;
   if (mode == MAT_FLUSH_ASSEMBLY) PetscFunctionReturn(0);
   
-  /* Since a MATSEQCSRPERM matrix is really just a MATSEQAIJ with some 
+  /* Since a MATSEQAIJPERM matrix is really just a MATSEQAIJ with some 
    * extra information, call the AssemblyEnd routine for a MATSEQAIJ. 
    * I'm not sure if this is the best way to do this, but it avoids 
    * a lot of code duplication.
-   * I also note that currently MATSEQCSRPERM doesn't know anything about 
+   * I also note that currently MATSEQAIJPERM doesn't know anything about 
    * the Mat_CompressedRow data structure that SeqAIJ now uses when there 
    * are many zero rows.  If the SeqAIJ assembly end routine decides to use 
    * this, this may break things.  (Don't know... haven't looked at it.) */
   a->inode.use = PETSC_FALSE;
-  ierr = MatAssemblyEnd_SeqAIJ(A, mode);
+  ierr = MatAssemblyEnd_SeqAIJ(A, mode);CHKERRQ(ierr);
 
   /* Now calculate the permutation and grouping information. */
-  ierr = SeqCSRPERM_create_perm(A);CHKERRQ(ierr);
+  ierr = SeqAIJPERM_create_perm(A);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "MatMult_SeqCSRPERM"
-PetscErrorCode MatMult_SeqCSRPERM(Mat A,Vec xx,Vec yy)
+#define __FUNCT__ "MatMult_SeqAIJPERM"
+PetscErrorCode MatMult_SeqAIJPERM(Mat A,Vec xx,Vec yy)
 {
-  Mat_SeqAIJ      *a = (Mat_SeqAIJ*)A->data;
-  PetscScalar     *x,*y;
-  const MatScalar *aa;
-  PetscErrorCode  ierr;
-  PetscInt        *aj,*ai;
-#if !(defined(PETSC_USE_FORTRAN_KERNEL_MULTCSRPERM) && defined(notworking))
-  PetscInt        i,j;
+  Mat_SeqAIJ        *a = (Mat_SeqAIJ*)A->data;
+  const PetscScalar *x;
+  PetscScalar       *y;
+  const MatScalar   *aa;
+  PetscErrorCode    ierr;
+  const PetscInt    *aj,*ai;
+#if !(defined(PETSC_USE_FORTRAN_KERNEL_MULTAIJPERM) && defined(notworking))
+  PetscInt          i,j;
 #endif
 
   /* Variables that don't appear in MatMult_SeqAIJ. */
-  Mat_SeqCSRPERM *csrperm = (Mat_SeqCSRPERM *) A->spptr;
+  Mat_SeqAIJPERM *aijperm = (Mat_SeqAIJPERM *) A->spptr;
   PetscInt       *iperm;  /* Points to the permutation vector. */
   PetscInt       *xgroup;
     /* Denotes where groups of rows with same number of nonzeros 
@@ -311,20 +312,20 @@ PetscErrorCode MatMult_SeqCSRPERM(Mat A,Vec xx,Vec yy)
 #endif
 
   PetscFunctionBegin;
-  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(xx,&x);CHKERRQ(ierr);
   ierr = VecGetArray(yy,&y);CHKERRQ(ierr);
   aj  = a->j;    /* aj[k] gives column index for element aa[k]. */
   aa  = a->a;  /* Nonzero elements stored row-by-row. */
   ai  = a->i;   /* ai[k] is the position in aa and aj where row k starts. */
 
   /* Get the info we need about the permutations and groupings. */
-  iperm  = csrperm->iperm;
-  ngroup = csrperm->ngroup;
-  xgroup = csrperm->xgroup;
-  nzgroup = csrperm->nzgroup;
+  iperm  = aijperm->iperm;
+  ngroup = aijperm->ngroup;
+  xgroup = aijperm->xgroup;
+  nzgroup = aijperm->nzgroup;
   
-#if defined(PETSC_USE_FORTRAN_KERNEL_MULTCSRPERM) && defined(notworking)
-  fortranmultcsrperm_(&m,x,ii,aj,aa,y);
+#if defined(PETSC_USE_FORTRAN_KERNEL_MULTAIJPERM) && defined(notworking)
+  fortranmultaijperm_(&m,x,ii,aj,aa,y);
 #else
 
   for (igroup=0; igroup<ngroup; igroup++) {
@@ -371,11 +372,11 @@ PetscErrorCode MatMult_SeqCSRPERM(Mat A,Vec xx,Vec yy)
          * the chunk, we should vectorize along nz, that is, perform the 
          * mat-vec one row at a time as in the usual CSR case. */
         if (nz > isize) {
-#if defined(PETSC_HAVE_CRAYC)
+#if defined(PETSC_HAVE_CRAY_VECTOR)
 #pragma _CRI preferstream
 #endif
           for (i=0; i<isize; i++) {
-#if defined(PETSC_HAVE_CRAYC)
+#if defined(PETSC_HAVE_CRAY_VECTOR)
 #pragma _CRI prefervector
 #endif
             for (j=0; j<nz; j++) {
@@ -395,7 +396,7 @@ PetscErrorCode MatMult_SeqCSRPERM(Mat A,Vec xx,Vec yy)
           }
         }
 
-#if defined(PETSC_HAVE_CRAYC)
+#if defined(PETSC_HAVE_CRAY_VECTOR)
 #pragma _CRI ivdep
 #endif
         /* Put results from yp[] into non-permuted result vector y. */
@@ -407,50 +408,51 @@ PetscErrorCode MatMult_SeqCSRPERM(Mat A,Vec xx,Vec yy)
   } /* End loop over igroup. */
 #endif
   ierr = PetscLogFlops(2.0*a->nz - A->rmap->n);CHKERRQ(ierr);
-  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(xx,&x);CHKERRQ(ierr);
   ierr = VecRestoreArray(yy,&y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-/* MatMultAdd_SeqCSRPERM() calculates yy = ww + A * xx.
+/* MatMultAdd_SeqAIJPERM() calculates yy = ww + A * xx.
  * Note that the names I used to designate the vectors differs from that 
  * used in MatMultAdd_SeqAIJ().  I did this to keep my notation consistent 
- * with the MatMult_SeqCSRPERM() routine, which is very similar to this one. */
+ * with the MatMult_SeqAIJPERM() routine, which is very similar to this one. */
 /*
     I hate having virtually identical code for the mult and the multadd!!!
 */
 #undef __FUNCT__  
-#define __FUNCT__ "MatMultAdd_SeqCSRPERM"
-PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
+#define __FUNCT__ "MatMultAdd_SeqAIJPERM"
+PetscErrorCode MatMultAdd_SeqAIJPERM(Mat A,Vec xx,Vec ww,Vec yy)
 {
-  Mat_SeqAIJ      *a = (Mat_SeqAIJ*)A->data;
-  PetscScalar     *x,*y,*w;
-  const MatScalar *aa;
-  PetscErrorCode  ierr;
-  PetscInt        *aj,*ai;
-#if !defined(PETSC_USE_FORTRAN_KERNEL_MULTADDCSRPERM)
-  PetscInt        i,j;
+  Mat_SeqAIJ        *a = (Mat_SeqAIJ*)A->data;
+  const PetscScalar *x;
+  PetscScalar       *y,*w;
+  const MatScalar   *aa;
+  PetscErrorCode    ierr;
+  const PetscInt    *aj,*ai;
+#if !defined(PETSC_USE_FORTRAN_KERNEL_MULTADDAIJPERM)
+  PetscInt          i,j;
 #endif
 
   /* Variables that don't appear in MatMultAdd_SeqAIJ. */
-  Mat_SeqCSRPERM *csrperm;
-  PetscInt       *iperm;  /* Points to the permutation vector. */
-  PetscInt       *xgroup;
+  Mat_SeqAIJPERM *  aijperm;
+  PetscInt         *iperm;  /* Points to the permutation vector. */
+  PetscInt         *xgroup;
     /* Denotes where groups of rows with same number of nonzeros 
      * begin and end in iperm. */
-  PetscInt       *nzgroup;
-  PetscInt       ngroup;
-  PetscInt       igroup;
-  PetscInt       jstart,jend;
+  PetscInt         *nzgroup;
+  PetscInt         ngroup;
+  PetscInt         igroup;
+  PetscInt         jstart,jend;
     /* jstart is used in loops to denote the position in iperm where a 
      * group starts; jend denotes the position where it ends.
      * (jend + 1 is where the next group starts.) */
-  PetscInt       iold,nz;
-  PetscInt       istart,iend,isize;
-  PetscInt       ipos;
-  PetscScalar    yp[NDIM];
-  PetscInt       ip[NDIM];
+  PetscInt         iold,nz;
+  PetscInt         istart,iend,isize;
+  PetscInt         ipos;
+  PetscScalar      yp[NDIM];
+  PetscInt         ip[NDIM];
     /* yp[] and ip[] are treated as vector "registers" for performing 
      * the mat-vec. */
 
@@ -459,7 +461,7 @@ PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
 #endif
 
   PetscFunctionBegin;
-  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(xx,&x);CHKERRQ(ierr);
   ierr = VecGetArray(yy,&y);CHKERRQ(ierr);
   if (yy != ww) {
     ierr = VecGetArray(ww,&w);CHKERRQ(ierr);
@@ -472,14 +474,14 @@ PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
   ai  = a->i;  /* ai[k] is the position in aa and aj where row k starts. */
 
   /* Get the info we need about the permutations and groupings. */
-  csrperm = (Mat_SeqCSRPERM *) A->spptr;
-  iperm = csrperm->iperm;
-  ngroup = csrperm->ngroup;
-  xgroup = csrperm->xgroup;
-  nzgroup = csrperm->nzgroup;
+  aijperm = (Mat_SeqAIJPERM *) A->spptr;
+  iperm = aijperm->iperm;
+  ngroup = aijperm->ngroup;
+  xgroup = aijperm->xgroup;
+  nzgroup = aijperm->nzgroup;
   
-#if defined(PETSC_USE_FORTRAN_KERNEL_MULTADDCSRPERM)
-  fortranmultaddcsrperm_(&m,x,ii,aj,aa,y,w);
+#if defined(PETSC_USE_FORTRAN_KERNEL_MULTADDAIJPERM)
+  fortranmultaddaijperm_(&m,x,ii,aj,aa,y,w);
 #else
 
   for(igroup=0; igroup<ngroup; igroup++) {
@@ -531,11 +533,11 @@ PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
          * the chunk, we should vectorize along nz, that is, perform the 
          * mat-vec one row at a time as in the usual CSR case. */
         if(nz > isize) {
-#if defined(PETSC_HAVE_CRAYC)
+#if defined(PETSC_HAVE_CRAY_VECTOR)
 #pragma _CRI preferstream
 #endif
           for(i=0; i<isize; i++) {
-#if defined(PETSC_HAVE_CRAYC)
+#if defined(PETSC_HAVE_CRAY_VECTOR)
 #pragma _CRI prefervector
 #endif
             for(j=0; j<nz; j++) {
@@ -556,7 +558,7 @@ PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
           }
         }
 
-#if defined(PETSC_HAVE_CRAYC)
+#if defined(PETSC_HAVE_CRAY_VECTOR)
 #pragma _CRI ivdep
 #endif
         /* Put results from yp[] into non-permuted result vector y. */
@@ -570,7 +572,7 @@ PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
 
 #endif
   ierr = PetscLogFlops(2.0*a->nz);CHKERRQ(ierr);
-  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(xx,&x);CHKERRQ(ierr);
   ierr = VecRestoreArray(yy,&y);CHKERRQ(ierr);
   if (yy != ww) {
     ierr = VecRestoreArray(ww,&w);CHKERRQ(ierr);
@@ -579,43 +581,43 @@ PetscErrorCode MatMultAdd_SeqCSRPERM(Mat A,Vec xx,Vec ww,Vec yy)
 }
 
 
-/* MatConvert_SeqAIJ_SeqCSRPERM converts a SeqAIJ matrix into a 
- * SeqCSRPERM matrix.  This routine is called by the MatCreate_SeqCSRPERM() 
+/* MatConvert_SeqAIJ_SeqAIJPERM converts a SeqAIJ matrix into a 
+ * SeqAIJPERM matrix.  This routine is called by the MatCreate_SeqAIJPERM() 
  * routine, but can also be used to convert an assembled SeqAIJ matrix 
- * into a SeqCSRPERM one. */
+ * into a SeqAIJPERM one. */
 EXTERN_C_BEGIN
 #undef __FUNCT__
-#define __FUNCT__ "MatConvert_SeqAIJ_SeqCSRPERM"
-PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqAIJ_SeqCSRPERM(Mat A,const MatType type,MatReuse reuse,Mat *newmat)
+#define __FUNCT__ "MatConvert_SeqAIJ_SeqAIJPERM"
+PetscErrorCode  MatConvert_SeqAIJ_SeqAIJPERM(Mat A,const MatType type,MatReuse reuse,Mat *newmat)
 {
   PetscErrorCode ierr;
   Mat            B = *newmat;
-  Mat_SeqCSRPERM *csrperm;
+  Mat_SeqAIJPERM *aijperm;
 
   PetscFunctionBegin;
   if (reuse == MAT_INITIAL_MATRIX) {
     ierr = MatDuplicate(A,MAT_COPY_VALUES,&B);CHKERRQ(ierr);
   }
 
-  ierr = PetscNewLog(B,Mat_SeqCSRPERM,&csrperm);CHKERRQ(ierr);
-  B->spptr = (void *) csrperm;
+  ierr = PetscNewLog(B,Mat_SeqAIJPERM,&aijperm);CHKERRQ(ierr);
+  B->spptr = (void *) aijperm;
 
   /* Set function pointers for methods that we inherit from AIJ but override. */
-  B->ops->duplicate   = MatDuplicate_SeqCSRPERM;
-  B->ops->assemblyend = MatAssemblyEnd_SeqCSRPERM;
-  B->ops->destroy     = MatDestroy_SeqCSRPERM;
-  B->ops->mult        = MatMult_SeqCSRPERM;
-  B->ops->multadd     = MatMultAdd_SeqCSRPERM;
+  B->ops->duplicate   = MatDuplicate_SeqAIJPERM;
+  B->ops->assemblyend = MatAssemblyEnd_SeqAIJPERM;
+  B->ops->destroy     = MatDestroy_SeqAIJPERM;
+  B->ops->mult        = MatMult_SeqAIJPERM;
+  B->ops->multadd     = MatMultAdd_SeqAIJPERM;
 
   /* If A has already been assembled, compute the permutation. */
   if (A->assembled) {
-    ierr = SeqCSRPERM_create_perm(B);CHKERRQ(ierr);
+    ierr = SeqAIJPERM_create_perm(B);CHKERRQ(ierr);
   }
  
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatConvert_seqcsrperm_seqaij_C",
-                                           "MatConvert_SeqCSRPERM_SeqAIJ",MatConvert_SeqCSRPERM_SeqAIJ);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatConvert_seqaijperm_seqaij_C",
+                                           "MatConvert_SeqAIJPERM_SeqAIJ",MatConvert_SeqAIJPERM_SeqAIJ);CHKERRQ(ierr);
 
-  ierr = PetscObjectChangeTypeName((PetscObject)B,MATSEQCSRPERM);CHKERRQ(ierr);
+  ierr = PetscObjectChangeTypeName((PetscObject)B,MATSEQAIJPERM);CHKERRQ(ierr);
   *newmat = B;
   PetscFunctionReturn(0);
 }
@@ -623,9 +625,9 @@ EXTERN_C_END
 
 
 #undef __FUNCT__
-#define __FUNCT__ "MatCreateSeqCSRPERM"
+#define __FUNCT__ "MatCreateSeqAIJPERM"
 /*@C
-   MatCreateSeqCSRPERM - Creates a sparse matrix of type SEQCSRPERM.
+   MatCreateSeqAIJPERM - Creates a sparse matrix of type SEQAIJPERM.
    This type inherits from AIJ, but calculates some additional permutation 
    information that is used to allow better vectorization of some 
    operations.  At the cost of increased storage, the AIJ formatted 
@@ -655,30 +657,30 @@ EXTERN_C_END
 
 .keywords: matrix, cray, sparse, parallel
 
-.seealso: MatCreate(), MatCreateMPICSRPERM(), MatSetValues()
+.seealso: MatCreate(), MatCreateMPIAIJPERM(), MatSetValues()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatCreateSeqCSRPERM(MPI_Comm comm,PetscInt m,PetscInt n,PetscInt nz,const PetscInt nnz[],Mat *A)
+PetscErrorCode  MatCreateSeqAIJPERM(MPI_Comm comm,PetscInt m,PetscInt n,PetscInt nz,const PetscInt nnz[],Mat *A)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = MatCreate(comm,A);CHKERRQ(ierr);
   ierr = MatSetSizes(*A,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(*A,MATSEQCSRPERM);CHKERRQ(ierr);
-  ierr = MatSeqAIJSetPreallocation_SeqAIJ(*A,nz,(PetscInt*)nnz);CHKERRQ(ierr);
+  ierr = MatSetType(*A,MATSEQAIJPERM);CHKERRQ(ierr);
+  ierr = MatSeqAIJSetPreallocation_SeqAIJ(*A,nz,nnz);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__
-#define __FUNCT__ "MatCreate_SeqCSRPERM"
-PetscErrorCode PETSCMAT_DLLEXPORT MatCreate_SeqCSRPERM(Mat A)
+#define __FUNCT__ "MatCreate_SeqAIJPERM"
+PetscErrorCode  MatCreate_SeqAIJPERM(Mat A)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = MatSetType(A,MATSEQAIJ);CHKERRQ(ierr);
-  ierr = MatConvert_SeqAIJ_SeqCSRPERM(A,MATSEQCSRPERM,MAT_REUSE_MATRIX,&A);CHKERRQ(ierr);
+  ierr = MatConvert_SeqAIJ_SeqAIJPERM(A,MATSEQAIJPERM,MAT_REUSE_MATRIX,&A);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
