@@ -1,6 +1,6 @@
 
 #if !defined(__FMULT_H)
-#include "petscsys.h"
+#include <petscsys.h>
 #if defined(PETSC_USE_FORTRAN_KERNEL_MULTAIJ)
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define fortranmultaij_                FORTRANMULTAIJ
@@ -10,8 +10,8 @@
 #define fortranmulttransposeaddaij_    fortranmulttransposeaddaij
 #endif
 EXTERN_C_BEGIN
-EXTERN void fortranmultaij_(PetscInt*,const PetscScalar*,const PetscInt*,const PetscInt*,const MatScalar*,PetscScalar*);
-EXTERN void fortranmulttransposeaddaij_(PetscInt*,void*,PetscInt*,PetscInt*,void*,void*);
+extern void fortranmultaij_(PetscInt*,const PetscScalar*,const PetscInt*,const PetscInt*,const MatScalar*,PetscScalar*);
+extern void fortranmulttransposeaddaij_(PetscInt*,void*,PetscInt*,PetscInt*,void*,void*);
 EXTERN_C_END
 #endif
 #endif
