@@ -1,6 +1,6 @@
 
 #if !defined(__FSOLVE_H)
-#include "petscsys.h"
+#include <petscsys.h>
 #if defined(PETSC_USE_FORTRAN_KERNEL_SOLVEAIJ)
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define fortransolveaij_   FORTRANSOLVEAIJ
@@ -8,7 +8,7 @@
 #define fortransolveaij_   fortransolveaij
 #endif
 EXTERN_C_BEGIN
-EXTERN void fortransolveaij_(const PetscInt*,void*,const PetscInt*,const PetscInt*,const PetscInt*,const void*,const void*);
+extern void fortransolveaij_(const PetscInt*,void*,const PetscInt*,const PetscInt*,const PetscInt*,const void*,const void*);
 EXTERN_C_END
 #endif
 #endif
