@@ -1,35 +1,48 @@
-#define PETSCMAT_DLL
 
 /* 
-        Provides an interface for the Matlab engine sparse solver
+        Provides an interface for the MATLAB engine sparse solver
 
 */
-#include "../src/mat/impls/aij/seq/aij.h"
+#include <../src/mat/impls/aij/seq/aij.h>
 
-#include "engine.h"   /* Matlab include file */
-#include "mex.h"      /* Matlab include file */
+#include <engine.h>   /* MATLAB include file */
+#include <mex.h>      /* MATLAB include file */
+
+EXTERN_C_BEGIN
+#undef __FUNCT__  
+#define __FUNCT__ "MatSeqAIJToMatlab"
+mxArray *MatSeqAIJToMatlab(Mat B)
+{
+  PetscErrorCode ierr;
+  Mat_SeqAIJ     *aij = (Mat_SeqAIJ*)B->data;
+  mwIndex        *ii,*jj;
+  mxArray        *mat;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  mat  = mxCreateSparse(B->cmap->n,B->rmap->n,aij->nz,mxREAL);
+  ierr = PetscMemcpy(mxGetPr(mat),aij->a,aij->nz*sizeof(PetscScalar));
+  /* MATLAB stores by column, not row so we pass in the transpose of the matrix */
+  jj = mxGetIr(mat);
+  for (i=0; i<aij->nz; i++) jj[i] = aij->j[i];
+  ii = mxGetJc(mat);
+  for (i=0; i<B->rmap->n+1; i++) ii[i] = aij->i[i];
+  
+  PetscFunctionReturn(mat);
+}
+EXTERN_C_END
 
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "MatlabEnginePut_SeqAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatlabEnginePut_SeqAIJ(PetscObject obj,void *mengine)
+PetscErrorCode  MatlabEnginePut_SeqAIJ(PetscObject obj,void *mengine)
 {
   PetscErrorCode ierr;
-  Mat            B = (Mat)obj;
   mxArray        *mat; 
-  Mat_SeqAIJ     *aij = (Mat_SeqAIJ*)B->data;
 
   PetscFunctionBegin;
-  mat  = mxCreateSparse(B->cmap->n,B->rmap->n,aij->nz,mxREAL);
-  //mat  = mxCreateSparse(((PetscObject)B)->cmap.n,((PetscObject)B)->rmap.n,((Mat_SeqAIJ*)aij)->nz,mxREAL); 
-  ierr = PetscMemcpy(mxGetPr(mat),aij->a,aij->nz*sizeof(PetscScalar));CHKERRQ(ierr);
-  /* Matlab stores by column, not row so we pass in the transpose of the matrix */
-  ierr = PetscMemcpy(mxGetIr(mat),aij->j,aij->nz*sizeof(int));CHKERRQ(ierr);
-  ierr = PetscMemcpy(mxGetJc(mat),aij->i,(B->rmap->n+1)*sizeof(int));CHKERRQ(ierr);
-
-  /* Matlab indices start at 0 for sparse (what a surprise) */
-  
+  mat = MatSeqAIJToMatlab((Mat)obj);if (!mat) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"Cannot create MATLAB matrix");
   ierr = PetscObjectName(obj);CHKERRQ(ierr);
   engPutVariable((Engine *)mengine,obj->name,mat);
   PetscFunctionReturn(0);
@@ -38,36 +51,86 @@ EXTERN_C_END
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "MatlabEngineGet_SeqAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatlabEngineGet_SeqAIJ(PetscObject obj,void *mengine)
+#define __FUNCT__ "MatSeqAIJFromMatlab"
+/*@C
+    MatSeqAIJFromMatlab - Given a MATLAB sparse matrix, fills a SeqAIJ matrix with its transpose.
+
+   Not Collective
+
+   Input Parameters:
++     mmat - a MATLAB sparse matris
+-     mat - a already created MATSEQAIJ
+
+  Level: intermediate
+
+@*/
+PetscErrorCode  MatSeqAIJFromMatlab(mxArray *mmat,Mat mat)
 {
   PetscErrorCode ierr;
-  int            ii;
-  Mat            mat = (Mat)obj;
+  PetscInt       nz,n,m,*i,*j,k;
+  mwIndex        nnz,nn,nm,*ii,*jj;
   Mat_SeqAIJ     *aij = (Mat_SeqAIJ*)mat->data;
-  mxArray        *mmat; 
 
   PetscFunctionBegin;
-  ierr = MatSeqXAIJFreeAIJ(mat,&aij->a,&aij->j,&aij->i);CHKERRQ(ierr);
+  nn  = mxGetN(mmat);   /* rows of transpose of matrix */
+  nm  = mxGetM(mmat);
+  nnz = (mxGetJc(mmat))[nn];
+  ii  = mxGetJc(mmat);
+  jj  = mxGetIr(mmat);
+  n   = (PetscInt) nn;  
+  m   = (PetscInt) nm;
+  nz  = (PetscInt) nnz;
 
-  mmat = engGetVariable((Engine *)mengine,obj->name);
-
-  aij->nz           = (mxGetJc(mmat))[mat->rmap->n];
-  ierr  = PetscMalloc3(aij->nz,PetscScalar,&aij->a,aij->nz,PetscInt,&aij->j,mat->rmap->n+1,PetscInt,&aij->i);CHKERRQ(ierr);
-  aij->singlemalloc = PETSC_TRUE;
+  if (mat->rmap->n < 0 && mat->cmap->n < 0) {
+    /* matrix has not yet had its size set */
+    ierr = MatSetSizes(mat,n,m,PETSC_DETERMINE,PETSC_DETERMINE);CHKERRQ(ierr);
+    ierr = MatPreallocated(mat);CHKERRQ(ierr);
+  } else {
+    if (mat->rmap->n != n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change size of PETSc matrix %D to %D",mat->rmap->n,n);
+    if (mat->cmap->n != m) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change size of PETSc matrix %D to %D",mat->cmap->n,m);
+  }
+  if (nz != aij->nz) {
+    /* number of nonzeros in matrix has changed, so need new data structure */
+    ierr = MatSeqXAIJFreeAIJ(mat,&aij->a,&aij->j,&aij->i);CHKERRQ(ierr);
+    aij->nz           = nz;
+    ierr  = PetscMalloc3(aij->nz,PetscScalar,&aij->a,aij->nz,PetscInt,&aij->j,mat->rmap->n+1,PetscInt,&aij->i);CHKERRQ(ierr);
+    aij->singlemalloc = PETSC_TRUE;
+  }
 
   ierr = PetscMemcpy(aij->a,mxGetPr(mmat),aij->nz*sizeof(PetscScalar));CHKERRQ(ierr);
-  /* Matlab stores by column, not row so we pass in the transpose of the matrix */
-  ierr = PetscMemcpy(aij->j,mxGetIr(mmat),aij->nz*sizeof(int));CHKERRQ(ierr);
-  ierr = PetscMemcpy(aij->i,mxGetJc(mmat),(mat->rmap->n+1)*sizeof(int));CHKERRQ(ierr);
+  /* MATLAB stores by column, not row so we pass in the transpose of the matrix */
+  i = aij->i;
+  for (k=0; k<n+1; k++) {
+    i[k] = (PetscInt) ii[k];
+  }
+  j = aij->j;
+  for (k=0; k<nz; k++) {
+    j[k] = (PetscInt) jj[k];
+  }
 
-  for (ii=0; ii<mat->rmap->n; ii++) {
-    aij->ilen[ii] = aij->imax[ii] = aij->i[ii+1] - aij->i[ii];
+  for (k=0; k<mat->rmap->n; k++) {
+    aij->ilen[k] = aij->imax[k] = aij->i[k+1] - aij->i[k];
   }
 
   ierr = MatAssemblyBegin(mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+EXTERN_C_END
 
+
+EXTERN_C_BEGIN
+#undef __FUNCT__  
+#define __FUNCT__ "MatlabEngineGet_SeqAIJ"
+PetscErrorCode  MatlabEngineGet_SeqAIJ(PetscObject obj,void *mengine)
+{
+  PetscErrorCode ierr;
+  Mat            mat = (Mat)obj;
+  mxArray        *mmat; 
+
+  PetscFunctionBegin;
+  mmat = engGetVariable((Engine *)mengine,obj->name);
+  ierr = MatSeqAIJFromMatlab(mmat,mat);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -105,10 +168,10 @@ PetscErrorCode MatLUFactorNumeric_Matlab(Mat F,Mat A,const MatFactorInfo *info)
   PetscReal      dtcol = info->dtcol;
 
   PetscFunctionBegin;
-  if (F->factor == MAT_FACTOR_ILU || info->dt > 0) {
+  if (F->factortype == MAT_FACTOR_ILU || info->dt > 0) {
     if (info->dtcol == PETSC_DEFAULT)  dtcol = .01;
     F->ops->solve           = MatSolve_Matlab;
-    F->factor               = MAT_FACTOR_LU;
+    F->factortype           = MAT_FACTOR_LU;
     ierr = PetscMatlabEnginePut(PETSC_MATLAB_ENGINE_(((PetscObject)A)->comm),(PetscObject)A);CHKERRQ(ierr);
     _A   = ((PetscObject)A)->name;
     ierr = PetscMatlabEngineEvaluate(PETSC_MATLAB_ENGINE_(((PetscObject)A)->comm),"info_%s = struct('droptol',%g,'thresh',%g);",_A,info->dt,dtcol);CHKERRQ(ierr);
@@ -140,7 +203,7 @@ PetscErrorCode MatLUFactorNumeric_Matlab(Mat F,Mat A,const MatFactorInfo *info)
 PetscErrorCode MatLUFactorSymbolic_Matlab(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
 {
   PetscFunctionBegin;
-  if (A->cmap->N != A->rmap->N) SETERRQ(PETSC_ERR_ARG_SIZ,"matrix must be square"); 
+  if (A->cmap->N != A->rmap->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"matrix must be square"); 
   F->ops->lufactornumeric    = MatLUFactorNumeric_Matlab;
   F->assembled = PETSC_TRUE;
   PetscFunctionReturn(0);
@@ -152,11 +215,12 @@ EXTERN_C_BEGIN
 PetscErrorCode MatFactorGetSolverPackage_seqaij_matlab(Mat A,const MatSolverPackage *type)
 {
   PetscFunctionBegin;
-  *type = MAT_SOLVER_MATLAB;
+  *type = MATSOLVERMATLAB;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
 
+EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "MatGetFactor_seqaij_matlab"
 PetscErrorCode MatGetFactor_seqaij_matlab(Mat A,MatFactorType ftype,Mat *F)
@@ -164,7 +228,7 @@ PetscErrorCode MatGetFactor_seqaij_matlab(Mat A,MatFactorType ftype,Mat *F)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (A->cmap->N != A->rmap->N) SETERRQ(PETSC_ERR_ARG_SIZ,"matrix must be square"); 
+  if (A->cmap->N != A->rmap->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"matrix must be square"); 
   ierr                         = MatCreate(((PetscObject)A)->comm,F);CHKERRQ(ierr);
   ierr                         = MatSetSizes(*F,A->rmap->n,A->cmap->n,A->rmap->n,A->cmap->n);CHKERRQ(ierr);
   ierr                         = MatSetType(*F,((PetscObject)A)->type_name);CHKERRQ(ierr);
@@ -173,10 +237,10 @@ PetscErrorCode MatGetFactor_seqaij_matlab(Mat A,MatFactorType ftype,Mat *F)
   (*F)->ops->ilufactorsymbolic = MatLUFactorSymbolic_Matlab;
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)(*F),"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_seqaij_matlab",MatFactorGetSolverPackage_seqaij_matlab);CHKERRQ(ierr);
 
-  (*F)->factor                = ftype;
+  (*F)->factortype             = ftype;
   PetscFunctionReturn(0);
 }
-
+EXTERN_C_END
 
 /* --------------------------------------------------------------------------------*/
 
@@ -187,7 +251,7 @@ PetscErrorCode MatFactorInfo_Matlab(Mat A,PetscViewer viewer)
   PetscErrorCode ierr;
   
   PetscFunctionBegin; 
-  ierr = PetscViewerASCIIPrintf(viewer,"Matlab run parameters:  -- not written yet!\n");CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"MATLAB run parameters:  -- not written yet!\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -196,12 +260,12 @@ PetscErrorCode MatFactorInfo_Matlab(Mat A,PetscViewer viewer)
 PetscErrorCode MatView_Matlab(Mat A,PetscViewer viewer) 
 {
   PetscErrorCode    ierr;
-  PetscTruth        iascii;
+  PetscBool         iascii;
   PetscViewerFormat format;
 
   PetscFunctionBegin;
   ierr = MatView_SeqAIJ(A,viewer);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_FACTOR_INFO) {
@@ -213,14 +277,14 @@ PetscErrorCode MatView_Matlab(Mat A,PetscViewer viewer)
 
 
 /*MC
-  MAT_SOLVER_MATLAB - "matlab" - Providing direct solvers (LU and QR) and drop tolerance
-  based ILU factorization (ILUDT) for sequential matrices via the external package Matlab.
+  MATSOLVERMATLAB - "matlab" - Providing direct solvers (LU and QR) and drop tolerance
+  based ILU factorization (ILUDT) for sequential matrices via the external package MATLAB.
 
 
   Works with MATSEQAIJ matrices.
 
   Options Database Keys:
-. -pc_factor_mat_solver_type matlab - selects Matlab to do the sparse factorization
+. -pc_factor_mat_solver_type matlab - selects MATLAB to do the sparse factorization
 
 
   Level: beginner
