@@ -1,4 +1,3 @@
-#define PETSCMAT_DLL
 
 /*  
               This file creating by running f2c 
@@ -12,10 +11,9 @@
        Used by the sparse factorization routines in 
      src/mat/impls/baij/seq
 
-       See also src/inline/ilu.h
 */
 
-#include "petsc.h"
+#include <petscsys.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "LINPACKdgedi"
