@@ -2,7 +2,7 @@
 #if !defined(__is_h)
 #define __is_h
 
-#include "include/private/matimpl.h"
+#include <private/matimpl.h>
 
 typedef struct {
   Mat                    A;             /* the local Neumann matrix */
@@ -10,7 +10,7 @@ typedef struct {
   Vec                    x,y;           /* work space for ghost values for matrix vector product */
   ISLocalToGlobalMapping mapping;
   int                    rstart,rend;   /* local row ownership */
-  PetscTruth             pure_neumann;
+  PetscBool              pure_neumann;
 } Mat_IS;
 
 #endif
