@@ -1,11 +1,10 @@
-#define PETSCMAT_DLL
 
 /* 
    Provides an interface to the Spooles parallel sparse solver (MPI SPOOLES)
 */
 
-#include "../src/mat/impls/aij/seq/spooles/spooles.h"
-#include "../src/mat/impls/sbaij/mpi/mpisbaij.h"
+#include <../src/mat/impls/aij/seq/spooles/spooles.h>
+#include <../src/mat/impls/sbaij/mpi/mpisbaij.h>
 
 #if !defined(PETSC_USE_COMPLEX)
 /* 
@@ -19,9 +18,9 @@
 #define __FUNCT__ "MatGetInertia_MPISBAIJSpooles"
 PetscErrorCode MatGetInertia_MPISBAIJSpooles(Mat F,int *nneg,int *nzero,int *npos)
 { 
-  Mat_Spooles *lu = (Mat_Spooles*)F->spptr; 
+  Mat_Spooles    *lu = (Mat_Spooles*)F->spptr; 
   PetscErrorCode ierr;
-  int neg,zero,pos,sbuf[3],rbuf[3];
+  int            neg,zero,pos,sbuf[3],rbuf[3];
 
   PetscFunctionBegin;
   FrontMtx_inertia(lu->frontmtx, &neg, &zero, &pos);
@@ -37,52 +36,39 @@ PetscErrorCode MatGetInertia_MPISBAIJSpooles(Mat F,int *nneg,int *nzero,int *npo
 #define __FUNCT__ "MatCholeskyFactorSymbolic_MPISBAIJSpooles"
 PetscErrorCode MatCholeskyFactorSymbolic_MPISBAIJSpooles(Mat B,Mat A,IS r,const MatFactorInfo *info)
 {
-  Mat_Spooles   *lu;   
-  PetscErrorCode ierr;
-  
   PetscFunctionBegin;	
-
-  lu                       = (Mat_Spooles*)(B->spptr);
-  lu->options.pivotingflag = SPOOLES_NO_PIVOTING; 
-  lu->flg                  = DIFFERENT_NONZERO_PATTERN;
-  lu->options.useQR        = PETSC_FALSE;
-  lu->options.symflag      = SPOOLES_SYMMETRIC;  /* default */
-
-  ierr = MPI_Comm_dup(((PetscObject)A)->comm,&(lu->comm_spooles));CHKERRQ(ierr);
   (B)->ops->choleskyfactornumeric  = MatFactorNumeric_MPISpooles;
   PetscFunctionReturn(0); 
 }
 
 extern PetscErrorCode MatDestroy_MPISBAIJ(Mat);
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatDestroy_MPISBAIJSpooles"
 PetscErrorCode MatDestroy_MPISBAIJSpooles(Mat A)
 {
-  Mat_Spooles   *lu = (Mat_Spooles*)A->spptr; 
+  Mat_Spooles   *lu = (Mat_Spooles*)A->spptr;
   PetscErrorCode ierr;
-  
+
   PetscFunctionBegin;
-  if (lu->CleanUpSpooles) {
-    FrontMtx_free(lu->frontmtx);        
-    IV_free(lu->newToOldIV);            
-    IV_free(lu->oldToNewIV); 
+  if (lu && lu->CleanUpSpooles) {
+    FrontMtx_free(lu->frontmtx);
+    IV_free(lu->newToOldIV);
+    IV_free(lu->oldToNewIV);
     IV_free(lu->vtxmapIV);
-    InpMtx_free(lu->mtxA);             
-    ETree_free(lu->frontETree);          
-    IVL_free(lu->symbfacIVL);         
-    SubMtxManager_free(lu->mtxmanager);    
+    InpMtx_free(lu->mtxA);
+    ETree_free(lu->frontETree);
+    IVL_free(lu->symbfacIVL);
+    SubMtxManager_free(lu->mtxmanager);
     DenseMtx_free(lu->mtxX);
     DenseMtx_free(lu->mtxY);
     ierr = MPI_Comm_free(&(lu->comm_spooles));CHKERRQ(ierr);
-    if ( lu->scat ){
-      ierr = VecDestroy(lu->vec_spooles);CHKERRQ(ierr); 
-      ierr = ISDestroy(lu->iden);CHKERRQ(ierr); 
-      ierr = ISDestroy(lu->is_petsc);CHKERRQ(ierr);
-      ierr = VecScatterDestroy(lu->scat);CHKERRQ(ierr);
-    }
+    ierr = VecDestroy(&lu->vec_spooles);CHKERRQ(ierr);
+    ierr = ISDestroy(&lu->iden);CHKERRQ(ierr);
+    ierr = ISDestroy(&lu->is_petsc);CHKERRQ(ierr);
+    ierr = VecScatterDestroy(&lu->scat);CHKERRQ(ierr);
   }
+  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
   ierr = MatDestroy_MPISBAIJ(A);CHKERRQ(ierr);
-
   PetscFunctionReturn(0);
 }
 
@@ -92,7 +78,7 @@ EXTERN_C_BEGIN
 PetscErrorCode MatFactorGetSolverPackage_mpisbaij_spooles(Mat A,const MatSolverPackage *type)
 {
   PetscFunctionBegin;
-  *type = MAT_SOLVER_SPOOLES;
+  *type = MATSOLVERSPOOLES;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -107,7 +93,6 @@ PetscErrorCode MatGetFactor_mpisbaij_spooles(Mat A,MatFactorType ftype,Mat *F)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;	
-
   /* Create the factorization matrix F */  
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,A->rmap->N,A->cmap->N);CHKERRQ(ierr);
@@ -121,27 +106,26 @@ PetscErrorCode MatGetFactor_mpisbaij_spooles(Mat A,MatFactorType ftype,Mat *F)
 
   if (ftype == MAT_FACTOR_CHOLESKY) {
     B->ops->choleskyfactorsymbolic = MatCholeskyFactorSymbolic_MPISBAIJSpooles;
-    B->ops->destroy         = MatDestroy_MPISBAIJSpooles;  
+    B->ops->view                   = MatView_Spooles;
+    B->ops->destroy                = MatDestroy_MPISBAIJSpooles;  
     ierr = PetscObjectComposeFunctionDynamic((PetscObject)B,"MatFactorGetSolverPackage_C","MatFactorGetSolverPackage_mpisbaij_spooles",MatFactorGetSolverPackage_mpisbaij_spooles);CHKERRQ(ierr);
 
-    lu->options.symflag      = SPOOLES_NONSYMMETRIC;
-    lu->options.pivotingflag = SPOOLES_NO_PIVOTING; 
     lu->options.symflag      = SPOOLES_SYMMETRIC;
-  } else SETERRQ(PETSC_ERR_SUP,"Only Cholesky for SBAIJ matrices");
-  B->factor = ftype;
+    lu->options.pivotingflag = SPOOLES_NO_PIVOTING; 
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Only Cholesky for SBAIJ matrices, use AIJ for LU");
 
+  B->factortype = ftype;
   ierr = MPI_Comm_dup(((PetscObject)A)->comm,&(lu->comm_spooles));CHKERRQ(ierr);
-
   *F = B;
   PetscFunctionReturn(0); 
 }
 EXTERN_C_END
 
 /*MC
-  MAT_SOLVER_SPOOLES - "spooles" - a matrix type providing direct solvers (LU and Cholesky) for distributed symmetric
+  MATSOLVERSPOOLES - "spooles" - a matrix type providing direct solvers (LU and Cholesky) for distributed symmetric
   and non-symmetric  matrices via the external package Spooles.
 
-  If Spooles is installed (run config/configure.py with the option --download-spooles)
+  If Spooles is installed (run ./configure with the option --download-spooles)
 
   Options Database Keys:
 + -mat_spooles_tau <tau> - upper bound on the magnitude of the largest element in L or U
@@ -161,6 +145,6 @@ EXTERN_C_END
 
    Level: beginner
 
-.seealso: MAT_SOLVER_SUPERLU, MAT_SOLVER_MUMPS, MAT_SOLVER_SUPERLU_DIST, PCFactorSetMatSolverPackage(), MatSolverPackage 
+.seealso: MATSOLVERSUPERLU, MATSOLVERMUMPS, MATSOLVERSUPERLU_DIST, PCFactorSetMatSolverPackage(), MatSolverPackage 
 M*/
 
