@@ -1,20 +1,19 @@
-#define PETSCMAT_DLL
 
-#include "../src/mat/impls/aij/seq/aij.h"
-#include "../src/mat/impls/baij/seq/baij.h"
-#include "../src/mat/impls/sbaij/seq/sbaij.h"
+#include <../src/mat/impls/aij/seq/aij.h>
+#include <../src/mat/impls/baij/seq/baij.h>
+#include <../src/mat/impls/sbaij/seq/sbaij.h>
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "MatConvert_SeqSBAIJ_SeqAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat) 
+PetscErrorCode  MatConvert_SeqSBAIJ_SeqAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat) 
 {
   Mat            B;
   Mat_SeqSBAIJ   *a = (Mat_SeqSBAIJ*)A->data; 
   Mat_SeqAIJ     *b;
   PetscErrorCode ierr;
   PetscInt       *ai=a->i,*aj=a->j,m=A->rmap->N,n=A->cmap->n,i,j,k,*bi,*bj,*rowlengths,nz,*rowstart,itmp;
-  PetscInt       bs=A->rmap->bs,bs2=bs*bs,mbs=A->rmap->N/bs;
+  PetscInt       bs=A->rmap->bs,bs2=bs*bs,mbs=A->rmap->N/bs,diagcnt=0;
   MatScalar      *av,*bv;
 
   PetscFunctionBegin;
@@ -26,11 +25,13 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqAIJ(Mat A, MatType newt
   k = 0;
   for (i=0; i<mbs; i++) {
     nz = ai[i+1] - ai[i];
-    aj++; /* skip diagonal */
-    for (j=1; j<nz; j++) { /* no. of lower triangular blocks */
-      rowlengths[(*aj)*bs]++; aj++;
+    if (nz) {
+      rowlengths[k] += nz;   /* no. of upper triangular blocks */
+      if (*aj == i) {aj++;diagcnt++;nz--;} /* skip diagonal */
+      for (j=0; j<nz; j++) { /* no. of lower triangular blocks */
+	rowlengths[(*aj)*bs]++; aj++;
+      }
     }
-    rowlengths[k] += nz;   /* no. of upper triangular blocks */
     rowlengths[k] *= bs;
     for (j=1; j<bs; j++) {
       rowlengths[k+j] = rowlengths[k];
@@ -41,7 +42,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqAIJ(Mat A, MatType newt
   
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(B,newtype);CHKERRQ(ierr);
+  ierr = MatSetType(B,MATSEQAIJ);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(B,0,rowlengths);CHKERRQ(ierr);
   ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_FALSE);CHKERRQ(ierr);
   B->rmap->bs = A->rmap->bs;
@@ -55,35 +56,38 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqAIJ(Mat A, MatType newt
   bi[0] = 0; rowstart[0] = 0;
   for (i=0; i<mbs; i++){
     for (j=0; j<bs; j++){
-      b->ilen[i*bs+j]  = rowlengths[i*bs];
+      b->ilen[i*bs+j]    = rowlengths[i*bs];
       rowstart[i*bs+j+1] = rowstart[i*bs+j] + rowlengths[i*bs];
     }
     bi[i+1]     = bi[i] + rowlengths[i*bs]/bs; 
   }
-  if (bi[mbs] != 2*a->nz - mbs) SETERRQ2(PETSC_ERR_PLIB,"bi[mbs]: %D != 2*a->nz-mbs: %D\n",bi[mbs],2*a->nz - mbs);
+  if (bi[mbs] != 2*a->nz - diagcnt) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"bi[mbs]: %D != 2*a->nz-diagcnt: %D\n",bi[mbs],2*a->nz - diagcnt);
 
   /* set b->j and b->a */
   aj = a->j; av = a->a;
   for (i=0; i<mbs; i++) {
+    nz = ai[i+1] - ai[i];
     /* diagonal block */
-    for (j=0; j<bs; j++){   /* row i*bs+j */
-      itmp = i*bs+j;
-      for (k=0; k<bs; k++){ /* col i*bs+k */
-        *(bj + rowstart[itmp]) = (*aj)*bs+k;
-        *(bv + rowstart[itmp]) = *(av+k*bs+j); 
-        rowstart[itmp]++;
+    if (nz && *aj == i) {
+      nz--;
+      for (j=0; j<bs; j++){   /* row i*bs+j */
+	itmp = i*bs+j;
+	for (k=0; k<bs; k++){ /* col i*bs+k */
+	  *(bj + rowstart[itmp]) = (*aj)*bs+k;
+	  *(bv + rowstart[itmp]) = *(av+k*bs+j); 
+	  rowstart[itmp]++;
+	}
       }
+      aj++; av += bs2; 
     }
-    aj++; av += bs2; 
     
-    nz = ai[i+1] - ai[i] -1;
     while (nz--){
       /* lower triangular blocks */
       for (j=0; j<bs; j++){   /* row (*aj)*bs+j */
         itmp = (*aj)*bs+j;
         for (k=0; k<bs; k++){ /* col i*bs+k */
           *(bj + rowstart[itmp]) = i*bs+k;
-          *(bv + rowstart[itmp]) = *(av+k*bs+j); 
+          *(bv + rowstart[itmp]) = *(av+j*bs+k); 
           rowstart[itmp]++;
         }
       }
@@ -115,7 +119,7 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "MatConvert_SeqAIJ_SeqSBAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqAIJ_SeqSBAIJ(Mat A,const MatType newtype,MatReuse reuse,Mat *newmat) {
+PetscErrorCode  MatConvert_SeqAIJ_SeqSBAIJ(Mat A,const MatType newtype,MatReuse reuse,Mat *newmat) {
   Mat            B;
   Mat_SeqAIJ     *a = (Mat_SeqAIJ*)A->data; 
   Mat_SeqSBAIJ   *b;
@@ -124,7 +128,8 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqAIJ_SeqSBAIJ(Mat A,const MatType
   MatScalar      *av,*bv;
 
   PetscFunctionBegin;
-  if (n != m) SETERRQ(PETSC_ERR_ARG_WRONG,"Matrix must be square");
+  if (!A->symmetric) SETERRQ(((PetscObject)A)->comm,PETSC_ERR_USER,"Matrix must be symmetric. Call MatSetOption(mat,MAT_SYMMETRIC,PETSC_TRUE)");
+  if (n != m) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Matrix must be square");
 
   ierr = PetscMalloc(m*sizeof(PetscInt),&rowlengths);CHKERRQ(ierr);
   for (i=0; i<m; i++) {
@@ -132,7 +137,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqAIJ_SeqSBAIJ(Mat A,const MatType
   }
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(B,newtype);CHKERRQ(ierr);
+  ierr = MatSetType(B,MATSEQSBAIJ);CHKERRQ(ierr);
   ierr = MatSeqSBAIJSetPreallocation_SeqSBAIJ(B,1,0,rowlengths);CHKERRQ(ierr);
 
   ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_TRUE);CHKERRQ(ierr);
@@ -157,9 +162,6 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqAIJ_SeqSBAIJ(Mat A,const MatType
   ierr = PetscFree(rowlengths);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  if (A->hermitian){
-    ierr = MatSetOption(B,MAT_HERMITIAN,PETSC_TRUE);CHKERRQ(ierr);
-  }
 
   if (reuse == MAT_REUSE_MATRIX) {
     ierr = MatHeaderReplace(A,B);CHKERRQ(ierr);
@@ -172,15 +174,15 @@ EXTERN_C_END
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "MatConvert_SeqSBAI_SeqBAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqBAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat) 
+#define __FUNCT__ "MatConvert_SeqSBAIJ_SeqBAIJ"
+PetscErrorCode  MatConvert_SeqSBAIJ_SeqBAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat) 
 {
   Mat            B;
   Mat_SeqSBAIJ   *a = (Mat_SeqSBAIJ*)A->data; 
   Mat_SeqBAIJ    *b;
   PetscErrorCode ierr;
   PetscInt       *ai=a->i,*aj=a->j,m=A->rmap->N,n=A->cmap->n,i,k,*bi,*bj,*browlengths,nz,*browstart,itmp;
-  PetscInt       bs=A->rmap->bs,bs2=bs*bs,mbs=m/bs;
+  PetscInt       bs=A->rmap->bs,bs2=bs*bs,mbs=m/bs,col,row;
   MatScalar      *av,*bv;
 
   PetscFunctionBegin;
@@ -199,7 +201,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqBAIJ(Mat A, MatType new
   
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(B,newtype);CHKERRQ(ierr);
+  ierr = MatSetType(B,MATSEQBAIJ);CHKERRQ(ierr);
   ierr = MatSeqBAIJSetPreallocation(B,bs,0,browlengths);CHKERRQ(ierr);
   ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_TRUE);CHKERRQ(ierr);
   
@@ -211,11 +213,11 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqBAIJ(Mat A, MatType new
   /* set b->i */
   bi[0] = 0;
   for (i=0; i<mbs; i++){
-    b->ilen[i]    = browlengths[i];
-    bi[i+1]       = bi[i] + browlengths[i]; 
-    browstart[i]  = bi[i];
+    b->ilen[i]   = browlengths[i];
+    bi[i+1]      = bi[i] + browlengths[i]; 
+    browstart[i] = bi[i];
   }
-  if (bi[mbs] != 2*a->nz - mbs) SETERRQ2(PETSC_ERR_PLIB,"bi[mbs]: %D != 2*a->nz - mbs: %D\n",bi[mbs],2*a->nz - mbs);
+  if (bi[mbs] != 2*a->nz - mbs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"bi[mbs]: %D != 2*a->nz - mbs: %D\n",bi[mbs],2*a->nz - mbs);
   
   /* set b->j and b->a */
   aj = a->j; av = a->a;
@@ -230,20 +232,24 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqBAIJ(Mat A, MatType new
     
     nz = ai[i+1] - ai[i] -1;
     while (nz--){
-      /* lower triangular blocks */   
-      *(bj + browstart[*aj]) = i;
-      itmp = bs2*browstart[*aj];
-      for (k=0; k<bs2; k++){
-        *(bv + itmp + k) = *(av + k);
+      /* lower triangular blocks - transpose blocks of A */   
+      *(bj + browstart[*aj]) = i; /* block col index */
+      itmp = bs2*browstart[*aj];  /* row index */
+      for (col=0; col<bs; col++){
+        k = col;
+        for (row=0; row<bs; row++){
+          bv[itmp + col*bs+row] = av[k]; k+=bs;
+        }
       }
       browstart[*aj]++;
 
       /* upper triangular blocks */
       *(bj + browstart[i]) = *aj; aj++;
-      itmp = bs2*browstart[i];
+      itmp = bs2*browstart[i]; 
       for (k=0; k<bs2; k++){
-        *(bv + itmp + k) = *av; av++;
+        bv[itmp + k] = av[k]; 
       }
+      av += bs2;
       browstart[i]++;
     }
   }
@@ -261,24 +267,25 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqSBAIJ_SeqBAIJ(Mat A, MatType new
 EXTERN_C_END
 
 EXTERN_C_BEGIN
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MatConvert_SeqBAIJ_SeqSBAIJ"
-PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqBAIJ_SeqSBAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat) 
+PetscErrorCode  MatConvert_SeqBAIJ_SeqSBAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat)
 {
   Mat            B;
-  Mat_SeqBAIJ    *a = (Mat_SeqBAIJ*)A->data; 
+  Mat_SeqBAIJ    *a = (Mat_SeqBAIJ*)A->data;
   Mat_SeqSBAIJ   *b;
   PetscErrorCode ierr;
   PetscInt       *ai=a->i,*aj,m=A->rmap->N,n=A->cmap->n,i,j,k,*bi,*bj,*browlengths;
   PetscInt       bs=A->rmap->bs,bs2=bs*bs,mbs=m/bs,dd;
   MatScalar      *av,*bv;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  if (n != m) SETERRQ(PETSC_ERR_ARG_WRONG,"Matrix must be square");
+  if (!A->symmetric) SETERRQ(((PetscObject)A)->comm,PETSC_ERR_USER,"Matrix must be symmetric. Call MatSetOption(mat,MAT_SYMMETRIC,PETSC_TRUE)");
+  if (n != m) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Matrix must be square");
   ierr = MatMissingDiagonal_SeqBAIJ(A,&flg,&dd);CHKERRQ(ierr); /* check for missing diagonals, then mark diag */
-  if (flg) SETERRQ1(PETSC_ERR_ARG_WRONGSTATE,"Matrix is missing diagonal %D",dd);
-  
+  if (flg) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Matrix is missing diagonal %D",dd);
+
   ierr = PetscMalloc(mbs*sizeof(PetscInt),&browlengths);CHKERRQ(ierr);
   for (i=0; i<mbs; i++) {
     browlengths[i] = ai[i+1] - a->diag[i];
@@ -286,10 +293,10 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqBAIJ_SeqSBAIJ(Mat A, MatType new
 
   ierr = MatCreate(((PetscObject)A)->comm,&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(B,newtype);CHKERRQ(ierr);
+  ierr = MatSetType(B,MATSEQSBAIJ);CHKERRQ(ierr);
   ierr = MatSeqSBAIJSetPreallocation_SeqSBAIJ(B,bs,0,browlengths);CHKERRQ(ierr);
   ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_TRUE);CHKERRQ(ierr);
-  
+
   b  = (Mat_SeqSBAIJ*)(B->data);
   bi = b->i;
   bj = b->j;
@@ -298,9 +305,9 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatConvert_SeqBAIJ_SeqSBAIJ(Mat A, MatType new
   bi[0] = 0;
   for (i=0; i<mbs; i++) {
     aj = a->j + a->diag[i];
-    av = a->a + (a->diag[i])*bs2;   
+    av = a->a + (a->diag[i])*bs2;
     for (j=0; j<browlengths[i]; j++){
-      *bj = *aj; bj++; aj++; 
+      *bj = *aj; bj++; aj++;
       for (k=0; k<bs2; k++){
         *bv = *av; bv++; av++;
       }
