@@ -1,7 +1,6 @@
-#define PETSCMAT_DLL
 
-#include "../src/mat/impls/sbaij/seq/sbaij.h"
-#include "../src/inline/ilu.h"
+#include <../src/mat/impls/sbaij/seq/sbaij.h>
+#include <../src/mat/blockinvert.h>
 
 /* Version for when blocks are 4 by 4  */
 #undef __FUNCT__  
@@ -15,20 +14,18 @@ PetscErrorCode MatCholeskyFactorNumeric_SeqSBAIJ_4(Mat C,Mat A,const MatFactorIn
   PetscInt       i,j,*a2anew,k,k1,jmin,jmax,*jl,*il,vj,nexti,ili;
   MatScalar      *ba = b->a,*aa,*ap,*dk,*uik;
   MatScalar      *u,*diag,*rtmp,*rtmp_ptr;
-  PetscTruth     pivotinblocks = b->pivotinblocks;
-  PetscReal      shift = info->shiftinblocks;
+  PetscBool      pivotinblocks = b->pivotinblocks;
+  PetscReal      shift = info->shiftamount;
 
   PetscFunctionBegin;
   /* initialization */
   ierr = PetscMalloc(16*mbs*sizeof(MatScalar),&rtmp);CHKERRQ(ierr);
   ierr = PetscMemzero(rtmp,16*mbs*sizeof(MatScalar));CHKERRQ(ierr); 
-  ierr = PetscMalloc(2*mbs*sizeof(PetscInt),&il);CHKERRQ(ierr);
-  jl   = il + mbs;
+  ierr = PetscMalloc2(mbs,PetscInt,&il,mbs,PetscInt,&jl);CHKERRQ(ierr);
   for (i=0; i<mbs; i++) {
     jl[i] = mbs; il[0] = 0;
   }
-  ierr = PetscMalloc(32*sizeof(MatScalar),&dk);CHKERRQ(ierr);
-  uik  = dk + 16;     
+  ierr = PetscMalloc2(16,MatScalar,&dk,16,MatScalar,&uik);CHKERRQ(ierr);
   ierr = ISGetIndices(perm,&perm_ptr);CHKERRQ(ierr);
 
   /* check permutation */
@@ -135,7 +132,7 @@ PetscErrorCode MatCholeskyFactorNumeric_SeqSBAIJ_4(Mat C,Mat A,const MatFactorIn
       dk[14]+= uik[8]*u[12] + uik[9]*u[13] + uik[10]*u[14]+ uik[11]*u[15];
       dk[15]+= uik[12]*u[12]+ uik[13]*u[13]+ uik[14]*u[14]+ uik[15]*u[15];
 
-      ierr = PetscLogFlops(64*4);CHKERRQ(ierr);
+      ierr = PetscLogFlops(64.0*4.0);CHKERRQ(ierr);
       
       /* update -U(i,k) */
       ierr = PetscMemcpy(ba+ili*16,uik,16*sizeof(MatScalar));CHKERRQ(ierr); 
@@ -167,7 +164,7 @@ PetscErrorCode MatCholeskyFactorNumeric_SeqSBAIJ_4(Mat C,Mat A,const MatFactorIn
           rtmp_ptr[14]+= uik[8]*u[12] + uik[9]*u[13] + uik[10]*u[14]+ uik[11]*u[15];
           rtmp_ptr[15]+= uik[12]*u[12]+ uik[13]*u[13]+ uik[14]*u[14]+ uik[15]*u[15];
         }
-        ierr = PetscLogFlops(2*64*(jmax-jmin));CHKERRQ(ierr);
+        ierr = PetscLogFlops(2.0*64.0*(jmax-jmin));CHKERRQ(ierr);
       
         /* ... add i to row list for next nonzero entry */
         il[i] = jmin;             /* update il(i) in column k+1, ... mbs-1 */
@@ -209,15 +206,15 @@ PetscErrorCode MatCholeskyFactorNumeric_SeqSBAIJ_4(Mat C,Mat A,const MatFactorIn
   } 
 
   ierr = PetscFree(rtmp);CHKERRQ(ierr);
-  ierr = PetscFree(il);CHKERRQ(ierr); 
-  ierr = PetscFree(dk);CHKERRQ(ierr);
+  ierr = PetscFree2(il,jl);CHKERRQ(ierr); 
+  ierr = PetscFree2(dk,uik);CHKERRQ(ierr);
   if (a->permute){
     ierr = PetscFree(aa);CHKERRQ(ierr);
   }
 
   ierr = ISRestoreIndices(perm,&perm_ptr);CHKERRQ(ierr);
-  C->ops->solve          = MatSolve_SeqSBAIJ_4;
-  C->ops->solvetranspose = MatSolve_SeqSBAIJ_4;
+  C->ops->solve          = MatSolve_SeqSBAIJ_4_inplace;
+  C->ops->solvetranspose = MatSolve_SeqSBAIJ_4_inplace;
   C->assembled = PETSC_TRUE;
   C->preallocated = PETSC_TRUE;
   ierr = PetscLogFlops(1.3333*64*b->mbs);CHKERRQ(ierr); /* from inverting diagonal blocks */
