@@ -1,4 +1,3 @@
-#define PETSCMAT_DLL
 
 /*
    This provides a simple shell for Fortran (and C programmers) to 
@@ -6,18 +5,168 @@
   much of anything.
 */
 
-#include "private/matimpl.h"        /*I "petscmat.h" I*/
-#include "private/vecimpl.h"  
+#include <private/matimpl.h>        /*I "petscmat.h" I*/
+#include <private/vecimpl.h>  
 
 typedef struct {
   PetscErrorCode (*destroy)(Mat);
   PetscErrorCode (*mult)(Mat,Vec,Vec);
   PetscErrorCode (*multtranspose)(Mat,Vec,Vec);
   PetscErrorCode (*getdiagonal)(Mat,Vec);
-  PetscTruth     scale,shift;
   PetscScalar    vscale,vshift;
+  Vec            dshift;
+  Vec            left,right;
+  Vec            dshift_owned,left_owned,right_owned;
+  Vec            left_work,right_work;
+  PetscBool      usingscaled;
   void           *ctx;
-} Mat_Shell;      
+} Mat_Shell;
+/*
+ The most general expression for the matrix is
+
+ S = L (a A + B) R
+
+ where
+ A is the matrix defined by the user's function
+ a is a scalar multiple
+ L is left scaling
+ R is right scaling
+ B is a diagonal shift defined by
+   diag(dshift) if the vector dshift is non-NULL
+   vscale*identity otherwise
+
+ The following identities apply:
+
+ Scale by c:
+  c [L (a A + B) R] = L [(a c) A + c B] R
+
+ Shift by c:
+  [L (a A + B) R] + c = L [a A + (B + c Linv Rinv)] R
+
+ Diagonal scaling is achieved by simply multiplying with existing L and R vectors
+
+ In the data structure:
+
+ vscale=1.0  means no special scaling will be applied
+ dshift=NULL means a constant diagonal shift (fall back to vshift)
+ vshift=0.0  means no constant diagonal shift, note that vshift is only used if dshift is NULL
+*/
+
+static PetscErrorCode MatMult_Shell(Mat,Vec,Vec);
+static PetscErrorCode MatMultTranspose_Shell(Mat,Vec,Vec);
+static PetscErrorCode MatGetDiagonal_Shell(Mat,Vec);
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatShellUseScaledMethods"
+static PetscErrorCode MatShellUseScaledMethods(Mat Y)
+{
+  Mat_Shell *shell = (Mat_Shell*)Y->data;
+
+  PetscFunctionBegin;
+  if (shell->usingscaled) PetscFunctionReturn(0);
+  shell->mult  = Y->ops->mult;
+  Y->ops->mult = MatMult_Shell;
+  if (Y->ops->multtranspose) {
+    shell->multtranspose  = Y->ops->multtranspose;
+    Y->ops->multtranspose = MatMultTranspose_Shell;
+  }
+  if (Y->ops->getdiagonal) {
+    shell->getdiagonal  = Y->ops->getdiagonal;
+    Y->ops->getdiagonal = MatGetDiagonal_Shell;
+  }
+  shell->usingscaled = PETSC_TRUE;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatShellPreScaleLeft"
+static PetscErrorCode MatShellPreScaleLeft(Mat A,Vec x,Vec *xx)
+{
+  Mat_Shell *shell = (Mat_Shell*)A->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  *xx = PETSC_NULL;
+  if (!shell->left) {
+    *xx = x;
+  } else {
+    if (!shell->left_work) {ierr = VecDuplicate(shell->left,&shell->left_work);CHKERRQ(ierr);}
+    ierr = VecPointwiseMult(shell->left_work,x,shell->left);CHKERRQ(ierr);
+    *xx = shell->left_work;
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatShellPreScaleRight"
+static PetscErrorCode MatShellPreScaleRight(Mat A,Vec x,Vec *xx)
+{
+  Mat_Shell *shell = (Mat_Shell*)A->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  *xx = PETSC_NULL;
+  if (!shell->right) {
+    *xx = x;
+  } else {
+    if (!shell->right_work) {ierr = VecDuplicate(shell->right,&shell->right_work);CHKERRQ(ierr);}
+    ierr = VecPointwiseMult(shell->right_work,x,shell->right);CHKERRQ(ierr);
+    *xx = shell->right_work;
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatShellPostScaleLeft"
+static PetscErrorCode MatShellPostScaleLeft(Mat A,Vec x)
+{
+  Mat_Shell *shell = (Mat_Shell*)A->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (shell->left) {ierr = VecPointwiseMult(x,x,shell->left);CHKERRQ(ierr);}
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatShellPostScaleRight"
+static PetscErrorCode MatShellPostScaleRight(Mat A,Vec x)
+{
+  Mat_Shell *shell = (Mat_Shell*)A->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (shell->right) {ierr = VecPointwiseMult(x,x,shell->right);CHKERRQ(ierr);}
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatShellShiftAndScale"
+static PetscErrorCode MatShellShiftAndScale(Mat A,Vec X,Vec Y)
+{
+  Mat_Shell *shell = (Mat_Shell*)A->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (shell->dshift) {          /* get arrays because there is no VecPointwiseMultAdd() */
+    PetscInt          i,m;
+    const PetscScalar *x,*d;
+    PetscScalar       *y;
+    ierr = VecGetLocalSize(X,&m);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(shell->dshift,&d);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(X,&x);CHKERRQ(ierr);
+    ierr = VecGetArray(Y,&y);CHKERRQ(ierr);
+    for (i=0; i<m; i++) y[i] = shell->vscale*y[i] + d[i]*x[i];
+    ierr = VecRestoreArrayRead(shell->dshift,&d);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(X,&x);CHKERRQ(ierr);
+    ierr = VecRestoreArray(Y,&y);CHKERRQ(ierr);
+  } else if (PetscAbsScalar(shell->vshift) != 0) {
+    ierr = VecAXPBY(Y,shell->vshift,shell->vscale,X);CHKERRQ(ierr);
+  } else if (shell->vscale != 1.0) {
+    ierr = VecScale(Y,shell->vscale);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "MatShellGetContext"
@@ -42,17 +191,17 @@ typedef struct {
 
 .seealso: MatCreateShell(), MatShellSetOperation(), MatShellSetContext()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatShellGetContext(Mat mat,void **ctx)
+PetscErrorCode  MatShellGetContext(Mat mat,void *ctx)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
-  PetscValidPointer(ctx,2); 
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  PetscValidPointer(ctx,2);
   ierr = PetscTypeCompare((PetscObject)mat,MATSHELL,&flg);CHKERRQ(ierr);
-  if (!flg) *ctx = 0; 
-  else      *ctx = ((Mat_Shell*)(mat->data))->ctx; 
+  if (flg) *(void**)ctx = ((Mat_Shell*)(mat->data))->ctx;
+  else SETERRQ(((PetscObject)mat)->comm,PETSC_ERR_SUP,"Cannot get context from non-shell matrix");
   PetscFunctionReturn(0);
 }
 
@@ -61,12 +210,18 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatShellGetContext(Mat mat,void **ctx)
 PetscErrorCode MatDestroy_Shell(Mat mat)
 {
   PetscErrorCode ierr;
-  Mat_Shell      *shell;
+  Mat_Shell      *shell = (Mat_Shell*)mat->data;
 
   PetscFunctionBegin;
-  shell = (Mat_Shell*)mat->data;
-  if (shell->destroy) {ierr = (*shell->destroy)(mat);CHKERRQ(ierr);}
-  ierr = PetscFree(shell);CHKERRQ(ierr);
+  if (shell->destroy) {
+    ierr = (*shell->destroy)(mat);CHKERRQ(ierr);
+  }
+  ierr = VecDestroy(&shell->left_owned);CHKERRQ(ierr);
+  ierr = VecDestroy(&shell->right_owned);CHKERRQ(ierr);
+  ierr = VecDestroy(&shell->dshift_owned);CHKERRQ(ierr);
+  ierr = VecDestroy(&shell->left_work);CHKERRQ(ierr);
+  ierr = VecDestroy(&shell->right_work);CHKERRQ(ierr);
+  ierr = PetscFree(mat->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -74,18 +229,15 @@ PetscErrorCode MatDestroy_Shell(Mat mat)
 #define __FUNCT__ "MatMult_Shell"
 PetscErrorCode MatMult_Shell(Mat A,Vec x,Vec y)
 {
-  Mat_Shell      *shell = (Mat_Shell*)A->data;  
+  Mat_Shell      *shell = (Mat_Shell*)A->data;
   PetscErrorCode ierr;
+  Vec            xx;
 
   PetscFunctionBegin;
-  ierr = (*shell->mult)(A,x,y);CHKERRQ(ierr);
-  if (shell->shift && shell->scale) {
-    ierr = VecAXPBY(y,shell->vshift,shell->vscale,x);CHKERRQ(ierr);
-  } else if (shell->scale) {
-    ierr = VecScale(y,shell->vscale);CHKERRQ(ierr);
-  } else {
-    ierr = VecAXPY(y,shell->vshift,x);CHKERRQ(ierr);
-  }
+  ierr = MatShellPreScaleRight(A,x,&xx);CHKERRQ(ierr);
+  ierr = (*shell->mult)(A,xx,y);CHKERRQ(ierr);
+  ierr = MatShellShiftAndScale(A,xx,y);CHKERRQ(ierr);
+  ierr = MatShellPostScaleLeft(A,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -93,18 +245,15 @@ PetscErrorCode MatMult_Shell(Mat A,Vec x,Vec y)
 #define __FUNCT__ "MatMultTranspose_Shell"
 PetscErrorCode MatMultTranspose_Shell(Mat A,Vec x,Vec y)
 {
-  Mat_Shell      *shell = (Mat_Shell*)A->data;  
+  Mat_Shell      *shell = (Mat_Shell*)A->data;
   PetscErrorCode ierr;
+  Vec            xx;
 
   PetscFunctionBegin;
-  ierr = (*shell->multtranspose)(A,x,y);CHKERRQ(ierr);
-  if (shell->shift && shell->scale) {
-    ierr = VecAXPBY(y,shell->vshift,shell->vscale,x);CHKERRQ(ierr);
-  } else if (shell->scale) {
-    ierr = VecScale(y,shell->vscale);CHKERRQ(ierr);
-  } else {
-    ierr = VecAXPY(y,shell->vshift,x);CHKERRQ(ierr);
-  }
+  ierr = MatShellPreScaleLeft(A,x,&xx);CHKERRQ(ierr);
+  ierr = (*shell->multtranspose)(A,xx,y);CHKERRQ(ierr);
+  ierr = MatShellShiftAndScale(A,xx,y);CHKERRQ(ierr);
+  ierr = MatShellPostScaleRight(A,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -117,12 +266,14 @@ PetscErrorCode MatGetDiagonal_Shell(Mat A,Vec v)
 
   PetscFunctionBegin;
   ierr = (*shell->getdiagonal)(A,v);CHKERRQ(ierr);
-  if (shell->scale) {
-    ierr = VecScale(v,shell->vscale);CHKERRQ(ierr);
-  }
-  if (shell->shift) {
+  ierr = VecScale(v,shell->vscale);CHKERRQ(ierr);
+  if (shell->dshift) {
+    ierr = VecPointwiseMult(v,v,shell->dshift);CHKERRQ(ierr);
+  } else {
     ierr = VecShift(v,shell->vshift);CHKERRQ(ierr);
-  } 
+  }
+  if (shell->left)  {ierr = VecPointwiseMult(v,v,shell->left);CHKERRQ(ierr);}
+  if (shell->right) {ierr = VecPointwiseMult(v,v,shell->right);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
@@ -130,25 +281,20 @@ PetscErrorCode MatGetDiagonal_Shell(Mat A,Vec v)
 #define __FUNCT__ "MatShift_Shell"
 PetscErrorCode MatShift_Shell(Mat Y,PetscScalar a)
 {
-  Mat_Shell *shell = (Mat_Shell*)Y->data;  
+  Mat_Shell *shell = (Mat_Shell*)Y->data;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (shell->scale || shell->shift) {
-    shell->vshift += a;
-  } else {
-    shell->mult  = Y->ops->mult;
-    Y->ops->mult = MatMult_Shell;
-    if (Y->ops->multtranspose) {
-      shell->multtranspose  = Y->ops->multtranspose;
-      Y->ops->multtranspose = MatMultTranspose_Shell;
-    }
-    if (Y->ops->getdiagonal) {
-      shell->getdiagonal  = Y->ops->getdiagonal;
-      Y->ops->getdiagonal = MatGetDiagonal_Shell;
-    }
-    shell->vshift = a;
-  }
-  shell->shift  =  PETSC_TRUE;
+  if (shell->left || shell->right || shell->dshift) {
+    if (!shell->dshift) {
+      if (!shell->dshift_owned) {ierr = VecDuplicate(shell->left ? shell->left : shell->right, &shell->dshift_owned);CHKERRQ(ierr);}
+      shell->dshift = shell->dshift_owned;
+      ierr = VecSet(shell->dshift,shell->vshift+a);CHKERRQ(ierr);
+    } else {ierr = VecScale(shell->dshift,a);CHKERRQ(ierr);}
+    if (shell->left)  {ierr = VecPointwiseDivide(shell->dshift,shell->dshift,shell->left);CHKERRQ(ierr);}
+    if (shell->right) {ierr = VecPointwiseDivide(shell->dshift,shell->dshift,shell->right);CHKERRQ(ierr);}
+  } else shell->vshift += a;
+  ierr = MatShellUseScaledMethods(Y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -156,25 +302,42 @@ PetscErrorCode MatShift_Shell(Mat Y,PetscScalar a)
 #define __FUNCT__ "MatScale_Shell"
 PetscErrorCode MatScale_Shell(Mat Y,PetscScalar a)
 {
-  Mat_Shell *shell = (Mat_Shell*)Y->data;  
+  Mat_Shell      *shell = (Mat_Shell*)Y->data;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (shell->scale || shell->shift) {
-    shell->vscale *= a;
-  } else {
-    shell->mult  = Y->ops->mult;
-    Y->ops->mult = MatMult_Shell;
-    if (Y->ops->multtranspose) {
-      shell->multtranspose  = Y->ops->multtranspose;
-      Y->ops->multtranspose = MatMultTranspose_Shell;
+  shell->vscale *= a;
+  if (shell->dshift) {ierr = VecScale(shell->dshift,a);CHKERRQ(ierr);}
+  else shell->vshift *= a;
+  ierr = MatShellUseScaledMethods(Y);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatDiagonalScale_Shell"
+static PetscErrorCode MatDiagonalScale_Shell(Mat Y,Vec left,Vec right)
+{
+  Mat_Shell *shell = (Mat_Shell*)Y->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (left) {
+    if (!shell->left_owned) {ierr = VecDuplicate(left,&shell->left_owned);CHKERRQ(ierr);}
+    if (shell->left) {ierr = VecPointwiseMult(shell->left,shell->left,left);CHKERRQ(ierr);}
+    else {
+      shell->left = shell->left_owned;
+      ierr = VecCopy(left,shell->left);CHKERRQ(ierr);
     }
-    if (Y->ops->getdiagonal) {
-      shell->getdiagonal  = Y->ops->getdiagonal;
-      Y->ops->getdiagonal = MatGetDiagonal_Shell;
-    }
-    shell->vscale = a;
   }
-  shell->scale  =  PETSC_TRUE;
+  if (right) {
+    if (!shell->right_owned) {ierr = VecDuplicate(right,&shell->right_owned);CHKERRQ(ierr);}
+    if (shell->right) {ierr = VecPointwiseMult(shell->right,shell->right,right);CHKERRQ(ierr);}
+    else {
+      shell->right = shell->right_owned;
+      ierr = VecCopy(right,shell->right);CHKERRQ(ierr);
+    }
+  }
+  ierr = MatShellUseScaledMethods(Y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -185,25 +348,40 @@ PetscErrorCode MatAssemblyEnd_Shell(Mat Y,MatAssemblyType t)
   Mat_Shell *shell = (Mat_Shell*)Y->data;  
 
   PetscFunctionBegin;
-  if ((shell->shift || shell->scale) && t == MAT_FINAL_ASSEMBLY) {
-    shell->scale  = PETSC_FALSE;
-    shell->shift  = PETSC_FALSE;
-    shell->vshift = 0.0;
-    shell->vscale = 1.0;
-    Y->ops->mult          = shell->mult;
-    Y->ops->multtranspose = shell->multtranspose;
-    Y->ops->getdiagonal   = shell->getdiagonal;
+  if (t == MAT_FINAL_ASSEMBLY) {
+    shell->vshift      = 0.0;
+    shell->vscale      = 1.0;
+    shell->dshift      = PETSC_NULL;
+    shell->left        = PETSC_NULL;
+    shell->right       = PETSC_NULL;
+    if (shell->mult) {
+      Y->ops->mult = shell->mult;
+      shell->mult  = PETSC_NULL;
+    }
+    if (shell->multtranspose) {
+      Y->ops->multtranspose = shell->multtranspose;
+      shell->multtranspose  = PETSC_NULL;
+    }
+    if (shell->getdiagonal) {
+      Y->ops->getdiagonal = shell->getdiagonal;
+      shell->getdiagonal  = PETSC_NULL;
+    }
+    shell->usingscaled = PETSC_FALSE;
   }
   PetscFunctionReturn(0);
 }
 
-EXTERN PetscErrorCode MatConvert_Shell(Mat, const MatType,MatReuse,Mat*);
+extern PetscErrorCode MatConvert_Shell(Mat, const MatType,MatReuse,Mat*);
 
 #undef __FUNCT__  
 #define __FUNCT__ "MatSetBlockSize_Shell"
 PetscErrorCode MatSetBlockSize_Shell(Mat A,PetscInt bs)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  ierr = PetscLayoutSetBlockSize(A->rmap,bs);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(A->cmap,bs);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -225,84 +403,83 @@ static struct _MatOps MatOps_Values = {0,
 /*15*/ 0,
        0,
        0,
-       0,
+       MatDiagonalScale_Shell,
        0,
 /*20*/ 0,
        MatAssemblyEnd_Shell,
        0,
        0,
-       0,
-/*25*/ 0,
+/*24*/ 0,
        0,
        0,
        0,
        0,
-/*30*/ 0,
+/*29*/ 0,
        0,
        0,
        0,
        0,
-/*35*/ 0,
+/*34*/ 0,
        0,
        0,
        0,
        0,
-/*40*/ 0,
+/*39*/ 0,
        0,
        0,
        0,
        0,
-/*45*/ 0,
+/*44*/ 0,
        MatScale_Shell,
        MatShift_Shell,
        0,
        0,
-/*50*/ MatSetBlockSize_Shell,
+/*49*/ MatSetBlockSize_Shell,
        0,
        0,
        0,
        0,
-/*55*/ 0,
+/*54*/ 0,
        0,
        0,
        0,
        0,
-/*60*/ 0,
+/*59*/ 0,
        MatDestroy_Shell,
        0,
        0,
        0,
-/*65*/ 0,
+/*64*/ 0,
        0,
        0,
        0,
        0,
-/*70*/ 0,
+/*69*/ 0,
        0,
        MatConvert_Shell,
        0,
        0,
-/*75*/ 0,
+/*74*/ 0,
        0,
        0,
        0,
        0,
-/*80*/ 0,
+/*79*/ 0,
        0,
        0,
        0,
        0,
-/*85*/ 0,
+/*84*/ 0,
        0,
        0,
        0,
        0,
-/*90*/ 0,
+/*89*/ 0,
        0,
        0,
        0,
        0,
-/*95*/ 0,
+/*94*/ 0,
        0,
        0,
        0};
@@ -318,7 +495,7 @@ M*/
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "MatCreate_Shell"
-PetscErrorCode PETSCMAT_DLLEXPORT MatCreate_Shell(Mat A)
+PetscErrorCode  MatCreate_Shell(Mat A)
 {
   Mat_Shell      *b;
   PetscErrorCode ierr;
@@ -329,18 +506,12 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatCreate_Shell(Mat A)
   ierr = PetscNewLog(A,Mat_Shell,&b);CHKERRQ(ierr);
   A->data = (void*)b;
 
-  if (A->rmap->n == PETSC_DECIDE || A->cmap->n == PETSC_DECIDE) {
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Must give local row and column count for matrix");
-  }
-
-  ierr = PetscMapSetBlockSize(A->rmap,1);CHKERRQ(ierr);
-  ierr = PetscMapSetBlockSize(A->cmap,1);CHKERRQ(ierr);
-  ierr = PetscMapSetUp(A->rmap);CHKERRQ(ierr);
-  ierr = PetscMapSetUp(A->cmap);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(A->rmap,1);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(A->cmap,1);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(A->rmap);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(A->cmap);CHKERRQ(ierr);
 
   b->ctx           = 0;
-  b->scale         = PETSC_FALSE;
-  b->shift         = PETSC_FALSE;
   b->vshift        = 0.0;
   b->vscale        = 1.0;
   b->mult          = 0;
@@ -419,7 +590,7 @@ $
 
 .seealso: MatShellSetOperation(), MatHasOperation(), MatShellGetContext(), MatShellSetContext()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatCreateShell(MPI_Comm comm,PetscInt m,PetscInt n,PetscInt M,PetscInt N,void *ctx,Mat *A)
+PetscErrorCode  MatCreateShell(MPI_Comm comm,PetscInt m,PetscInt n,PetscInt M,PetscInt N,void *ctx,Mat *A)
 {
   PetscErrorCode ierr;
 
@@ -434,10 +605,10 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatCreateShell(MPI_Comm comm,PetscInt m,PetscI
 
 #undef __FUNCT__  
 #define __FUNCT__ "MatShellSetContext"
-/*@C
+/*@
     MatShellSetContext - sets the context for a shell matrix
 
-   Collective on Mat
+   Logically Collective on Mat
 
     Input Parameters:
 +   mat - the shell matrix
@@ -450,18 +621,18 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatCreateShell(MPI_Comm comm,PetscInt m,PetscI
 
 .seealso: MatCreateShell(), MatShellGetContext(), MatShellGetOperation()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatShellSetContext(Mat mat,void *ctx)
+PetscErrorCode  MatShellSetContext(Mat mat,void *ctx)
 {
   Mat_Shell      *shell = (Mat_Shell*)mat->data;
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   ierr = PetscTypeCompare((PetscObject)mat,MATSHELL,&flg);CHKERRQ(ierr);
   if (flg) {
     shell->ctx = ctx;
-  }
+  } else SETERRQ(((PetscObject)mat)->comm,PETSC_ERR_SUP,"Cannot attach context to non-shell matrix");
   PetscFunctionReturn(0);
 }
 
@@ -471,7 +642,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatShellSetContext(Mat mat,void *ctx)
     MatShellSetOperation - Allows user to set a matrix operation for
                            a shell matrix.
 
-   Collective on Mat
+   Logically Collective on Mat
 
     Input Parameters:
 +   mat - the shell matrix
@@ -504,17 +675,20 @@ $       MatMult(Mat,Vec,Vec) -> usermult(Mat,Vec,Vec)
     MatShellGetContext() to obtain the user-defined context that was
     set by MatCreateShell().
 
+    Fortran Notes: For MatGetVecs() the user code should check if the input left or right matrix is -1 and in that case not
+       generate a matrix. See src/mat/examples/tests/ex120f.F
+
 .keywords: matrix, shell, set, operation
 
 .seealso: MatCreateShell(), MatShellGetContext(), MatShellGetOperation(), MatShellSetContext()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatShellSetOperation(Mat mat,MatOperation op,void (*f)(void))
+PetscErrorCode  MatShellSetOperation(Mat mat,MatOperation op,void (*f)(void))
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   if (op == MATOP_DESTROY) {
     ierr = PetscTypeCompare((PetscObject)mat,MATSHELL,&flg);CHKERRQ(ierr);
     if (flg) {
@@ -564,13 +738,13 @@ $       MatMult(Mat,Vec,Vec) -> usermult(Mat,Vec,Vec)
 
 .seealso: MatCreateShell(), MatShellGetContext(), MatShellSetOperation(), MatShellSetContext()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatShellGetOperation(Mat mat,MatOperation op,void(**f)(void))
+PetscErrorCode  MatShellGetOperation(Mat mat,MatOperation op,void(**f)(void))
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   if (op == MATOP_DESTROY) {
     ierr = PetscTypeCompare((PetscObject)mat,MATSHELL,&flg);CHKERRQ(ierr);
     if (flg) {
