@@ -1,11 +1,10 @@
-#define PETSCMAT_DLL
 
 /*
      Mechanism for register PETSc matrix types
 */
-#include "private/matimpl.h"      /*I "petscmat.h" I*/
+#include <private/matimpl.h>      /*I "petscmat.h" I*/
 
-PetscTruth MatRegisterAllCalled = PETSC_FALSE;
+PetscBool  MatRegisterAllCalled = PETSC_FALSE;
 
 /*
    Contains the list of registered Mat routines
@@ -36,35 +35,47 @@ PetscFList MatList = 0;
 
 .seealso: PCSetType(), VecSetType(), MatCreate(), MatType, Mat
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatSetType(Mat mat, const MatType matype)
+PetscErrorCode  MatSetType(Mat mat, const MatType matype)
 {
   PetscErrorCode ierr,(*r)(Mat);
-  PetscTruth     sametype;
+  PetscBool      sametype,found;
+  MatBaseName    names = MatBaseNameList;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+
+  while (names) {
+    ierr = PetscStrcmp(matype,names->bname,&found);CHKERRQ(ierr);
+    if (found) {
+      PetscMPIInt size;
+      ierr = MPI_Comm_size(((PetscObject)mat)->comm,&size);CHKERRQ(ierr);
+      if (size == 1) matype = names->sname;
+      else matype = names->mname;
+      break;
+    }
+    names = names->next;
+  }
 
   ierr = PetscTypeCompare((PetscObject)mat,matype,&sametype);CHKERRQ(ierr);
   if (sametype) PetscFunctionReturn(0);
 
-  ierr =  PetscFListFind(MatList,((PetscObject)mat)->comm,matype,(void(**)(void))&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown Mat type given: %s",matype);
+  ierr =  PetscFListFind(MatList,((PetscObject)mat)->comm,matype,PETSC_TRUE,(void(**)(void))&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown Mat type given: %s",matype);
   
   /* free the old data structure if it existed */
   if (mat->ops->destroy) {
-    ierr = MatPreallocated(mat);CHKERRQ(ierr);
     ierr = (*mat->ops->destroy)(mat);CHKERRQ(ierr);
     mat->ops->destroy = PETSC_NULL;
-    mat->preallocated = PETSC_FALSE;
   }
+  mat->preallocated = PETSC_FALSE;
 
   /* create the new data structure */
-  if (mat->rmap->n < 0 && mat->rmap->N < 0 && mat->cmap->n < 0 && mat->cmap->N < 0) {
-    mat->ops->create = r;
-  } else {
-    ierr = (*r)(mat);CHKERRQ(ierr);
+  ierr = (*r)(mat);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_AMS)
+  if (PetscAMSPublishAll) {
+    /*    ierr = PetscObjectAMSPublish((PetscObject)mat);CHKERRQ(ierr); */
   }
-  ierr = PetscPublishAll(mat);CHKERRQ(ierr);
+#endif
   PetscFunctionReturn(0);
 }
 
@@ -83,7 +94,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatSetType(Mat mat, const MatType matype)
 
 .seealso: MatRegister(), MatRegisterAll(), MatRegisterDynamic()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatRegisterDestroy(void)
+PetscErrorCode  MatRegisterDestroy(void)
 {
   PetscErrorCode ierr;
 
@@ -112,10 +123,10 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatRegisterDestroy(void)
 
 .seealso: MatSetType()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatGetType(Mat mat,const MatType *type)
+PetscErrorCode  MatGetType(Mat mat,const MatType *type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   PetscValidPointer(type,2);
   *type = ((PetscObject)mat)->type_name;
   PetscFunctionReturn(0);
@@ -129,7 +140,7 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatGetType(Mat mat,const MatType *type)
 
   Level: advanced
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatRegister(const char sname[],const char path[],const char name[],PetscErrorCode (*function)(Mat))
+PetscErrorCode  MatRegister(const char sname[],const char path[],const char name[],PetscErrorCode (*function)(Mat))
 {
   PetscErrorCode ierr;
   char           fullname[PETSC_MAX_PATH_LEN];
@@ -140,6 +151,41 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatRegister(const char sname[],const char path
   PetscFunctionReturn(0);
 }
 
+MatBaseName MatBaseNameList = 0;
+
+#undef __FUNCT__  
+#define __FUNCT__ "MatRegisterBaseName"
+/*@C
+      MatRegisterBaseName - Registers a name that can be used for either a sequential or its corresponding parallel matrix type.
+
+  Input Parameters:
++     bname - the basename, for example, MATAIJ
+.     sname - the name of the sequential matrix type, for example, MATSEQAIJ
+-     mname - the name of the parallel matrix type, for example, MATMPIAIJ
+
+
+  Level: advanced
+@*/
+PetscErrorCode  MatRegisterBaseName(const char bname[],const char sname[],const char mname[])
+{
+  PetscErrorCode ierr;
+  MatBaseName    names;
+
+  PetscFunctionBegin;
+  ierr = PetscNew(struct _p_MatBaseName,&names);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(bname,&names->bname);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(sname,&names->sname);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(mname,&names->mname);CHKERRQ(ierr);
+  if (!MatBaseNameList) {
+    MatBaseNameList = names;
+  } else {
+    MatBaseName next = MatBaseNameList;
+    while (next->next) next = next->next;
+    next->next = names;
+  }
+  PetscFunctionReturn(0);
+}
+
 
 
 
