@@ -1,8 +1,7 @@
-#define PETSCMAT_DLL 
 
 /* qmdmrg.f -- translated by f2c (version 19931217).*/
 
-#include "petscsys.h"
+#include <petscsys.h>
 
 /******************************************************************/
 /***********     QMDMRG ..... QUOT MIN DEG MERGE       ************/
