@@ -1,8 +1,7 @@
-#define PETSCMAT_DLL
 
 /* rootls.f -- translated by f2c (version 19931217).*/
 
-#include "petsc.h"
+#include <petscsys.h>
 
 /*****************************************************************/
 /*********     ROOTLS ..... ROOTED LEVEL STRUCTURE      **********/
