@@ -1,5 +1,5 @@
-#include "private/fortranimpl.h"
-#include "petscmat.h"
+#include <private/fortranimpl.h>
+#include <petscmat.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define matpartitioningpartysetglobal_   MATPARTITIONINGPARTYSETGLOBAL
