@@ -1,8 +1,7 @@
-#define PETSCMAT_DLL
 
-#include "include/private/matimpl.h"       /*I "petscmat.h"  I*/
-#include "petscsys.h"
+#include <private/matimpl.h>       /*I "petscmat.h"  I*/
 
+#if 0
 #undef __FUNCT__  
 #define __FUNCT__ "MatPublish_Base"
 static PetscErrorCode MatPublish_Base(PetscObject obj)
@@ -10,7 +9,7 @@ static PetscErrorCode MatPublish_Base(PetscObject obj)
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
-
+#endif
 
 #undef __FUNCT__  
 #define __FUNCT__ "MatCreate"
@@ -34,9 +33,6 @@ static PetscErrorCode MatPublish_Base(PetscObject obj)
    Options Database Keys:
 +    -mat_type seqaij   - AIJ type, uses MatCreateSeqAIJ()
 .    -mat_type mpiaij   - AIJ type, uses MatCreateMPIAIJ()
-.    -mat_type seqbdiag - block diagonal type, uses MatCreateSeqBDiag()
-.    -mat_type mpibdiag - block diagonal type, uses MatCreateMPIBDiag()
-.    -mat_type mpirowbs - rowbs type, uses MatCreateMPIRowbs()
 .    -mat_type seqdense - dense type, uses MatCreateSeqDense()
 .    -mat_type mpidense - dense type, uses MatCreateMPIDense()
 .    -mat_type seqbaij  - block AIJ type, uses MatCreateSeqBAIJ()
@@ -57,13 +53,12 @@ static PetscErrorCode MatPublish_Base(PetscObject obj)
 .keywords: matrix, create
 
 .seealso: MatCreateSeqAIJ(), MatCreateMPIAIJ(), 
-          MatCreateSeqBDiag(),MatCreateMPIBDiag(),
           MatCreateSeqDense(), MatCreateMPIDense(), 
-          MatCreateMPIRowbs(), MatCreateSeqBAIJ(), MatCreateMPIBAIJ(),
+          MatCreateSeqBAIJ(), MatCreateMPIBAIJ(),
           MatCreateSeqSBAIJ(), MatCreateMPISBAIJ(),
           MatConvert()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatCreate(MPI_Comm comm,Mat *A)
+PetscErrorCode  MatCreate(MPI_Comm comm,Mat *A)
 {
   Mat            B;
   PetscErrorCode ierr;
@@ -76,11 +71,10 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatCreate(MPI_Comm comm,Mat *A)
   ierr = MatInitializePackage(PETSC_NULL);CHKERRQ(ierr);
 #endif
 
-  ierr = PetscHeaderCreate(B,_p_Mat,struct _MatOps,MAT_COOKIE,0,"Mat",comm,MatDestroy,MatView);CHKERRQ(ierr);
-  ierr = PetscMapInitialize(comm,&B->rmap);CHKERRQ(ierr);
-  ierr = PetscMapInitialize(comm,&B->cmap);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(B,_p_Mat,struct _MatOps,MAT_CLASSID,0,"Mat","Matrix","Mat",comm,MatDestroy,MatView);CHKERRQ(ierr);
+  ierr = PetscLayoutCreate(comm,&B->rmap);CHKERRQ(ierr);
+  ierr = PetscLayoutCreate(comm,&B->cmap);CHKERRQ(ierr);
   B->preallocated  = PETSC_FALSE;
-  B->bops->publish = MatPublish_Base;
   *A               = B;
   PetscFunctionReturn(0);
 }
@@ -115,26 +109,27 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatCreate(MPI_Comm comm,Mat *A)
 
 .seealso: MatGetSize(), PetscSplitOwnership()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatSetSizes(Mat A, PetscInt m, PetscInt n, PetscInt M, PetscInt N)
+PetscErrorCode  MatSetSizes(Mat A, PetscInt m, PetscInt n, PetscInt M, PetscInt N)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(A,MAT_COOKIE,1); 
-  if (M > 0 && m > M) SETERRQ2(PETSC_ERR_ARG_INCOMP,"Local column size %D cannot be larger than global column size %D",m,M);
-  if (N > 0 && n > N) SETERRQ2(PETSC_ERR_ARG_INCOMP,"Local row size %D cannot be larger than global row size %D",n,N);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,1); 
+  if (M > 0 && m > M) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Local column size %D cannot be larger than global column size %D",m,M);
+  if (N > 0 && n > N) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Local row size %D cannot be larger than global row size %D",n,N);
   if (A->ops->setsizes) {
     /* Since this will not be set until the type has been set, this will NOT be called on the initial
        call of MatSetSizes() (which must be called BEFORE MatSetType() */
     ierr = (*A->ops->setsizes)(A,m,n,M,N);CHKERRQ(ierr);
   } else {
-    if ((A->rmap.n >= 0 || A->rmap.N >= 0) && (A->rmap.n != m || A->rmap.N != M)) SETERRQ4(PETSC_ERR_SUP,"Cannot change/reset row sizes to %D local %D global after previously setting them to %D local %D global",m,M,A->rmap.n,A->rmap.N);
-    if ((A->cmap.n >= 0 || A->cmap.N >= 0) && (A->cmap.n != n || A->cmap.N != N)) SETERRQ4(PETSC_ERR_SUP,"Cannot change/reset column sizes to %D local %D global after previously setting them to %D local %D global",n,N,A->cmap.n,A->cmap.N);
+    if ((A->rmap->n >= 0 || A->rmap->N >= 0) && (A->rmap->n != m || A->rmap->N != M)) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change/reset row sizes to %D local %D global after previously setting them to %D local %D global",m,M,A->rmap->n,A->rmap->N);
+    if ((A->cmap->n >= 0 || A->cmap->N >= 0) && (A->cmap->n != n || A->cmap->N != N)) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change/reset column sizes to %D local %D global after previously setting them to %D local %D global",n,N,A->cmap->n,A->cmap->N);
   }
-  A->rmap.n = m;
-  A->cmap.n = n;
-  A->rmap.N = M;
-  A->cmap.N = N;
+  A->rmap->n = m;
+  A->cmap->n = n;
+  A->rmap->N = M;
+  A->cmap->N = N;
+
   PetscFunctionReturn(0);
 }
 
@@ -155,9 +150,6 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatSetSizes(Mat A, PetscInt m, PetscInt n, Pet
    Options Database Keys:
 +    -mat_type seqaij   - AIJ type, uses MatCreateSeqAIJ()
 .    -mat_type mpiaij   - AIJ type, uses MatCreateMPIAIJ()
-.    -mat_type seqbdiag - block diagonal type, uses MatCreateSeqBDiag()
-.    -mat_type mpibdiag - block diagonal type, uses MatCreateMPIBDiag()
-.    -mat_type mpirowbs - rowbs type, uses MatCreateMPIRowbs()
 .    -mat_type seqdense - dense type, uses MatCreateSeqDense()
 .    -mat_type mpidense - dense type, uses MatCreateMPIDense()
 .    -mat_type seqbaij  - block AIJ type, uses MatCreateSeqBAIJ()
@@ -172,51 +164,70 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatSetSizes(Mat A, PetscInt m, PetscInt n, Pet
 .keywords: matrix, create
 
 .seealso: MatCreateSeqAIJ((), MatCreateMPIAIJ(), 
-          MatCreateSeqBDiag(),MatCreateMPIBDiag(),
           MatCreateSeqDense(), MatCreateMPIDense(), 
-          MatCreateMPIRowbs(), MatCreateSeqBAIJ(), MatCreateMPIBAIJ(),
+          MatCreateSeqBAIJ(), MatCreateMPIBAIJ(),
           MatCreateSeqSBAIJ(), MatCreateMPISBAIJ(),
           MatConvert()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatSetFromOptions(Mat B)
+PetscErrorCode  MatSetFromOptions(Mat B)
 {
   PetscErrorCode ierr;
-  char           mtype[256];
-  PetscTruth     flg;
+  const char     *deft = MATAIJ;
+  char           type[256];
+  PetscBool      flg,set;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetString(B->prefix,"-mat_type",mtype,256,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = MatSetType(B,mtype);CHKERRQ(ierr);
-  }
-  if (!B->type_name) {
-    ierr = MatSetType(B,MATAIJ);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(B,MAT_CLASSID,1);
+
+  ierr = PetscObjectOptionsBegin((PetscObject)B);CHKERRQ(ierr);
+    ierr = PetscOptionsList("-mat_type","Matrix type","MatSetType",MatList,deft,type,256,&flg);CHKERRQ(ierr);
+    if (flg) {
+      ierr = MatSetType(B,type);CHKERRQ(ierr);
+    } else if (!((PetscObject)B)->type_name) {
+      ierr = MatSetType(B,deft);CHKERRQ(ierr);
+    }
+
+    if (B->ops->setfromoptions) {
+      ierr = (*B->ops->setfromoptions)(B);CHKERRQ(ierr);
+    }
+
+    flg = PETSC_FALSE;
+    ierr = PetscOptionsBool("-mat_new_nonzero_location_err","Generate an error if new nonzeros are created in the matrix structure (useful to test preallocation)","MatSetOption",flg,&flg,&set);CHKERRQ(ierr);
+    if (set) {ierr = MatSetOption(B,MAT_NEW_NONZERO_LOCATION_ERR,flg);CHKERRQ(ierr);}
+    flg = PETSC_FALSE;
+    ierr = PetscOptionsBool("-mat_new_nonzero_allocation_err","Generate an error if new nonzeros are allocated in the matrix structure (useful to test preallocation)","MatSetOption",flg,&flg,&set);CHKERRQ(ierr);
+    if (set) {ierr = MatSetOption(B,MAT_NEW_NONZERO_ALLOCATION_ERR,flg);CHKERRQ(ierr);}
+
+    /* process any options handlers added with PetscObjectAddOptionsHandler() */
+    ierr = PetscObjectProcessOptionsHandlers((PetscObject)B);CHKERRQ(ierr);
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "MatSetUpPreallocation"
-/*@C
-   MatSetUpPreallocation
+/*@
+   MatSetUpPreallocation - If the user has not set preallocation for this matrix then a default preallocation that is likely to be inefficient is used.
 
    Collective on Mat
 
    Input Parameter:
 .  A - the matrix
 
-   Level: beginner
+   Level: advanced
+
+   Notes: See the Performance chapter of the PETSc users manual for how to preallocate matrices
 
 .keywords: matrix, create
 
 .seealso: MatCreateSeqAIJ((), MatCreateMPIAIJ(), 
-          MatCreateSeqBDiag(),MatCreateMPIBDiag(),
           MatCreateSeqDense(), MatCreateMPIDense(), 
-          MatCreateMPIRowbs(), MatCreateSeqBAIJ(), MatCreateMPIBAIJ(),
+          MatCreateSeqBAIJ(), MatCreateMPIBAIJ(),
           MatCreateSeqSBAIJ(), MatCreateMPISBAIJ(),
           MatConvert()
 @*/
-PetscErrorCode PETSCMAT_DLLEXPORT MatSetUpPreallocation(Mat B)
+PetscErrorCode  MatSetUpPreallocation(Mat B)
 {
   PetscErrorCode ierr;
 
@@ -230,74 +241,95 @@ PetscErrorCode PETSCMAT_DLLEXPORT MatSetUpPreallocation(Mat B)
 }
 
 /*
-        Copies from Cs header to A
+        Merges some information from Cs header to A; the C object is then destroyed
+
+        This is somewhat different from MatHeaderReplace() it would be nice to merge the code
 */
 #undef __FUNCT__  
-#define __FUNCT__ "MatHeaderCopy"
-PetscErrorCode MatHeaderCopy(Mat A,Mat C)
+#define __FUNCT__ "MatHeaderMerge"
+PetscErrorCode MatHeaderMerge(Mat A,Mat C)
 {
-  PetscErrorCode ierr;
-  PetscInt       refct;
-  PetscOps       *Abops;
-  MatOps         Aops;
-  char           *mtype,*mname;
-  void           *spptr;
+  PetscErrorCode         ierr;
+  PetscInt               refct;
+  PetscOps               *Abops;
+  MatOps                 Aops;
+  char                   *mtype,*mname;
+  void                   *spptr;
 
   PetscFunctionBegin;
   /* save the parts of A we need */
-  Abops = A->bops;
+  Abops = ((PetscObject)A)->bops;
   Aops  = A->ops;
-  refct = A->refct;
-  mtype = A->type_name; A->type_name = 0;
-  mname = A->name; A->name = 0;
+  refct = ((PetscObject)A)->refct;
+  mtype = ((PetscObject)A)->type_name;
+  mname = ((PetscObject)A)->name;
   spptr = A->spptr;
 
+  /* zero these so the destroy below does not free them */
+  ((PetscObject)A)->type_name = 0;
+  ((PetscObject)A)->name      = 0;
+
   /* free all the interior data structures from mat */
   ierr = (*A->ops->destroy)(A);CHKERRQ(ierr);
 
   ierr = PetscFree(C->spptr);CHKERRQ(ierr);
 
-  ierr = PetscFree(A->rmap.range);CHKERRQ(ierr);
-  ierr = PetscFree(A->cmap.range);CHKERRQ(ierr);
+  ierr = PetscLayoutDestroy(&A->rmap);CHKERRQ(ierr);
+  ierr = PetscLayoutDestroy(&A->cmap);CHKERRQ(ierr);
+  ierr = PetscFListDestroy(&((PetscObject)A)->qlist);CHKERRQ(ierr);
+  ierr = PetscOListDestroy(&((PetscObject)A)->olist);CHKERRQ(ierr);
 
   /* copy C over to A */
   ierr  = PetscMemcpy(A,C,sizeof(struct _p_Mat));CHKERRQ(ierr);
 
   /* return the parts of A we saved */
-  A->bops      = Abops;
-  A->ops       = Aops;
-  A->qlist     = 0;
-  A->refct     = refct;
-  A->type_name = mtype;
-  A->name      = mname;
-  A->spptr     = spptr;
+  ((PetscObject)A)->bops      = Abops;
+  A->ops                      = Aops;
+  ((PetscObject)A)->refct     = refct;
+  ((PetscObject)A)->type_name = mtype;
+  ((PetscObject)A)->name      = mname;
+  A->spptr                    = spptr;
 
-  ierr = PetscHeaderDestroy(C);CHKERRQ(ierr);
+  /* since these two are copied into A we do not want them destroyed in C */
+  ((PetscObject)C)->qlist = 0;
+  ((PetscObject)C)->olist = 0;
+  ierr = PetscHeaderDestroy(&C);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 /*
-        Replace A's header with that of C
-        This is essentially code moved from MatDestroy
+        Replace A's header with that of C; the C object is then destroyed
+
+        This is essentially code moved from MatDestroy()
+
+        This is somewhat different from MatHeaderMerge() it would be nice to merge the code
 */
 #undef __FUNCT__  
 #define __FUNCT__ "MatHeaderReplace"
 PetscErrorCode MatHeaderReplace(Mat A,Mat C)
 {
   PetscErrorCode ierr;
+  PetscInt refct;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
+  PetscValidHeaderSpecific(C,MAT_CLASSID,2);
+  if (A == C) PetscFunctionReturn(0);
+  PetscCheckSameComm(A,1,C,2);
+  if (((PetscObject)C)->refct != 1) SETERRQ1(((PetscObject)C)->comm,PETSC_ERR_ARG_WRONGSTATE,"Object C has refct %D > 1, would leave hanging reference",((PetscObject)C)->refct);
+
   /* free all the interior data structures from mat */
   ierr = (*A->ops->destroy)(A);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy_Private((PetscObject)A);CHKERRQ(ierr);
-  ierr = PetscFree(A->rmap.range);CHKERRQ(ierr);
-  ierr = PetscFree(A->cmap.range);CHKERRQ(ierr);
+  ierr = PetscFree(A->ops);CHKERRQ(ierr);
+  ierr = PetscLayoutDestroy(&A->rmap);CHKERRQ(ierr);
+  ierr = PetscLayoutDestroy(&A->cmap);CHKERRQ(ierr);
   ierr = PetscFree(A->spptr);CHKERRQ(ierr);
-  
+
   /* copy C over to A */
-  if (C) {
-    ierr = PetscMemcpy(A,C,sizeof(struct _p_Mat));CHKERRQ(ierr);
-    ierr = PetscLogObjectDestroy((PetscObject)C);CHKERRQ(ierr);
-    ierr = PetscFree(C);CHKERRQ(ierr);
-  }
+  refct = ((PetscObject)A)->refct;
+  ierr = PetscMemcpy(A,C,sizeof(struct _p_Mat));CHKERRQ(ierr);
+  ((PetscObject)A)->refct = refct;
+  ierr = PetscLogObjectDestroy((PetscObject)C);CHKERRQ(ierr);
+  ierr = PetscFree(C);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
