@@ -1,6 +1,5 @@
-#define PETSCMAT_DLL
 
-#include "private/matimpl.h"
+#include <private/matimpl.h>
 
 #define DEFAULT_STASH_SIZE   10000
 
@@ -23,8 +22,8 @@
 PetscErrorCode MatStashCreate_Private(MPI_Comm comm,PetscInt bs,MatStash *stash)
 {
   PetscErrorCode ierr;
-  PetscInt       max,*opt,nopt;
-  PetscTruth     flg;
+  PetscInt       max,*opt,nopt,i;
+  PetscBool      flg;
 
   PetscFunctionBegin;
   /* Require 2 tags,get the second using PetscCommGetNewTag() */
@@ -33,6 +32,9 @@ PetscErrorCode MatStashCreate_Private(MPI_Comm comm,PetscInt bs,MatStash *stash)
   ierr = PetscCommGetNewTag(stash->comm,&stash->tag2);CHKERRQ(ierr);
   ierr = MPI_Comm_size(stash->comm,&stash->size);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(stash->comm,&stash->rank);CHKERRQ(ierr);
+  ierr  = PetscMalloc(2*stash->size*sizeof(PetscMPIInt),&stash->flg_v);CHKERRQ(ierr);
+  for (i=0; i<2*stash->size; i++) stash->flg_v[i] = -1;
+
 
   nopt = stash->size;
   ierr = PetscMalloc(nopt*sizeof(PetscInt),&opt);CHKERRQ(ierr);
@@ -65,8 +67,10 @@ PetscErrorCode MatStashCreate_Private(MPI_Comm comm,PetscInt bs,MatStash *stash)
   stash->svalues     = 0;
   stash->rvalues     = 0;
   stash->rindices    = 0;
-  stash->nprocs      = 0;
   stash->nprocessed  = 0;
+
+  stash->reproduce   = PETSC_FALSE;
+  ierr = PetscOptionsGetBool(PETSC_NULL,"-matstash_reproduce",&stash->reproduce,PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -80,18 +84,16 @@ PetscErrorCode MatStashDestroy_Private(MatStash *stash)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (stash->space_head){
-    ierr = PetscMatStashSpaceDestroy(stash->space_head);CHKERRQ(ierr);
-    stash->space_head = 0;
-    stash->space      = 0;
-  }
+  ierr = PetscMatStashSpaceDestroy(&stash->space_head);CHKERRQ(ierr);
+  stash->space = 0;
+  ierr = PetscFree(stash->flg_v);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /* 
-   MatStashScatterEnd_Private - This is called as the fial stage of
-   scatter. The final stages of messagepassing is done here, and
-   all the memory used for messagepassing is cleanedu up. This
+   MatStashScatterEnd_Private - This is called as the final stage of
+   scatter. The final stages of message passing is done here, and
+   all the memory used for message passing is cleaned up. This
    routine also resets the stash, and deallocates the memory used
    for the stash. It also keeps track of the current memory usage
    so that the same value can be used the next time through.
@@ -101,10 +103,11 @@ PetscErrorCode MatStashDestroy_Private(MatStash *stash)
 PetscErrorCode MatStashScatterEnd_Private(MatStash *stash)
 { 
   PetscErrorCode ierr;
-  PetscInt       nsends=stash->nsends,bs2,oldnmax;
+  PetscInt       nsends=stash->nsends,bs2,oldnmax,i;
   MPI_Status     *send_status;
 
   PetscFunctionBegin;
+  for (i=0; i<2*stash->size; i++) stash->flg_v[i] = -1;
   /* wait on sends */
   if (nsends) {
     ierr = PetscMalloc(2*nsends*sizeof(MPI_Status),&send_status);CHKERRQ(ierr);
@@ -125,23 +128,15 @@ PetscErrorCode MatStashScatterEnd_Private(MatStash *stash)
   stash->n          = 0;
   stash->reallocs   = -1;
   stash->nprocessed = 0;
-  if (stash->space_head){
-    ierr = PetscMatStashSpaceDestroy(stash->space_head);CHKERRQ(ierr);
-    stash->space_head = 0;
-    stash->space      = 0;
-  }
+  ierr = PetscMatStashSpaceDestroy(&stash->space_head);CHKERRQ(ierr);
+  stash->space      = 0;
   ierr = PetscFree(stash->send_waits);CHKERRQ(ierr);
-  stash->send_waits = 0;
   ierr = PetscFree(stash->recv_waits);CHKERRQ(ierr);
-  stash->recv_waits = 0;
-  ierr = PetscFree(stash->svalues);CHKERRQ(ierr);
-  stash->svalues = 0;
+  ierr = PetscFree2(stash->svalues,stash->sindices);CHKERRQ(ierr);
+  ierr = PetscFree(stash->rvalues[0]);CHKERRQ(ierr);
   ierr = PetscFree(stash->rvalues);CHKERRQ(ierr);
-  stash->rvalues = 0;
+  ierr = PetscFree(stash->rindices[0]);CHKERRQ(ierr);
   ierr = PetscFree(stash->rindices);CHKERRQ(ierr);
-  stash->rindices = 0;
-  ierr = PetscFree(stash->nprocs);CHKERRQ(ierr);
-  stash->nprocs = 0;
   PetscFunctionReturn(0);
 }
 
@@ -239,10 +234,10 @@ static PetscErrorCode MatStashExpand_Private(MatStash *stash,PetscInt incr)
 */
 #undef __FUNCT__
 #define __FUNCT__ "MatStashValuesRow_Private"
-PetscErrorCode MatStashValuesRow_Private(MatStash *stash,PetscInt row,PetscInt n,const PetscInt idxn[],const PetscScalar values[])
+PetscErrorCode MatStashValuesRow_Private(MatStash *stash,PetscInt row,PetscInt n,const PetscInt idxn[],const PetscScalar values[],PetscBool  ignorezeroentries)
 {
   PetscErrorCode     ierr;
-  PetscInt           i,k;
+  PetscInt           i,k,cnt = 0;
   PetscMatStashSpace space=stash->space;
 
   PetscFunctionBegin;
@@ -253,14 +248,16 @@ PetscErrorCode MatStashValuesRow_Private(MatStash *stash,PetscInt row,PetscInt n
   space = stash->space;
   k     = space->local_used;
   for (i=0; i<n; i++) {
+    if (ignorezeroentries && (values[i] == 0.0)) continue;
     space->idx[k] = row;
     space->idy[k] = idxn[i];
     space->val[k] = values[i];
     k++;
+    cnt++;
   }
-  stash->n               += n; 
-  space->local_used      += n;
-  space->local_remaining -= n;
+  stash->n               += cnt; 
+  space->local_used      += cnt;
+  space->local_remaining -= cnt;
   PetscFunctionReturn(0);
 }
 
@@ -280,10 +277,10 @@ PetscErrorCode MatStashValuesRow_Private(MatStash *stash,PetscInt row,PetscInt n
 */
 #undef __FUNCT__  
 #define __FUNCT__ "MatStashValuesCol_Private"
-PetscErrorCode MatStashValuesCol_Private(MatStash *stash,PetscInt row,PetscInt n,const PetscInt idxn[],const PetscScalar values[],PetscInt stepval)
+PetscErrorCode MatStashValuesCol_Private(MatStash *stash,PetscInt row,PetscInt n,const PetscInt idxn[],const PetscScalar values[],PetscInt stepval,PetscBool  ignorezeroentries)
 {
   PetscErrorCode     ierr;
-  PetscInt           i,k; 
+  PetscInt           i,k,cnt = 0; 
   PetscMatStashSpace space=stash->space;
 
   PetscFunctionBegin;
@@ -294,14 +291,16 @@ PetscErrorCode MatStashValuesCol_Private(MatStash *stash,PetscInt row,PetscInt n
   space = stash->space;
   k = space->local_used;
   for (i=0; i<n; i++) {
+    if (ignorezeroentries && (values[i*stepval] == 0.0)) continue;
     space->idx[k] = row;
     space->idy[k] = idxn[i]; 
     space->val[k] = values[i*stepval]; 
     k++;
+    cnt++;
   }
-  stash->n               += n; 
-  space->local_used      += n;
-  space->local_remaining -= n;
+  stash->n               += cnt; 
+  space->local_used      += cnt;
+  space->local_remaining -= cnt;
   PetscFunctionReturn(0);
 }
 
@@ -451,11 +450,12 @@ PetscErrorCode MatStashScatterBegin_Private(Mat mat,MatStash *stash,PetscInt *ow
   bs2 = stash->bs*stash->bs;
   
   /*  first count number of contributors to each processor */
-  ierr  = PetscMalloc(2*size*sizeof(PetscMPIInt),&nprocs);CHKERRQ(ierr);
-  ierr  = PetscMemzero(nprocs,2*size*sizeof(PetscMPIInt));CHKERRQ(ierr);
+  ierr  = PetscMalloc(size*sizeof(PetscMPIInt),&nprocs);CHKERRQ(ierr);
+  ierr  = PetscMemzero(nprocs,size*sizeof(PetscMPIInt));CHKERRQ(ierr);
+  ierr  = PetscMalloc(size*sizeof(PetscMPIInt),&nlengths);CHKERRQ(ierr);
+  ierr  = PetscMemzero(nlengths,size*sizeof(PetscMPIInt));CHKERRQ(ierr);
   ierr  = PetscMalloc((stash->n+1)*sizeof(PetscInt),&owner);CHKERRQ(ierr);
 
-  nlengths = nprocs+size;
   i = j    = 0;
   lastidx  = -1; 
   space    = stash->space_head;
@@ -498,16 +498,14 @@ PetscErrorCode MatStashScatterBegin_Private(Mat mat,MatStash *stash,PetscInt *ow
       1) starts[i] gives the starting index in svalues for stuff going to 
          the ith processor
   */
-  ierr     = PetscMalloc((stash->n+1)*(bs2*sizeof(PetscScalar)+2*sizeof(PetscInt)),&svalues);CHKERRQ(ierr);
-  sindices = (PetscInt*)(svalues + bs2*stash->n);
-  ierr     = PetscMalloc(2*(nsends+1)*sizeof(MPI_Request),&send_waits);CHKERRQ(ierr);
-  ierr     = PetscMalloc(2*size*sizeof(PetscInt),&startv);CHKERRQ(ierr);
-  starti   = startv + size;
+  ierr     = PetscMalloc2(bs2*stash->n,PetscScalar,&svalues,2*(stash->n+1),PetscInt,&sindices);CHKERRQ(ierr);
+  ierr     = PetscMalloc(2*nsends*sizeof(MPI_Request),&send_waits);CHKERRQ(ierr);
+  ierr     = PetscMalloc2(size,PetscInt,&startv,size,PetscInt,&starti);CHKERRQ(ierr);
   /* use 2 sends the first with all_a, the next with all_i and all_j */
   startv[0]  = 0; starti[0] = 0;
   for (i=1; i<size; i++) { 
     startv[i] = startv[i-1] + nlengths[i-1];
-    starti[i] = starti[i-1] + nlengths[i-1]*2;
+    starti[i] = starti[i-1] + 2*nlengths[i-1];
   } 
   
   i     = 0;
@@ -553,14 +551,13 @@ PetscErrorCode MatStashScatterBegin_Private(Mat mat,MatStash *stash,PetscInt *ow
     }
   }
 #endif
+  ierr = PetscFree(nlengths);CHKERRQ(ierr);
   ierr = PetscFree(owner);CHKERRQ(ierr);
-  ierr = PetscFree(startv);CHKERRQ(ierr);
-  /* This memory is reused in scatter end  for a different purpose*/
-  for (i=0; i<2*size; i++) nprocs[i] = -1;
-  stash->nprocs = nprocs;
+  ierr = PetscFree2(startv,starti);CHKERRQ(ierr);
+  ierr = PetscFree(nprocs);CHKERRQ(ierr);
   
   /* recv_waits need to be contiguous for MatStashScatterGetMesg_Private() */
-  ierr  = PetscMalloc((nreceives+1)*2*sizeof(MPI_Request),&recv_waits);CHKERRQ(ierr);
+  ierr  = PetscMalloc(2*nreceives*sizeof(MPI_Request),&recv_waits);CHKERRQ(ierr);
 
   for (i=0; i<nreceives; i++) { 
     recv_waits[2*i]   = recv_waits1[i];
@@ -570,9 +567,14 @@ PetscErrorCode MatStashScatterBegin_Private(Mat mat,MatStash *stash,PetscInt *ow
   ierr = PetscFree(recv_waits1);CHKERRQ(ierr);
   ierr = PetscFree(recv_waits2);CHKERRQ(ierr);
 
-  stash->svalues    = svalues;    stash->rvalues     = rvalues;
-  stash->rindices   = rindices;   stash->send_waits  = send_waits;
-  stash->nsends     = nsends;     stash->nrecvs      = nreceives;
+  stash->svalues     = svalues;
+  stash->sindices    = sindices;
+  stash->rvalues     = rvalues;
+  stash->rindices    = rindices; 
+  stash->send_waits  = send_waits;
+  stash->nsends      = nsends;  
+  stash->nrecvs      = nreceives;
+  stash->reproduce_count = 0;
   PetscFunctionReturn(0);
 } 
 
@@ -599,10 +601,10 @@ PetscErrorCode MatStashScatterBegin_Private(Mat mat,MatStash *stash,PetscInt *ow
 PetscErrorCode MatStashScatterGetMesg_Private(MatStash *stash,PetscMPIInt *nvals,PetscInt **rows,PetscInt** cols,PetscScalar **vals,PetscInt *flg)
 {
   PetscErrorCode ierr;
-  PetscMPIInt    i,*flg_v,i1,i2;
+  PetscMPIInt    i,*flg_v = stash->flg_v,i1,i2;
   PetscInt       bs2;
   MPI_Status     recv_status;
-  PetscTruth     match_found = PETSC_FALSE;
+  PetscBool      match_found = PETSC_FALSE;
 
   PetscFunctionBegin;
 
@@ -610,13 +612,21 @@ PetscErrorCode MatStashScatterGetMesg_Private(MatStash *stash,PetscMPIInt *nvals
   /* Return if no more messages to process */
   if (stash->nprocessed == stash->nrecvs) { PetscFunctionReturn(0); } 
 
-  flg_v = stash->nprocs;
   bs2   = stash->bs*stash->bs;
-  /* If a matching pair of receieves are found, process them, and return the data to
+  /* If a matching pair of receives are found, process them, and return the data to
      the calling function. Until then keep receiving messages */
   while (!match_found) {
-    ierr = MPI_Waitany(2*stash->nrecvs,stash->recv_waits,&i,&recv_status);CHKERRQ(ierr);
-    /* Now pack the received message into a structure which is useable by others */
+    CHKMEMQ;
+    if (stash->reproduce) {
+      i = stash->reproduce_count++;
+      ierr = MPI_Wait(stash->recv_waits+i,&recv_status);CHKERRQ(ierr);
+    } else {
+      ierr = MPI_Waitany(2*stash->nrecvs,stash->recv_waits,&i,&recv_status);CHKERRQ(ierr);
+    }
+    CHKMEMQ;
+    if (recv_status.MPI_SOURCE < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Negative MPI source!");
+
+    /* Now pack the received message into a structure which is usable by others */
     if (i % 2) { 
       ierr = MPI_Get_count(&recv_status,MPIU_SCALAR,nvals);CHKERRQ(ierr);
       flg_v[2*recv_status.MPI_SOURCE] = i/2; 
