@@ -50,9 +50,9 @@ static char help[] =
 
   ------------------------------------------------------------------------- */
 
-#include "petscsnes.h"
-#include "petscda.h"
-#include "petscdmmg.h"
+#include <petscsnes.h>
+#include <petscdmda.h>
+#include <petscdmmg.h>
 
 #define VISC_CONST   0
 #define VISC_DIFN    1
@@ -84,14 +84,14 @@ typedef struct { /* physical and miscelaneous parameters */
   PetscReal    L, V, lid_depth, fault_depth;
   ViscParam    diffusion, dislocation;
   PetscInt     ivisc, adv_scheme, ibound, output_ivisc;
-  PetscTruth   quiet, param_test, output_to_file, pv_analytic;
-  PetscTruth   interrupted, stop_solve, toggle_kspmon, kspmon;
+  PetscBool    quiet, param_test, output_to_file, pv_analytic;
+  PetscBool    interrupted, stop_solve, toggle_kspmon, kspmon;
   char         filename[PETSC_MAX_PATH_LEN];
 } Parameter;
 
 typedef struct { /* grid parameters */
-  DAPeriodicType periodic;
-  DAStencilType  stencil;
+  DMDABoundaryType bx,by;
+  DMDAStencilType  stencil;
   PetscInt       corner,ni,nj,jlid,jfault,inose;
   PetscInt       dof,stencil_width,mglevels;
   PassiveScalar  dx,dz;
@@ -105,7 +105,7 @@ typedef struct { /* application context */
 
 /* Callback functions (static interface) */
 extern PetscErrorCode FormInitialGuess(DMMG,Vec);
-extern PetscErrorCode FormFunctionLocal(DALocalInfo*,Field**,Field**,void*);
+extern PetscErrorCode FormFunctionLocal(DMDALocalInfo*,Field**,Field**,void*);
 
 /* Main routines */
 extern PetscErrorCode SetParams(Parameter *param, GridInfo *grid);
@@ -141,7 +141,7 @@ extern PetscErrorCode ViscosityField(DMMG,Vec,Vec);
 extern PetscErrorCode StressField(DMMG *dmmg);
 extern PetscErrorCode SNESConverged_Interactive(SNES, PetscInt, PetscReal, PetscReal, PetscReal, SNESConvergedReason *, void *);
 extern PetscErrorCode InteractiveHandler(int, void *);
-extern PetscTruth OptionsHasName(const char pre[],const char name[]);
+extern PetscBool  OptionsHasName(const char pre[],const char name[]);
 
 /*-----------------------------------------------------------------------*/
 #undef __FUNCT__
@@ -156,7 +156,7 @@ int main(int argc,char **argv)
   PetscInt       nits;
   PetscErrorCode ierr;
   MPI_Comm       comm;
-  DA             da;
+  DM             da;
 
   PetscInitialize(&argc,&argv,(char *)0,help);
   PetscOptionsSetValue("-file","ex30_output");
@@ -178,7 +178,7 @@ int main(int argc,char **argv)
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create user context, set problem data, create vector data structures.
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */   
-  ierr = PetscMalloc(sizeof(AppCtx),&user); CHKERRQ(ierr);
+  ierr = PetscMalloc(sizeof(AppCtx),&user);CHKERRQ(ierr);
   user->param = &param;
   user->grid  = &grid;
 
@@ -187,50 +187,51 @@ int main(int argc,char **argv)
      for principal unknowns (x) and governing residuals (f)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
   ierr = DMMGCreate(comm,grid.mglevels,user,&dmmg);CHKERRQ(ierr); 
-  ierr = DACreate2d(comm,grid.periodic,grid.stencil,grid.ni,grid.nj,PETSC_DECIDE,PETSC_DECIDE,grid.dof,grid.stencil_width,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(comm,grid.bx,grid.by,grid.stencil,grid.ni,grid.nj,PETSC_DECIDE,PETSC_DECIDE,grid.dof,grid.stencil_width,0,0,&da);CHKERRQ(ierr);
   ierr = DMMGSetDM(dmmg,(DM)da);CHKERRQ(ierr);
-  ierr = DADestroy(da);CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),0,"x-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),1,"y-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),2,"pressure");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),3,"temperature");CHKERRQ(ierr);
-  ierr = VecDuplicate(dmmg[0]->x, &(user->Xguess)); CHKERRQ(ierr);
+  ierr = DMDestroy(&da);CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),0,"x-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),1,"y-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),2,"pressure");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),3,"temperature");CHKERRQ(ierr);
+  ierr = VecDuplicate(dmmg[0]->x, &(user->Xguess));CHKERRQ(ierr);
 #else
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create distributed array multigrid object (DMMG) to manage parallel grid and vectors
      for principal unknowns (x) and governing residuals (f)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
   ierr = DMMGCreate(comm,grid.mglevels,&user,&dmmg);CHKERRQ(ierr); 
-  ierr = DACreate2d(comm,grid.periodic,grid.stencil,grid.ni,grid.nj,PETSC_DECIDE,PETSC_DECIDE,grid.dof,grid.stencil_width,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(comm,grid.bx,grid.by,grid.stencil,grid.ni,grid.nj,PETSC_DECIDE,PETSC_DECIDE,grid.dof,grid.stencil_width,0,0,&da);CHKERRQ(ierr);
   ierr = DMMGSetDM(dmmg,(DM)da);CHKERRQ(ierr);
-  ierr = DADestroy(da);CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),0,"x-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),1,"y-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),2,"pressure");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),3,"temperature");CHKERRQ(ierr);
+  ierr = DMDestroy(&da);CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),0,"x-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),1,"y-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),2,"pressure");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),3,"temperature");CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create user context, set problem data, create vector data structures.
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */   
-  ierr = PetscMalloc(sizeof(AppCtx),&user); CHKERRQ(ierr);
+  ierr = PetscMalloc(sizeof(AppCtx),&user);CHKERRQ(ierr);
   user->param   = &param;
   user->grid    = &grid;
   dmmg[0]->user = user; 
-  ierr = VecDuplicate(dmmg[0]->x, &(user->Xguess)); CHKERRQ(ierr);
+  ierr = VecDuplicate(dmmg[0]->x, &(user->Xguess));CHKERRQ(ierr);
 #endif
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Set up the SNES solver with callback functions.
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = DMMGSetSNESLocal(dmmg,FormFunctionLocal,0,0,0);CHKERRQ(ierr);
+  ierr = DMMGSetFromOptions(dmmg);CHKERRQ(ierr);
   ierr = DMMGSetInitialGuess(dmmg,FormInitialGuess);CHKERRQ(ierr);
-  ierr = SNESSetConvergenceTest(DMMGGetSNES(dmmg),SNESConverged_Interactive,(void*)user);CHKERRQ(ierr);
+  ierr = SNESSetConvergenceTest(DMMGGetSNES(dmmg),SNESConverged_Interactive,(void*)user,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscPushSignalHandler(InteractiveHandler,(void*)user);CHKERRQ(ierr);    
    
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Initialize and solve the nonlinear system
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-  ierr = Initialize(dmmg); CHKERRQ(ierr);
+  ierr = Initialize(dmmg);CHKERRQ(ierr);
   ierr = UpdateSolution(dmmg,user,&nits);CHKERRQ(ierr); 
   
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -241,11 +242,11 @@ int main(int argc,char **argv)
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Free work space. 
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-  ierr = VecDestroy(user->Xguess); CHKERRQ(ierr);
-  ierr = PetscFree(user); CHKERRQ(ierr);
-  ierr = DMMGDestroy(dmmg); CHKERRQ(ierr);
+  ierr = VecDestroy(&user->Xguess);CHKERRQ(ierr);
+  ierr = PetscFree(user);CHKERRQ(ierr);
+  ierr = DMMGDestroy(dmmg);CHKERRQ(ierr);
   
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 
@@ -267,7 +268,7 @@ PetscErrorCode UpdateSolution(DMMG *dmmg, AppCtx *user, PetscInt *nits)
   PassiveScalar       cont_incr=0.3;
   PetscInt            its;
   PetscErrorCode      ierr;
-  PetscTruth          q = PETSC_FALSE;
+  PetscBool           q = PETSC_FALSE;
 
   PetscFunctionBegin;
   snes = DMMGGetSNES(dmmg);
@@ -341,7 +342,7 @@ PetscErrorCode FormInitialGuess(DMMG dmmg,Vec X)
   AppCtx         *user = (AppCtx*)dmmg->user;
   PetscErrorCode ierr;
 
-  ierr = VecCopy(user->Xguess, X); CHKERRQ(ierr);
+  ierr = VecCopy(user->Xguess, X);CHKERRQ(ierr);
   return 0;
 } 
 
@@ -354,7 +355,7 @@ PetscErrorCode FormInitialGuess(DMMG dmmg,Vec X)
 #define __FUNCT__ "FormFunctionLocal"
 /*  main call-back function that computes the processor-local piece 
     of the residual */
-PetscErrorCode FormFunctionLocal(DALocalInfo *info,Field **x,Field **f,void *ptr)
+PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *ptr)
 /*---------------------------------------------------------------------*/
 {
   AppCtx         *user = (AppCtx*)ptr;
@@ -926,8 +927,9 @@ PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
   param->depth         = grid->dz*(grid->nj-2);                            /* km */
   grid->inose          = 0;                                         /* gridpoints*/
   ierr = PetscOptionsGetInt(PETSC_NULL,"-inose",&(grid->inose),PETSC_NULL);CHKERRQ(ierr);
-  grid->periodic       = DA_NONPERIODIC;
-  grid->stencil        = DA_STENCIL_BOX;
+  grid->bx       = DMDA_BOUNDARY_NONE;
+  grid->by       = DMDA_BOUNDARY_NONE;
+  grid->stencil        = DMDA_STENCIL_BOX;
   grid->dof            = 4;
   grid->stencil_width  = 2;
   grid->mglevels       = 1;
@@ -1103,15 +1105,15 @@ PetscErrorCode Initialize(DMMG *dmmg)
   AppCtx         *user = (AppCtx*)dmmg[0]->user;
   Parameter      *param = user->param;
   GridInfo       *grid  = user->grid;
-  DA             da;
+  DM             da;
   PetscInt       i,j,is,js,im,jm;
   PetscErrorCode ierr;
   Field          **x;
 
   /* Get the fine grid */
-  da = (DA)(dmmg[0]->dm); 
-  ierr = DAGetCorners(da,&is,&js,PETSC_NULL,&im,&jm,PETSC_NULL);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
+  da = (dmmg[0]->dm); 
+  ierr = DMDAGetCorners(da,&is,&js,PETSC_NULL,&im,&jm,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
 
   /* Compute initial guess */
   for (j=js; j<js+jm; j++) {
@@ -1140,7 +1142,7 @@ PetscErrorCode Initialize(DMMG *dmmg)
   }
 
   /* Restore x to Xguess */
-  ierr = DAVecRestoreArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
 
   return 0;
 } 
@@ -1166,7 +1168,7 @@ PetscErrorCode DoOutput(DMMG *dmmg, PetscInt its)
 
   /* compute final residual and final viscosity/strain rate fields */
   ierr = SNESGetFunction(DMMGGetSNES(dmmg), &res, PETSC_NULL, PETSC_NULL);CHKERRQ(ierr);
-  ierr = ViscosityField(DMMGGetDMMG(dmmg), DMMGGetx(dmmg), ((AppCtx *)dmmg[0]->user)->Xguess);CHKERRQ(ierr); 
+  ierr = ViscosityField(DMMGGetFine(dmmg), DMMGGetx(dmmg), ((AppCtx *)dmmg[0]->user)->Xguess);CHKERRQ(ierr); 
 
   /* get the communicator and the rank of the processor */
   ierr = PetscObjectGetComm((PetscObject)DMMGGetSNES(dmmg), &comm);CHKERRQ(ierr);
@@ -1221,8 +1223,8 @@ PetscErrorCode DoOutput(DMMG *dmmg, PetscInt its)
     ierr = VecView(pars, viewer);CHKERRQ(ierr);
     
     /* destroy viewer and vector */
-    ierr = PetscViewerDestroy(viewer);CHKERRQ(ierr);
-    ierr = VecDestroy(pars);CHKERRQ(ierr);
+    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
+    ierr = VecDestroy(&pars);CHKERRQ(ierr);
   } 
   
   param->ivisc = ivt;
@@ -1236,7 +1238,7 @@ PetscErrorCode DoOutput(DMMG *dmmg, PetscInt its)
 PetscErrorCode ViscosityField(DMMG dmmg, Vec X, Vec V)
 /* ------------------------------------------------------------------- */
 {
-  DA             da    = (DA) dmmg->dm;
+  DM             da    =  dmmg->dm;
   AppCtx         *user  = (AppCtx *) dmmg->user;
   Parameter      *param = user->param;
   GridInfo       *grid  = user->grid;
@@ -1250,18 +1252,18 @@ PetscErrorCode ViscosityField(DMMG dmmg, Vec X, Vec V)
   ivt          = param->ivisc;
   param->ivisc = param->output_ivisc;
 
-  ierr = DACreateLocalVector(da, &localX);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalBegin(da, X, INSERT_VALUES, localX);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalEnd(da, X, INSERT_VALUES, localX);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da,localX,(void**)&x);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da,V,(void**)&v);CHKERRQ(ierr);
+  ierr = DMCreateLocalVector(da, &localX);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalBegin(da, X, INSERT_VALUES, localX);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalEnd(da, X, INSERT_VALUES, localX);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,localX,(void**)&x);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,V,(void**)&v);CHKERRQ(ierr);
 
   /* Parameters */
   dx   = grid->dx;   dz   = grid->dz;
   ilim = grid->ni-1; jlim = grid->nj-1;
 
   /* Compute real temperature, strain rate and viscosity */
-  ierr = DAGetCorners(da,&is,&js,PETSC_NULL,&im,&jm,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAGetCorners(da,&is,&js,PETSC_NULL,&im,&jm,PETSC_NULL);CHKERRQ(ierr);
   for (j=js; j<js+jm; j++) {
     for (i=is; i<is+im; i++) {
       T  = param->potentialT * x[j][i].T * exp( (j-0.5)*dz*param->z_scale );
@@ -1278,8 +1280,8 @@ PetscErrorCode ViscosityField(DMMG dmmg, Vec X, Vec V)
       v[j][i].T = Viscosity(TC,epsC,dz*j,param);
     }
   }
-  ierr = DAVecRestoreArray(da,V,(void**)&v);CHKERRQ(ierr);
-  ierr = DAVecRestoreArray(da,localX,(void**)&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,V,(void**)&v);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,localX,(void**)&x);CHKERRQ(ierr);
   param->ivisc = ivt;
   PetscFunctionReturn(0);
 }
@@ -1294,19 +1296,19 @@ PetscErrorCode StressField(DMMG *dmmg)
   AppCtx         *user = (AppCtx*)dmmg[0]->user;
   PetscInt       i,j,is,js,im,jm;
   PetscErrorCode ierr;
-  DA             da;
+  DM             da;
   Vec            locVec;
   Field          **x, **y;
 
   /* Get the fine grid of Xguess and X */
-  da = (DA)(dmmg[0]->dm); 
-  ierr = DAGetCorners(da,&is,&js,PETSC_NULL,&im,&jm,PETSC_NULL);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
+  da = (dmmg[0]->dm); 
+  ierr = DMDAGetCorners(da,&is,&js,PETSC_NULL,&im,&jm,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
 
-  ierr = DACreateLocalVector(da, &locVec);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalBegin(da, DMMGGetx(dmmg), INSERT_VALUES, locVec);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalEnd(da, DMMGGetx(dmmg), INSERT_VALUES, locVec);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da,locVec,(void**)&y);CHKERRQ(ierr);
+  ierr = DMCreateLocalVector(da, &locVec);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalBegin(da, DMMGGetx(dmmg), INSERT_VALUES, locVec);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalEnd(da, DMMGGetx(dmmg), INSERT_VALUES, locVec);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,locVec,(void**)&y);CHKERRQ(ierr);
 
   /* Compute stress on the corner points */
   for (j=js; j<js+jm; j++) {
@@ -1320,8 +1322,8 @@ PetscErrorCode StressField(DMMG *dmmg)
   }
 
   /* Restore the fine grid of Xguess and X */
-  ierr = DAVecRestoreArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
-  ierr = DAVecRestoreArray(da,locVec,(void**)&y);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,((AppCtx*)dmmg[0]->user)->Xguess,(void**)&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,locVec,(void**)&y);CHKERRQ(ierr);
 
   return 0;
 }
@@ -1375,7 +1377,7 @@ PassiveScalar PlateModel(PetscInt j, PetscInt plate, AppCtx *user)
 #if defined (PETSC_HAVE_ERF)
   return erf(z*param->L/2.0/param->skt);
 #else
-  SETERRQ(1,"erf() not available on this machine");
+  SETERRQ(PETSC_COMM_SELF,1,"erf() not available on this machine");
 #endif
 }
 
@@ -1467,10 +1469,10 @@ PetscScalar Pressure(PetscInt i, PetscInt j, AppCtx *user)
 #undef __FUNCT__
 #define __FUNCT__ "OptionsHasName"
 /*  utility function */
-PetscTruth OptionsHasName(const char pre[],const char name[])
+PetscBool  OptionsHasName(const char pre[],const char name[])
 /* ------------------------------------------------------------------- */
 {
-  PetscTruth     retval; 
+  PetscBool      retval; 
   PetscErrorCode ierr;
   ierr = PetscOptionsHasName(pre,name,&retval);
   return retval;
@@ -1525,10 +1527,14 @@ PetscErrorCode InteractiveHandler(int signum, void *ctx)
 
   if (signum == SIGILL) {
     param->toggle_kspmon = PETSC_TRUE;
+#if !defined(PETSC_HAVE_MISSING_SIGCONT)
   } else if (signum == SIGCONT) {
     param->interrupted = PETSC_TRUE;
+#endif
+#if !defined(PETSC_HAVE_MISSING_SIGURG)
   } else if (signum == SIGURG) {
     param->stop_solve = PETSC_TRUE;
+#endif
   } 
   return 0;
 }
