@@ -9,7 +9,7 @@ The flow can be driven with the lid or with bouyancy or both:\n\
 
 /*T
    Concepts: SNES^solving a system of nonlinear equations (parallel multicomponent example);
-   Concepts: DA^using distributed arrays;
+   Concepts: DMDA^using distributed arrays;
    Concepts: multicomponent
    Processors: n
 T*/
@@ -49,9 +49,10 @@ T*/
         without a preconditioner due to ill-conditioning).
 
   ------------------------------------------------------------------------- */
-#include "petscsnes.h"
-#include "petscda.h"
-#include "petscdmmg.h"
+#include <petscsnes.h>
+#include <petscdmda.h>
+#include <petscdmcomposite.h>
+#include <petscdmmg.h>
 
 /* User-defined routines and data structure */
 typedef struct {
@@ -68,11 +69,11 @@ typedef struct {
 
 typedef struct {
   PassiveReal  lidvelocity,prandtl,grashof;  /* physical parameters */
-  PetscTruth   draw_contours;                /* flag - 1 indicates drawing contours */
+  PetscBool    draw_contours;                /* flag - 1 indicates drawing contours */
   DMMG         *dmmg,*dmmg_comp;             /* used by MySolutionView() */
-  DMMG         *dmmg1,*dmmg2; /* passing objects of sub-physics into the composite physics - used by FormInitialGuessLocalComp() */
-  DMComposite  pack;
-  PetscTruth   COMPOSITE_MODEL;
+  DMMG         *dmmg1,*dmmg2; /* passing objects of sub-physics into the composite physics - used by FormInitialGuessComp() */
+  DM           pack;
+  PetscBool    COMPOSITE_MODEL;
   Field        **x;   /* passing DMMGGetx(dmmg) - final solution of original coupled physics */
   Field1       **x1;  /* passing local ghosted vector array of Physics 1 */
   Field2       **x2;  /* passing local ghosted vector array of Physics 2 */
@@ -81,12 +82,12 @@ typedef struct {
 extern PetscErrorCode FormInitialGuessLocal(DMMG,Vec);
 extern PetscErrorCode FormInitialGuessLocal1(DMMG,Vec);
 extern PetscErrorCode FormInitialGuessLocal2(DMMG,Vec);
-extern PetscErrorCode FormInitialGuessLocalComp(DMMG,Vec);
+extern PetscErrorCode FormInitialGuessComp(DMMG,Vec);
 
-extern PetscErrorCode FormFunctionLocal(DALocalInfo*,Field**,Field**,void*);
-extern PetscErrorCode FormFunctionLocal1(DALocalInfo*,Field1**,Field1**,void*);
-extern PetscErrorCode FormFunctionLocal2(DALocalInfo*,Field2**,Field2**,void*);
-extern PetscErrorCode FormFunctionLocalComp(SNES,Vec,Vec,void*);
+extern PetscErrorCode FormFunctionLocal(DMDALocalInfo*,Field**,Field**,void*);
+extern PetscErrorCode FormFunctionLocal1(DMDALocalInfo*,Field1**,Field1**,void*);
+extern PetscErrorCode FormFunctionLocal2(DMDALocalInfo*,Field2**,Field2**,void*);
+extern PetscErrorCode FormFunctionComp(SNES,Vec,Vec,void*);
 
 extern PetscErrorCode MySolutionView(MPI_Comm,PetscInt,void*);
 
@@ -100,8 +101,8 @@ int main(int argc,char **argv)
   PetscErrorCode ierr;
   MPI_Comm       comm;
   SNES           snes;
-  DA             da;
-  PetscTruth     ViewSolu=PETSC_FALSE,CompSolu=PETSC_TRUE,DoSubPhysics=PETSC_FALSE;
+  DM             da;
+  PetscBool      ViewSolu=PETSC_FALSE,CompSolu=PETSC_TRUE,DoSubPhysics=PETSC_FALSE;
   Vec            solu_true,solu_local;
 
   PetscInitialize(&argc,&argv,(char *)0,help);
@@ -115,12 +116,12 @@ int main(int argc,char **argv)
     for principal unknowns (x) and governing residuals (f)
   */
   dof  = 4;
-  ierr = DACreate2d(comm,DA_NONPERIODIC,DA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,dof,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,dof,1,0,0,&da);CHKERRQ(ierr);
   ierr = DMMGSetDM(dmmg,(DM)da);CHKERRQ(ierr);
-  ierr = DADestroy(da);CHKERRQ(ierr);
+  ierr = DMDestroy(&da);CHKERRQ(ierr);
 
-  ierr = DAGetInfo(DMMGGetDA(dmmg),0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
-                     PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(DMMGGetDM(dmmg),0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
+                     PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
 
   /* Problem parameters (velocity of lid, prandtl, and grashof numbers) */
   user.lidvelocity = 1.0/(mx*my);
@@ -133,10 +134,10 @@ int main(int argc,char **argv)
   ierr = PetscOptionsHasName(PETSC_NULL,"-view_solu",&ViewSolu);CHKERRQ(ierr);
   ierr = PetscOptionsHasName(PETSC_NULL,"-do_subphysics",&DoSubPhysics);CHKERRQ(ierr);
 
-  ierr = DASetFieldName(DMMGGetDA(dmmg),0,"x-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),1,"y-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),2,"Omega");CHKERRQ(ierr);
-  ierr = DASetFieldName(DMMGGetDA(dmmg),3,"temperature");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),0,"x-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),1,"y-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),2,"Omega");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(DMMGGetDM(dmmg),3,"temperature");CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create user context, set problem data, create vector data structures.
@@ -145,6 +146,7 @@ int main(int argc,char **argv)
 
   /* Create nonlinear solver context */
   ierr = DMMGSetSNESLocal(dmmg,FormFunctionLocal,0,ad_FormFunctionLocal,admf_FormFunctionLocal);CHKERRQ(ierr);
+  ierr = DMMGSetFromOptions(dmmg);CHKERRQ(ierr);
   ierr = PetscPrintf(comm,"lid velocity = %G, prandtl # = %G, grashof # = %G\n",
 		       user.lidvelocity,user.prandtl,user.grashof);CHKERRQ(ierr);
   ierr = DMMGSetInitialGuess(dmmg,FormInitialGuessLocal);CHKERRQ(ierr);
@@ -157,12 +159,12 @@ int main(int argc,char **argv)
   ierr = PetscPrintf(comm,"Origianl Physics: Number of Newton iterations = %D\n\n", its);CHKERRQ(ierr);
  
   /* Save the ghosted local solu_true to be used by Physics 1 and Physics 2 */
-  da        = DMMGGetDA(dmmg);
+  da        = DMMGGetDM(dmmg);
   solu_true = DMMGGetx(dmmg);
-  ierr = DACreateLocalVector(da,&solu_local);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalBegin(da,solu_true,INSERT_VALUES,solu_local);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalEnd(da,solu_true,INSERT_VALUES,solu_local);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da,solu_local,(Field **)&user.x);CHKERRQ(ierr);
+  ierr = DMCreateLocalVector(da,&solu_local);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalBegin(da,solu_true,INSERT_VALUES,solu_local);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalEnd(da,solu_true,INSERT_VALUES,solu_local);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,solu_local,(Field **)&user.x);CHKERRQ(ierr);
 
   /* Visualize solution */
   if (user.draw_contours) {
@@ -183,15 +185,16 @@ int main(int argc,char **argv)
   ierr = DMMGCreate(comm,nlevels,&user,&dmmg1);CHKERRQ(ierr);
   dof  = 3;
   user.COMPOSITE_MODEL = PETSC_FALSE;
-  ierr = DACreate2d(comm,DA_NONPERIODIC,DA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,dof,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,dof,1,0,0,&da);CHKERRQ(ierr);
   ierr = DMMGSetDM(dmmg1,(DM)da);CHKERRQ(ierr);
-  ierr = DASetFieldName(da,0,"x-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(da,1,"y-velocity");CHKERRQ(ierr);
-  ierr = DASetFieldName(da,2,"Omega");CHKERRQ(ierr);
-  ierr = DADestroy(da);CHKERRQ(ierr);
+  ierr = DMDASetFieldName(da,0,"x-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(da,1,"y-velocity");CHKERRQ(ierr);
+  ierr = DMDASetFieldName(da,2,"Omega");CHKERRQ(ierr);
+  ierr = DMDestroy(&da);CHKERRQ(ierr);
 
   if (DoSubPhysics){
     ierr = DMMGSetSNESLocal(dmmg1,FormFunctionLocal1,0,ad_FormFunctionLocal,admf_FormFunctionLocal);CHKERRQ(ierr);
+    ierr = DMMGSetFromOptions(dmmg1);CHKERRQ(ierr);
     ierr = DMMGSetInitialGuess(dmmg1,FormInitialGuessLocal1);CHKERRQ(ierr);
 
     ierr = DMMGSolve(dmmg1);CHKERRQ(ierr); 
@@ -213,13 +216,14 @@ int main(int argc,char **argv)
   ierr = DMMGCreate(comm,nlevels,&user,&dmmg2);CHKERRQ(ierr);
   dof  = 1;
   user.COMPOSITE_MODEL = PETSC_FALSE;
-  ierr = DACreate2d(comm,DA_NONPERIODIC,DA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,dof,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,dof,1,0,0,&da);CHKERRQ(ierr);
   ierr = DMMGSetDM(dmmg2,(DM)da);CHKERRQ(ierr);
-  ierr = DASetFieldName(da,0,"temperature");CHKERRQ(ierr);
-  ierr = DADestroy(da);CHKERRQ(ierr);
+  ierr = DMDASetFieldName(da,0,"temperature");CHKERRQ(ierr);
+  ierr = DMDestroy(&da);CHKERRQ(ierr);
 
   if (DoSubPhysics){
     ierr = DMMGSetSNESLocal(dmmg2,FormFunctionLocal2,0,ad_FormFunctionLocal,admf_FormFunctionLocal);CHKERRQ(ierr);
+    ierr = DMMGSetFromOptions(dmmg2);CHKERRQ(ierr);
     ierr = DMMGSetInitialGuess(dmmg2,FormInitialGuessLocal2);CHKERRQ(ierr);
 
     ierr = DMMGSolve(dmmg2);CHKERRQ(ierr); 
@@ -236,8 +240,8 @@ int main(int argc,char **argv)
     Create the DMComposite object to manage the two grids/physics. 
     - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = DMCompositeCreate(comm,&user.pack);CHKERRQ(ierr);
-  ierr = DMCompositeAddDA(user.pack,DMMGGetDA(dmmg1));CHKERRQ(ierr);
-  ierr = DMCompositeAddDA(user.pack,DMMGGetDA(dmmg2));CHKERRQ(ierr);
+  ierr = DMCompositeAddDM(user.pack,(DM)DMMGGetDM(dmmg1));CHKERRQ(ierr);
+  ierr = DMCompositeAddDM(user.pack,(DM)DMMGGetDM(dmmg2));CHKERRQ(ierr);
 
   /* Create the solver object and attach the grid/physics info */
   ierr = DMMGCreate(comm,nlevels,&user,&dmmg_comp);CHKERRQ(ierr);
@@ -248,8 +252,9 @@ int main(int argc,char **argv)
   user.dmmg1 = dmmg1;
   user.dmmg2 = dmmg2;
   user.COMPOSITE_MODEL = PETSC_TRUE;
-  ierr = DMMGSetInitialGuess(dmmg_comp,FormInitialGuessLocalComp);CHKERRQ(ierr);
-  ierr = DMMGSetSNES(dmmg_comp,FormFunctionLocalComp,0);CHKERRQ(ierr);
+  ierr = DMMGSetInitialGuess(dmmg_comp,FormInitialGuessComp);CHKERRQ(ierr);
+  ierr = DMMGSetSNES(dmmg_comp,FormFunctionComp,0);CHKERRQ(ierr);
+  ierr = DMMGSetFromOptions(dmmg_comp);CHKERRQ(ierr);
 
   /* Solve the nonlinear system */
   ierr = DMMGSolve(dmmg_comp);CHKERRQ(ierr); 
@@ -269,14 +274,14 @@ int main(int argc,char **argv)
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Free spaces 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-  ierr = DAVecRestoreArray(DMMGGetDA(dmmg),solu_local,(Field **)&user.x);CHKERRQ(ierr);
-  ierr = VecDestroy(solu_local);CHKERRQ(ierr);
-  ierr = DMCompositeDestroy(user.pack);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(DMMGGetDM(dmmg),solu_local,(Field **)&user.x);CHKERRQ(ierr);
+  ierr = VecDestroy(&solu_local);CHKERRQ(ierr);
+  ierr = DMDestroy(&user.pack);CHKERRQ(ierr);
   ierr = DMMGDestroy(dmmg);CHKERRQ(ierr);
   ierr = DMMGDestroy(dmmg1);CHKERRQ(ierr);
   ierr = DMMGDestroy(dmmg2);CHKERRQ(ierr);
   ierr = DMMGDestroy(dmmg_comp);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 
@@ -289,7 +294,7 @@ int main(int argc,char **argv)
 
    Input Parameters:
      user - user-defined application context
-     X    - vector (DA local vector)
+     X    - vector (DMDA local vector)
 
    Output Parameter:
      X - vector with the local values set
@@ -297,22 +302,22 @@ int main(int argc,char **argv)
 PetscErrorCode FormInitialGuessLocal(DMMG dmmg,Vec X)
 {
   AppCtx         *user = (AppCtx*)dmmg->user;
-  DA             da = (DA)dmmg->dm;
+  DM             da = dmmg->dm;
   PetscInt       i,j,mx,xs,ys,xm,ym;
   PetscErrorCode ierr;
   PetscReal      grashof,dx;
   Field          **x;
 
   grashof = user->grashof;
-  ierr = DAGetInfo(da,0,&mx,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(da,0,&mx,0,0,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
   dx  = 1.0/(mx-1);
 
   /*
-     Get local grid boundaries (for 2-dimensional DA):
+     Get local grid boundaries (for 2-dimensional DMDA):
        xs, ys   - starting grid indices (no ghost points)
        xm, ym   - widths of local grid (no ghost points)
   */
-  ierr = DAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
 
   /*
      Get a pointer to vector data.
@@ -321,7 +326,7 @@ PetscErrorCode FormInitialGuessLocal(DMMG dmmg,Vec X)
        - You MUST call VecResetArraystoreArray() when you no longer need access to
          the array.
   */
-  ierr = DAVecGetArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,X,&x);CHKERRQ(ierr);
 
   /*
      Compute initial guess over the locally owned part of the grid
@@ -338,7 +343,7 @@ PetscErrorCode FormInitialGuessLocal(DMMG dmmg,Vec X)
   }
 
   /* Restore vector */
-  ierr = DAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
   return 0;
 }
 
@@ -347,16 +352,15 @@ PetscErrorCode FormInitialGuessLocal(DMMG dmmg,Vec X)
 /* Form initial guess for Physic 1 */
 PetscErrorCode FormInitialGuessLocal1(DMMG dmmg,Vec X)
 {
-  AppCtx         *user = (AppCtx*)dmmg->user;
-  DA             da = (DA)dmmg->dm;
+  DM             da = dmmg->dm;
   PetscInt       i,j,mx,xs,ys,xm,ym;
   PetscErrorCode ierr;
   Field1         **x;
 
-  ierr = DAGetInfo(da,0,&mx,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
-  ierr = DAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(da,0,&mx,0,0,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
+  ierr = DMDAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
 
-  ierr = DAVecGetArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,X,&x);CHKERRQ(ierr);
   for (j=ys; j<ys+ym; j++) {
     for (i=xs; i<xs+xm; i++) {
       x[j][i].u     = 0.0;
@@ -364,7 +368,7 @@ PetscErrorCode FormInitialGuessLocal1(DMMG dmmg,Vec X)
       x[j][i].omega = 0.0;
     }
   }
-  ierr = DAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
   return 0;
 }
 
@@ -374,41 +378,41 @@ PetscErrorCode FormInitialGuessLocal1(DMMG dmmg,Vec X)
 PetscErrorCode FormInitialGuessLocal2(DMMG dmmg,Vec X)
 {
   AppCtx         *user = (AppCtx*)dmmg->user;
-  DA             da = (DA)dmmg->dm;
+  DM             da = dmmg->dm;
   PetscInt       i,j,mx,xs,ys,xm,ym;
   PetscErrorCode ierr;
   PetscReal      grashof,dx;
   Field2         **x;
 
   grashof = user->grashof;
-  ierr = DAGetInfo(da,0,&mx,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(da,0,&mx,0,0,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
   dx  = 1.0/(mx-1);
 
-  ierr = DAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
 
-  ierr = DAVecGetArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,X,&x);CHKERRQ(ierr);
   for (j=ys; j<ys+ym; j++) {
     for (i=xs; i<xs+xm; i++) {
       x[j][i].temp  = (grashof>0)*i*dx;  
     }
   }
-  ierr = DAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
   return 0;
 }
 
 #undef __FUNCT__
-#define __FUNCT__ "FormInitialGuessLocalComp"
+#define __FUNCT__ "FormInitialGuessComp"
 /* 
-   FormInitialGuessLocalComp - 
+   FormInitialGuessComp - 
               Forms the initial guess for the composite model
               Unwraps the global solution vector and passes its local pieces into the user functions
  */
-PetscErrorCode FormInitialGuessLocalComp(DMMG dmmg,Vec X)
+PetscErrorCode FormInitialGuessComp(DMMG dmmg,Vec X)
 {
   PetscErrorCode ierr;
   AppCtx         *user = (AppCtx*)dmmg->user;
   DMMG           *dmmg1 = user->dmmg1,*dmmg2=user->dmmg2;
-  DMComposite    dm = (DMComposite)dmmg->dm;
+  DM             dm = dmmg->dm;
   Vec            X1,X2;
 
   PetscFunctionBegin;
@@ -429,9 +433,9 @@ PetscErrorCode FormInitialGuessLocalComp(DMMG dmmg,Vec X)
    FormFunctionLocal - Function evaluation for this process
 
    Input Parameters:
-     info - DALocalInfo context
-     x    - (DA local) vector array including ghost points
-     f    - (DA local) vector array to be evaluated 
+     info - DMDALocalInfo context
+     x    - (DMDA local) vector array including ghost points
+     f    - (DMDA local) vector array to be evaluated 
      ptr  - user-defined application context
 
    Output Parameter:
@@ -442,7 +446,7 @@ PetscErrorCode FormInitialGuessLocalComp(DMMG dmmg,Vec X)
      f.omega = - Lap(Omega) + Div([U*Omega,V*Omega]) - GR*Grad_x(T)
      f.temp  = - Lap(T) + PR*Div([U*T,V*T])
 */
-PetscErrorCode FormFunctionLocal(DALocalInfo *info,Field **x,Field **f,void *ptr)
+PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *ptr)
  {
   AppCtx         *user = (AppCtx*)ptr;
   PetscErrorCode ierr;
@@ -567,7 +571,7 @@ PetscErrorCode FormFunctionLocal(DALocalInfo *info,Field **x,Field **f,void *ptr
   }
 
   /* Flop count (multiply-adds are counted as 2 operations) */
-  ierr = PetscLogFlops(84*info->ym*info->xm);CHKERRQ(ierr);
+  ierr = PetscLogFlops(84.0*info->ym*info->xm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 } 
 
@@ -578,20 +582,19 @@ PetscErrorCode FormFunctionLocal(DALocalInfo *info,Field **x,Field **f,void *ptr
       same as FormFunctionLocal() except without f.temp and x.temp.
       the input x.temp comes from the solu_true 
 */
-PetscErrorCode FormFunctionLocal1(DALocalInfo *info,Field1 **x,Field1 **f,void *ptr)
+PetscErrorCode FormFunctionLocal1(DMDALocalInfo *info,Field1 **x,Field1 **f,void *ptr)
  {
   AppCtx         *user = (AppCtx*)ptr;
-  PetscErrorCode ierr;
   PetscInt       xints,xinte,yints,yinte,i,j;
   PetscReal      hx,hy,dhx,dhy,hxdhy,hydhx;
-  PetscReal      grashof,prandtl,lid;
+  PetscReal      grashof,lid; /* ,prandtl */
   PetscScalar    u,uxx,uyy,vx,vy,avx,avy,vxp,vxm,vyp,vym;
   Field          **solu=user->x;
   Field2         **solu2=user->x2;
 
   PetscFunctionBegin;
   grashof = user->grashof;  
-  prandtl = user->prandtl;
+  /* prandtl = user->prandtl; */
   lid     = user->lidvelocity;
 
   dhx = (PetscReal)(info->mx-1);  dhy = (PetscReal)(info->my-1);
@@ -693,13 +696,12 @@ PetscErrorCode FormFunctionLocal1(DALocalInfo *info,Field1 **x,Field1 **f,void *
       same as FormFunctionLocal() but only has f.temp and x.temp.
       the input x.u and x.v come from solu_true 
 */
-PetscErrorCode FormFunctionLocal2(DALocalInfo *info,Field2 **x,Field2 **f,void *ptr)
+PetscErrorCode FormFunctionLocal2(DMDALocalInfo *info,Field2 **x,Field2 **f,void *ptr)
  {
   AppCtx         *user = (AppCtx*)ptr;
-  PetscErrorCode ierr;
   PetscInt       xints,xinte,yints,yinte,i,j;
   PetscReal      hx,hy,dhx,dhy,hxdhy,hydhx;
-  PetscReal      grashof,prandtl,lid;
+  PetscReal      grashof,prandtl; /* ,lid */
   PetscScalar    u,uxx,uyy,vx,vy,avx,avy,vxp,vxm,vyp,vym;
   Field          **solu=user->x;
   Field1         **solu1=user->x1;
@@ -707,7 +709,7 @@ PetscErrorCode FormFunctionLocal2(DALocalInfo *info,Field2 **x,Field2 **f,void *
   PetscFunctionBegin;
   grashof = user->grashof;  
   prandtl = user->prandtl;
-  lid     = user->lidvelocity;
+  /* lid     = user->lidvelocity; */
 
   dhx = (PetscReal)(info->mx-1);  dhy = (PetscReal)(info->my-1);
   hx = 1.0/dhx;                   hy = 1.0/dhy;
@@ -783,42 +785,42 @@ PetscErrorCode FormFunctionLocal2(DALocalInfo *info,Field2 **x,Field2 **f,void *
 } 
 
 #undef __FUNCT__
-#define __FUNCT__ "FormFunctionLocalComp"
+#define __FUNCT__ "FormFunctionComp"
 /* 
-   FormFunctionLocalComp  - Unwraps the input vector and passes its local ghosted pieces into the user function
+   FormFunctionComp  - Unwraps the input vector and passes its local ghosted pieces into the user function
 */
-PetscErrorCode FormFunctionLocalComp(SNES snes,Vec X,Vec F,void *ctx)
+PetscErrorCode FormFunctionComp(SNES snes,Vec X,Vec F,void *ctx)
 {
   PetscErrorCode ierr;
   DMMG           dmmg = (DMMG)ctx;
   AppCtx         *user = (AppCtx*)dmmg->user;
-  DMComposite    dm = (DMComposite)dmmg->dm;
-  DALocalInfo    info1,info2;
-  DA             da1,da2;
+  DM             dm = dmmg->dm;
+  DMDALocalInfo    info1,info2;
+  DM             da1,da2;
   Field1         **x1,**f1;
   Field2         **x2,**f2;
   Vec            X1,X2,F1,F2;
 
   PetscFunctionBegin;
   ierr = DMCompositeGetEntries(dm,&da1,&da2);CHKERRQ(ierr);
-  ierr = DAGetLocalInfo(da1,&info1);CHKERRQ(ierr);
-  ierr = DAGetLocalInfo(da2,&info2);CHKERRQ(ierr);
+  ierr = DMDAGetLocalInfo(da1,&info1);CHKERRQ(ierr);
+  ierr = DMDAGetLocalInfo(da2,&info2);CHKERRQ(ierr);
 
   /* Get local vectors to hold ghosted parts of X */
   ierr = DMCompositeGetLocalVectors(dm,&X1,&X2);CHKERRQ(ierr);
   ierr = DMCompositeScatter(dm,X,X1,X2);CHKERRQ(ierr); 
 
   /* Access the arrays inside the subvectors of X */
-  ierr = DAVecGetArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
 
   /* Access the subvectors in F. 
      These are not ghosted and directly access the memory locations in F */
   ierr = DMCompositeGetAccess(dm,F,&F1,&F2);CHKERRQ(ierr);
 
   /* Access the arrays inside the subvectors of F */  
-  ierr = DAVecGetArray(da1,F1,(void**)&f1);CHKERRQ(ierr);
-  ierr = DAVecGetArray(da2,F2,(void**)&f2);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da1,F1,(void**)&f1);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da2,F2,(void**)&f2);CHKERRQ(ierr);
 
   /* Evaluate local user provided function */    
   user->x2 = x2; /* used by FormFunctionLocal1() */
@@ -826,11 +828,11 @@ PetscErrorCode FormFunctionLocalComp(SNES snes,Vec X,Vec F,void *ctx)
   user->x1 = x1; /* used by FormFunctionLocal2() */
   ierr = FormFunctionLocal2(&info2,x2,f2,(void**)user);CHKERRQ(ierr);
 
-  ierr = DAVecRestoreArray(da1,F1,(void**)&f1);CHKERRQ(ierr);
-  ierr = DAVecRestoreArray(da2,F2,(void**)&f2);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da1,F1,(void**)&f1);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da2,F2,(void**)&f2);CHKERRQ(ierr);
   ierr = DMCompositeRestoreAccess(dm,F,&F1,&F2);CHKERRQ(ierr);
-  ierr = DAVecRestoreArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
-  ierr = DAVecRestoreArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
   ierr = DMCompositeRestoreLocalVectors(dm,&X1,&X2);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -845,15 +847,19 @@ PetscErrorCode MySolutionView(MPI_Comm comm,PetscInt phy_num,void *ctx)
   PetscErrorCode ierr;
   AppCtx         *user = (AppCtx*)ctx;
   DMMG           *dmmg = user->dmmg;
-  DA             da=DMMGGetDA(dmmg);
+  DM             da=DMMGGetDM(dmmg);
+#if !defined(PETSC_USE_COMPLEX)
   Field          **x = user->x;
-  Field1         **x1 = user->x1;
-  PetscInt       i,j,mx,xs,ys,xm,ym;
+#endif
+  PetscInt       i,j,xs,ys,xm,ym;
   PetscMPIInt    size;
 
   PetscFunctionBegin;
+#if defined(PETSC_USE_COMPLEX)
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This routine does not support for scalar type complex yet\n");
+#endif
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
-  ierr = DAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
   
   switch (phy_num){
   case 0:
@@ -862,82 +868,90 @@ PetscErrorCode MySolutionView(MPI_Comm comm,PetscInt phy_num,void *ctx)
       ierr = PetscPrintf(PETSC_COMM_SELF,"-----------------------------------\n");
       for (j=ys; j<ys+ym; j++) {
         for (i=xs; i<xs+xm; i++) {
+#if !defined(PETSC_USE_COMPLEX)
           ierr = PetscPrintf(PETSC_COMM_SELF,"x[%d,%d] = %g, %g, %g, %g\n",j,i,x[j][i].u,x[j][i].v,x[j][i].omega,x[j][i].temp);
+#endif
         }
       }    
     }
     break;
   case 1:
     if (size == 1){
-      ierr = PetscPrintf(PETSC_COMM_SELF,"SubPhysics %d: U,V,Omega: \n",phy_num);
-      ierr = PetscPrintf(PETSC_COMM_SELF,"------------------------\n");
       DMMG *dmmg1=user->dmmg1;
       Vec  solu_true = DMMGGetx(dmmg1);
-      DA   da=DMMGGetDA(dmmg1);
+      DM   da=DMMGGetDM(dmmg1);
       Field1 **x1;
-      ierr = DAVecGetArray(da,solu_true,&x1);CHKERRQ(ierr);
+      ierr = PetscPrintf(PETSC_COMM_SELF,"SubPhysics %d: U,V,Omega: \n",phy_num);
+      ierr = PetscPrintf(PETSC_COMM_SELF,"------------------------\n");
+      ierr = DMDAVecGetArray(da,solu_true,&x1);CHKERRQ(ierr);
       for (j=ys; j<ys+ym; j++) {
         for (i=xs; i<xs+xm; i++) {
+#if !defined(PETSC_USE_COMPLEX)
           ierr = PetscPrintf(PETSC_COMM_SELF,"x[%d,%d] = %g, %g, %g\n",j,i,x1[j][i].u,x1[j][i].v,x1[j][i].omega);
+#endif
         }
       } 
-      ierr = DAVecRestoreArray(da,solu_true,&x1);CHKERRQ(ierr);
+      ierr = DMDAVecRestoreArray(da,solu_true,&x1);CHKERRQ(ierr);
     }
     break;
   case 2:
     if (size == 1){
-      ierr = PetscPrintf(PETSC_COMM_SELF,"SubPhysics %d: Temperature: \n",phy_num);
-      ierr = PetscPrintf(PETSC_COMM_SELF,"--------------------------\n");
       DMMG *dmmg2=user->dmmg2;
       Vec  solu_true = DMMGGetx(dmmg2);
-      DA   da=DMMGGetDA(dmmg2);
+      DM   da=DMMGGetDM(dmmg2);
       Field2 **x2;
-      ierr = DAVecGetArray(da,solu_true,&x2);CHKERRQ(ierr);
+      ierr = PetscPrintf(PETSC_COMM_SELF,"SubPhysics %d: Temperature: \n",phy_num);
+      ierr = PetscPrintf(PETSC_COMM_SELF,"--------------------------\n");
+      ierr = DMDAVecGetArray(da,solu_true,&x2);CHKERRQ(ierr);
       for (j=ys; j<ys+ym; j++) {
         for (i=xs; i<xs+xm; i++) {
+#if !defined(PETSC_USE_COMPLEX)
           ierr = PetscPrintf(PETSC_COMM_SELF,"x[%d,%d] = %g\n",j,i,x2[j][i].temp);
+#endif
         }
       } 
-      ierr = DAVecRestoreArray(da,solu_true,&x2);CHKERRQ(ierr);
+      ierr = DMDAVecRestoreArray(da,solu_true,&x2);CHKERRQ(ierr);
     }
     break;
   default:
     if (size == 1){
       DMMG        *dmmg_comp=user->dmmg_comp;
-      DA          da1,da2,da=DMMGGetDA(dmmg);
+      DM          da1,da2,da=DMMGGetDM(dmmg);
       Vec         X1,X2,solu_true = DMMGGetx(dmmg);
       Field       **x;
       Field1      **x1;
       Field2      **x2;
-      DMComposite dm = (DMComposite)(*dmmg_comp)->dm;
+      DM          dm = (*dmmg_comp)->dm;
       PetscReal   err,err_tmp;
       if (phy_num == 3){
         ierr = PetscPrintf(PETSC_COMM_SELF,"Composite physics %d, U,V,Omega,Temp: \n",phy_num);
         ierr = PetscPrintf(PETSC_COMM_SELF,"------------------------------------\n");
         ierr = PetscPrintf(PETSC_COMM_SELF,"Composite physics, U,V,Omega,Temp: \n");CHKERRQ(ierr);
       }
-      ierr = DAVecGetArray(da,solu_true,&x);CHKERRQ(ierr);
+      ierr = DMDAVecGetArray(da,solu_true,&x);CHKERRQ(ierr);
       ierr = DMCompositeGetEntries(dm,&da1,&da2);CHKERRQ(ierr);
       ierr = DMCompositeGetLocalVectors(dm,&X1,&X2);CHKERRQ(ierr);
-      ierr = DAVecGetArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
-      ierr = DAVecGetArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
+      ierr = DMDAVecGetArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
+      ierr = DMDAVecGetArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
 
       err = 0.0;
       for (j=ys; j<ys+ym; j++) {
         for (i=xs; i<xs+xm; i++) {
-          err_tmp = PetscAbs(x[j][i].u-x1[j][i].u) + PetscAbs(x[j][i].v-x1[j][i].v) + PetscAbs(x[j][i].omega-x1[j][i].omega);
-          err_tmp += PetscAbs(x[j][i].temp-x2[j][i].temp);
+          err_tmp = PetscAbsScalar(x[j][i].u-x1[j][i].u) + PetscAbsScalar(x[j][i].v-x1[j][i].v) + PetscAbsScalar(x[j][i].omega-x1[j][i].omega);
+          err_tmp += PetscAbsScalar(x[j][i].temp-x2[j][i].temp);
           if (err < err_tmp) err = err_tmp; 
           if (phy_num == 3){
+#if !defined(PETSC_USE_COMPLEX)
             ierr = PetscPrintf(PETSC_COMM_SELF,"x[%d,%d] = %g, %g, %g, %g\n",j,i,x1[j][i].u,x1[j][i].v,x1[j][i].omega,x2[j][i].temp);
+#endif
           }
         }
       }
       ierr = PetscPrintf(PETSC_COMM_SELF,"|solu - solu_comp| =  %g\n",err);CHKERRQ(ierr);
-      ierr = DAVecRestoreArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
-      ierr = DAVecRestoreArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
+      ierr = DMDAVecRestoreArray(da1,X1,(void**)&x1);CHKERRQ(ierr);
+      ierr = DMDAVecRestoreArray(da2,X2,(void**)&x2);CHKERRQ(ierr);
       ierr = DMCompositeRestoreLocalVectors(dm,&X1,&X2);CHKERRQ(ierr);
-      ierr = DAVecRestoreArray(da,solu_true,&x);CHKERRQ(ierr);
+      ierr = DMDAVecRestoreArray(da,solu_true,&x);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
