@@ -2,11 +2,11 @@
 
 static char help[] = "Nonlinear PDE in 2d.\n\
 We solve the Lane-Emden equation in a 2D rectangular\n\
-domain, using distributed arrays (DAs) to partition the parallel grid.\n\n";
+domain, using distributed arrays (DMDAs) to partition the parallel grid.\n\n";
 
 /*T
    Concepts: SNES^parallel Lane-Emden example
-   Concepts: DA^using distributed arrays;
+   Concepts: DMDA^using distributed arrays;
    Processors: n
 T*/
 
@@ -26,7 +26,7 @@ T*/
   ------------------------------------------------------------------------- */
 
 /* 
-   Include "petscda.h" so that we can use distributed arrays (DAs).
+   Include "petscdmda.h" so that we can use distributed arrays (DMDAs).
    Include "petscsnes.h" so that we can use SNES solvers.  Note that this
    file automatically includes:
      petscsys.h       - base PETSc routines   petscvec.h - vectors
@@ -35,20 +35,20 @@ T*/
      petscviewer.h - viewers               petscpc.h  - preconditioners
      petscksp.h   - linear solvers
 */
-#include "petscda.h"
-#include "petscdmmg.h"
-#include "petscsnes.h"
-#include "../src/snes/impls/ls/ls.h"
+#include <petscdmda.h>
+#include <petscdmmg.h>
+#include <petscsnes.h>
+#include <../src/snes/impls/ls/lsimpl.h>
 /* 
    User-defined application context - contains data needed by the 
    application-provided call-back routines, FormJacobianLocal() and
    FormFunctionLocal().
 */
 typedef struct {
-  //  DA        da;             /* distributed array data structure */
+  //  DM        da;             /* distributed array data structure */
    PetscReal alpha;          /* parameter controlling linearity */
    PetscReal lambda;         /* parameter controlling nonlinearity */
-  PetscTruth     draw_contours;                /* flag - 1 indicates drawing contours */
+  PetscBool      draw_contours;                /* flag - 1 indicates drawing contours */
 } AppCtx;
 
 
@@ -56,10 +56,10 @@ typedef struct {
    User-defined routines
 */
 extern PetscErrorCode FormInitialGuess(DMMG,Vec);
-extern PetscErrorCode FormFunctionLocal(DALocalInfo*,PetscScalar**,PetscScalar**,AppCtx*);
-extern PetscErrorCode FormFunctionLocali(DALocalInfo*,MatStencil*,PetscScalar**,PetscScalar*,AppCtx*);
-extern PetscErrorCode FormFunctionLocali4(DALocalInfo*,MatStencil*,PetscScalar**,PetscScalar*,AppCtx*);
-extern PetscErrorCode FormJacobianLocal(DALocalInfo*,PetscScalar**,Mat,AppCtx*);
+extern PetscErrorCode FormFunctionLocal(DMDALocalInfo*,PetscScalar**,PetscScalar**,AppCtx*);
+extern PetscErrorCode FormFunctionLocali(DMDALocalInfo*,MatStencil*,PetscScalar**,PetscScalar*,AppCtx*);
+extern PetscErrorCode FormFunctionLocali4(DMDALocalInfo*,MatStencil*,PetscScalar**,PetscScalar*,AppCtx*);
+extern PetscErrorCode FormJacobianLocal(DMDALocalInfo*,PetscScalar**,Mat,AppCtx*);
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -76,7 +76,7 @@ int main(int argc,char **argv)
   PetscReal              lambda_max = 6.81, lambda_min = 0.0;
   MPI_Comm       comm;
    PetscInt       mx,my;
-   DA                    da;
+   DM                    da;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Initialize program
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -91,11 +91,11 @@ comm = PETSC_COMM_WORLD;
   ierr = PetscOptionsGetReal(PETSC_NULL,"-alpha",&user.alpha,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscOptionsGetReal(PETSC_NULL,"-lambda",&user.lambda,PETSC_NULL);CHKERRQ(ierr);
   if (user.lambda > lambda_max || user.lambda < lambda_min) {
-    SETERRQ3(1,"Lambda %g is out of range [%g, %g]", user.lambda, lambda_min, lambda_max);
+    SETERRQ3(PETSC_COMM_SELF,1,"Lambda %g is out of range [%g, %g]", user.lambda, lambda_min, lambda_max);
   }
 
 
-  // in order only run once, I block it: PreLoadBegin(PETSC_TRUE,"SetUp");
+  // in order only run once, I block it: PetscPreLoadBegin(PETSC_TRUE,"SetUp");
     ierr = DMMGCreate(comm,2,&user,&dmmg);CHKERRQ(ierr);
 
 
@@ -103,13 +103,13 @@ comm = PETSC_COMM_WORLD;
       Create distributed array multigrid object (DMMG) to manage parallel grid and vectors
       for principal unknowns (x) and governing residuals (f)
     */
-    ierr = DACreate2d(PETSC_COMM_WORLD,DA_NONPERIODIC,DA_STENCIL_BOX,-4,-4,PETSC_DECIDE,PETSC_DECIDE,1,1,PETSC_NULL,PETSC_NULL,&da);CHKERRQ(ierr);
+    ierr = DMDACreate2d(PETSC_COMM_WORLD, DMDA_BOUNDARY_NONE, DMDA_BOUNDARY_NONE,DMDA_STENCIL_BOX,-4,-4,PETSC_DECIDE,PETSC_DECIDE,1,1,PETSC_NULL,PETSC_NULL,&da);CHKERRQ(ierr);
     
     ierr = DMMGSetDM(dmmg,(DM)da);CHKERRQ(ierr);
-    ierr = DADestroy(da);CHKERRQ(ierr);
+    ierr = DMDestroy(&da);CHKERRQ(ierr);
 
-    ierr = DAGetInfo(DMMGGetDA(dmmg),0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
-                     PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
+    ierr = DMDAGetInfo(DMMGGetDM(dmmg),0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
+                     PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
     ierr = PetscPrintf(comm,"mx = %d, my= %d\n",
 		       mx,my);CHKERRQ(ierr);
  
@@ -135,7 +135,7 @@ comm = PETSC_COMM_WORLD;
        - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
     ierr = DMMGSetInitialGuess(dmmg,FormInitialGuess);CHKERRQ(ierr);
 
-    //I block it:  PreLoadStage("Solve");
+    //I block it:  PetscPreLoadStage("Solve");
     ierr = DMMGSolve(dmmg);CHKERRQ(ierr); 
 
     snes = DMMGGetSNES(dmmg);
@@ -157,9 +157,9 @@ comm = PETSC_COMM_WORLD;
        - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
     ierr = DMMGDestroy(dmmg);CHKERRQ(ierr);
-    //PreLoadEnd();
+    //PetscPreLoadEnd();
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 
 }
@@ -179,15 +179,15 @@ comm = PETSC_COMM_WORLD;
 PetscErrorCode FormInitialGuess(DMMG dmmg,Vec X)
 {
   AppCtx         *user = (AppCtx*)dmmg->user;
-  DA             da = (DA)dmmg->dm;
+  DM             da = dmmg->dm;
   PetscInt       i,j,Mx,My,xs,ys,xm,ym;
   PetscErrorCode ierr;
   PetscReal      lambda,temp1,temp,hx,hy;
   PetscScalar    **x;
 
   PetscFunctionBegin;
-  ierr = DAGetInfo(da,PETSC_IGNORE,&Mx,&My,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
-                   PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);
+  ierr = DMDAGetInfo(da,PETSC_IGNORE,&Mx,&My,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
+                   PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);
  
   lambda = user->lambda;
   hx     = 1.0/(PetscReal)(Mx-1);
@@ -205,15 +205,15 @@ PetscErrorCode FormInitialGuess(DMMG dmmg,Vec X)
        - You MUST call VecRestoreArray() when you no longer need access to
          the array.
   */
-  ierr = DAVecGetArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecGetArray(da,X,&x);CHKERRQ(ierr);
 
   /*
-     Get local grid boundaries (for 2-dimensional DA):
+     Get local grid boundaries (for 2-dimensional DMDA):
        xs, ys   - starting grid indices (no ghost points)
        xm, ym   - widths of local grid (no ghost points)
 
   */
-  ierr = DAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMDAGetCorners(da,&xs,&ys,PETSC_NULL,&xm,&ym,PETSC_NULL);CHKERRQ(ierr);
 
   /*
      Compute initial guess over the locally owned part of the grid
@@ -234,7 +234,7 @@ PetscErrorCode FormInitialGuess(DMMG dmmg,Vec X)
   /*
      Restore vector
   */
-  ierr = DAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArray(da,X,&x);CHKERRQ(ierr);
 
   PetscFunctionReturn(0);
 }
@@ -248,7 +248,7 @@ PetscErrorCode FormInitialGuess(DMMG dmmg,Vec X)
        Process adiC(36): FormFunctionLocal nonlinearResidual FormFunctionLocali FormFunctionLocali4
 
  */
-PetscErrorCode FormFunctionLocal(DALocalInfo *info,PetscScalar **x,PetscScalar **f,AppCtx *user)
+PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,PetscScalar **x,PetscScalar **f,AppCtx *user)
 
 {
   PetscScalar    uLocal[4];
@@ -365,7 +365,7 @@ PetscErrorCode FormFunctionLocal(DALocalInfo *info,PetscScalar **x,PetscScalar *
 #undef __FUNCT__
 #define __FUNCT__ "FormFunctionLocali"
 
-PetscErrorCode FormFunctionLocali(DALocalInfo *info,MatStencil *st,PetscScalar **x,PetscScalar *f,AppCtx *user)
+PetscErrorCode FormFunctionLocali(DMDALocalInfo *info,MatStencil *st,PetscScalar **x,PetscScalar *f,AppCtx *user)
 {
   PetscScalar    uLocal[4];
   PetscScalar    rLocal[4];
@@ -532,7 +532,7 @@ PetscErrorCode FormFunctionLocali(DALocalInfo *info,MatStencil *st,PetscScalar *
 
 #undef __FUNCT__
 #define __FUNCT__ "FormFunctionLocali4"
-PetscErrorCode FormFunctionLocali4(DALocalInfo *info,MatStencil *st,PetscScalar **x,PetscScalar *f,AppCtx *user)
+PetscErrorCode FormFunctionLocali4(DMDALocalInfo *info,MatStencil *st,PetscScalar **x,PetscScalar *f,AppCtx *user)
 { 
   PetscScalar    uLocal[4],fLocal[5][5];
   PetscScalar    rLocal[4];
