@@ -1,9 +1,8 @@
-#define PETSCSNES_DLL
 
-#include "private/snesimpl.h"
+#include <private/snesimpl.h>
 
 typedef struct {
-  PetscTruth complete_print;
+  PetscBool  complete_print;
 } SNES_Test;
 
 /*
@@ -24,9 +23,7 @@ PetscErrorCode SNESSolve_Test(SNES snes)
 
   PetscFunctionBegin;
 
-  if (A != snes->jacobian_pre) {
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Cannot test with alternative preconditioner");
-  }
+  if (A != snes->jacobian_pre) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Cannot test with alternative preconditioner");
 
   ierr = PetscPrintf(((PetscObject)snes)->comm,"Testing hand-coded Jacobian, if the ratio is\n");CHKERRQ(ierr);
   ierr = PetscPrintf(((PetscObject)snes)->comm,"O(1.e-8), the hand-coded Jacobian is probably correct.\n");CHKERRQ(ierr);
@@ -36,11 +33,17 @@ PetscErrorCode SNESSolve_Test(SNES snes)
   }
 
   for (i=0; i<3; i++) {
+    static const char *const loc[] = {"user-defined state","constant state -1.0","constant state 1.0"};
     if (i == 1) {ierr = VecSet(x,-1.0);CHKERRQ(ierr);}
     else if (i == 2) {ierr = VecSet(x,1.0);CHKERRQ(ierr);}
 
     /* evaluate the function at this point because SNESDefaultComputeJacobianColor() assumes that the function has been evaluated and put into snes->vec_func */
     ierr = SNESComputeFunction(snes,x,f);CHKERRQ(ierr);
+    if (snes->domainerror) {
+      ierr = PetscPrintf(((PetscObject)snes)->comm,"Domain error at %s\n",loc[i]);CHKERRQ(ierr);
+      snes->domainerror = PETSC_FALSE;
+      continue;
+    }
 
     /* compute both versions of Jacobian */
     ierr = SNESComputeJacobian(snes,x,&A,&A,&flg);CHKERRQ(ierr);
@@ -49,7 +52,7 @@ PetscErrorCode SNESSolve_Test(SNES snes)
     if (neP->complete_print) {
       MPI_Comm    comm;
       PetscViewer viewer;
-      ierr = PetscPrintf(((PetscObject)snes)->comm,"Finite difference Jacobian\n");CHKERRQ(ierr);
+      ierr = PetscPrintf(((PetscObject)snes)->comm,"Finite difference Jacobian (%s)\n",loc[i]);CHKERRQ(ierr);
       ierr = PetscObjectGetComm((PetscObject)B,&comm);CHKERRQ(ierr);
       ierr = PetscViewerASCIIGetStdout(comm,&viewer);CHKERRQ(ierr);
       ierr = MatView(B,viewer);CHKERRQ(ierr);
@@ -61,17 +64,17 @@ PetscErrorCode SNESSolve_Test(SNES snes)
     if (neP->complete_print) {
       MPI_Comm    comm;
       PetscViewer viewer;
-      ierr = PetscPrintf(((PetscObject)snes)->comm,"Hand-coded Jacobian\n");CHKERRQ(ierr);
+      ierr = PetscPrintf(((PetscObject)snes)->comm,"Hand-coded Jacobian (%s)\n",loc[i]);CHKERRQ(ierr);
       ierr = PetscObjectGetComm((PetscObject)B,&comm);CHKERRQ(ierr);
       ierr = PetscViewerASCIIGetStdout(comm,&viewer);CHKERRQ(ierr);
       ierr = MatView(A,viewer);CHKERRQ(ierr);
-      ierr = PetscPrintf(((PetscObject)snes)->comm,"Hand-coded minus finite difference Jacobian\n");CHKERRQ(ierr);
+      ierr = PetscPrintf(((PetscObject)snes)->comm,"Hand-coded minus finite difference Jacobian (%s)\n",loc[i]);CHKERRQ(ierr);
       ierr = MatView(B,viewer);CHKERRQ(ierr);
     }
     if (!gnorm) gnorm = 1; /* just in case */
-    ierr = PetscPrintf(((PetscObject)snes)->comm,"Norm of matrix ratio %G difference %G\n",nrm/gnorm,nrm);CHKERRQ(ierr);
+    ierr = PetscPrintf(((PetscObject)snes)->comm,"Norm of matrix ratio %G difference %G (%s)\n",nrm/gnorm,nrm,loc[i]);CHKERRQ(ierr);
   }
-  ierr = MatDestroy(B);CHKERRQ(ierr);
+  ierr = MatDestroy(&B);CHKERRQ(ierr);
   /*
          Return error code cause Jacobian not good
   */
@@ -98,7 +101,7 @@ static PetscErrorCode SNESSetFromOptions_Test(SNES snes)
   PetscFunctionBegin;
 
   ierr = PetscOptionsHead("Hand-coded Jacobian tester options");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-snes_test_display","Display difference between hand-coded and finite difference Jacobians","None",ls->complete_print,&ls->complete_print,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-snes_test_display","Display difference between hand-coded and finite difference Jacobians","None",ls->complete_print,&ls->complete_print,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -118,20 +121,21 @@ M*/
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "SNESCreate_Test"
-PetscErrorCode PETSCSNES_DLLEXPORT SNESCreate_Test(SNES  snes)
+PetscErrorCode  SNESCreate_Test(SNES  snes)
 {
   SNES_Test      *neP;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  snes->ops->setup	     = 0;
-  snes->ops->solve	     = SNESSolve_Test;
-  snes->ops->destroy	     = SNESDestroy_Test;
+  snes->ops->solve           = SNESSolve_Test;
+  snes->ops->destroy         = SNESDestroy_Test;
   snes->ops->setfromoptions  = SNESSetFromOptions_Test;
   snes->ops->view            = 0;
+  snes->ops->setup           = 0;
+  snes->ops->reset           = 0;
 
-  ierr			= PetscNewLog(snes,SNES_Test,&neP);CHKERRQ(ierr);
-  snes->data    	= (void*)neP;
+  ierr                  = PetscNewLog(snes,SNES_Test,&neP);CHKERRQ(ierr);
+  snes->data            = (void*)neP;
   neP->complete_print   = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
