@@ -1,26 +1,28 @@
-#define PETSCSNES_DLL
  
-#include "petscda.h"            /*I "petscda.h"   I*/
-#include "petscksp.h"           /*I "petscksp.h"  I*/
-#include "petscmg.h"            /*I "petscmg.h"   I*/
-#include "petscdmmg.h"          /*I "petscdmmg.h" I*/
-#include "private/pcimpl.h"     /*I "petscpc.h"   I*/
+#include <petscdm.h>            /*I "petscdm.h"   I*/
+#include <petscksp.h>           /*I "petscksp.h"  I*/
+#include <petscpcmg.h>            /*I "petscpcmg.h"   I*/
+#include <petscdmmg.h>          /*I "petscdmmg.h" I*/
+#include <private/pcimpl.h>     /*I "petscpc.h"   I*/
 
 /*
-   Code for almost fully managing multigrid/multi-level linear solvers for DA grids
+   Code for almost fully managing multigrid/multi-level linear solvers for DM grids
 */
 
 #undef __FUNCT__  
 #define __FUNCT__ "DMMGCreate"
 /*@C
-    DMMGCreate - Creates a DA based multigrid solver object. This allows one to 
+    DMMGCreate - Creates a DM based multigrid solver object. This allows one to 
       easily implement MG methods on regular grids.
 
+     This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
     Collective on MPI_Comm
 
     Input Parameter:
 +   comm - the processors that will share the grids and solution process
-.   nlevels - number of multigrid levels 
+.   nlevels - number of multigrid levels (if this is negative it CANNOT be reset with -dmmg_nlevels
 -   user - an optional user context
 
     Output Parameters:
@@ -28,7 +30,7 @@
 
     Options Database:
 +     -dmmg_nlevels <levels> - number of levels to use
-.     -dmmg_galerkin - use Galerkin approach to compute coarser matrices
+.     -pc_mg_galerkin - use Galerkin approach to compute coarser matrices
 -     -dmmg_mat_type <type> - matrix type that DMMG should create, defaults to MATAIJ
 
     Notes:
@@ -37,16 +39,16 @@
 
     Level: advanced
 
-.seealso DMMGDestroy(), DMMGSetUser(), DMMGGetUser(), DMMGSetMatType(), DMMGSetUseGalerkin(), DMMGSetNullSpace(), DMMGSetInitialGuess(),
+.seealso DMMGDestroy(), DMMGSetUser(), DMMGGetUser(), DMMGSetMatType(),  DMMGSetNullSpace(), DMMGSetInitialGuess(),
          DMMGSetISColoringType()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGCreate(MPI_Comm comm,PetscInt nlevels,void *user,DMMG **dmmg)
+PetscErrorCode  DMMGCreate(MPI_Comm comm,PetscInt nlevels,void *user,DMMG **dmmg)
 {
   PetscErrorCode ierr;
   PetscInt       i;
   DMMG           *p;
-  PetscTruth     galerkin,ftype;
+  PetscBool      ftype;
   char           mtype[256];
 
   PetscFunctionBegin;
@@ -55,7 +57,7 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGCreate(MPI_Comm comm,PetscInt nlevels,voi
   } else {
     ierr = PetscOptionsGetInt(0,"-dmmg_nlevels",&nlevels,PETSC_IGNORE);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsHasName(0,"-dmmg_galerkin",&galerkin);CHKERRQ(ierr);
+  if (nlevels < 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_USER,"Cannot set levels less than 1");
 
   ierr = PetscMalloc(nlevels*sizeof(DMMG),&p);CHKERRQ(ierr);
   for (i=0; i<nlevels; i++) {
@@ -63,13 +65,11 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGCreate(MPI_Comm comm,PetscInt nlevels,voi
     p[i]->nlevels  = nlevels - i;
     p[i]->comm     = comm;
     p[i]->user     = user;
-    p[i]->galerkin = galerkin;
     p[i]->updatejacobianperiod = 1;
     p[i]->updatejacobian       = PETSC_TRUE;
-    p[i]->isctype  = IS_COLORING_GHOSTED;   /* default to faster version, requires DMMGSetSNESLocal() */
+    p[i]->isctype  = IS_COLORING_GLOBAL; 
     ierr           = PetscStrallocpy(MATAIJ,&p[i]->mtype);CHKERRQ(ierr);
   }
-  p[nlevels-1]->galerkin = PETSC_FALSE;
   *dmmg = p;
 
   ierr = PetscOptionsGetString(PETSC_NULL,"-dmmg_mat_type",mtype,256,&ftype);CHKERRQ(ierr);
@@ -84,7 +84,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGCreate(MPI_Comm comm,PetscInt nlevels,voi
 /*@C
     DMMGSetMatType - Sets the type of matrices that DMMG will create for its solvers.
 
-    Collective on MPI_Comm 
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on MPI_Comm 
 
     Input Parameters:
 +    dmmg - the DMMG object created with DMMGCreate()
@@ -95,13 +98,14 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGCreate(MPI_Comm comm,PetscInt nlevels,voi
 .seealso DMMGDestroy(), DMMGSetUser(), DMMGGetUser(), DMMGCreate(), DMMGSetNullSpace()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetMatType(DMMG *dmmg,const MatType mtype)
+PetscErrorCode  DMMGSetMatType(DMMG *dmmg,const MatType mtype)
 {
   PetscInt       i;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   for (i=0; i<dmmg[0]->nlevels; i++) {
+    ierr = PetscFree(dmmg[i]->mtype);CHKERRQ(ierr);
     ierr = PetscStrallocpy(mtype,&dmmg[i]->mtype);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -112,7 +116,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetMatType(DMMG *dmmg,const MatType mtype
 /*@C
     DMMGSetOptionsPrefix - Sets the prefix used for the solvers inside a DMMG
 
-    Collective on MPI_Comm 
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on MPI_Comm 
 
     Input Parameters:
 +    dmmg - the DMMG object created with DMMGCreate()
@@ -123,7 +130,7 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetMatType(DMMG *dmmg,const MatType mtype
 .seealso DMMGDestroy(), DMMGSetUser(), DMMGGetUser(), DMMGCreate(), DMMGSetNullSpace()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetOptionsPrefix(DMMG *dmmg,const char prefix[])
+PetscErrorCode  DMMGSetOptionsPrefix(DMMG *dmmg,const char prefix[])
 {
   PetscInt       i;
   PetscErrorCode ierr;
@@ -135,50 +142,13 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetOptionsPrefix(DMMG *dmmg,const char pr
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "DMMGSetUseGalerkinCoarse"
-/*@C
-    DMMGSetUseGalerkinCoarse - Courses the DMMG to use R*A_f*R^T to form
-       the coarser matrices from finest 
-
-    Collective on DMMG
-
-    Input Parameter:
-.    dmmg - the context
-
-    Options Database Keys:
-.    -dmmg_galerkin 
-
-    Level: advanced
-
-    Notes: After you have called this you can manually set dmmg[0]->galerkin = PETSC_FALSE
-       to have the coarsest grid not compute via Galerkin but still have the intermediate
-       grids computed via Galerkin.
-
-       The default behavior of this should be idential to using -pc_mg_galerkin; this offers
-       more potential flexibility since you can select exactly which levels are done via
-       Galerkin and which are done via user provided function.
-
-.seealso DMMGCreate(), PCMGSetUseGalerkin(), DMMGSetMatType(), DMMGSetNullSpace()
-
-@*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetUseGalerkinCoarse(DMMG* dmmg)
-{
-  PetscInt  i,nlevels = dmmg[0]->nlevels;
-
-  PetscFunctionBegin;
-  if (!dmmg) SETERRQ(PETSC_ERR_ARG_NULL,"Passing null as DMMG");
-
-  for (i=0; i<nlevels-1; i++) {
-    dmmg[i]->galerkin = PETSC_TRUE;
-  }
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "DMMGDestroy"
 /*@C
-    DMMGDestroy - Destroys a DA based multigrid solver object. 
+    DMMGDestroy - Destroys a DM based multigrid solver object. 
+
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
 
     Collective on DMMG
 
@@ -190,35 +160,35 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetUseGalerkinCoarse(DMMG* dmmg)
 .seealso DMMGCreate()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGDestroy(DMMG *dmmg)
+PetscErrorCode  DMMGDestroy(DMMG *dmmg)
 {
   PetscErrorCode ierr;
   PetscInt       i,nlevels = dmmg[0]->nlevels;
 
   PetscFunctionBegin;
-  if (!dmmg) SETERRQ(PETSC_ERR_ARG_NULL,"Passing null as DMMG");
+  if (!dmmg) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Passing null as DMMG");
 
   for (i=1; i<nlevels; i++) {
-    if (dmmg[i]->R) {ierr = MatDestroy(dmmg[i]->R);CHKERRQ(ierr);}
+    ierr = MatDestroy(&dmmg[i]->R);CHKERRQ(ierr);
   }
   for (i=0; i<nlevels; i++) {
-    ierr = PetscStrfree(dmmg[i]->prefix);CHKERRQ(ierr);
-    ierr = PetscStrfree(dmmg[i]->mtype);CHKERRQ(ierr);
-    if (dmmg[i]->dm)      {ierr = DMDestroy(dmmg[i]->dm);CHKERRQ(ierr);}
-    if (dmmg[i]->x)       {ierr = VecDestroy(dmmg[i]->x);CHKERRQ(ierr);}
-    if (dmmg[i]->b)       {ierr = VecDestroy(dmmg[i]->b);CHKERRQ(ierr);}
-    if (dmmg[i]->r)       {ierr = VecDestroy(dmmg[i]->r);CHKERRQ(ierr);}
-    if (dmmg[i]->work1)   {ierr = VecDestroy(dmmg[i]->work1);CHKERRQ(ierr);}
-    if (dmmg[i]->w)       {ierr = VecDestroy(dmmg[i]->w);CHKERRQ(ierr);}
-    if (dmmg[i]->work2)   {ierr = VecDestroy(dmmg[i]->work2);CHKERRQ(ierr);}
-    if (dmmg[i]->lwork1)  {ierr = VecDestroy(dmmg[i]->lwork1);CHKERRQ(ierr);}
-    if (dmmg[i]->B)         {ierr = MatDestroy(dmmg[i]->B);CHKERRQ(ierr);}
-    if (dmmg[i]->J)         {ierr = MatDestroy(dmmg[i]->J);CHKERRQ(ierr);}
-    if (dmmg[i]->Rscale)    {ierr = VecDestroy(dmmg[i]->Rscale);CHKERRQ(ierr);}
-    if (dmmg[i]->fdcoloring){ierr = MatFDColoringDestroy(dmmg[i]->fdcoloring);CHKERRQ(ierr);}
-    if (dmmg[i]->ksp && !dmmg[i]->snes) {ierr = KSPDestroy(dmmg[i]->ksp);CHKERRQ(ierr);}
-    if (dmmg[i]->snes)      {ierr = PetscObjectDestroy((PetscObject)dmmg[i]->snes);CHKERRQ(ierr);} 
-    if (dmmg[i]->inject)    {ierr = VecScatterDestroy(dmmg[i]->inject);CHKERRQ(ierr);} 
+    ierr = PetscFree(dmmg[i]->prefix);CHKERRQ(ierr);
+    ierr = PetscFree(dmmg[i]->mtype);CHKERRQ(ierr);
+    ierr = DMDestroy(&dmmg[i]->dm);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->x);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->b);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->r);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->work1);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->w);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->work2);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->lwork1);CHKERRQ(ierr);
+    ierr = MatDestroy(&dmmg[i]->B);CHKERRQ(ierr);
+    ierr = MatDestroy(&dmmg[i]->J);CHKERRQ(ierr);
+    ierr = VecDestroy(&dmmg[i]->Rscale);CHKERRQ(ierr);
+    ierr = MatFDColoringDestroy(&dmmg[i]->fdcoloring);CHKERRQ(ierr);
+    if (dmmg[i]->ksp && !dmmg[i]->snes) {ierr = KSPDestroy(&dmmg[i]->ksp);CHKERRQ(ierr);}
+    ierr = PetscObjectDestroy((PetscObject*)&dmmg[i]->snes);CHKERRQ(ierr);
+    ierr = VecScatterDestroy(&dmmg[i]->inject);CHKERRQ(ierr);
     ierr = PetscFree(dmmg[i]);CHKERRQ(ierr);
   }
   ierr = PetscFree(dmmg);CHKERRQ(ierr);
@@ -230,71 +200,57 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGDestroy(DMMG *dmmg)
 /*@C
     DMMGSetDM - Sets the coarse grid information for the grids
 
-    Collective on DMMG
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on DMMG and DM
 
     Input Parameter:
 +   dmmg - the context
--   dm - the DA or DMComposite object
+-   dm - the DMDA or DMComposite object
 
     Options Database Keys:
-+   -dmmg_refine: Use the input problem as the coarse level and refine. Otherwise, use it as the fine level and coarsen.
--   -dmmg_hierarchy: Construct all grids at once
+.   -dmmg_refine: Use the input problem as the coarse level and refine.
+.   -dmmg_refine false: Use the input problem as the fine level and coarsen.
 
     Level: advanced
 
-.seealso DMMGCreate(), DMMGDestroy(), DMMGSetUseGalerkin(), DMMGSetMatType()
+.seealso DMMGCreate(), DMMGDestroy(), DMMGSetMatType()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetDM(DMMG *dmmg, DM dm)
+PetscErrorCode  DMMGSetDM(DMMG *dmmg, DM dm)
 {
   PetscInt       nlevels     = dmmg[0]->nlevels;
-  PetscTruth     doRefine    = PETSC_TRUE;
-  PetscTruth     doHierarchy = PETSC_FALSE;
+  PetscBool      doRefine    = PETSC_TRUE;
   PetscInt       i;
+  DM             *hierarchy;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!dmmg) SETERRQ(PETSC_ERR_ARG_NULL,"Passing null as DMMG");
+  if (!dmmg) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Passing null as DMMG");
 
   /* Create DM data structure for all the levels */
-  ierr = PetscOptionsGetTruth(PETSC_NULL, "-dmmg_refine", &doRefine, PETSC_IGNORE);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(PETSC_NULL, "-dmmg_hierarchy", &doHierarchy);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(PETSC_NULL, "-dmmg_refine", &doRefine, PETSC_IGNORE);CHKERRQ(ierr);
   ierr = PetscObjectReference((PetscObject) dm);CHKERRQ(ierr);
+  ierr = PetscMalloc(nlevels*sizeof(DM),&hierarchy);CHKERRQ(ierr);
   if (doRefine) {
+    ierr = DMRefineHierarchy(dm,nlevels-1,hierarchy);CHKERRQ(ierr);
     dmmg[0]->dm = dm;
-    if (doHierarchy) {
-/*       DM *hierarchy; */
-
-/*       ierr = DMRefineHierarchy(dm, nlevels-1, &hierarchy);CHKERRQ(ierr); */
-/*       for(i = 1; i < nlevels; ++i) { */
-/*         dmmg[i]->dm = hierarchy[i-1]; */
-/*       } */
-      SETERRQ(PETSC_ERR_SUP, "Refinement hierarchy not yet implemented");
-    } else {
-      for(i = 1; i < nlevels; ++i) {
-        ierr = DMRefine(dmmg[i-1]->dm, dmmg[i]->comm, &dmmg[i]->dm);CHKERRQ(ierr);
-      }
+    for(i=1; i<nlevels; ++i) {
+      dmmg[i]->dm = hierarchy[i-1];
     }
   } else {
     dmmg[nlevels-1]->dm = dm;
-    if (doHierarchy) {
-      DM *hierarchy;
-
-      ierr = DMCoarsenHierarchy(dm, nlevels-1, &hierarchy);CHKERRQ(ierr);
-      for(i = 0; i < nlevels-1; ++i) {
-        dmmg[nlevels-2-i]->dm = hierarchy[i];
-      }
-    } else {
-/*       for(i = nlevels-2; i >= 0; --i) { */
-/*         ierr = DMCoarsen(dmmg[i+1]->dm, dmmg[i]->comm, &dmmg[i]->dm);CHKERRQ(ierr); */
-/*       } */
-      SETERRQ(PETSC_ERR_SUP, "Sequential coarsening not yet implemented");
+    ierr = DMCoarsenHierarchy(dm,nlevels-1,hierarchy);CHKERRQ(ierr);
+    for(i=0; i<nlevels-1; ++i) {
+      dmmg[nlevels-2-i]->dm = hierarchy[i];
     }
   }
+  ierr = PetscFree(hierarchy);CHKERRQ(ierr);
   /* Cleanup old structures (should use some private Destroy() instead) */
   for(i = 0; i < nlevels; ++i) {
-    if (dmmg[i]->B) {ierr = MatDestroy(dmmg[i]->B);CHKERRQ(ierr); dmmg[i]->B = PETSC_NULL;}
-    if (dmmg[i]->J) {ierr = MatDestroy(dmmg[i]->J);CHKERRQ(ierr); dmmg[i]->J = PETSC_NULL;}
+    ierr = MatDestroy(&dmmg[i]->B);CHKERRQ(ierr);
+    ierr = MatDestroy(&dmmg[i]->J);CHKERRQ(ierr);
   }
 
   /* Create work vectors and matrix for each level */
@@ -317,6 +273,9 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetDM(DMMG *dmmg, DM dm)
 /*@C
     DMMGSolve - Actually solves the (non)linear system defined with the DMMG
 
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
     Collective on DMMG
 
     Input Parameter:
@@ -335,15 +294,15 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetDM(DMMG *dmmg, DM dm)
 .seealso DMMGCreate(), DMMGDestroy(), DMMG, DMMGSetSNES(), DMMGSetKSP(), DMMGSetUp(), DMMGSetMatType()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSolve(DMMG *dmmg)
+PetscErrorCode  DMMGSolve(DMMG *dmmg)
 {
   PetscErrorCode ierr;
   PetscInt       i,nlevels = dmmg[0]->nlevels;
-  PetscTruth     gridseq,vecmonitor,flg;
+  PetscBool      gridseq = PETSC_FALSE,vecmonitor = PETSC_FALSE,flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHasName(0,"-dmmg_grid_sequence",&gridseq);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(0,"-dmmg_monitor_solution",&vecmonitor);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(0,"-dmmg_grid_sequence",&gridseq,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(0,"-dmmg_monitor_solution",&vecmonitor,PETSC_NULL);CHKERRQ(ierr);
   if (gridseq) {
     if (dmmg[0]->initialguess) {
       ierr = (*dmmg[0]->initialguess)(dmmg[0],dmmg[0]->x);CHKERRQ(ierr);
@@ -374,13 +333,15 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSolve(DMMG *dmmg)
      ierr = VecView(dmmg[nlevels-1]->x,PETSC_VIEWER_DRAW_(dmmg[nlevels-1]->comm));CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsHasName(PETSC_NULL,"-dmmg_view",&flg);CHKERRQ(ierr);
+  flg  = PETSC_FALSE;
+  ierr = PetscOptionsGetBool(PETSC_NULL,"-dmmg_view",&flg,PETSC_NULL);CHKERRQ(ierr);
   if (flg && !PetscPreLoadingOn) {
     PetscViewer viewer;
     ierr = PetscViewerASCIIGetStdout(dmmg[0]->comm,&viewer);CHKERRQ(ierr);
     ierr = DMMGView(dmmg,viewer);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsHasName(PETSC_NULL,"-dmmg_view_binary",&flg);CHKERRQ(ierr);
+  flg  = PETSC_FALSE;
+  ierr = PetscOptionsGetBool(PETSC_NULL,"-dmmg_view_binary",&flg,PETSC_NULL);CHKERRQ(ierr);
   if (flg && !PetscPreLoadingOn) {
     ierr = DMMGView(dmmg,PETSC_VIEWER_BINARY_(dmmg[0]->comm));CHKERRQ(ierr);
   }
@@ -389,7 +350,7 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSolve(DMMG *dmmg)
 
 #undef __FUNCT__  
 #define __FUNCT__ "DMMGSolveKSP"
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSolveKSP(DMMG *dmmg,PetscInt level)
+PetscErrorCode  DMMGSolveKSP(DMMG *dmmg,PetscInt level)
 {
   PetscErrorCode ierr;
 
@@ -413,17 +374,17 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSolveKSP(DMMG *dmmg,PetscInt level)
 */
 #undef __FUNCT__  
 #define __FUNCT__ "DMMGSetUpLevel"
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetUpLevel(DMMG *dmmg,KSP ksp,PetscInt nlevels)
+PetscErrorCode  DMMGSetUpLevel(DMMG *dmmg,KSP ksp,PetscInt nlevels)
 {
   PetscErrorCode          ierr;
   PetscInt                i;
   PC                      pc;
-  PetscTruth              ismg,ismf,isshell,ismffd;
+  PetscBool               ismg,ismf,isshell,ismffd;
   KSP                     lksp; /* solver internal to the multigrid preconditioner */
   MPI_Comm                *comms;
 
   PetscFunctionBegin;
-  if (!dmmg) SETERRQ(PETSC_ERR_ARG_NULL,"Passing null as DMMG");
+  if (!dmmg) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Passing null as DMMG");
 
   /* use fgmres on outer iteration by default */
   ierr  = KSPSetType(ksp,KSPFGMRES);CHKERRQ(ierr);
@@ -476,7 +437,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetUpLevel(DMMG *dmmg,KSP ksp,PetscInt nl
 /*@C
     DMMGSetKSP - Sets the linear solver object that will use the grid hierarchy
 
-    Collective on DMMG
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on DMMG
 
     Input Parameter:
 +   dmmg - the context
@@ -493,64 +457,55 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetUpLevel(DMMG *dmmg,KSP ksp,PetscInt nl
 .seealso DMMGCreate(), DMMGDestroy, DMMGSetDM(), DMMGSolve(), DMMGSetMatType()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetKSP(DMMG *dmmg,PetscErrorCode (*rhs)(DMMG,Vec),PetscErrorCode (*func)(DMMG,Mat,Mat))
+PetscErrorCode  DMMGSetKSP(DMMG *dmmg,PetscErrorCode (*rhs)(DMMG,Vec),PetscErrorCode (*func)(DMMG,Mat,Mat))
 {
   PetscErrorCode ierr;
   PetscInt       i,nlevels = dmmg[0]->nlevels,level;
-  PetscTruth     galerkin,ismg;
+  PetscBool      ismg,galerkin=PETSC_FALSE;
   PC             pc;
   KSP            lksp;
-
+  
   PetscFunctionBegin;
-  if (!dmmg) SETERRQ(PETSC_ERR_ARG_NULL,"Passing null as DMMG");
-  galerkin = dmmg[nlevels - 2 > 0 ? nlevels - 2 : 0]->galerkin;  
-
-  if (galerkin) {
-    ierr = DMGetMatrix(dmmg[nlevels-1]->dm,dmmg[nlevels-1]->mtype,&dmmg[nlevels-1]->B);CHKERRQ(ierr);
-    if (!dmmg[nlevels-1]->J) {
-      dmmg[nlevels-1]->J = dmmg[nlevels-1]->B;
-      ierr = PetscObjectReference((PetscObject) dmmg[nlevels-1]->J);CHKERRQ(ierr);
-    }
-    if (func) {
-      ierr = (*func)(dmmg[nlevels-1],dmmg[nlevels-1]->J,dmmg[nlevels-1]->B);CHKERRQ(ierr);
-    }
-    for (i=nlevels-2; i>-1; i--) {
-      if (dmmg[i]->galerkin) {
-        ierr = MatPtAP(dmmg[i+1]->B,dmmg[i+1]->R,MAT_INITIAL_MATRIX,1.0,&dmmg[i]->B);CHKERRQ(ierr);
-        if (!dmmg[i]->J) {
-          dmmg[i]->J = dmmg[i]->B;
-          ierr = PetscObjectReference((PetscObject) dmmg[i]->J);CHKERRQ(ierr);
-        }
-      }
-    }
-  }
+  if (!dmmg) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Passing null as DMMG");
 
   if (!dmmg[0]->ksp) {
     /* create solvers for each level if they don't already exist*/
     for (i=0; i<nlevels; i++) {
 
-      if (!dmmg[i]->B && !dmmg[i]->galerkin) {
-        ierr = DMGetMatrix(dmmg[i]->dm,dmmg[nlevels-1]->mtype,&dmmg[i]->B);CHKERRQ(ierr);
-      } 
-      if (!dmmg[i]->J) {
-        dmmg[i]->J = dmmg[i]->B;
-        ierr = PetscObjectReference((PetscObject) dmmg[i]->J);CHKERRQ(ierr);
-      }
-
       ierr = KSPCreate(dmmg[i]->comm,&dmmg[i]->ksp);CHKERRQ(ierr);
       ierr = PetscObjectIncrementTabLevel((PetscObject)dmmg[i]->ksp,PETSC_NULL,nlevels-i);CHKERRQ(ierr);
       ierr = KSPSetOptionsPrefix(dmmg[i]->ksp,dmmg[i]->prefix);CHKERRQ(ierr);
       ierr = DMMGSetUpLevel(dmmg,dmmg[i]->ksp,i+1);CHKERRQ(ierr);
       ierr = KSPSetFromOptions(dmmg[i]->ksp);CHKERRQ(ierr);
+
+      /*  if the multigrid is being run with Galerkin then these matrices do not need to be created except on the finest level
+          we do not take advantage of this because it may be that Galerkin has not yet been selected for the KSP object 
+          These are also used if grid sequencing is selected for the linear problem. We should probably turn off grid sequencing
+          for the linear problem */
+      if (!dmmg[i]->B) {
+	ierr = DMGetMatrix(dmmg[i]->dm,dmmg[nlevels-1]->mtype,&dmmg[i]->B);CHKERRQ(ierr);
+      } 
+      if (!dmmg[i]->J) {
+	dmmg[i]->J = dmmg[i]->B;
+	ierr = PetscObjectReference((PetscObject) dmmg[i]->J);CHKERRQ(ierr);
+      }
+
       dmmg[i]->solve = DMMGSolveKSP;
       dmmg[i]->rhs   = rhs;
     }
   }
 
   /* evalute matrix on each level */
-  for (i=0; i<nlevels; i++) {
-    if (!dmmg[i]->galerkin) {
-      if (func) {
+  ierr = KSPGetPC(dmmg[nlevels-1]->ksp,&pc);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)pc,PCMG,&ismg);CHKERRQ(ierr);
+  if (ismg) {
+    ierr = PCMGGetGalerkin(pc,&galerkin);CHKERRQ(ierr);
+  }
+  if (func) {
+    if (galerkin) {
+      ierr = (*func)(dmmg[nlevels-1],dmmg[nlevels-1]->J,dmmg[nlevels-1]->B);CHKERRQ(ierr);
+    } else {
+      for (i=0; i<nlevels; i++) {
         ierr = (*func)(dmmg[i],dmmg[i]->J,dmmg[i]->B);CHKERRQ(ierr);
       }
     }
@@ -563,7 +518,6 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetKSP(DMMG *dmmg,PetscErrorCode (*rhs)(D
   for (level=0; level<nlevels; level++) {
     ierr = KSPSetOperators(dmmg[level]->ksp,dmmg[level]->J,dmmg[level]->B,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
     ierr = KSPGetPC(dmmg[level]->ksp,&pc);CHKERRQ(ierr);
-    ierr = PetscTypeCompare((PetscObject)pc,PCMG,&ismg);CHKERRQ(ierr);
     if (ismg) {
       for (i=0; i<=level; i++) {
         ierr = PCMGGetSmoother(pc,i,&lksp);CHKERRQ(ierr); 
@@ -578,7 +532,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetKSP(DMMG *dmmg,PetscErrorCode (*rhs)(D
 #undef __FUNCT__  
 #define __FUNCT__ "DMMGView"
 /*@C
-    DMMGView - prints information on a DA based multi-level preconditioner
+    DMMGView - prints information on a DM based multi-level preconditioner
+
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
 
     Collective on DMMG and PetscViewer
 
@@ -588,28 +545,26 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetKSP(DMMG *dmmg,PetscErrorCode (*rhs)(D
 
     Level: advanced
 
-.seealso DMMGCreate(), DMMGDestroy(), DMMGSetMatType(), DMMGSetUseGalerkin()
+.seealso DMMGCreate(), DMMGDestroy(), DMMGSetMatType()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGView(DMMG *dmmg,PetscViewer viewer)
+PetscErrorCode  DMMGView(DMMG *dmmg,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscInt       i,nlevels = dmmg[0]->nlevels;
   PetscMPIInt    flag;
   MPI_Comm       comm;
-  PetscTruth     iascii,isbinary;
+  PetscBool      iascii,isbinary;
 
   PetscFunctionBegin;
   PetscValidPointer(dmmg,1);
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,2);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
   ierr = MPI_Comm_compare(comm,dmmg[0]->comm,&flag);CHKERRQ(ierr);
-  if (flag != MPI_CONGRUENT && flag != MPI_IDENT) {
-    SETERRQ(PETSC_ERR_ARG_NOTSAMECOMM,"Different communicators in the DMMG and the PetscViewer");
-  }
+  if (flag != MPI_CONGRUENT && flag != MPI_IDENT) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NOTSAMECOMM,"Different communicators in the DMMG and the PetscViewer");
 
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&iascii);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_BINARY,&isbinary);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
   if (isbinary) {
     for (i=0; i<nlevels; i++) {
       ierr = MatView(dmmg[i]->J,viewer);CHKERRQ(ierr);
@@ -620,6 +575,11 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGView(DMMG *dmmg,PetscViewer viewer)
   } else {
     if (iascii) {
       ierr = PetscViewerASCIIPrintf(viewer,"DMMG Object with %D levels\n",nlevels);CHKERRQ(ierr);
+      if (dmmg[0]->isctype == IS_COLORING_GLOBAL) {
+        ierr = PetscViewerASCIIPrintf(viewer,"Using global (nonghosted) Jacobian coloring computation\n");CHKERRQ(ierr);
+      } else {
+        ierr = PetscViewerASCIIPrintf(viewer,"Using ghosted Jacobian coloring computation\n");CHKERRQ(ierr);
+      }
     }
     for (i=0; i<nlevels; i++) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -627,17 +587,14 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGView(DMMG *dmmg,PetscViewer viewer)
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
     if (iascii) {
-      ierr = PetscViewerASCIIPrintf(viewer,"%s Object on finest level\n",dmmg[nlevels-1]->ksp ? "KSP" : "SNES");CHKERRQ(ierr);
-      if (dmmg[nlevels-2 > 0 ? nlevels-2 : 0]->galerkin) {
-	ierr = PetscViewerASCIIPrintf(viewer,"Using Galerkin R^T*A*R process to compute coarser matrices\n");CHKERRQ(ierr);
-      }
       ierr = PetscViewerASCIIPrintf(viewer,"Using matrix type %s\n",dmmg[nlevels-1]->mtype);CHKERRQ(ierr);
     }
-    if (dmmg[nlevels-1]->ksp) {
-      ierr = KSPView(dmmg[nlevels-1]->ksp,viewer);CHKERRQ(ierr);
-    } else {
-      /* use of PetscObjectView() means we do not have to link with libpetscsnes if SNES is not being used */
-      ierr = PetscObjectView((PetscObject)dmmg[nlevels-1]->snes,viewer);CHKERRQ(ierr);
+    if (DMMGGetKSP(dmmg)) {
+      ierr = KSPView(DMMGGetKSP(dmmg),viewer);CHKERRQ(ierr);
+    } else if (DMMGGetSNES(dmmg)) {
+      ierr = SNESView(DMMGGetSNES(dmmg),viewer);CHKERRQ(ierr);
+    } else if (iascii) {
+      ierr = PetscViewerASCIIPrintf(viewer,"DMMG does not have a SNES or KSP set\n");CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
@@ -648,7 +605,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGView(DMMG *dmmg,PetscViewer viewer)
 /*@C
     DMMGSetNullSpace - Indicates the null space in the linear operator (this is needed by the linear solver)
 
-    Collective on DMMG
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on DMMG
 
     Input Parameter:
 +   dmmg - the context
@@ -658,10 +618,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGView(DMMG *dmmg,PetscViewer viewer)
 
     Level: advanced
 
-.seealso DMMGCreate(), DMMGDestroy, DMMGSetDM(), DMMGSolve(), MatNullSpaceCreate(), KSPSetNullSpace(), DMMGSetUseGalerkin(), DMMGSetMatType()
+.seealso DMMGCreate(), DMMGDestroy, DMMGSetDM(), DMMGSolve(), MatNullSpaceCreate(), KSPSetNullSpace(), DMMGSetMatType()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetNullSpace(DMMG *dmmg,PetscTruth has_cnst,PetscInt n,PetscErrorCode (*func)(DMMG,Vec[]))
+PetscErrorCode  DMMGSetNullSpace(DMMG *dmmg,PetscBool  has_cnst,PetscInt n,PetscErrorCode (*func)(DMMG,Vec[]))
 {
   PetscErrorCode ierr;
   PetscInt       i,j,nlevels = dmmg[0]->nlevels;
@@ -669,13 +629,13 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetNullSpace(DMMG *dmmg,PetscTruth has_cn
   MatNullSpace   nullsp;
   KSP            iksp;
   PC             pc,ipc;
-  PetscTruth     ismg,isred;
+  PetscBool      ismg,isred;
 
   PetscFunctionBegin;
-  if (!dmmg) SETERRQ(PETSC_ERR_ARG_NULL,"Passing null as DMMG");
-  if (!dmmg[0]->ksp) SETERRQ(PETSC_ERR_ORDER,"Must call AFTER DMMGSetKSP() or DMMGSetSNES()");
-  if ((n && !func) || (!n && func)) SETERRQ(PETSC_ERR_ARG_INCOMP,"Both n and func() must be set together");
-  if (n < 0) SETERRQ1(PETSC_ERR_ARG_OUTOFRANGE,"Cannot have negative number of vectors in null space n = %D",n)
+  if (!dmmg) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Passing null as DMMG");
+  if (!dmmg[0]->ksp) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Must call AFTER DMMGSetKSP() or DMMGSetSNES()");
+  if ((n && !func) || (!n && func)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Both n and func() must be set together");
+  if (n < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Cannot have negative number of vectors in null space n = %D",n);
 
   for (i=0; i<nlevels; i++) {
     if (n) {
@@ -692,9 +652,9 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetNullSpace(DMMG *dmmg,PetscTruth has_cn
         ierr = KSPSetNullSpace(iksp, nullsp);CHKERRQ(ierr);
       }
     }
-    ierr = MatNullSpaceDestroy(nullsp);CHKERRQ(ierr);
+    ierr = MatNullSpaceDestroy(&nullsp);CHKERRQ(ierr);
     if (n) {
-      ierr = PetscFree(nulls);CHKERRQ(ierr);
+      ierr = VecDestroyVecs(n,&nulls);CHKERRQ(ierr);
     }
   }
   /* make all the coarse grid solvers have LU shift since they are singular */
@@ -706,9 +666,11 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetNullSpace(DMMG *dmmg,PetscTruth has_cn
       ierr = KSPGetPC(iksp,&ipc);CHKERRQ(ierr);
       ierr = PetscTypeCompare((PetscObject)ipc,PCREDUNDANT,&isred);CHKERRQ(ierr);
       if (isred) {
-        ierr = PCRedundantGetPC(ipc,&ipc);CHKERRQ(ierr);
+        KSP iksp;
+        ierr = PCRedundantGetKSP(ipc,&iksp);CHKERRQ(ierr);
+        ierr = KSPGetPC(iksp,&ipc);CHKERRQ(ierr);
       }
-      ierr = PCFactorSetShiftPd(ipc,PETSC_TRUE);CHKERRQ(ierr); 
+      ierr = PCFactorSetShiftType(ipc,MAT_SHIFT_POSITIVE_DEFINITE);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
@@ -722,7 +684,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetNullSpace(DMMG *dmmg,PetscTruth has_cn
        problems zero is used for the initial guess (unless grid sequencing is used). For nonlinear 
        problems this is not needed; it always uses the previous solution as the initial guess.
 
-    Collective on DMMG
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on DMMG
 
     Input Parameter:
 +   dmmg - the context
@@ -733,7 +698,7 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetNullSpace(DMMG *dmmg,PetscTruth has_cn
 .seealso DMMGCreate(), DMMGDestroy, DMMGSetKSP(), DMMGSetSNES(), DMMGSetInitialGuess()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGInitialGuessCurrent(DMMG dmmg,Vec vec)
+PetscErrorCode  DMMGInitialGuessCurrent(DMMG dmmg,Vec vec)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
@@ -744,7 +709,10 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGInitialGuessCurrent(DMMG dmmg,Vec vec)
 /*@C
     DMMGSetInitialGuess - Sets the function that computes an initial guess.
 
-    Collective on DMMG
+    This is being deprecated. Use KSPSetDM() for linear problems and SNESSetDM() for nonlinear problems. 
+    See src/ksp/ksp/examples/tutorials/ex45.c and src/snes/examples/tutorials/ex57.c 
+
+    Logically Collective on DMMG
 
     Input Parameter:
 +   dmmg - the context
@@ -766,7 +734,7 @@ PetscErrorCode PETSCSNES_DLLEXPORT DMMGInitialGuessCurrent(DMMG dmmg,Vec vec)
 .seealso DMMGCreate(), DMMGDestroy, DMMGSetKSP(), DMMGSetSNES(), DMMGInitialGuessCurrent(), DMMGSetGalekin(), DMMGSetMatType(), DMMGSetNullSpace()
 
 @*/
-PetscErrorCode PETSCSNES_DLLEXPORT DMMGSetInitialGuess(DMMG *dmmg,PetscErrorCode (*guess)(DMMG,Vec))
+PetscErrorCode  DMMGSetInitialGuess(DMMG *dmmg,PetscErrorCode (*guess)(DMMG,Vec))
 {
   PetscInt       i,nlevels = dmmg[0]->nlevels;
   PetscErrorCode ierr;
