@@ -1,8 +1,8 @@
-#define PETSC_DLL
+
 /*
        Provides the calling sequences for all the basic PetscDraw routines.
 */
-#include "../src/sys/draw/drawimpl.h"  /*I "petscdraw.h" I*/
+#include <../src/sys/draw/drawimpl.h>  /*I "petscdraw.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscDrawFlush" 
@@ -20,11 +20,11 @@
 
 .seealso: PetscDrawSynchronizedFlush()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscDrawFlush(PetscDraw draw)
+PetscErrorCode  PetscDrawFlush(PetscDraw draw)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(draw,PETSC_DRAW_COOKIE,1);
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   if (draw->ops->flush) {
     ierr = (*draw->ops->flush)(draw);CHKERRQ(ierr);
   }
