@@ -1,8 +1,8 @@
-#define PETSC_DLL
+
 /*
        Provides the calling sequences for all the basic PetscDraw routines.
 */
-#include "../src/sys/draw/drawimpl.h"  /*I "petscdraw.h" I*/
+#include <../src/sys/draw/drawimpl.h>  /*I "petscdraw.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscDrawPause" 
@@ -23,11 +23,11 @@
 
 .seealso: PetscDrawSetPause(), PetscDrawGetPause()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscDrawPause(PetscDraw draw)
+PetscErrorCode  PetscDrawPause(PetscDraw draw)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(draw,PETSC_DRAW_COOKIE,1);
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   if (draw->ops->pause) {
     ierr = (*draw->ops->pause)(draw);CHKERRQ(ierr);
   }
