@@ -1,14 +1,17 @@
-#define PETSC_DLL
+
 /*
        Provides the calling sequences for all the basic PetscDraw routines.
 */
-#include "../src/sys/draw/drawimpl.h"  /*I "petscdraw.h" I*/
+#include <../src/sys/draw/drawimpl.h>  /*I "petscdraw.h" I*/
 
 EXTERN_C_BEGIN
-EXTERN PetscErrorCode PetscDrawCreate_X(PetscDraw);
-EXTERN PetscErrorCode PetscDrawCreate_PS(PetscDraw);
-EXTERN PetscErrorCode PetscDrawCreate_Null(PetscDraw);
-EXTERN PetscErrorCode PetscDrawCreate_Win32(PetscDraw);
+#if defined(PETSC_HAVE_X11)
+extern PetscErrorCode PetscDrawCreate_X(PetscDraw);
+#endif
+extern PetscErrorCode PetscDrawCreate_Null(PetscDraw);
+#if defined(PETSC_USE_WINDOWS_GRAPHICS)
+extern PetscErrorCode PetscDrawCreate_Win32(PetscDraw);
+#endif
 EXTERN_C_END
   
 #undef __FUNCT__  
@@ -22,7 +25,7 @@ EXTERN_C_END
 
 .seealso:  PetscDrawRegisterDestroy()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscDrawRegisterAll(const char *path)
+PetscErrorCode  PetscDrawRegisterAll(const char *path)
 {
   PetscErrorCode ierr;
 
@@ -30,11 +33,10 @@ PetscErrorCode PETSC_DLLEXPORT PetscDrawRegisterAll(const char *path)
   
 #if defined(PETSC_HAVE_X11)
   ierr = PetscDrawRegisterDynamic(PETSC_DRAW_X,     path,"PetscDrawCreate_X",     PetscDrawCreate_X);CHKERRQ(ierr);
-#elif defined(PETSC_HAVE_WINDOWS_H)
+#elif defined(PETSC_USE_WINDOWS_GRAPHICS)
   ierr = PetscDrawRegisterDynamic(PETSC_DRAW_WIN32, path,"PetscDrawCreate_Win32", PetscDrawCreate_Win32);CHKERRQ(ierr);
 #endif
   ierr = PetscDrawRegisterDynamic(PETSC_DRAW_NULL,  path,"PetscDrawCreate_Null",  PetscDrawCreate_Null);CHKERRQ(ierr);
-  ierr = PetscDrawRegisterDynamic(PETSC_DRAW_PS,    path,"PetscDrawCreate_PS",    PetscDrawCreate_PS);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
