@@ -1,5 +1,4 @@
-#include "zpetsc.h"
-#include "petscsys.h"
+#include <private/fortranimpl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscdrawaxissetlabels_   PETSCDRAWAXISSETLABELS
