@@ -2,14 +2,14 @@
    Demonstrates PETSc error handlers.
  */
 
-#include "petscsys.h"
+#include <petscsys.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "CreateError"
 int CreateError(int n)
 {
   PetscErrorCode ierr;
-  if (!n) SETERRQ(1,"Error Created");
+  if (!n) SETERRQ(PETSC_COMM_SELF,1,"Error Created");
   ierr = CreateError(n-1);CHKERRQ(ierr);
   return 0;
 }
@@ -24,7 +24,7 @@ int main(int argc,char **argv)
   ierr = PetscFPrintf(PETSC_COMM_WORLD,stdout,"The error is a contrived error to test error handling\n");CHKERRQ(ierr);
   ierr = PetscSynchronizedFlush(PETSC_COMM_WORLD);CHKERRQ(ierr);
   ierr = CreateError(5);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
  
