@@ -3,7 +3,7 @@
    Tests PetscOptionsGetScalar() for complex numbers
  */
 
-#include "petscsys.h"
+#include <petscsys.h>
 
 
 #undef __FUNCT__
@@ -16,7 +16,7 @@ int main(int argc,char **argv)
   PetscInitialize(&argc,&argv,(char *)0,0);
   ierr = PetscOptionsGetScalar(PETSC_NULL,"-a",&a,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_SELF,"Scalar a = %G + %Gi\n",PetscRealPart(a),PetscImaginaryPart(a));CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
  
