@@ -3,7 +3,7 @@
    Demonstrates PetscPopUpSelect()
  */
 
-#include "petscsys.h"
+#include <petscsys.h>
 
 
 #undef __FUNCT__
@@ -17,7 +17,7 @@ int main(int argc,char **argv)
   ierr = PetscPopUpSelect(PETSC_COMM_WORLD,PETSC_NULL,"Select one of ",2,choices,&choice);CHKERRQ(ierr);
   ierr = PetscSynchronizedPrintf(PETSC_COMM_WORLD,"You selected %s\n",choices[choice]);CHKERRQ(ierr);
   ierr = PetscSynchronizedFlush(PETSC_COMM_WORLD);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
  
