@@ -1,7 +1,7 @@
 
 static char help[] = "Tests string options with spaces";
 
-#include "petsc.h"
+#include <petscsys.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -9,7 +9,7 @@ int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
   char           option2[20],option3[30];
-  PetscTruth     flg;
+  PetscBool      flg;
   PetscInt       option1;
 
   PetscInitialize(&argc,&argv,"ex19options",help);
@@ -18,6 +18,6 @@ int main(int argc,char **argv)
   ierr = PetscPrintf(PETSC_COMM_WORLD,"%s\n",option2);CHKERRQ(ierr);
   ierr = PetscOptionsGetString(0,"-option3",option3,30,&flg);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"%s\n",option3);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
