@@ -9,7 +9,7 @@ static char help[] = "Introductory example that illustrates printing.\n\n";
    Processors: n
 T*/
  
-#include "petscsys.h"
+#include <petscsys.h>
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
@@ -62,6 +62,6 @@ int main(int argc,char **argv)
          options are chosen (e.g., -log_summary).  See PetscFinalize()
      manpage for more information.
   */
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
