@@ -1,5 +1,5 @@
-#include "private/fortranimpl.h"
-#include "petsc.h"
+#include <private/fortranimpl.h>
+#include <petscsys.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscfixfilename_          PETSCFIXFILENAME
