@@ -2,12 +2,11 @@
 #if !defined(__MPRINT_H)
 #define __MPRINT_H
 
-#include "petscsys.h"             /*I    "petscsys.h"   I*/
+#include <petscsys.h>             /*I    "petscsys.h"   I*/
 #include <stdarg.h>
 #if defined(PETSC_HAVE_STDLIB_H)
 #include <stdlib.h>
 #endif
-#include "petscfix.h"
 
 
 /* ----------------------------------------------------------------------- */
