@@ -1,14 +1,17 @@
-#define PETSC_DLL
+
 /*
      Provides utility routines for manipulating any type of PETSc object.
 */
-#include "petscsys.h"  /*I   "petscsys.h"    I*/
+#include <petscsys.h>  /*I   "petscsys.h"    I*/
 
-EXTERN PetscErrorCode PetscObjectGetComm_Petsc(PetscObject,MPI_Comm *);
-EXTERN PetscErrorCode PetscObjectCompose_Petsc(PetscObject,const char[],PetscObject);
-EXTERN PetscErrorCode PetscObjectQuery_Petsc(PetscObject,const char[],PetscObject *);
-EXTERN PetscErrorCode PetscObjectComposeFunction_Petsc(PetscObject,const char[],const char[],void (*)(void));
-EXTERN PetscErrorCode PetscObjectQueryFunction_Petsc(PetscObject,const char[],void (**)(void));
+PetscObject *PetscObjects = 0;
+PetscInt    PetscObjectsCounts = 0, PetscObjectsMaxCounts = 0;
+
+extern PetscErrorCode PetscObjectGetComm_Petsc(PetscObject,MPI_Comm *);
+extern PetscErrorCode PetscObjectCompose_Petsc(PetscObject,const char[],PetscObject);
+extern PetscErrorCode PetscObjectQuery_Petsc(PetscObject,const char[],PetscObject *);
+extern PetscErrorCode PetscObjectComposeFunction_Petsc(PetscObject,const char[],const char[],void (*)(void));
+extern PetscErrorCode PetscObjectQueryFunction_Petsc(PetscObject,const char[],void (**)(void));
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscHeaderCreate_Private"
@@ -16,16 +19,20 @@ EXTERN PetscErrorCode PetscObjectQueryFunction_Petsc(PetscObject,const char[],vo
    PetscHeaderCreate_Private - Creates a base PETSc object header and fills
    in the default values.  Called by the macro PetscHeaderCreate().
 */
-PetscErrorCode PETSC_DLLEXPORT PetscHeaderCreate_Private(PetscObject h,PetscCookie cookie,PetscInt type,const char class_name[],MPI_Comm comm,
-                                         PetscErrorCode (*des)(PetscObject),PetscErrorCode (*vie)(PetscObject,PetscViewer))
+PetscErrorCode  PetscHeaderCreate_Private(PetscObject h,PetscClassId classid,PetscInt type,const char class_name[],const char descr[],const char mansec[],
+                                          MPI_Comm comm,PetscErrorCode (*des)(PetscObject*),PetscErrorCode (*vie)(PetscObject,PetscViewer))
 {
   static PetscInt idcnt = 1;
   PetscErrorCode  ierr;
+  PetscObject     *newPetscObjects;
+  PetscInt         newPetscObjectsMaxCounts,i;
 
   PetscFunctionBegin;
-  h->cookie                 = cookie;
+  h->classid                = classid;
   h->type                   = type;
   h->class_name             = (char*)class_name;
+  h->description            = (char*)descr;
+  h->mansec                 = (char*)mansec;
   h->prefix                 = 0;
   h->refct                  = 1;
   h->amem                   = -1;
@@ -33,6 +40,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscHeaderCreate_Private(PetscObject h,PetscCook
   h->parentid               = 0;
   h->qlist                  = 0;
   h->olist                  = 0;
+  h->precision              = (PetscPrecision) sizeof(PetscScalar);
   h->bops->destroy          = des;
   h->bops->view             = vie;
   h->bops->getcomm          = PetscObjectGetComm_Petsc;
@@ -41,10 +49,30 @@ PetscErrorCode PETSC_DLLEXPORT PetscHeaderCreate_Private(PetscObject h,PetscCook
   h->bops->composefunction  = PetscObjectComposeFunction_Petsc;
   h->bops->queryfunction    = PetscObjectQueryFunction_Petsc;
   ierr = PetscCommDuplicate(comm,&h->comm,&h->tag);CHKERRQ(ierr);
+
+  /* Keep a record of object created */
+  PetscObjectsCounts++;
+  for (i=0; i<PetscObjectsMaxCounts; i++) {
+    if (!PetscObjects[i]) {
+      PetscObjects[i] = h;
+      PetscFunctionReturn(0);
+    }
+  }
+  /* Need to increase the space for storing PETSc objects */
+  if (!PetscObjectsMaxCounts) newPetscObjectsMaxCounts = 100;
+  else                        newPetscObjectsMaxCounts = 2*PetscObjectsMaxCounts;
+  ierr = PetscMalloc(newPetscObjectsMaxCounts*sizeof(PetscObject),&newPetscObjects);CHKERRQ(ierr);
+  ierr = PetscMemcpy(newPetscObjects,PetscObjects,PetscObjectsMaxCounts*sizeof(PetscObject));CHKERRQ(ierr);
+  ierr = PetscMemzero(newPetscObjects+PetscObjectsMaxCounts,(newPetscObjectsMaxCounts - PetscObjectsMaxCounts)*sizeof(PetscObject));CHKERRQ(ierr);
+  ierr = PetscFree(PetscObjects);CHKERRQ(ierr);
+  PetscObjects                        = newPetscObjects;
+  PetscObjects[PetscObjectsMaxCounts] = h;
+  PetscObjectsMaxCounts               = newPetscObjectsMaxCounts;
+
   PetscFunctionReturn(0);
 }
 
-extern PetscTruth     PetscMemoryCollectMaximumUsage;
+extern PetscBool      PetscMemoryCollectMaximumUsage;
 extern PetscLogDouble PetscMemoryMaximumUsage;
 
 #undef __FUNCT__  
@@ -53,11 +81,17 @@ extern PetscLogDouble PetscMemoryMaximumUsage;
     PetscHeaderDestroy_Private - Destroys a base PETSc object header. Called by 
     the macro PetscHeaderDestroy().
 */
-PetscErrorCode PETSC_DLLEXPORT PetscHeaderDestroy_Private(PetscObject h)
+PetscErrorCode  PetscHeaderDestroy_Private(PetscObject h)
 {
   PetscErrorCode ierr;
+  PetscInt       i;
 
   PetscFunctionBegin;
+#if defined(PETSC_HAVE_AMS)
+  if (PetscAMSPublishAll) {
+    ierr = PetscObjectUnPublish((PetscObject)h);CHKERRQ(ierr);
+  }
+#endif
   if (PetscMemoryCollectMaximumUsage) {
     PetscLogDouble usage;
     ierr = PetscMemoryGetCurrentUsage(&usage);CHKERRQ(ierr);
@@ -71,33 +105,260 @@ PetscErrorCode PETSC_DLLEXPORT PetscHeaderDestroy_Private(PetscObject h)
     h->python_destroy = 0;
     ierr = (*python_destroy)(python_context);CHKERRQ(ierr);
   }
-  ierr = PetscOListDestroy(h->olist);CHKERRQ(ierr);
+  ierr = PetscOListDestroy(&h->olist);CHKERRQ(ierr);
   ierr = PetscCommDestroy(&h->comm);CHKERRQ(ierr);
   /* next destroy other things */
-  h->cookie = PETSCFREEDHEADER;
+  h->classid = PETSCFREEDHEADER;
   ierr = PetscFree(h->bops);CHKERRQ(ierr);
   ierr = PetscFListDestroy(&h->qlist);CHKERRQ(ierr);
-  ierr = PetscStrfree(h->type_name);CHKERRQ(ierr);
-  ierr = PetscStrfree(h->name);CHKERRQ(ierr);
-  ierr = PetscStrfree(h->prefix);CHKERRQ(ierr);
+  ierr = PetscFree(h->type_name);CHKERRQ(ierr);
+  ierr = PetscFree(h->name);CHKERRQ(ierr);
+  ierr = PetscFree(h->prefix);CHKERRQ(ierr);
   ierr = PetscFree(h->fortran_func_pointers);CHKERRQ(ierr);
-  ierr = PetscFree(h->intcomposeddata);CHKERRQ(ierr);
-  ierr = PetscFree(h->intcomposedstate);CHKERRQ(ierr);
-  ierr = PetscFree(h->realcomposeddata);CHKERRQ(ierr);
-  ierr = PetscFree(h->realcomposedstate);CHKERRQ(ierr);
-  ierr = PetscFree(h->scalarcomposeddata);CHKERRQ(ierr);
-  ierr = PetscFree(h->scalarcomposedstate);CHKERRQ(ierr);
+
+  /* Record object removal from list of all objects */
+  for (i=0; i<PetscObjectsMaxCounts; i++) {
+    if (PetscObjects[i] == h) {
+      PetscObjects[i] = 0;
+      PetscObjectsCounts--;
+      break;
+    }
+  }
+  if (!PetscObjectsCounts) {
+    ierr = PetscFree(PetscObjects);CHKERRQ(ierr);
+    PetscObjectsMaxCounts = 0;
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "PetscObjectCopyFortranFunctionPointers"
+/*@C
+   PetscObjectCopyFortranFunctionPointers - Copy function pointers to another object
+
+   Logically Collective on PetscObject
+
+   Input Parameter:
++  src - source object
+-  dest - destination object
+
+   Level: developer
+
+   Note:
+   Both objects must have the same class.
+@*/
+PetscErrorCode PetscObjectCopyFortranFunctionPointers(PetscObject src,PetscObject dest)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeader(src,1);
+  PetscValidHeader(dest,2);
+  if (src->classid != dest->classid) SETERRQ(src->comm,PETSC_ERR_ARG_INCOMP,"Objects must be of the same class");
+
+  ierr = PetscFree(dest->fortran_func_pointers);CHKERRQ(ierr);
+  ierr = PetscMalloc(src->num_fortran_func_pointers*sizeof(void(*)(void)),&dest->fortran_func_pointers);CHKERRQ(ierr);
+  ierr = PetscMemcpy(dest->fortran_func_pointers,src->fortran_func_pointers,src->num_fortran_func_pointers*sizeof(void(*)(void)));CHKERRQ(ierr);
+  dest->num_fortran_func_pointers = src->num_fortran_func_pointers;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "PetscObjectReference"
+#define __FUNCT__ "PetscObjectsView"
 /*@C
+   PetscObjectsView - Prints the currently existing objects.
+
+   Logically Collective on PetscViewer
+
+   Input Parameter:
+.  viewer - must be an PETSCVIEWERASCII viewer
+
+   Level: advanced
+
+   Concepts: options database^printing
+
+@*/
+PetscErrorCode  PetscObjectsView(PetscViewer viewer) 
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscBool      isascii;
+  PetscObject    h;
+
+  PetscFunctionBegin;
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_WORLD;
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (!isascii) SETERRQ(((PetscObject)viewer)->comm,PETSC_ERR_SUP,"Only supports ASCII viewer");
+
+  for (i=0; i<PetscObjectsMaxCounts; i++) {
+    if ((h = PetscObjects[i])) {
+      ierr = PetscObjectName(h);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"%s %s %s\n",h->class_name,h->type_name,h->name);CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectsGetObject"
+/*@C
+   PetscObjectsGetObject - Get a pointer to a named object
+
+   Not collective
+
+   Input Parameter:
+.  name - the name of an object
+
+   Output Parameter:
+.   obj - the object or null if there is no object
+
+   Level: advanced
+
+   Concepts: options database^printing
+
+@*/
+PetscErrorCode  PetscObjectsGetObject(const char* name,PetscObject *obj,char **classname) 
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscObject    h;
+  PetscBool      flg;
+
+  PetscFunctionBegin;
+  *obj = PETSC_NULL;
+  for (i=0; i<PetscObjectsMaxCounts; i++) {
+    if ((h = PetscObjects[i])) {
+      ierr = PetscObjectName(h);CHKERRQ(ierr);
+      ierr = PetscStrcmp(h->name,name,&flg);CHKERRQ(ierr);
+      if (flg) {
+        *obj = h;
+        if (classname) *classname = h->class_name;
+        PetscFunctionReturn(0);
+      }
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectsGetObjectMatlab"
+char* PetscObjectsGetObjectMatlab(const char* name,PetscObject *obj) 
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscObject    h;
+  PetscBool      flg;
+
+  PetscFunctionBegin;
+  *obj = PETSC_NULL;
+  for (i=0; i<PetscObjectsMaxCounts; i++) {
+    if ((h = PetscObjects[i])) {
+      ierr = PetscObjectName(h);if (ierr) PetscFunctionReturn(0);
+      ierr = PetscStrcmp(h->name,name,&flg);if (ierr) PetscFunctionReturn(0);
+      if (flg) {
+        *obj = h;
+        PetscFunctionReturn(h->class_name);
+      }
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectAddOptionsHandler"
+/*@C
+    PetscObjectAddOptionsHandler - Adds an additional function to check for options when XXXSetFromOptions() is called.
+
+    Not Collective
+
+    Input Parameter:
++   obj - the PETSc object
+.   handle - function that checks for options
+.   destroy - function to destroy context if provided
+-   ctx - optional context for check function
+
+    Level: developer
+
+
+.seealso: KSPSetFromOptions(), PCSetFromOptions(), SNESSetFromOptions(), PetscObjectProcessOptionsHandlers(), PetscObjectDestroyOptionsHandlers()
+
+@*/
+PetscErrorCode  PetscObjectAddOptionsHandler(PetscObject obj,PetscErrorCode (*handle)(PetscObject,void*),PetscErrorCode (*destroy)(PetscObject,void*),void *ctx)
+{
+  PetscFunctionBegin;
+  if (obj->noptionhandler >= PETSC_MAX_OPTIONS_HANDLER) SETERRQ(obj->comm,PETSC_ERR_ARG_OUTOFRANGE,"To many options handlers added");
+  obj->optionhandler[obj->noptionhandler]   = handle;
+  obj->optiondestroy[obj->noptionhandler]   = destroy;
+  obj->optionctx[obj->noptionhandler++]     = ctx;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectProcessOptionsHandlers"
+/*@C
+    PetscObjectProcessOptionsHandlers - Calls all the options handler attached to an object
+
+    Not Collective
+
+    Input Parameter:
+.   obj - the PETSc object
+
+    Level: developer
+
+
+.seealso: KSPSetFromOptions(), PCSetFromOptions(), SNESSetFromOptions(), PetscObjectAddOptionsHandler(), PetscObjectDestroyOptionsHandlers()
+
+@*/
+PetscErrorCode  PetscObjectProcessOptionsHandlers(PetscObject obj)
+{
+  PetscInt       i;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  for (i=0; i<obj->noptionhandler; i++) {
+    ierr = (*obj->optionhandler[i])(obj,obj->optionctx[i]);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectDestroyOptionsHandlers"
+/*@C
+    PetscObjectDestroyOptionsHandlers - Destroys all the option handlers attached to an objeft
+
+    Not Collective
+
+    Input Parameter:
+.   obj - the PETSc object
+
+    Level: developer
+
+
+.seealso: KSPSetFromOptions(), PCSetFromOptions(), SNESSetFromOptions(), PetscObjectAddOptionsHandler(), PetscObjectProcessOptionsHandlers()
+
+@*/
+PetscErrorCode  PetscObjectDestroyOptionsHandlers(PetscObject obj)
+{
+  PetscInt       i;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  for (i=0; i<obj->noptionhandler; i++) {
+    ierr = (*obj->optiondestroy[i])(obj,obj->optionctx[i]);CHKERRQ(ierr);
+  }
+  obj->noptionhandler = 0;
+  PetscFunctionReturn(0);
+}
+
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectReference"
+/*@
    PetscObjectReference - Indicates to any PetscObject that it is being
    referenced by another PetscObject. This increases the reference
    count for that object by one.
 
-   Collective on PetscObject
+   Logically Collective on PetscObject
 
    Input Parameter:
 .  obj - the PETSc object. This must be cast with (PetscObject), for example, 
@@ -107,7 +368,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscHeaderDestroy_Private(PetscObject h)
 
 .seealso: PetscObjectCompose(), PetscObjectDereference()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectReference(PetscObject obj)
+PetscErrorCode  PetscObjectReference(PetscObject obj)
 {
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
@@ -117,7 +378,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectReference(PetscObject obj)
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscObjectGetReference"
-/*@C
+/*@
    PetscObjectGetReference - Gets the current reference count for 
    any PETSc object.
 
@@ -134,7 +395,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectReference(PetscObject obj)
 
 .seealso: PetscObjectCompose(), PetscObjectDereference(), PetscObjectReference()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectGetReference(PetscObject obj,PetscInt *cnt)
+PetscErrorCode  PetscObjectGetReference(PetscObject obj,PetscInt *cnt)
 {
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
@@ -150,27 +411,27 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectGetReference(PetscObject obj,PetscInt
    referenced by one less PetscObject. This decreases the reference
    count for that object by one.
 
-   Collective on PetscObject
+   Collective on PetscObject if reference reaches 0 otherwise Logically Collective
 
    Input Parameter:
 .  obj - the PETSc object; this must be cast with (PetscObject), for example, 
          PetscObjectDereference((PetscObject)mat);
 
+   Notes: PetscObjectDestroy(PetscObject *obj)  sets the obj pointer to null after the call, this routine does not.
+
    Level: advanced
 
 .seealso: PetscObjectCompose(), PetscObjectReference()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectDereference(PetscObject obj)
+PetscErrorCode  PetscObjectDereference(PetscObject obj)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
   if (obj->bops->destroy) {
-    ierr = (*obj->bops->destroy)(obj);CHKERRQ(ierr);
-  } else if (!--obj->refct) {
-    SETERRQ(PETSC_ERR_SUP,"This PETSc object does not have a generic destroy routine");
-  }
+    ierr = (*obj->bops->destroy)(&obj);CHKERRQ(ierr);
+  } else if (!--obj->refct) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This PETSc object does not have a generic destroy routine");
   PetscFunctionReturn(0);
 }
 
@@ -188,19 +449,29 @@ PetscErrorCode PetscObjectGetComm_Petsc(PetscObject obj,MPI_Comm *comm)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectRemoveReference"
+PetscErrorCode PetscObjectRemoveReference(PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscOListRemoveReference(&obj->olist,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "PetscObjectCompose_Petsc"
 PetscErrorCode PetscObjectCompose_Petsc(PetscObject obj,const char name[],PetscObject ptr)
 {
   PetscErrorCode ierr;
   char           *tname;
+  PetscBool      skipreference;
 
   PetscFunctionBegin;
   if (ptr) {
-    ierr = PetscOListReverseFind(ptr->olist,obj,&tname);CHKERRQ(ierr);
-    if (tname){
-      SETERRQ(PETSC_ERR_ARG_INCOMP,"An object cannot be composed with an object that was compose with it");
-    }
+    ierr = PetscOListReverseFind(ptr->olist,obj,&tname,&skipreference);CHKERRQ(ierr);
+    if (tname && !skipreference) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"An object cannot be composed with an object that was compose with it");
   }
   ierr = PetscOListAdd(&obj->olist,name,ptr);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -235,13 +506,10 @@ PetscErrorCode PetscObjectQueryFunction_Petsc(PetscObject obj,const char name[],
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscFListFind(obj->qlist,obj->comm,name,ptr);CHKERRQ(ierr);
+  ierr = PetscFListFind(obj->qlist,obj->comm,name,PETSC_FALSE,ptr);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-/*
-        These are the versions that are usable to any CCA compliant objects
-*/
 #undef __FUNCT__  
 #define __FUNCT__ "PetscObjectCompose"
 /*@C
@@ -277,7 +545,7 @@ PetscErrorCode PetscObjectQueryFunction_Petsc(PetscObject obj,const char name[],
 
 .seealso: PetscObjectQuery(), PetscContainerCreate()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectCompose(PetscObject obj,const char name[],PetscObject ptr)
+PetscErrorCode  PetscObjectCompose(PetscObject obj,const char name[],PetscObject ptr)
 {
   PetscErrorCode ierr;
 
@@ -289,7 +557,29 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectCompose(PetscObject obj,const char nam
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "PetscObjectSetPrecision"
+/*@C
+   PetscObjectSetPrecision - sets the precision used within a given object.
+                       
+   Collective on the PetscObject
 
+   Input Parameters:
++  obj - the PETSc object; this must be cast with (PetscObject), for example, 
+         PetscObjectCompose((PetscObject)mat,...);
+-  precision - the precision
+
+   Level: advanced
+
+.seealso: PetscObjectQuery(), PetscContainerCreate()
+@*/
+PetscErrorCode  PetscObjectSetPrecision(PetscObject obj,PetscPrecision precision)
+{
+  PetscFunctionBegin;
+  PetscValidHeader(obj,1);
+  obj->precision = precision;
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscObjectQuery"
@@ -303,19 +593,21 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectCompose(PetscObject obj,const char nam
          Thus must be cast with a (PetscObject), for example, 
          PetscObjectCompose((PetscObject)mat,...);
 .  name - name associated with child object 
--  ptr - the other PETSc object associated with the PETSc object, this must also be 
-         cast with (PetscObject)
+-  ptr - the other PETSc object associated with the PETSc object, this must be 
+         cast with (PetscObject *)
 
    Level: advanced
 
+   The reference count of neither object is increased in this call
+
    Concepts: objects^composing
    Concepts: composing objects
    Concepts: objects^querying
    Concepts: querying objects
 
-.seealso: PetscObjectQuery()
+.seealso: PetscObjectCompose()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectQuery(PetscObject obj,const char name[],PetscObject *ptr)
+PetscErrorCode  PetscObjectQuery(PetscObject obj,const char name[],PetscObject *ptr)
 {
   PetscErrorCode ierr;
 
@@ -329,14 +621,14 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectQuery(PetscObject obj,const char name[
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscObjectComposeFunction"
-PetscErrorCode PETSC_DLLEXPORT PetscObjectComposeFunction(PetscObject obj,const char name[],const char fname[],void (*ptr)(void))
+PetscErrorCode  PetscObjectComposeFunction(PetscObject obj,const char name[],const char fname[],void (*ptr)(void))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
   PetscValidCharPointer(name,2);
-  PetscValidCharPointer(fname,2);
+  PetscValidCharPointer(fname,3);
   ierr = (*obj->bops->composefunction)(obj,name,fname,ptr);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -346,7 +638,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectComposeFunction(PetscObject obj,const
 /*@C
    PetscObjectQueryFunction - Gets a function associated with a given object.
                        
-   Collective on PetscObject
+   Logically Collective on PetscObject
 
    Input Parameters:
 +  obj - the PETSc object; this must be cast with (PetscObject), for example, 
@@ -366,7 +658,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectComposeFunction(PetscObject obj,const
 
 .seealso: PetscObjectComposeFunctionDynamic()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectQueryFunction(PetscObject obj,const char name[],void (**ptr)(void))
+PetscErrorCode  PetscObjectQueryFunction(PetscObject obj,const char name[],void (**ptr)(void))
 {
   PetscErrorCode ierr;
 
@@ -388,7 +680,7 @@ struct _p_PetscContainer {
 /*@C
    PetscContainerGetPointer - Gets the pointer value contained in the container.
 
-   Collective on PetscContainer
+   Not Collective
 
    Input Parameter:
 .  obj - the object created with PetscContainerCreate()
@@ -401,10 +693,10 @@ struct _p_PetscContainer {
 .seealso: PetscContainerCreate(), PetscContainerDestroy(), 
           PetscContainerSetPointer()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscContainerGetPointer(PetscContainer obj,void **ptr)
+PetscErrorCode  PetscContainerGetPointer(PetscContainer obj,void **ptr)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_COOKIE,1);
+  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_CLASSID,1);
   PetscValidPointer(ptr,2);
   *ptr = obj->ptr;
   PetscFunctionReturn(0);
@@ -416,7 +708,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscContainerGetPointer(PetscContainer obj,void
 /*@C
    PetscContainerSetPointer - Sets the pointer value contained in the container.
 
-   Collective on PetscContainer
+   Logically Collective on PetscContainer
 
    Input Parameters:
 +  obj - the object created with PetscContainerCreate()
@@ -427,10 +719,10 @@ PetscErrorCode PETSC_DLLEXPORT PetscContainerGetPointer(PetscContainer obj,void
 .seealso: PetscContainerCreate(), PetscContainerDestroy(), 
           PetscContainerGetPointer()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscContainerSetPointer(PetscContainer obj,void *ptr)
+PetscErrorCode  PetscContainerSetPointer(PetscContainer obj,void *ptr)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_COOKIE,1);
+  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_CLASSID,1);
   if (ptr) PetscValidPointer(ptr,2);
   obj->ptr = ptr;
   PetscFunctionReturn(0);
@@ -448,15 +740,16 @@ PetscErrorCode PETSC_DLLEXPORT PetscContainerSetPointer(PetscContainer obj,void
 
    Level: advanced
 
-.seealso: PetscContainerCreate()
+.seealso: PetscContainerCreate(), PetscContainerSetUserDestroy()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscContainerDestroy(PetscContainer obj)
+PetscErrorCode  PetscContainerDestroy(PetscContainer *obj)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_COOKIE,1);
-  if (--((PetscObject)obj)->refct > 0) PetscFunctionReturn(0);
-  if (obj->userdestroy) (*obj->userdestroy)(obj->ptr);
+  if (!*obj) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(*obj,PETSC_CONTAINER_CLASSID,1);
+  if (--((PetscObject)(*obj))->refct > 0) PetscFunctionReturn(0);
+  if ((*obj)->userdestroy) (*(*obj)->userdestroy)((*obj)->ptr);
   ierr = PetscHeaderDestroy(obj);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -466,7 +759,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscContainerDestroy(PetscContainer obj)
 /*@C
    PetscContainerSetUserDestroy - Sets name of the user destroy function.
 
-   Collective on PetscContainer
+   Logically Collective on PetscContainer
 
    Input Parameter:
 +  obj - an object that was created with PetscContainerCreate()
@@ -476,15 +769,15 @@ PetscErrorCode PETSC_DLLEXPORT PetscContainerDestroy(PetscContainer obj)
 
 .seealso: PetscContainerDestroy()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscContainerSetUserDestroy(PetscContainer obj, PetscErrorCode (*des)(void*))
+PetscErrorCode  PetscContainerSetUserDestroy(PetscContainer obj, PetscErrorCode (*des)(void*))
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_COOKIE,1);
+  PetscValidHeaderSpecific(obj,PETSC_CONTAINER_CLASSID,1);
   obj->userdestroy = des;
   PetscFunctionReturn(0);
 }
 
-PetscCookie PETSC_DLLEXPORT PETSC_CONTAINER_COOKIE;
+PetscClassId  PETSC_CONTAINER_CLASSID;
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscContainerCreate"
@@ -492,7 +785,7 @@ PetscCookie PETSC_DLLEXPORT PETSC_CONTAINER_COOKIE;
    PetscContainerCreate - Creates a PETSc object that has room to hold
    a single pointer. This allows one to attach any type of data (accessible
    through a pointer) with the PetscObjectCompose() function to a PetscObject.
-   The data item itself is attached by a call to PetscContainerSetPointer.
+   The data item itself is attached by a call to PetscContainerSetPointer().
 
    Collective on MPI_Comm
 
@@ -506,14 +799,14 @@ PetscCookie PETSC_DLLEXPORT PETSC_CONTAINER_COOKIE;
 
 .seealso: PetscContainerDestroy(), PetscContainerSetPointer(), PetscContainerGetPointer()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscContainerCreate(MPI_Comm comm,PetscContainer *container)
+PetscErrorCode  PetscContainerCreate(MPI_Comm comm,PetscContainer *container)
 {
   PetscErrorCode ierr;
   PetscContainer contain;
 
   PetscFunctionBegin;
   PetscValidPointer(container,2);
-  ierr = PetscHeaderCreate(contain,_p_PetscContainer,PetscInt,PETSC_CONTAINER_COOKIE,0,"PetscContainer",comm,PetscContainerDestroy,0);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(contain,_p_PetscContainer,PetscInt,PETSC_CONTAINER_CLASSID,0,"PetscContainer","Container","Sys",comm,PetscContainerDestroy,0);CHKERRQ(ierr);
   *container = contain;
   PetscFunctionReturn(0);
 }
@@ -538,7 +831,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscContainerCreate(MPI_Comm comm,PetscContainer
 .keywords: set, options, database
 .seealso: PetscObjectSetOptionsPrefix(), PetscObjectGetOptionsPrefix()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectSetFromOptions(PetscObject obj)
+PetscErrorCode  PetscObjectSetFromOptions(PetscObject obj)
 {
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
@@ -563,7 +856,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscObjectSetFromOptions(PetscObject obj)
 .keywords: setup
 .seealso: PetscObjectDestroy()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscObjectSetUp(PetscObject obj)
+PetscErrorCode  PetscObjectSetUp(PetscObject obj)
 {
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
