@@ -1,4 +1,3 @@
-#define PETSC_DLL
 
 #include <petscsys.h>
 #include <Windows.h>
@@ -8,9 +7,9 @@ EXTERN_C_BEGIN
 
 #undef __FUNCT__  
 #define __FUNCT__ "nt_time"
-PetscLogDouble PETSC_DLLEXPORT nt_time(void) 
+PetscLogDouble  nt_time(void)
 {
-  static PetscTruth     flag = PETSC_TRUE;
+  static PetscBool      flag = PETSC_TRUE;
   PetscErrorCode ierr;
 
   static LARGE_INTEGER  StartTime,PerfFreq,CurTime; 
