@@ -1,4 +1,4 @@
-#include "private/fortranimpl.h"
+#include <private/fortranimpl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscgetarchtype_                  PETSCGETARCHTYPE
