@@ -1,4 +1,3 @@
-#define PETSC_DLL
 
 /*
     We define the memory operations here. The reason we just do not use 
@@ -6,9 +5,9 @@
   they are broken.
 
 */
-#include "petscsys.h"        /*I  "petscsys.h"   I*/
-#include "petscbt.h"
-#include "../src/sys/utils/ftn-kernels/fcopy.h"
+#include <petscsys.h>        /*I  "petscsys.h"   I*/
+#include <petscbt.h>
+#include <../src/sys/utils/ftn-kernels/fcopy.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscMemcmp"
@@ -31,13 +30,13 @@
    Note: 
    This routine is anologous to memcmp()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscMemcmp(const void *str1,const void *str2,size_t len,PetscTruth *e)
+PetscErrorCode  PetscMemcmp(const void *str1,const void *str2,size_t len,PetscBool  *e)
 {
   int r;
 
   PetscFunctionBegin;
-  if (len > 0 && !str1) SETERRQ(PETSC_ERR_ARG_NULL,"Trying to compare at a null pointer");
-  if (len > 0 && !str2) SETERRQ(PETSC_ERR_ARG_NULL,"Trying to compare at a null pointer");
+  if (len > 0 && !str1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Trying to compare at a null pointer");
+  if (len > 0 && !str2) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Trying to compare at a null pointer");
   r = memcmp((char *)str1,(char *)str2,len);
   if (!r) *e = PETSC_TRUE;
   else    *e = PETSC_FALSE;
@@ -72,11 +71,11 @@ PetscErrorCode PETSC_DLLEXPORT PetscMemcmp(const void *str1,const void *str2,siz
 
 .seealso: PetscMemcpy()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscMemmove(void *a,void *b,size_t n)
+PetscErrorCode  PetscMemmove(void *a,void *b,size_t n)
 {
   PetscFunctionBegin;
-  if (n > 0 && !a) SETERRQ(PETSC_ERR_ARG_NULL,"Trying to copy to null pointer");
-  if (n > 0 && !b) SETERRQ(PETSC_ERR_ARG_NULL,"Trying to copy from a null pointer");
+  if (n > 0 && !a) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Trying to copy to null pointer");
+  if (n > 0 && !b) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Trying to copy from a null pointer");
 #if !defined(PETSC_HAVE_MEMMOVE)
   if (a < b) {
     if (a <= b - n) {
