@@ -1,10 +1,10 @@
-#define PETSC_DLL
+
 /*
     Code for tracing mistakes in MPI usage. For example, sends that are never received,
   nonblocking messages that are not correctly waited for, etc.
 */
 
-#include "petscsys.h"           /*I "petscsys.h" I*/
+#include <petscsys.h>           /*I "petscsys.h" I*/
 
 #if defined(PETSC_USE_LOG) && !defined(__MPIUNI_H)
 
@@ -26,7 +26,7 @@
 
 .seealso:  PetscMallocDump()
  @*/
-PetscErrorCode PETSC_DLLEXPORT PetscMPIDump(FILE *fd)
+PetscErrorCode  PetscMPIDump(FILE *fd)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank;
@@ -39,22 +39,22 @@ PetscErrorCode PETSC_DLLEXPORT PetscMPIDump(FILE *fd)
    
   /* Did we wait on all the non-blocking sends and receives? */
   ierr = PetscSequentialPhaseBegin(PETSC_COMM_WORLD,1);CHKERRQ(ierr);
-  if (irecv_ct + isend_ct != sum_of_waits_ct) {
+  if (petsc_irecv_ct + petsc_isend_ct != petsc_sum_of_waits_ct) {
     ierr = PetscFPrintf(PETSC_COMM_SELF,fd,"[%d]You have not waited on all non-blocking sends and receives",rank);CHKERRQ(ierr);
-    ierr = PetscFPrintf(PETSC_COMM_SELF,fd,"[%d]Number non-blocking sends %g receives %g number of waits %g\n",rank,isend_ct,irecv_ct,sum_of_waits_ct);CHKERRQ(ierr);
+    ierr = PetscFPrintf(PETSC_COMM_SELF,fd,"[%d]Number non-blocking sends %g receives %g number of waits %g\n",rank,petsc_isend_ct,petsc_irecv_ct,petsc_sum_of_waits_ct);CHKERRQ(ierr);
     err = fflush(fd);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");        
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");        
   }
   ierr = PetscSequentialPhaseEnd(PETSC_COMM_WORLD,1);CHKERRQ(ierr);
   /* Did we receive all the messages that we sent? */
-  work = irecv_ct + recv_ct;
+  work = petsc_irecv_ct + petsc_recv_ct;
   ierr = MPI_Reduce(&work,&trecvs,1,MPI_DOUBLE,MPI_SUM,0,PETSC_COMM_WORLD);CHKERRQ(ierr);
-  work = isend_ct + send_ct;
+  work = petsc_isend_ct + petsc_send_ct;
   ierr = MPI_Reduce(&work,&tsends,1,MPI_DOUBLE,MPI_SUM,0,PETSC_COMM_WORLD);CHKERRQ(ierr);
   if (!rank && tsends != trecvs) {
     ierr = PetscFPrintf(PETSC_COMM_SELF,fd,"Total number sends %g not equal receives %g\n",tsends,trecvs);CHKERRQ(ierr);
     err = fflush(fd);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");        
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");        
   }
   PetscFunctionReturn(0);
 }
@@ -63,7 +63,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscMPIDump(FILE *fd)
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscMPIDump"
-PetscErrorCode PETSC_DLLEXPORT PetscMPIDump(FILE *fd)
+PetscErrorCode  PetscMPIDump(FILE *fd)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
