@@ -1,6 +1,5 @@
-#define PETSC_DLL
 
-#include "petscsys.h"        
+#include <petscsys.h>        
 #include <stdarg.h>
 #if defined(PETSC_HAVE_STDLIB_H)
 #include <stdlib.h>
@@ -36,12 +35,12 @@
     put it in a universal location like a .chsrc file
 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscOptionsGetenv(MPI_Comm comm,const char name[],char env[],size_t len,PetscTruth *flag)
+PetscErrorCode  PetscOptionsGetenv(MPI_Comm comm,const char name[],char env[],size_t len,PetscBool  *flag)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank;
   char           *str,work[256];
-  PetscTruth     flg = PETSC_FALSE,spetsc;
+  PetscBool      flg = PETSC_FALSE,spetsc;
    
   PetscFunctionBegin;
 
@@ -85,41 +84,62 @@ PetscErrorCode PETSC_DLLEXPORT PetscOptionsGetenv(MPI_Comm comm,const char name[
 */
 static char PetscDisplay[256]; 
 
+#undef __FUNCT__  
+#define __FUNCT__ "PetscWorldIsSingleHost"
+static PetscErrorCode PetscWorldIsSingleHost(PetscBool  *onehost)
+{
+  PetscErrorCode ierr;
+  char           hostname[256],roothostname[256];
+  PetscMPIInt    localmatch,allmatch;
+  PetscBool      flag;
+
+  PetscFunctionBegin;
+  ierr = PetscGetHostName(hostname,256);CHKERRQ(ierr);
+  ierr = PetscMemcpy(roothostname,hostname,256);CHKERRQ(ierr);
+  ierr = MPI_Bcast(roothostname,256,MPI_CHAR,0,PETSC_COMM_WORLD);CHKERRQ(ierr);
+  ierr = PetscStrcmp(hostname,roothostname,&flag);CHKERRQ(ierr);
+  localmatch = (PetscMPIInt)flag;
+  ierr = MPI_Allreduce(&localmatch,&allmatch,1,MPI_INT,MPI_LAND,PETSC_COMM_WORLD);CHKERRQ(ierr);
+  *onehost = (PetscBool)allmatch;
+  PetscFunctionReturn(0);
+}
+
+
 #undef __FUNCT__  
 #define __FUNCT__ "PetscSetDisplay" 
-PetscErrorCode PETSC_DLLEXPORT PetscSetDisplay(void)
+PetscErrorCode  PetscSetDisplay(void)
 {
   PetscErrorCode ierr;
   PetscMPIInt    size,rank;
-  PetscTruth     flag;
-  char           *str,display[256];
+  PetscBool      flag,singlehost=PETSC_FALSE;
+  char           display[sizeof PetscDisplay];
+  const char     *str;
 
   PetscFunctionBegin;
-  ierr = PetscMemzero(display,256*sizeof(char));CHKERRQ(ierr);
-  ierr = PetscOptionsGetString(PETSC_NULL,"-display",PetscDisplay,256,&flag);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-display",PetscDisplay,sizeof PetscDisplay,&flag);CHKERRQ(ierr);
   if (flag) PetscFunctionReturn(0);
 
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr); 
-  if (!rank) {
-    str = getenv("DISPLAY");
-    if (!str || (str[0] == ':' && size > 1)) {
-      ierr = PetscGetHostName(display,255);CHKERRQ(ierr);
-      ierr = PetscStrcat(display,":0.0");CHKERRQ(ierr);
-    } else {
-      ierr = PetscStrncpy(display,str,256);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
+
+  ierr = PetscWorldIsSingleHost(&singlehost);CHKERRQ(ierr);
+
+  str = getenv("DISPLAY");
+  if (!str) str = ":0.0";
+  if (str[0] != ':' || singlehost) {
+    ierr = PetscStrncpy(display,str,sizeof display);CHKERRQ(ierr);
+  } else {
+    if (!rank) {
+      size_t len;
+      ierr = PetscGetHostName(display,sizeof display);CHKERRQ(ierr);
+      ierr = PetscStrlen(display,&len);CHKERRQ(ierr);
+      ierr = PetscStrncat(display,str,sizeof display-len-1);CHKERRQ(ierr);
     }
   }
-  ierr = MPI_Bcast(display,256,MPI_CHAR,0,PETSC_COMM_WORLD);CHKERRQ(ierr);
-  if (rank) {
-    str = getenv("DISPLAY");
-    /* assume that ssh port forwarding is working */
-    if (str && (str[0] != ':')) {
-      ierr = PetscStrcpy(display,str);CHKERRQ(ierr);
-    }
-  }
-  ierr = PetscStrcpy(PetscDisplay,display);CHKERRQ(ierr);
-  PetscFunctionReturn(0);  
+  ierr = MPI_Bcast(display,sizeof display,MPI_CHAR,0,PETSC_COMM_WORLD);CHKERRQ(ierr);
+  ierr = PetscMemcpy(PetscDisplay,display,sizeof PetscDisplay);CHKERRQ(ierr);
+  PetscDisplay[sizeof PetscDisplay-1] = 0;
+  PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
@@ -134,7 +154,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscSetDisplay(void)
 .   display - the display string
 
 */
-PetscErrorCode PETSC_DLLEXPORT PetscGetDisplay(char display[],size_t n)
+PetscErrorCode  PetscGetDisplay(char display[],size_t n)
 {
   PetscErrorCode ierr;
 
