@@ -1,5 +1,5 @@
-#define PETSC_DLL
-#include "petscsys.h"              /*I  "petscsys.h"  I*/
+
+#include <petscsys.h>              /*I  "petscsys.h"  I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscPopUpSelect"
@@ -33,7 +33,7 @@
    Concepts: menu
 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscPopUpSelect(MPI_Comm comm,const char *machine,const char *title,int n,const char **choices,int *choice)
+PetscErrorCode  PetscPopUpSelect(MPI_Comm comm,const char *machine,const char *title,int n,const char **choices,int *choice)
 {
   PetscMPIInt    rank;
   int            i,rows = n + 2;
@@ -43,8 +43,8 @@ PetscErrorCode PETSC_DLLEXPORT PetscPopUpSelect(MPI_Comm comm,const char *machin
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!title) SETERRQ(PETSC_ERR_ARG_NULL,"Must pass in a title line");
-  if (n < 1) SETERRQ(PETSC_ERR_ARG_WRONG,"Must pass in at least one selection");
+  if (!title) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Must pass in a title line");
+  if (n < 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must pass in at least one selection");
   if (n == 1) {*choice = 0; PetscFunctionReturn(0);}
 
   ierr = PetscStrlen(title,&cols);CHKERRQ(ierr);
@@ -76,13 +76,13 @@ PetscErrorCode PETSC_DLLEXPORT PetscPopUpSelect(MPI_Comm comm,const char *machin
     FILE *fd;
 
     ierr = PetscFOpen(PETSC_COMM_SELF,"${HOMEDIRECTORY}/.popuptmp","r",&fd);CHKERRQ(ierr);
-    fscanf(fd,"%d",choice);
+    if (fscanf(fd,"%d",choice) != 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fscanf() could not read numeric choice");
     *choice -= 1;
-    if (*choice < 0 || *choice > n-1) SETERRQ1(PETSC_ERR_ARG_OUTOFRANGE,"Selection %d out of range",*choice);
+    if (*choice < 0 || *choice > n-1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Selection %d out of range",*choice);
     ierr = PetscFClose(PETSC_COMM_SELF,fd);CHKERRQ(ierr);
   }
 #else
-  SETERRQ(PETSC_ERR_SUP_SYS,"Cannot run external programs on this machine");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP_SYS,"Cannot run external programs on this machine");
 #endif
   ierr = MPI_Bcast(choice,1,MPI_INT,0,comm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
