@@ -1,6 +1,5 @@
-#define PETSC_DLL
 
-#include "../src/sys/viewer/impls/ascii/asciiimpl.h"  /*I     "petsc.h"   I*/
+#include <../src/sys/viewer/impls/ascii/asciiimpl.h>  /*I     "petscsys.h"   I*/
 
 /* ---------------------------------------------------------------------*/
 /*
@@ -29,21 +28,24 @@ static PetscMPIInt Petsc_Viewer_Stdout_keyval = MPI_KEYVAL_INVALID;
           PETSC_VIEWER_STDOUT_SELF
 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerASCIIGetStdout(MPI_Comm comm,PetscViewer *viewer)
+PetscErrorCode  PetscViewerASCIIGetStdout(MPI_Comm comm,PetscViewer *viewer)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
+  MPI_Comm       ncomm;
 
   PetscFunctionBegin;
+  ierr = PetscCommDuplicate(comm,&ncomm,PETSC_NULL);CHKERRQ(ierr);
   if (Petsc_Viewer_Stdout_keyval == MPI_KEYVAL_INVALID) {
     ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,MPI_NULL_DELETE_FN,&Petsc_Viewer_Stdout_keyval,0);CHKERRQ(ierr);
   }
-  ierr = MPI_Attr_get(comm,Petsc_Viewer_Stdout_keyval,(void **)viewer,(PetscMPIInt*)&flg);CHKERRQ(ierr);
+  ierr = MPI_Attr_get(ncomm,Petsc_Viewer_Stdout_keyval,(void **)viewer,(PetscMPIInt*)&flg);CHKERRQ(ierr);
   if (!flg) { /* PetscViewer not yet created */
-    ierr = PetscViewerASCIIOpen(comm,"stdout",viewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIOpen(ncomm,"stdout",viewer);CHKERRQ(ierr);
     ierr = PetscObjectRegisterDestroy((PetscObject)*viewer);CHKERRQ(ierr);
-    ierr = MPI_Attr_put(comm,Petsc_Viewer_Stdout_keyval,(void*)*viewer);CHKERRQ(ierr);
+    ierr = MPI_Attr_put(ncomm,Petsc_Viewer_Stdout_keyval,(void*)*viewer);CHKERRQ(ierr);
   } 
+  ierr = PetscCommDestroy(&ncomm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -69,14 +71,14 @@ $      XXXView(XXX object,PETSC_VIEWER_STDOUT_(comm));
           PETSC_VIEWER_STDOUT_SELF
 
 @*/
-PetscViewer PETSC_DLLEXPORT PETSC_VIEWER_STDOUT_(MPI_Comm comm)
+PetscViewer  PETSC_VIEWER_STDOUT_(MPI_Comm comm)
 {
   PetscErrorCode ierr;
   PetscViewer    viewer;
 
   PetscFunctionBegin;
   ierr = PetscViewerASCIIGetStdout(comm,&viewer);
-  if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_STDOUT_",__FILE__,__SDIR__,1,1," "); PetscFunctionReturn(0);}
+  if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_STDOUT_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," "); PetscFunctionReturn(0);}
   PetscFunctionReturn(viewer);
 }
 
@@ -107,21 +109,24 @@ static PetscMPIInt Petsc_Viewer_Stderr_keyval = MPI_KEYVAL_INVALID;
           PETSC_VIEWER_STDERR_SELF
 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerASCIIGetStderr(MPI_Comm comm,PetscViewer *viewer)
+PetscErrorCode  PetscViewerASCIIGetStderr(MPI_Comm comm,PetscViewer *viewer)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
+  MPI_Comm       ncomm;
 
   PetscFunctionBegin;
+  ierr = PetscCommDuplicate(comm,&ncomm,PETSC_NULL);CHKERRQ(ierr);
   if (Petsc_Viewer_Stderr_keyval == MPI_KEYVAL_INVALID) {
     ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,MPI_NULL_DELETE_FN,&Petsc_Viewer_Stderr_keyval,0);CHKERRQ(ierr);
   }
-  ierr = MPI_Attr_get(comm,Petsc_Viewer_Stderr_keyval,(void **)viewer,(PetscMPIInt*)&flg);CHKERRQ(ierr);
+  ierr = MPI_Attr_get(ncomm,Petsc_Viewer_Stderr_keyval,(void **)viewer,(PetscMPIInt*)&flg);CHKERRQ(ierr);
   if (!flg) { /* PetscViewer not yet created */
-    ierr = PetscViewerASCIIOpen(comm,"stderr",viewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIOpen(ncomm,"stderr",viewer);CHKERRQ(ierr);
     ierr = PetscObjectRegisterDestroy((PetscObject)*viewer);CHKERRQ(ierr);
-    ierr = MPI_Attr_put(comm,Petsc_Viewer_Stderr_keyval,(void*)*viewer);CHKERRQ(ierr);
+    ierr = MPI_Attr_put(ncomm,Petsc_Viewer_Stderr_keyval,(void*)*viewer);CHKERRQ(ierr);
   } 
+  ierr = PetscCommDestroy(&ncomm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -146,14 +151,14 @@ $      XXXView(XXX object,PETSC_VIEWER_STDERR_(comm));
 .seealso: PETSC_VIEWER_DRAW_, PetscViewerASCIIOpen(), PETSC_VIEWER_STDOUT_, PETSC_VIEWER_STDOUT_WORLD,
           PETSC_VIEWER_STDOUT_SELF, PETSC_VIEWER_STDERR_WORLD, PETSC_VIEWER_STDERR_SELF
 @*/
-PetscViewer PETSC_DLLEXPORT PETSC_VIEWER_STDERR_(MPI_Comm comm)
+PetscViewer  PETSC_VIEWER_STDERR_(MPI_Comm comm)
 {
   PetscErrorCode ierr;
   PetscViewer    viewer;
 
   PetscFunctionBegin;
   ierr = PetscViewerASCIIGetStderr(comm,&viewer);
-  if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_STDERR_",__FILE__,__SDIR__,1,1," "); PetscFunctionReturn(0);}
+  if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_STDERR_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," "); PetscFunctionReturn(0);}
   PetscFunctionReturn(viewer);
 }
 
@@ -163,16 +168,18 @@ EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "Petsc_DelViewer" 
 /*
-   Private routine to delete internal storage when a communicator is freed.
+   Called with MPI_Comm_free() is called on a communicator that has a viewer as an attribute. The viewer is not actually destroyed because that is managed by 
+   PetscObjectDestroyRegisterAll(). PetscViewerASCIIGetStdout() registers the viewer with PetscObjectDestroyRegister() to be destroyed when PetscFinalize() is called.
+
   This is called by MPI, not by users.
 
 */
-PetscMPIInt PETSC_DLLEXPORT MPIAPI Petsc_DelViewer(MPI_Comm comm,PetscMPIInt keyval,void* attr_val,void* extra_state)
+PetscMPIInt  MPIAPI Petsc_DelViewer(MPI_Comm comm,PetscMPIInt keyval,void* attr_val,void* extra_state)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscInfo1(0,"Deleting viewer data in an MPI_Comm %ld\n",(long)comm);if (ierr) PetscFunctionReturn((PetscMPIInt)ierr);
+  ierr = PetscInfo1(0,"Removing viewer data attribute in an MPI_Comm %ld\n",(long)comm);if (ierr) PetscFunctionReturn((PetscMPIInt)ierr);
   PetscFunctionReturn(MPI_SUCCESS);
 }
 EXTERN_C_END
@@ -220,11 +227,11 @@ EXTERN_C_END
           PetscViewerASCIIGetPointer(), PetscViewerSetFormat(), PETSC_VIEWER_STDOUT_, PETSC_VIEWER_STDERR_,
           PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_STDOUT_SELF, 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerASCIIOpen(MPI_Comm comm,const char name[],PetscViewer *lab)
+PetscErrorCode  PetscViewerASCIIOpen(MPI_Comm comm,const char name[],PetscViewer *lab)
 {
   PetscErrorCode    ierr;
   PetscViewerLink   *vlink,*nv;
-  PetscTruth        flg,eq;
+  PetscBool         flg,eq;
   size_t            len;
 
   PetscFunctionBegin;
@@ -254,7 +261,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerASCIIOpen(MPI_Comm comm,const char nam
     }
   }
   ierr = PetscViewerCreate(comm,lab);CHKERRQ(ierr);
-  ierr = PetscViewerSetType(*lab,PETSC_VIEWER_ASCII);CHKERRQ(ierr);
+  ierr = PetscViewerSetType(*lab,PETSCVIEWERASCII);CHKERRQ(ierr);
   if (name) {
     ierr = PetscViewerFileSetName(*lab,name);CHKERRQ(ierr);
   }
@@ -276,4 +283,61 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerASCIIOpen(MPI_Comm comm,const char nam
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "PetscViewerASCIIOpenWithFILE" 
+/*@C
+   PetscViewerASCIIOpenWithFILE - Given an open file creates an ASCII viewer that prints to it.
+
+   Collective on MPI_Comm
+
+   Input Parameters:
++  comm - the communicator
+-  fd - the FILE pointer
+
+   Output Parameter:
+.  lab - the PetscViewer to use with the specified file
+
+   Level: beginner
+
+   Notes:
+   This PetscViewer can be destroyed with PetscViewerDestroy(), but the fd will NOT be closed.
+
+   If a multiprocessor communicator is used (such as PETSC_COMM_WORLD), 
+   then only the first processor in the group uses the file.  All other 
+   processors send their data to the first processor to print. 
+
+  Concepts: PetscViewerASCII^creating
+  Concepts: printf
+  Concepts: printing
+  Concepts: accessing remote file
+  Concepts: remote file
+
+.seealso: MatView(), VecView(), PetscViewerDestroy(), PetscViewerBinaryOpen(),
+          PetscViewerASCIIGetPointer(), PetscViewerSetFormat(), PETSC_VIEWER_STDOUT_, PETSC_VIEWER_STDERR_,
+          PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_STDOUT_SELF, PetscViewerASCIIOpen()
+@*/
+PetscErrorCode  PetscViewerASCIIOpenWithFILE(MPI_Comm comm,FILE *fd,PetscViewer *lab)
+{
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscViewerCreate(comm,lab);CHKERRQ(ierr);
+  ierr = PetscViewerSetType(*lab,PETSCVIEWERASCII);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetFILE(*lab,fd);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "PetscViewerASCIISetFILE" 
+PetscErrorCode  PetscViewerASCIISetFILE(PetscViewer viewer,FILE *fd)
+{
+  PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
+
+  PetscFunctionBegin;
+  vascii->fd        = fd;
+  vascii->closefile = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+
 
