@@ -1,28 +1,27 @@
-#define PETSC_DLL
 
-#include "private/viewerimpl.h"
-#include "mat.h"
+#include <private/viewerimpl.h>
+#include <mat.h>
 
 /*MC
-   PETSC_VIEWER_MATLAB - A viewer that saves the variables into a Matlab .mat file that may be read into Matlab
+   PETSCVIEWERMATLAB - A viewer that saves the variables into a MATLAB .mat file that may be read into MATLAB
        with load('filename').
 
    Level: intermediate
 
-       Note: Currently can only save PETSc vectors to .mat files, not matrices (use the PETSC_VIEWER_BINARY and 
+       Note: Currently can only save PETSc vectors to .mat files, not matrices (use the PETSCVIEWERBINARY and 
              ${PETSC_DIR}/bin/matlab/PetscBinaryRead.m to read matrices into matlab).
 
-             For parallel vectors obtained with DACreateGlobalVector() or DAGetGlobalVector() the vectors are saved to
-             the .mat file in natural ordering. You can use DAView() to save the DA information to the .mat file
-             the fields in the Matlab loaded da variable give the array dimensions so you can reshape the Matlab
+             For parallel vectors obtained with DMCreateGlobalVector() or DMGetGlobalVector() the vectors are saved to
+             the .mat file in natural ordering. You can use DMView() to save the DMDA information to the .mat file
+             the fields in the MATLAB loaded da variable give the array dimensions so you can reshape the MATLAB
              vector to the same multidimensional shape as it had in PETSc for plotting etc. For example,
 
-$             In your PETSc C/C++ code (assuming a two dimensional DA with one degree of freedom per node)
+$             In your PETSc C/C++ code (assuming a two dimensional DMDA with one degree of freedom per node)
 $                PetscObjectSetName((PetscObject)x,"x");
 $                VecView(x,PETSC_VIEWER_MATLAB_WORLD);
 $                PetscObjectSetName((PetscObject)da,"da");
-$                DAView(x,PETSC_VIEWER_MATLAB_WORLD);
-$             Then from Matlab
+$                DMView(x,PETSC_VIEWER_MATLAB_WORLD);
+$             Then from MATLAB
 $                load('matlaboutput.mat')   % matlaboutput.mat is the default filename
 $                xnew = zeros(da.n,da.m);
 $                xnew(:) = x;    % reshape one dimensional vector back to two dimensions
@@ -33,8 +32,8 @@ $                xnew(:) = x;    % reshape one dimensional vector back to two di
               Use PetscViewerMatlabPutArray() to just put an array of doubles into the .mat file
 
 .seealso:  PETSC_VIEWER_MATLAB_(),PETSC_VIEWER_MATLAB_SELF(), PETSC_VIEWER_MATLAB_WORLD(),PetscViewerCreate(),
-           PetscViewerMatlabOpen(), VecView(), DAView(), PetscViewerMatlabPutArray(), PETSC_VIEWER_BINARY,
-           PETSC_ASCII_VIEWER, DAView(), PetscViewerFileSetName(), PetscViewerFileSetMode()
+           PetscViewerMatlabOpen(), VecView(), DMView(), PetscViewerMatlabPutArray(), PETSCVIEWERBINARY,
+           PETSC_ASCII_VIEWER, PetscViewerFileSetName(), PetscViewerFileSetMode()
 
 M*/
 
@@ -47,7 +46,7 @@ typedef struct {
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerMatlabPutArray"
 /*@C
-    PetscViewerMatlabPutArray - Puts an array into the Matlab viewer.
+    PetscViewerMatlabPutArray - Puts an array into the MATLAB viewer.
 
       Not collective: only processor zero saves the array
 
@@ -62,7 +61,7 @@ typedef struct {
      Notes: Only writes array values on processor 0.
 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabPutArray(PetscViewer mfile,int m,int n,PetscScalar *array,char *name)
+PetscErrorCode  PetscViewerMatlabPutArray(PetscViewer mfile,int m,int n,const PetscScalar *array,const char *name)
 {
   PetscErrorCode     ierr;
   PetscViewer_Matlab *ml = (PetscViewer_Matlab*)mfile->data; 
@@ -70,7 +69,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabPutArray(PetscViewer mfile,int m
   
   PetscFunctionBegin;  
   if (!ml->rank) {
-    ierr = PetscInfo1(mfile,"Putting Matlab array %s\n",name);CHKERRQ(ierr);
+    ierr = PetscInfo1(mfile,"Putting MATLAB array %s\n",name);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
     mat  = mxCreateDoubleMatrix(m,n,mxREAL);
 #else
@@ -79,14 +78,14 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabPutArray(PetscViewer mfile,int m
     ierr = PetscMemcpy(mxGetPr(mat),array,m*n*sizeof(PetscScalar));CHKERRQ(ierr);
     matPutVariable(ml->ep,name,mat);
 
-    ierr = PetscInfo1(mfile,"Put Matlab array %s\n",name);CHKERRQ(ierr);
+    ierr = PetscInfo1(mfile,"Put MATLAB array %s\n",name);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerMatlabPutVariable"
-PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabPutVariable(PetscViewer viewer,const char* name,void* mat)
+PetscErrorCode  PetscViewerMatlabPutVariable(PetscViewer viewer,const char* name,void* mat)
 {
   PetscViewer_Matlab *ml = (PetscViewer_Matlab*)viewer->data; ;
 
@@ -98,12 +97,12 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabPutVariable(PetscViewer viewer,c
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerMatlabGetArray"
 /*@C
-    PetscViewerMatlabGetArray - Gets a variable from a Matlab viewer into an array
+    PetscViewerMatlabGetArray - Gets a variable from a MATLAB viewer into an array
 
     Not Collective; only processor zero reads in the array
 
     Input Parameters:
-+    mfile - the Matlab file viewer
++    mfile - the MATLAB file viewer
 .    m,n - the dimensions of the array
 .    array - the array (represented in one dimension)
 -    name - the name of the array
@@ -113,7 +112,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabPutVariable(PetscViewer viewer,c
      Notes: Only reads in array values on processor 0.
 
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabGetArray(PetscViewer mfile,int m,int n,PetscScalar *array,char *name)
+PetscErrorCode  PetscViewerMatlabGetArray(PetscViewer mfile,int m,int n,PetscScalar *array,const char *name)
 {
   PetscErrorCode     ierr;
   PetscViewer_Matlab *ml = (PetscViewer_Matlab*)mfile->data; 
@@ -121,11 +120,11 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabGetArray(PetscViewer mfile,int m
   
   PetscFunctionBegin;  
   if (!ml->rank) {
-    ierr = PetscInfo1(mfile,"Getting Matlab array %s\n",name);CHKERRQ(ierr);
+    ierr = PetscInfo1(mfile,"Getting MATLAB array %s\n",name);CHKERRQ(ierr);
     mat  = matGetVariable(ml->ep,name);
-    if (!mat) SETERRQ1(PETSC_ERR_LIB,"Unable to get array %s from matlab",name);
+    if (!mat) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Unable to get array %s from matlab",name);
     ierr = PetscMemcpy(array,mxGetPr(mat),m*n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = PetscInfo1(mfile,"Got Matlab array %s\n",name);CHKERRQ(ierr);
+    ierr = PetscInfo1(mfile,"Got MATLAB array %s\n",name);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -133,7 +132,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabGetArray(PetscViewer mfile,int m
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerFileSetMode_Matlab" 
-PetscErrorCode PETSC_DLLEXPORT PetscViewerFileSetMode_Matlab(PetscViewer viewer,PetscFileMode type)
+PetscErrorCode  PetscViewerFileSetMode_Matlab(PetscViewer viewer,PetscFileMode type)
 {
   PetscViewer_Matlab *vmatlab = (PetscViewer_Matlab*)viewer->data;
 
@@ -149,15 +148,13 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerFileSetName_Matlab" 
-PetscErrorCode PETSC_DLLEXPORT PetscViewerFileSetName_Matlab(PetscViewer viewer,const char name[])
+PetscErrorCode  PetscViewerFileSetName_Matlab(PetscViewer viewer,const char name[])
 {
   PetscViewer_Matlab  *vmatlab = (PetscViewer_Matlab*)viewer->data;
   PetscFileMode       type = vmatlab->btype;
 
   PetscFunctionBegin;
-  if (type == (PetscFileMode) -1) {
-    SETERRQ(PETSC_ERR_ORDER,"Must call PetscViewerFileSetMode() before PetscViewerFileSetName()");
-  }
+  if (type == (PetscFileMode) -1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Must call PetscViewerFileSetMode() before PetscViewerFileSetName()");
 
   /* only first processor opens file */
   if (!vmatlab->rank){
@@ -166,7 +163,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerFileSetName_Matlab(PetscViewer viewer,
     } else if (type == FILE_MODE_WRITE || type == FILE_MODE_WRITE) {
       vmatlab->ep = matOpen(name,"w");
     } else {
-      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Unknown file type");
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown file type");
     }
   }
   PetscFunctionReturn(0);
@@ -189,7 +186,7 @@ PetscErrorCode PetscViewerDestroy_Matlab(PetscViewer v)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerCreate_Matlab"
-PetscErrorCode PETSC_DLLEXPORT PetscViewerCreate_Matlab(PetscViewer viewer)
+PetscErrorCode  PetscViewerCreate_Matlab(PetscViewer viewer)
 {
   PetscErrorCode     ierr;
   PetscViewer_Matlab *e;
@@ -211,7 +208,7 @@ EXTERN_C_END
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerMatlabOpen" 
 /*@C
-   PetscViewerMatlabOpen - Opens a Matlab .mat file for input or output.
+   PetscViewerMatlabOpen - Opens a Matlab .mat file for output
 
    Collective on MPI_Comm
 
@@ -219,36 +216,35 @@ EXTERN_C_END
 +  comm - MPI communicator
 .  name - name of file 
 -  type - type of file
-$    FILE_MODE_WRITE - create new file for Matlab output
-$    FILE_MODE_READ - open existing file for Matlab input
-$    FILE_MODE_WRITE - open existing file for Matlab output
+$    FILE_MODE_WRITE - create new file for MATLAB output
+$    FILE_MODE_READ - open existing file for MATLAB input
+$    FILE_MODE_WRITE - open existing file for MATLAB output
 
    Output Parameter:
-.  binv - PetscViewer for Matlab input/output to use with the specified file
+.  binv - PetscViewer for MATLAB output to use with the specified file
 
    Level: beginner
 
-   Note:
-   This PetscViewer should be destroyed with PetscViewerDestroy().
+   Note: This PetscViewer should be destroyed with PetscViewerDestroy().
 
     For writing files it only opens the file on processor 0 in the communicator.
-    For readable files it opens the file on all nodes that have the file. If 
-    node 0 does not have the file it generates an error even if other nodes
-    do have the file.
 
-   Concepts: Matlab .mat files
+     This only saves Vecs it cannot be used to save Mats. We recommend using the PETSCVIEWERBINARY to save objects to be loaded into MATLAB 
+     instead of this routine.
+
+   Concepts: MATLAB .mat files
    Concepts: PetscViewerMatlab^creating
 
-.seealso: PetscViewerASCIIOpen(), PetscViewerSetFormat(), PetscViewerDestroy(),
+.seealso: PetscViewerASCIIOpen(), PetscViewerSetFormat(), PetscViewerDestroy(), PETSCVIEWERBINARY, PetscViewerBinaryOpen()
           VecView(), MatView(), VecLoad(), MatLoad()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerMatlabOpen(MPI_Comm comm,const char name[],PetscFileMode type,PetscViewer *binv)
+PetscErrorCode  PetscViewerMatlabOpen(MPI_Comm comm,const char name[],PetscFileMode type,PetscViewer *binv)
 {
   PetscErrorCode ierr;
   
   PetscFunctionBegin;
   ierr = PetscViewerCreate(comm,binv);CHKERRQ(ierr);
-  ierr = PetscViewerSetType(*binv,PETSC_VIEWER_MATLAB);CHKERRQ(ierr);
+  ierr = PetscViewerSetType(*binv,PETSCVIEWERMATLAB);CHKERRQ(ierr);
   ierr = PetscViewerFileSetMode(*binv,type);CHKERRQ(ierr);
   ierr = PetscViewerFileSetName(*binv,name);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -280,39 +276,43 @@ $    -viewer_matlab_filename <name>
      an error code.  The matlab PetscViewer is usually used in the form
 $       XXXView(XXX object,PETSC_VIEWER_MATLAB_(comm));
 
-     Use PETSC_VIEWER_SOCKET_() or PetscViewerSocketOpen() to communicator with an interactive Matlab session.
+     Use PETSC_VIEWER_SOCKET_() or PetscViewerSocketOpen() to communicator with an interactive MATLAB session.
 
 .seealso: PETSC_VIEWER_MATLAB_WORLD, PETSC_VIEWER_MATLAB_SELF, PetscViewerMatlabOpen(), PetscViewerCreate(),
           PetscViewerDestroy()
 @*/
-PetscViewer PETSC_DLLEXPORT PETSC_VIEWER_MATLAB_(MPI_Comm comm)
+PetscViewer  PETSC_VIEWER_MATLAB_(MPI_Comm comm)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
   PetscViewer    viewer;
   char           fname[PETSC_MAX_PATH_LEN];
+  MPI_Comm       ncomm;
 
   PetscFunctionBegin;
+  ierr = PetscCommDuplicate(comm,&ncomm,PETSC_NULL);if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   if (Petsc_Viewer_Matlab_keyval == MPI_KEYVAL_INVALID) {
     ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,MPI_NULL_DELETE_FN,&Petsc_Viewer_Matlab_keyval,0);
-    if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
+    if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   }
-  ierr = MPI_Attr_get(comm,Petsc_Viewer_Matlab_keyval,(void **)&viewer,(int*)&flg);
-  if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
+  ierr = MPI_Attr_get(ncomm,Petsc_Viewer_Matlab_keyval,(void **)&viewer,(int*)&flg);
+  if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   if (!flg) { /* PetscViewer not yet created */
-    ierr = PetscOptionsGetenv(comm,"PETSC_VIEWER_MATLAB_FILENAME",fname,PETSC_MAX_PATH_LEN,&flg);
-    if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
+    ierr = PetscOptionsGetenv(ncomm,"PETSC_VIEWER_MATLAB_FILENAME",fname,PETSC_MAX_PATH_LEN,&flg);
+    if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
     if (!flg) {
       ierr = PetscStrcpy(fname,"matlaboutput.mat");
-      if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
+      if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
     }
-    ierr = PetscViewerMatlabOpen(comm,fname,FILE_MODE_WRITE,&viewer); 
-    if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
+    ierr = PetscViewerMatlabOpen(ncomm,fname,FILE_MODE_WRITE,&viewer); 
+    if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
     ierr = PetscObjectRegisterDestroy((PetscObject)viewer);
-    if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
-    ierr = MPI_Attr_put(comm,Petsc_Viewer_Matlab_keyval,(void*)viewer);
-    if (ierr) {PetscError(__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,1,1," ");PetscFunctionReturn(0);}
+    if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
+    ierr = MPI_Attr_put(ncomm,Petsc_Viewer_Matlab_keyval,(void*)viewer);
+    if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   } 
+  ierr = PetscCommDestroy(&ncomm);
+  if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,__SDIR__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   PetscFunctionReturn(viewer);
 }
 
