@@ -1,6 +1,5 @@
-#define PETSC_DLL
 
-#include "private/viewerimpl.h"  /*I     "petscsys.h"   I*/
+#include <private/viewerimpl.h>  /*I     "petscsys.h"   I*/
 #include <stdarg.h>
 
 #define QUEUESTRINGSIZE 1024
@@ -15,7 +14,7 @@ typedef struct {
   FILE          *fd;
   PetscFileMode mode;     /* The mode in which to open the file */
   char          *filename;
-  PetscTruth    vecSeen;  /* The flag indicating whether any vector has been viewed so far */
+  PetscBool     vecSeen;  /* The flag indicating whether any vector has been viewed so far */
   PrintfQueue   queue, queueBase;
   int           queueLength;
 } PetscViewer_VU;
@@ -33,7 +32,7 @@ PetscErrorCode PetscViewerDestroy_VU(PetscViewer viewer)
   }
   ierr = PetscViewerVUFlushDeferred(viewer);CHKERRQ(ierr);
   ierr = PetscFClose(((PetscObject)viewer)->comm, vu->fd);CHKERRQ(ierr);
-  ierr = PetscStrfree(vu->filename);CHKERRQ(ierr);
+  ierr = PetscFree(vu->filename);CHKERRQ(ierr);
   ierr = PetscFree(vu);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -51,7 +50,7 @@ PetscErrorCode PetscViewerFlush_VU(PetscViewer viewer)
   ierr = MPI_Comm_rank(((PetscObject)viewer)->comm, &rank);CHKERRQ(ierr);
   if (!rank) {
     err = fflush(vu->fd);
-    if (err) SETERRQ(PETSC_ERR_SYS,"fflush() failed on file");        
+    if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");        
   }
   PetscFunctionReturn(0);  
 }
@@ -59,7 +58,7 @@ PetscErrorCode PetscViewerFlush_VU(PetscViewer viewer)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerFileGetName_VU" 
-PetscErrorCode PETSC_DLLEXPORT PetscViewerFileGetName_VU(PetscViewer viewer, char **name)
+PetscErrorCode  PetscViewerFileGetName_VU(PetscViewer viewer, const char **name)
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
 
@@ -72,7 +71,7 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerFileSetName_VU" 
-PetscErrorCode PETSC_DLLEXPORT PetscViewerFileSetName_VU(PetscViewer viewer, const char name[])
+PetscErrorCode  PetscViewerFileSetName_VU(PetscViewer viewer, const char name[])
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
   char           fname[PETSC_MAX_PATH_LEN];
@@ -113,10 +112,10 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerFileSetName_VU(PetscViewer viewer, con
     }
     break;
   default:
-    SETERRQ1(PETSC_ERR_ARG_WRONG, "Invalid file mode %d", vu->mode);
+    SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG, "Invalid file mode %d", vu->mode);
   }
 
-  if (!vu->fd) SETERRQ1(PETSC_ERR_FILE_OPEN, "Cannot open PetscViewer file: %s", fname);
+  if (!vu->fd) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN, "Cannot open PetscViewer file: %s", fname);
 #if defined(PETSC_USE_LOG)
   PetscLogObjectState((PetscObject) viewer, "File: %s", name);
 #endif
@@ -128,7 +127,7 @@ EXTERN_C_END
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "PetscViewerCreate_VU" 
-PetscErrorCode PETSC_DLLEXPORT PetscViewerCreate_VU(PetscViewer viewer)
+PetscErrorCode  PetscViewerCreate_VU(PetscViewer viewer)
 {
   PetscViewer_VU *vu;
   PetscErrorCode ierr;
@@ -181,12 +180,12 @@ EXTERN_C_END
 
 .seealso: PetscViewerASCIIGetPointer()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerVUGetPointer(PetscViewer viewer, FILE **fd)
+PetscErrorCode  PetscViewerVUGetPointer(PetscViewer viewer, FILE **fd)
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,1);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   PetscValidPointer(fd,2);
   *fd = vu->fd;
   PetscFunctionReturn(0);
@@ -208,7 +207,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerVUGetPointer(PetscViewer viewer, FILE
 .keywords: Viewer, file, get, pointer
 .seealso: PetscViewerASCIISetMode()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerVUSetMode(PetscViewer viewer, PetscFileMode mode)
+PetscErrorCode  PetscViewerVUSetMode(PetscViewer viewer, PetscFileMode mode)
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
 
@@ -234,7 +233,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerVUSetMode(PetscViewer viewer, PetscFil
 .keywords: Viewer, Vec
 .seealso: PetscViewerVUGetVecSeen()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerVUSetVecSeen(PetscViewer viewer, PetscTruth vecSeen)
+PetscErrorCode  PetscViewerVUSetVecSeen(PetscViewer viewer, PetscBool  vecSeen)
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
 
@@ -262,12 +261,12 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerVUSetVecSeen(PetscViewer viewer, Petsc
 .keywords: Viewer, Vec
 .seealso: PetscViewerVUGetVecSeen()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerVUGetVecSeen(PetscViewer viewer, PetscTruth *vecSeen)
+PetscErrorCode  PetscViewerVUGetVecSeen(PetscViewer viewer, PetscBool  *vecSeen)
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,1);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   PetscValidPointer(vecSeen,2);
   *vecSeen = vu->vecSeen;
   PetscFunctionReturn(0);
@@ -289,11 +288,11 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerVUGetVecSeen(PetscViewer viewer, Petsc
 .keywords: Viewer, print, deferred
 .seealso: PetscViewerVUFlushDeferred()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerVUPrintDeferred(PetscViewer viewer, const char format[], ...)
+PetscErrorCode  PetscViewerVUPrintDeferred(PetscViewer viewer, const char format[], ...)
 {
   PetscViewer_VU *vu = (PetscViewer_VU *) viewer->data;
   va_list        Argp;
-  int            fullLength;
+  size_t         fullLength;
   PrintfQueue    next;
   PetscErrorCode ierr;
 
@@ -330,7 +329,7 @@ PetscErrorCode PETSC_DLLEXPORT PetscViewerVUPrintDeferred(PetscViewer viewer, co
 .keywords: Viewer, flush, deferred
 .seealso: PetscViewerVUPrintDeferred()
 @*/
-PetscErrorCode PETSC_DLLEXPORT PetscViewerVUFlushDeferred(PetscViewer viewer)
+PetscErrorCode  PetscViewerVUFlushDeferred(PetscViewer viewer)
 {
   PetscViewer_VU *vu   = (PetscViewer_VU *) viewer->data;
   PrintfQueue    next = vu->queueBase;
