@@ -38,11 +38,11 @@ timestepping.  Runtime options include:\n\
    this file automatically includes "petscsys.h" and other lower-level
    PETSc include files.
 
-   Include the "petscda.h" to allow us to use the distributed array data 
+   Include the "petscdmda.h" to allow us to use the distributed array data 
    structures to manage the parallel grid.
 */
-#include "petscts.h"
-#include "petscda.h"
+#include <petscts.h>
+#include <petscdmda.h>
 
 /* 
    User-defined application context - contains data needed by the 
@@ -50,13 +50,13 @@ timestepping.  Runtime options include:\n\
 */
 typedef struct {
   MPI_Comm   comm;          /* communicator */
-  DA         da;            /* distributed array data structure */
+  DM         da;            /* distributed array data structure */
   Vec        localwork;     /* local ghosted work vector */
   Vec        u_local;       /* local ghosted approximate solution vector */
   Vec        solution;      /* global exact solution vector */
   PetscInt   m;             /* total number of grid points */
   PetscReal  h;             /* mesh width: h = 1/(m-1) */
-  PetscTruth debug;         /* flag (1 indicates activation of debugging printouts) */
+  PetscBool  debug;         /* flag (1 indicates activation of debugging printouts) */
 } AppCtx;
 
 /* 
@@ -81,24 +81,24 @@ int main(int argc,char **argv)
   TS             ts;                     /* timestepping context */
   Mat            A;                      /* Jacobian matrix data structure */
   Vec            u;                      /* approximate solution vector */
-  PetscInt       time_steps_max = 1000;  /* default max timesteps */
+  PetscInt       time_steps_max = 100;  /* default max timesteps */
   PetscErrorCode ierr;
-  PetscInt       steps;
-  PetscReal      ftime;                  /* final time */
   PetscReal      dt;
   PetscReal      time_total_max = 100.0; /* default max total time */
-  PetscTruth     flg;
+  PetscBool      flg,mymonitor = PETSC_FALSE;
+  PetscReal      bounds[] = {1.0, 3.3};
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Initialize program and set problem parameters
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  
   ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr); 
-
+  ierr = PetscViewerDrawSetBounds(PETSC_VIEWER_DRAW_(PETSC_COMM_WORLD),1,bounds);CHKERRQ(ierr);
   appctx.comm = PETSC_COMM_WORLD;
   appctx.m    = 60;
   ierr = PetscOptionsGetInt(PETSC_NULL,"-M",&appctx.m,PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscOptionsHasName(PETSC_NULL,"-debug",&appctx.debug);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(PETSC_NULL,"-mymonitor",&mymonitor);CHKERRQ(ierr);
   appctx.h    = 1.0/(appctx.m-1.0);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -106,20 +106,20 @@ int main(int argc,char **argv)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
   /*
-     Create distributed array (DA) to manage parallel grid and vectors
+     Create distributed array (DMDA) to manage parallel grid and vectors
      and to set up the ghost point communication pattern.  There are M 
      total grid values spread equally among all the processors.
   */ 
-  ierr = DACreate1d(PETSC_COMM_WORLD,DA_NONPERIODIC,appctx.m,1,1,PETSC_NULL,
+  ierr = DMDACreate1d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,appctx.m,1,1,PETSC_NULL,
                     &appctx.da);CHKERRQ(ierr);
 
   /*
-     Extract global and local vectors from DA; we use these to store the
+     Extract global and local vectors from DMDA; we use these to store the
      approximate solution.  Then duplicate these for remaining vectors that
      have the same types.
   */ 
-  ierr = DACreateGlobalVector(appctx.da,&u);CHKERRQ(ierr);
-  ierr = DACreateLocalVector(appctx.da,&appctx.u_local);CHKERRQ(ierr);
+  ierr = DMCreateGlobalVector(appctx.da,&u);CHKERRQ(ierr);
+  ierr = DMCreateLocalVector(appctx.da,&appctx.u_local);CHKERRQ(ierr);
 
   /*
      Create local work vector for use in evaluating right-hand-side function;
@@ -135,13 +135,15 @@ int main(int argc,char **argv)
 
   ierr = TSCreate(PETSC_COMM_WORLD,&ts);CHKERRQ(ierr);
   ierr = TSSetProblemType(ts,TS_NONLINEAR);CHKERRQ(ierr);
-  ierr = TSSetRHSFunction(ts,RHSFunction,&appctx);CHKERRQ(ierr);
+  ierr = TSSetRHSFunction(ts,PETSC_NULL,RHSFunction,&appctx);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Set optional user-defined monitoring routine
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = TSMonitorSet(ts,Monitor,&appctx,PETSC_NULL);CHKERRQ(ierr);
+  if (mymonitor) {
+    ierr = TSMonitorSet(ts,Monitor,&appctx,PETSC_NULL);CHKERRQ(ierr);
+  }
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      For nonlinear problems, the user can provide a Jacobian evaluation
@@ -166,7 +168,6 @@ int main(int argc,char **argv)
 
   dt   = appctx.h/2.0;
   ierr = TSSetInitialTimeStep(ts,0.0,dt);CHKERRQ(ierr);
-  ierr = TSSetSolution(ts,u);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Customize timestepping solver:  
@@ -194,20 +195,20 @@ int main(int argc,char **argv)
   /*
      Run the timestepping solver
   */
-  ierr = TSStep(ts,&steps,&ftime);CHKERRQ(ierr);
+  ierr = TSSolve(ts,u,PETSC_NULL);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Free work space.  All PETSc objects should be destroyed when they
      are no longer needed.
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = TSDestroy(ts);CHKERRQ(ierr);
-  ierr = VecDestroy(u);CHKERRQ(ierr);
-  ierr = MatDestroy(A);CHKERRQ(ierr);
-  ierr = DADestroy(appctx.da);CHKERRQ(ierr);
-  ierr = VecDestroy(appctx.localwork);CHKERRQ(ierr);
-  ierr = VecDestroy(appctx.solution);CHKERRQ(ierr);
-  ierr = VecDestroy(appctx.u_local);CHKERRQ(ierr);
+  ierr = TSDestroy(&ts);CHKERRQ(ierr);
+  ierr = VecDestroy(&u);CHKERRQ(ierr);
+  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  ierr = DMDestroy(&appctx.da);CHKERRQ(ierr);
+  ierr = VecDestroy(&appctx.localwork);CHKERRQ(ierr);
+  ierr = VecDestroy(&appctx.solution);CHKERRQ(ierr);
+  ierr = VecDestroy(&appctx.u_local);CHKERRQ(ierr);
 
   /*
      Always call PetscFinalize() before exiting a program.  This routine
@@ -215,7 +216,7 @@ int main(int argc,char **argv)
        - provides summary and diagnostic information if certain runtime
          options are chosen (e.g., -log_summary). 
   */
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
 /* --------------------------------------------------------------------- */
@@ -427,7 +428,7 @@ PetscErrorCode Monitor(TS ts,PetscInt step,PetscReal time,Vec u,void *ctx)
 PetscErrorCode RHSFunction(TS ts,PetscReal t,Vec global_in,Vec global_out,void *ctx)
 {
   AppCtx         *appctx = (AppCtx*) ctx;       /* user-defined application context */
-  DA             da = appctx->da;               /* distributed array */
+  DM             da = appctx->da;               /* distributed array */
   Vec            local_in = appctx->u_local;    /* local ghosted input vector */
   Vec            localwork = appctx->localwork; /* local ghosted work vector */
   PetscErrorCode ierr;
@@ -440,12 +441,12 @@ PetscErrorCode RHSFunction(TS ts,PetscReal t,Vec global_in,Vec global_out,void *
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   /*
      Scatter ghost points to local vector, using the 2-step process
-        DAGlobalToLocalBegin(), DAGlobalToLocalEnd().
+        DMGlobalToLocalBegin(), DMGlobalToLocalEnd().
      By placing code between these two statements, computations can be
      done while messages are in transition.
   */
-  ierr = DAGlobalToLocalBegin(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalEnd(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalBegin(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalEnd(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
 
   /*
       Access directly the values in our local INPUT work array
@@ -500,7 +501,8 @@ PetscErrorCode RHSFunction(TS ts,PetscReal t,Vec global_in,Vec global_out,void *
      Insert values from the local OUTPUT vector into the global 
      output vector
   */
-  ierr = DALocalToGlobal(da,localwork,INSERT_VALUES,global_out);CHKERRQ(ierr);
+  ierr = DMLocalToGlobalBegin(da,localwork,INSERT_VALUES,global_out);CHKERRQ(ierr);
+  ierr = DMLocalToGlobalEnd(da,localwork,INSERT_VALUES,global_out);CHKERRQ(ierr);
 
   /* Print debugging information if desired */
   if (appctx->debug) {
@@ -546,7 +548,7 @@ PetscErrorCode RHSJacobian(TS ts,PetscReal t,Vec global_in,Mat *AA,Mat *BB,MatSt
   Mat            B = *BB;                      /* Jacobian matrix */
   AppCtx         *appctx = (AppCtx*)ctx;     /* user-defined application context */
   Vec            local_in = appctx->u_local;   /* local ghosted input vector */
-  DA             da = appctx->da;              /* distributed array */
+  DM             da = appctx->da;              /* distributed array */
   PetscScalar    v[3],*localptr,sc;
   PetscErrorCode ierr;
   PetscInt       i,mstart,mend,mstarts,mends,idx[3],is;
@@ -556,12 +558,12 @@ PetscErrorCode RHSJacobian(TS ts,PetscReal t,Vec global_in,Mat *AA,Mat *BB,MatSt
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   /*
      Scatter ghost points to local vector, using the 2-step process
-        DAGlobalToLocalBegin(), DAGlobalToLocalEnd().
+        DMGlobalToLocalBegin(), DMGlobalToLocalEnd().
      By placing code between these two statements, computations can be
      done while messages are in transition.
   */
-  ierr = DAGlobalToLocalBegin(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
-  ierr = DAGlobalToLocalEnd(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalBegin(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
+  ierr = DMGlobalToLocalEnd(da,global_in,INSERT_VALUES,local_in);CHKERRQ(ierr);
 
   /*
      Get pointer to vector data
