@@ -1,14 +1,12 @@
-#define PETSCTS_DLL
-
-#include "private/tsimpl.h"      /*I "petscts.h"  I*/
+#include <private/tsimpl.h>      /*I "petscts.h"  I*/
 
 PetscFList TSList                       = PETSC_NULL;
-PetscTruth TSRegisterAllCalled          = PETSC_FALSE;
+PetscBool  TSRegisterAllCalled          = PETSC_FALSE;
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "TSSetType"
 /*@C
-  TSSetType - Sets the method for the timestepping solver.  
+  TSSetType - Sets the method for the timestepping solver.
 
   Collective on TS
 
@@ -43,36 +41,35 @@ PetscTruth TSRegisterAllCalled          = PETSC_FALSE;
 .keywords: TS, set, type
 
 @*/
-PetscErrorCode PETSCTS_DLLEXPORT TSSetType(TS ts,const TSType type)
+PetscErrorCode  TSSetType(TS ts,const TSType type)
 {
   PetscErrorCode (*r)(TS);
-  PetscTruth     match;
+  PetscBool      match;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ts, TS_COOKIE,1);
+  PetscValidHeaderSpecific(ts, TS_CLASSID,1);
   ierr = PetscTypeCompare((PetscObject) ts, type, &match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  ierr = PetscFListFind( TSList,((PetscObject)ts)->comm, type, (void (**)(void)) &r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PETSC_ERR_ARG_UNKNOWN_TYPE, "Unknown TS type: %s", type);
-  if (ts->ksp) {
-    ierr = KSPDestroy(ts->ksp);CHKERRQ(ierr);
-    ts->ksp = PETSC_NULL;
-  }
-  if (ts->snes) {
-    ierr = SNESDestroy(ts->snes);CHKERRQ(ierr);
-    ts->snes = PETSC_NULL;
-  }
+  ierr = PetscFListFind( TSList,((PetscObject)ts)->comm, type,PETSC_TRUE, (void (**)(void)) &r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE, "Unknown TS type: %s", type);
   if (ts->ops->destroy) {
     ierr = (*(ts)->ops->destroy)(ts);CHKERRQ(ierr);
   }
-  ierr = (*r)(ts);CHKERRQ(ierr);
+  ierr = PetscMemzero(ts->ops,sizeof(*ts->ops));CHKERRQ(ierr);
+  ts->setupcalled = PETSC_FALSE;
   ierr = PetscObjectChangeTypeName((PetscObject)ts, type);CHKERRQ(ierr);
+  ierr = (*r)(ts);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_AMS)
+  if (PetscAMSPublishAll) {
+    ierr = PetscObjectAMSPublish((PetscObject)ts);CHKERRQ(ierr);
+  }
+#endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "TSGetType"
 /*@C
   TSGetType - Gets the TS method type (as a string).
@@ -90,10 +87,10 @@ PetscErrorCode PETSCTS_DLLEXPORT TSSetType(TS ts,const TSType type)
 .keywords: TS, timestepper, get, type, name
 .seealso TSSetType()
 @*/
-PetscErrorCode PETSCTS_DLLEXPORT TSGetType(TS ts, const TSType *type)
+PetscErrorCode  TSGetType(TS ts, const TSType *type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ts,TS_COOKIE,1);
+  PetscValidHeaderSpecific(ts,TS_CLASSID,1);
   PetscValidPointer(type,2);
   *type = ((PetscObject)ts)->type_name;
   PetscFunctionReturn(0);
@@ -101,14 +98,14 @@ PetscErrorCode PETSCTS_DLLEXPORT TSGetType(TS ts, const TSType *type)
 
 /*--------------------------------------------------------------------------------------------------------------------*/
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "TSRegister"
 /*@C
   TSRegister - See TSRegisterDynamic()
 
   Level: advanced
 @*/
-PetscErrorCode PETSCTS_DLLEXPORT TSRegister(const char sname[], const char path[], const char name[], PetscErrorCode (*function)(TS))
+PetscErrorCode  TSRegister(const char sname[], const char path[], const char name[], PetscErrorCode (*function)(TS))
 {
   char           fullname[PETSC_MAX_PATH_LEN];
   PetscErrorCode ierr;
@@ -122,7 +119,7 @@ PetscErrorCode PETSCTS_DLLEXPORT TSRegister(const char sname[], const char path[
 }
 
 /*-------------------------------------------------------------------------------------------------------------------*/
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "TSRegisterDestroy"
 /*@C
    TSRegisterDestroy - Frees the list of timestepping routines that were registered by TSRegister()/TSRegisterDynamic().
@@ -134,7 +131,7 @@ PetscErrorCode PETSCTS_DLLEXPORT TSRegister(const char sname[], const char path[
 .keywords: TS, timestepper, register, destroy
 .seealso: TSRegister(), TSRegisterAll(), TSRegisterDynamic()
 @*/
-PetscErrorCode PETSCTS_DLLEXPORT TSRegisterDestroy(void)
+PetscErrorCode  TSRegisterDestroy(void)
 {
   PetscErrorCode ierr;
 
@@ -143,4 +140,3 @@ PetscErrorCode PETSCTS_DLLEXPORT TSRegisterDestroy(void)
   TSRegisterAllCalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
-
