@@ -1,18 +1,19 @@
-/*$Id: ex3.c,v 1.18 2001/04/10 19:34:49 bsmith Exp $*/
 /*
        Tests ISAllGather()
 */
 
 static char help[] = "Tests ISAllGather().\n\n";
 
-#include "petscis.h"
+#include <petscis.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
-  int        i,n,ierr,*indices,rank,size;
-  IS         is,newis;
+  PetscErrorCode ierr;
+  PetscInt       i,n,*indices;
+  PetscInt       rank,size;
+  IS             is,newis;
 
   ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr); 
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
@@ -22,11 +23,11 @@ int main(int argc,char **argv)
      Create IS
   */
   n = 4 + rank;
-  ierr = PetscMalloc(n*sizeof(int),&indices);CHKERRQ(ierr);
+  ierr = PetscMalloc(n*sizeof(PetscInt),&indices);CHKERRQ(ierr);
   for (i=0; i<n; i++) {
     indices[i] = rank + i;
   }
-  ierr = ISCreateGeneral(PETSC_COMM_WORLD,n,indices,&is);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_WORLD,n,indices,PETSC_COPY_VALUES,&is);CHKERRQ(ierr);
   ierr = PetscFree(indices);CHKERRQ(ierr);
 
   /*
@@ -38,9 +39,9 @@ int main(int argc,char **argv)
     ierr = ISView(newis,PETSC_VIEWER_STDOUT_SELF);CHKERRQ(ierr);
   }
 
-  ierr = ISDestroy(newis);CHKERRQ(ierr);
-  ierr = ISDestroy(is);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = ISDestroy(&newis);CHKERRQ(ierr);
+  ierr = ISDestroy(&is);CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
  
