@@ -1,5 +1,5 @@
-#include "private/fortranimpl.h"
-#include "petscis.h"
+#include <private/fortranimpl.h>
+#include <petscis.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define isblockgetindices_     ISBLOCKGETINDICES
