@@ -1,6 +1,6 @@
 
-#include "petscis.h"
-#include "../src/sys/f90/f90impl.h"
+#include <petscis.h>
+#include <../src/sys/f90-src/f90impl.h>
 
 #ifdef PETSC_HAVE_FORTRAN_CAPS
 #define isgetindicesf90_           ISGETINDICESF90
