@@ -2,11 +2,11 @@
 #ifndef _PFIMPL
 #define _PFIMPL
 
-#include "petscpf.h"
+#include <petscpf.h>
 
 typedef struct _PFOps *PFOps;
 struct _PFOps {
-  PetscErrorCode (*apply)(void*,PetscInt,PetscScalar*,PetscScalar*);
+  PetscErrorCode (*apply)(void*,PetscInt,const PetscScalar*,PetscScalar*);
   PetscErrorCode (*applyvec)(void*,Vec,Vec);
   PetscErrorCode (*destroy)(void*);
   PetscErrorCode (*view)(void*,PetscViewer);
