@@ -10,11 +10,11 @@ T*/
 /* 
   Include "petscvec.h" so that we can use vectors.  Note that this file
   automatically includes:
-     petsc.h       - base PETSc routines   petscis.h     - index sets
-     petscsys.h    - system routines       petscviewer.h - viewers
+     petscsys.h       - base PETSc routines   petscis.h     - index sets
+     petscviewer.h - viewers
 */
-#include "petscvec.h"
-#include "stdlib.h"
+#include <petscvec.h>
+#include <stdlib.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -25,15 +25,15 @@ int main(int argc,char **argv)
   PetscInt       i,istart,iend,n = 6,m,*indices;
   PetscScalar    *values;
   Vec            x;
-  PetscTruth     set_option_negidx = PETSC_FALSE, set_values_negidx = PETSC_FALSE, get_values_negidx = PETSC_FALSE;
+  PetscBool      set_option_negidx = PETSC_FALSE, set_values_negidx = PETSC_FALSE, get_values_negidx = PETSC_FALSE;
 
   ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr); 
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
 
   ierr = PetscOptionsGetInt(PETSC_NULL,"-n",&n,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetTruth(PETSC_NULL, "-set_option_negidx", &set_option_negidx, PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetTruth(PETSC_NULL, "-set_values_negidx", &set_values_negidx, PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetTruth(PETSC_NULL, "-get_values_negidx", &get_values_negidx, PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(PETSC_NULL, "-set_option_negidx", &set_option_negidx, PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(PETSC_NULL, "-set_values_negidx", &set_values_negidx, PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(PETSC_NULL, "-get_values_negidx", &get_values_negidx, PETSC_NULL);CHKERRQ(ierr);
   
   ierr = VecCreate(PETSC_COMM_WORLD,&x);CHKERRQ(ierr);
   ierr = VecSetSizes(x,PETSC_DECIDE,n);CHKERRQ(ierr);
@@ -49,11 +49,11 @@ int main(int argc,char **argv)
   /* Set the vectors */
   
   ierr = PetscMalloc(n*sizeof(PetscScalar),&values);CHKERRQ(ierr);
-  ierr = PetscMalloc(n * sizeof(PetscInt),&indices);CHKERRQ(ierr);
+  ierr = PetscMalloc(n*sizeof(PetscInt),&indices);CHKERRQ(ierr);
 
   for (i=istart; i<iend; i++) {
     values[i - istart]  = (rank + 1) * i * 2;
-    if (set_values_negidx == PETSC_TRUE) {
+    if (set_values_negidx) {
         indices[i - istart] = (-1 + 2*(i % 2)) * i;
     }
     else {
@@ -111,11 +111,11 @@ int main(int argc,char **argv)
      Free work space.
   */
 
-  ierr = VecDestroy(x);CHKERRQ(ierr);
+  ierr = VecDestroy(&x);CHKERRQ(ierr);
   ierr = PetscFree(values);CHKERRQ(ierr);
   ierr = PetscFree(indices);CHKERRQ(ierr);
 
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = PetscFinalize();
 
   return 0;
 }
