@@ -1,4 +1,3 @@
-
 /* Program usage:  mpiexec ex1 [-help] [all PETSc options] */
 
 static char help[] = "Basic vector routines.\n\n";
@@ -11,11 +10,11 @@ T*/
 /* 
   Include "petscvec.h" so that we can use vectors.  Note that this file
   automatically includes:
-     petsc.h       - base PETSc routines   petscis.h     - index sets
-     petscsys.h    - system routines       petscviewer.h - viewers
+     petscsys.h       - base PETSc routines   petscis.h     - index sets
+     petscviewer.h - viewers
 */
 
-#include "petscvec.h"
+#include <petscvec.h>
 
 #undef __FUNCT__
 #define __FUNCT__ "main"
@@ -23,10 +22,9 @@ int main(int argc,char **argv)
 {
   Vec            x,y,w;               /* vectors */
   Vec            *z;                    /* array of vectors */
-  PetscReal      norm,v,v1,v2;
-  PetscInt       n = 20;
+  PetscReal      norm,v,v1,v2,maxval;
+  PetscInt       n = 20,maxind;
   PetscErrorCode ierr;
-  PetscTruth     flg;
   PetscScalar    one = 1.0,two = 2.0,three = 3.0,dots[3],dot;
 
   ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr); 
@@ -48,13 +46,13 @@ int main(int argc,char **argv)
 
      With VecCreate(), VecSetSizes() and VecSetFromOptions() the option -vec_type mpi or 
      -vec_type shared causes the particular type of vector to be formed.
+y
 
   */
 
   ierr = VecCreate(PETSC_COMM_WORLD,&x);CHKERRQ(ierr);
   ierr = VecSetSizes(x,PETSC_DECIDE,n);CHKERRQ(ierr);
   ierr = VecSetFromOptions(x);CHKERRQ(ierr);
-
   /*
      Duplicate some work vectors (of the same format and
      partitioning as the initial vector).
@@ -62,8 +60,6 @@ int main(int argc,char **argv)
   ierr = VecDuplicate(x,&y);CHKERRQ(ierr);
   ierr = VecDuplicate(x,&w);CHKERRQ(ierr);
   ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
-
-
   /*
      Duplicate more work vectors (of the same format and
      partitioning as the initial vector).  Here we duplicate
@@ -71,7 +67,6 @@ int main(int argc,char **argv)
      duplicating individual ones.
   */
   ierr = VecDuplicateVecs(x,3,&z);CHKERRQ(ierr); 
-
   /*
      Set the vectors to entries to a constant value.
   */
@@ -80,7 +75,6 @@ int main(int argc,char **argv)
   ierr = VecSet(z[0],one);CHKERRQ(ierr);
   ierr = VecSet(z[1],two);CHKERRQ(ierr);
   ierr = VecSet(z[2],three);CHKERRQ(ierr);
-
   /*
      Demonstrate various basic vector routines.
   */
@@ -92,6 +86,9 @@ int main(int argc,char **argv)
      PETSC_USE_COMPLEX is defined in the makefiles; otherwise,
      (when using real numbers) it is undefined.
   */
+  //printf("Size Of Void* =  %ld, Size Of PetscErrorCode =  %ld\n",sizeof(void*),sizeof(PetscErrorCode));
+  //printf("Size of PetscBool = %ld\n",sizeof(PetscBool));
+
 #if defined(PETSC_USE_COMPLEX)
   ierr = PetscPrintf(PETSC_COMM_WORLD,"Vector length %D\n",(PetscInt) (PetscRealPart(dot)));CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"Vector length %D %D %D\n",(PetscInt)PetscRealPart(dots[0]),
@@ -101,52 +98,71 @@ int main(int argc,char **argv)
   ierr = PetscPrintf(PETSC_COMM_WORLD,"Vector length %D %D %D\n",(PetscInt)dots[0],
                              (PetscInt)dots[1],(PetscInt)dots[2]);CHKERRQ(ierr);
 #endif
+  //struct timeval t1, t2, result;
+  //gettimeofday(&t1,NULL);
+  ierr = VecMax(x,&maxind,&maxval);CHKERRQ(ierr);
+  //gettimeofday(&t2,NULL);
+  //timersub(&t2,&t1,&result);
+  //printf("Time for VecMax = %lf\n",result.tv_sec+result.tv_usec/1000000.0);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecMax %g, VecInd %D\n",(double)maxval,maxind);CHKERRQ(ierr);
 
+  ierr = VecMin(x,&maxind,&maxval);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecMin %g, VecInd %D\n",(double)maxval,maxind);CHKERRQ(ierr);
+  /*
+  const PetscInt ix_kds[] = {3,7,14};
+  const PetscScalar y_kds[] = {13.2,69.3,-8.7};
+  ierr = VecSetValues(x,3,ix_kds,y_kds,INSERT_VALUES);CHKERRQ(ierr);
+  ierr = VecMax(x,&maxind,&maxval);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecMax %g, VecInd %D\n",maxval,maxind);CHKERRQ(ierr);
+  ierr = VecMin(x,&maxind,&maxval);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecMax %g, VecInd %D\n",maxval,maxind);CHKERRQ(ierr);
+   */
   ierr = PetscPrintf(PETSC_COMM_WORLD,"All other values should be near zero\n");CHKERRQ(ierr);
 
+
   ierr = VecScale(x,two);CHKERRQ(ierr);
   ierr = VecNorm(x,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-2.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecScale %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecScale %g\n",(double)v);CHKERRQ(ierr);
 
 
   ierr = VecCopy(x,w);CHKERRQ(ierr);
   ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-2.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecCopy  %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecCopy  %g\n",(double)v);CHKERRQ(ierr);
 
   ierr = VecAXPY(y,three,x);CHKERRQ(ierr);
   ierr = VecNorm(y,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-8.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecAXPY %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecAXPY %g\n",(double)v);CHKERRQ(ierr);
 
   ierr = VecAYPX(y,two,x);CHKERRQ(ierr);
   ierr = VecNorm(y,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-18.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecAYPX %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecAYPX %g\n",(double)v);CHKERRQ(ierr);
 
   ierr = VecSwap(x,y);CHKERRQ(ierr);
   ierr = VecNorm(y,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-2.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecSwap  %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecSwap  %g\n",(double)v);CHKERRQ(ierr);
   ierr = VecNorm(x,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-18.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecSwap  %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecSwap  %g\n",(double)v);CHKERRQ(ierr);
 
   ierr = VecWAXPY(w,two,x,y);CHKERRQ(ierr);
   ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-38.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecWAXPY %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecWAXPY %g\n",(double)v);CHKERRQ(ierr);
 
   ierr = VecPointwiseMult(w,y,x);CHKERRQ(ierr);
   ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr); 
   v = norm-36.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecPointwiseMult %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecPointwiseMult %g\n",(double)v);CHKERRQ(ierr);
 
   ierr = VecPointwiseDivide(w,x,y);CHKERRQ(ierr);
   ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
   v = norm-9.0*sqrt((double)n); if (v > -PETSC_SMALL && v < PETSC_SMALL) v = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecPointwiseDivide %G\n",v);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecPointwiseDivide %g\n",(double)v);CHKERRQ(ierr);
 
   dots[0] = one;
   dots[1] = three;
@@ -159,24 +175,18 @@ int main(int argc,char **argv)
   v1 = norm-2.0*sqrt((double)n); if (v1 > -PETSC_SMALL && v1 < PETSC_SMALL) v1 = 0.0; 
   ierr = VecNorm(z[2],NORM_2,&norm);CHKERRQ(ierr);
   v2 = norm-3.0*sqrt((double)n); if (v2 > -PETSC_SMALL && v2 < PETSC_SMALL) v2 = 0.0; 
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecMAXPY %G %G %G \n",v,v1,v2);CHKERRQ(ierr);
-
-  /* 
-     Test whether vector has been corrupted (just to demonstrate this
-     routine) not needed in most application codes.
-  */
-  ierr = VecValid(x,&flg);CHKERRQ(ierr);
-  if (!flg) SETERRQ(1,"Corrupted vector.");
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"VecMAXPY %g %g %g \n",(double)v,(double)v1,(double)v2);CHKERRQ(ierr);
+  
 
   /* 
      Free work space.  All PETSc objects should be destroyed when they
      are no longer needed.
   */
-  ierr = VecDestroy(x);CHKERRQ(ierr);
-  ierr = VecDestroy(y);CHKERRQ(ierr);
-  ierr = VecDestroy(w);CHKERRQ(ierr);
-  ierr = VecDestroyVecs(z,3);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
+  ierr = VecDestroy(&x);CHKERRQ(ierr);
+  ierr = VecDestroy(&y);CHKERRQ(ierr);
+  ierr = VecDestroy(&w);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(3,&z);CHKERRQ(ierr);
+  ierr = PetscFinalize();
   return 0;
 }
  
