@@ -2,7 +2,7 @@
 
 static char help[] = "Tests Mathematica I/O of vectors and illustrates the use of user-defined event logging.\n\n";
 
-#include "petscvec.h"
+#include <petscvec.h>
 
 /* Note:  Most applications would not read and write a vector within
   the same program.  This example is intended only to demonstrate
@@ -26,7 +26,7 @@ int main(int argc, char *argv[])
 
   /* PART 1:  Generate vector, then write it to Mathematica */
 
-  ierr = PetscLogEventRegister("Generate Vector", VEC_COOKIE,&VECTOR_GENERATE);                           CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("Generate Vector", VEC_CLASSID,&VECTOR_GENERATE);                           CHKERRQ(ierr);
   ierr = PetscLogEventBegin(VECTOR_GENERATE, 0, 0, 0, 0);                                                 CHKERRQ(ierr);
   /* Generate vector */
   ierr = VecCreate(PETSC_COMM_WORLD, &u);                                                                 CHKERRQ(ierr);
@@ -61,13 +61,13 @@ int main(int argc, char *argv[])
 
   /* PART 2:  Read in vector in from Mathematica */
 
-  ierr = PetscLogEventRegister("Read Vector", VEC_COOKIE,&VECTOR_READ);                                   CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("Read Vector", VEC_CLASSID,&VECTOR_READ);                                   CHKERRQ(ierr);
   ierr = PetscLogEventBegin(VECTOR_READ, 0, 0, 0, 0);                                                     CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD, "reading vector from Mathematica...\n");                           CHKERRQ(ierr);
   /* Read new vector in binary format */
 #if 0
   ierr = PetscViewerMathematicaGetVector(viewer, u);                                                      CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(viewer);                                                                      CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&viewer);                                                                      CHKERRQ(ierr);
 #else
   ierr = PetscViewerMathematicaGetVector(PETSC_VIEWER_MATHEMATICA_WORLD, u);                              CHKERRQ(ierr);
 #endif
@@ -75,7 +75,7 @@ int main(int argc, char *argv[])
   ierr = VecView(u, PETSC_VIEWER_STDOUT_WORLD);                                                           CHKERRQ(ierr);
 
   /* Free data structures */
-  ierr = VecDestroy(u);                                                                                   CHKERRQ(ierr);
+  ierr = VecDestroy(&u);                                                                                   CHKERRQ(ierr);
   ierr = PetscFinalize();                                                                                 CHKERRQ(ierr);
   return 0;
 }
