@@ -1,19 +1,17 @@
-#define PETSCVEC_DLL
 
 /* 
    Defines some vector operation functions that are shared by 
   sequential and parallel vectors.
 */
-#include "../src/vec/vec/impls/dvecimpl.h"   
-#include "private/petscaxpy.h"
+#include <../src/vec/vec/impls/dvecimpl.h>   
+#include <private/petscaxpy.h>
 
 #if defined(PETSC_USE_FORTRAN_KERNEL_MDOT)
-#include "../src/vec/vec/impls/seq/ftn-kernels/fmdot.h"
+#include <../src/vec/vec/impls/seq/ftn-kernels/fmdot.h>
 #undef __FUNCT__  
 #define __FUNCT__ "VecMDot_Seq"
 PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
 {
-  Vec_Seq           *xv = (Vec_Seq *)xin->data;
   PetscErrorCode    ierr;
   PetscInt          i,nv_rem,n = xin->map->n;
   PetscScalar       sum0,sum1,sum2,sum3;
@@ -28,34 +26,34 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
   i      = nv;
   nv_rem = nv&0x3;
   yy     = (Vec*)yin;
-  x      = xv->array;
+  ierr   = VecGetArrayRead(xin,&x);CHKERRQ(ierr); 
 
   switch (nv_rem) {
   case 3:
-    ierr = VecGetArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[2],&yy2);CHKERRQ(ierr);
     fortranmdot3_(x,yy0,yy1,yy2,&n,&sum0,&sum1,&sum2);
-    ierr = VecRestoreArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[2],&yy2);CHKERRQ(ierr);
     z[0] = sum0;
     z[1] = sum1;
     z[2] = sum2;
     break;
   case 2:
-    ierr = VecGetArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
     fortranmdot2_(x,yy0,yy1,&n,&sum0,&sum1);
-    ierr = VecRestoreArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    z[0] = sum0;
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    z[0] = sum0; 
     z[1] = sum1;
     break;
   case 1:
-    ierr = VecGetArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
     fortranmdot1_(x,yy0,&n,&sum0);
-    ierr = VecRestoreArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
     z[0] = sum0;
     break;
   case 0:
@@ -70,15 +68,15 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     sum1 = 0.;
     sum2 = 0.;
     sum3 = 0.;
-    ierr = VecGetArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[3],(PetscScalar**)&yy3);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[3],&yy3);CHKERRQ(ierr);
     fortranmdot4_(x,yy0,yy1,yy2,yy3,&n,&sum0,&sum1,&sum2,&sum3);
-    ierr = VecRestoreArray(yy[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[3],(PetscScalar**)&yy3);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[3],&yy3);CHKERRQ(ierr);
     yy  += 4;
     z[0] = sum0;
     z[1] = sum1;
@@ -87,6 +85,7 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z   += 4;
     i   -= 4;
   }
+  ierr = VecRestoreArrayRead(xin,&x);CHKERRQ(ierr); 
   ierr = PetscLogFlops(PetscMax(nv*(2.0*xin->map->n-1),0.0));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -96,11 +95,10 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
 #define __FUNCT__ "VecMDot_Seq"
 PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
 {
-  Vec_Seq           *xv = (Vec_Seq *)xin->data;
   PetscErrorCode    ierr;
   PetscInt          n = xin->map->n,i,j,nv_rem,j_rem;
   PetscScalar       sum0,sum1,sum2,sum3,x0,x1,x2,x3;
-  const PetscScalar * PETSC_RESTRICT yy0,* PETSC_RESTRICT yy1,* PETSC_RESTRICT yy2,*PETSC_RESTRICT yy3,* PETSC_RESTRICT x;
+  const PetscScalar *yy0,*yy1,*yy2,*yy3,*x,*xbase;
   Vec               *yy;
 
   PetscFunctionBegin;
@@ -112,13 +110,14 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
   nv_rem = nv&0x3;
   yy     = (Vec *)yin;
   j      = n;
-  x      = xv->array;
+  ierr   = VecGetArrayRead(xin,&xbase);CHKERRQ(ierr); 
+  x      = xbase;
 
   switch (nv_rem) {
   case 3:
-    ierr = VecGetArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],(PetscScalar **)&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[2],(PetscScalar **)&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[2],&yy2);CHKERRQ(ierr);
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; 
@@ -155,13 +154,13 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z[0] = sum0;
     z[1] = sum1;
     z[2] = sum2;
-    ierr = VecRestoreArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],(PetscScalar **)&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[2],(PetscScalar **)&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[2],&yy2);CHKERRQ(ierr);
     break;
   case 2:
-    ierr = VecGetArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],(PetscScalar **)&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; 
@@ -193,11 +192,11 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z[0] = sum0;
     z[1] = sum1;
  
-    ierr = VecRestoreArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],(PetscScalar **)&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
     break;
   case 1:
-    ierr = VecGetArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; sum0 += x2*PetscConj(yy0[2]);
@@ -219,7 +218,7 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     }
     z[0] = sum0;
 
-    ierr = VecRestoreArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
     break;
   case 0:
     break;
@@ -233,13 +232,13 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     sum1 = 0.;
     sum2 = 0.;
     sum3 = 0.;
-    ierr = VecGetArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],(PetscScalar **)&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[2],(PetscScalar **)&yy2);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[3],(PetscScalar **)&yy3);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[3],&yy3);CHKERRQ(ierr);
 
     j = n;
-    x = xv->array;
+    x = xbase;
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; 
@@ -281,12 +280,13 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z[3] = sum3;
     z   += 4;
     i   -= 4;
-    ierr = VecRestoreArray(yy[0],(PetscScalar **)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],(PetscScalar **)&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[2],(PetscScalar **)&yy2);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[3],(PetscScalar **)&yy3);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[3],&yy3);CHKERRQ(ierr);
     yy  += 4;
   }
+  ierr = VecRestoreArrayRead(xin,&xbase);CHKERRQ(ierr); 
   ierr = PetscLogFlops(PetscMax(nv*(2.0*xin->map->n-1),0.0));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -297,11 +297,11 @@ PetscErrorCode VecMDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
 #define __FUNCT__ "VecMTDot_Seq"
 PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
 {
-  Vec_Seq        *xv = (Vec_Seq *)xin->data;
-  PetscErrorCode ierr;
-  PetscInt       n = xin->map->n,i,j,nv_rem,j_rem;
-  PetscScalar    sum0,sum1,sum2,sum3,*yy0,*yy1,*yy2,*yy3,x0,x1,x2,x3,*x;
-  Vec            *yy;
+  PetscErrorCode    ierr;
+  PetscInt          n = xin->map->n,i,j,nv_rem,j_rem;
+  PetscScalar       sum0,sum1,sum2,sum3,x0,x1,x2,x3;
+  const PetscScalar *yy0,*yy1,*yy2,*yy3,*x;
+  Vec               *yy;
   
   PetscFunctionBegin;
 
@@ -312,14 +312,14 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
   i      = nv;
   nv_rem = nv&0x3;
   yy     = (Vec*)yin;
-  j    = n;
-  x    = xv->array;
+  j      = n;
+  ierr   = VecGetArrayRead(xin,&x);CHKERRQ(ierr);
 
   switch (nv_rem) {
   case 3:
-    ierr = VecGetArray(yy[0],&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[2],&yy2);CHKERRQ(ierr);
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; 
@@ -356,13 +356,13 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z[0] = sum0;
     z[1] = sum1;
     z[2] = sum2;
-    ierr = VecRestoreArray(yy[0],&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[2],&yy2);CHKERRQ(ierr);
     break;
   case 2:
-    ierr = VecGetArray(yy[0],&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; 
@@ -394,11 +394,11 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z[0] = sum0;
     z[1] = sum1;
  
-    ierr = VecRestoreArray(yy[0],&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
     break;
   case 1:
-    ierr = VecGetArray(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; sum0 += x2*yy0[2];
@@ -418,7 +418,7 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     }
     z[0] = sum0;
 
-    ierr = VecRestoreArray(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
     break;
   case 0:
     break;
@@ -432,13 +432,12 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     sum1 = 0.;
     sum2 = 0.;
     sum3 = 0.;
-    ierr = VecGetArray(yy[0],&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[1],&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[2],&yy2);CHKERRQ(ierr);
-    ierr = VecGetArray(yy[3],&yy3);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(yy[3],&yy3);CHKERRQ(ierr);
 
     j = n;
-    x = xv->array;
     switch (j_rem=j&0x3) {
     case 3: 
       x2 = x[2]; 
@@ -480,12 +479,13 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
     z[3] = sum3;
     z   += 4;
     i   -= 4;
-    ierr = VecRestoreArray(yy[0],&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[1],&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[2],&yy2);CHKERRQ(ierr);
-    ierr = VecRestoreArray(yy[3],&yy3);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[2],&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(yy[3],&yy3);CHKERRQ(ierr);
     yy  += 4;
   }
+  ierr = VecRestoreArrayRead(xin,&x);CHKERRQ(ierr);
   ierr = PetscLogFlops(PetscMax(nv*(2.0*xin->map->n-1),0.0));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -495,14 +495,15 @@ PetscErrorCode VecMTDot_Seq(Vec xin,PetscInt nv,const Vec yin[],PetscScalar *z)
 #define __FUNCT__ "VecMax_Seq"
 PetscErrorCode VecMax_Seq(Vec xin,PetscInt* idx,PetscReal * z)
 { 
-  Vec_Seq      *x = (Vec_Seq*)xin->data;
-  PetscInt     i,j=0,n = xin->map->n;
-  PetscReal    max,tmp;
-  PetscScalar  *xx = x->array;
+  PetscInt          i,j=0,n = xin->map->n;
+  PetscReal         max,tmp;
+  const PetscScalar *xx;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
+  ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
   if (!n) {
-    max = PETSC_MIN;
+    max = PETSC_MIN_REAL;
     j   = -1;
   } else {
 #if defined(PETSC_USE_COMPLEX)
@@ -520,6 +521,7 @@ PetscErrorCode VecMax_Seq(Vec xin,PetscInt* idx,PetscReal * z)
   }
   *z   = max;
   if (idx) *idx = j;
+  ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -527,14 +529,15 @@ PetscErrorCode VecMax_Seq(Vec xin,PetscInt* idx,PetscReal * z)
 #define __FUNCT__ "VecMin_Seq"
 PetscErrorCode VecMin_Seq(Vec xin,PetscInt* idx,PetscReal * z)
 {
-  Vec_Seq      *x = (Vec_Seq*)xin->data;
-  PetscInt     i,j=0,n = xin->map->n;
-  PetscReal    min,tmp;
-  PetscScalar  *xx = x->array;
+  PetscInt          i,j=0,n = xin->map->n;
+  PetscReal         min,tmp;
+  const PetscScalar *xx;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
+  ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
   if (!n) {
-    min = PETSC_MAX;
+    min = PETSC_MAX_REAL;
     j   = -1;
   } else {
 #if defined(PETSC_USE_COMPLEX)
@@ -552,6 +555,7 @@ PetscErrorCode VecMin_Seq(Vec xin,PetscInt* idx,PetscReal * z)
   }
   *z   = min;
   if (idx) *idx = j;
+  ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -559,40 +563,26 @@ PetscErrorCode VecMin_Seq(Vec xin,PetscInt* idx,PetscReal * z)
 #define __FUNCT__ "VecSet_Seq"
 PetscErrorCode VecSet_Seq(Vec xin,PetscScalar alpha)
 {
-  Vec_Seq        *x = (Vec_Seq *)xin->data;
-  PetscErrorCode ierr;
   PetscInt       i,n = xin->map->n;
-  PetscScalar    *xx = x->array;
+  PetscScalar    *xx;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (alpha == 0.0) {
+  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
+  if (alpha == (PetscScalar)0.0) {
     ierr = PetscMemzero(xx,n*sizeof(PetscScalar));CHKERRQ(ierr);
   } else {
     for (i=0; i<n; i++) xx[i] = alpha;
   }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecSetRandom_Seq"
-PetscErrorCode VecSetRandom_Seq(Vec xin,PetscRandom r)
-{
-  PetscErrorCode ierr;
-  PetscInt       n = xin->map->n,i;
-  PetscScalar    *xx;
-
-  PetscFunctionBegin;
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
-  for (i=0; i<n; i++) {ierr = PetscRandomGetValue(r,&xx[i]);CHKERRQ(ierr);}
   ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+
 #undef __FUNCT__  
 #define __FUNCT__ "VecMAXPY_Seq"
 PetscErrorCode VecMAXPY_Seq(Vec xin, PetscInt nv,const PetscScalar *alpha,Vec *y)
 {
-  Vec_Seq           *xdata = (Vec_Seq*)xin->data;
   PetscErrorCode    ierr;
   PetscInt          n = xin->map->n,j,j_rem;
   const PetscScalar *yy0,*yy1,*yy2,*yy3;
@@ -604,47 +594,46 @@ PetscErrorCode VecMAXPY_Seq(Vec xin, PetscInt nv,const PetscScalar *alpha,Vec *y
 
   PetscFunctionBegin;
   ierr = PetscLogFlops(nv*2.0*n);CHKERRQ(ierr);
-
-  xx = xdata->array;  
+  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
   switch (j_rem=nv&0x3) {
   case 3: 
-    ierr = VecGetArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(y[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(y[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[2],&yy2);CHKERRQ(ierr);
     alpha0 = alpha[0]; 
     alpha1 = alpha[1]; 
     alpha2 = alpha[2]; 
     alpha += 3;
     PetscAXPY3(xx,alpha0,alpha1,alpha2,yy0,yy1,yy2,n);
-    ierr = VecRestoreArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(y[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(y[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[2],&yy2);CHKERRQ(ierr);
     y     += 3;
     break;
   case 2: 
-    ierr = VecGetArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(y[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[1],&yy1);CHKERRQ(ierr);
     alpha0 = alpha[0]; 
     alpha1 = alpha[1]; 
     alpha +=2;
     PetscAXPY2(xx,alpha0,alpha1,yy0,yy1,n);
-    ierr = VecRestoreArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(y[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[1],&yy1);CHKERRQ(ierr);
     y     +=2;
     break;
   case 1: 
-    ierr = VecGetArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[0],&yy0);CHKERRQ(ierr);
     alpha0 = *alpha++; 
     PetscAXPY(xx,alpha0,yy0,n);
-    ierr = VecRestoreArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[0],&yy0);CHKERRQ(ierr);
     y     +=1;
     break;
   }
   for (j=j_rem; j<nv; j+=4) {
-    ierr = VecGetArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecGetArray(y[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecGetArray(y[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
-    ierr = VecGetArray(y[3],(PetscScalar**)&yy3);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[0],&yy0);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[1],&yy1);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[2],&yy2);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(y[3],&yy3);CHKERRQ(ierr);
     alpha0 = alpha[0];
     alpha1 = alpha[1];
     alpha2 = alpha[2];
@@ -652,41 +641,44 @@ PetscErrorCode VecMAXPY_Seq(Vec xin, PetscInt nv,const PetscScalar *alpha,Vec *y
     alpha  += 4;
 
     PetscAXPY4(xx,alpha0,alpha1,alpha2,alpha3,yy0,yy1,yy2,yy3,n);
-    ierr = VecRestoreArray(y[0],(PetscScalar**)&yy0);CHKERRQ(ierr);
-    ierr = VecRestoreArray(y[1],(PetscScalar**)&yy1);CHKERRQ(ierr);
-    ierr = VecRestoreArray(y[2],(PetscScalar**)&yy2);CHKERRQ(ierr);
-    ierr = VecRestoreArray(y[3],(PetscScalar**)&yy3);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[0],&yy0);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[1],&yy1);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[2],&yy2);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(y[3],&yy3);CHKERRQ(ierr);
     y      += 4;
   }
+  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 } 
 
-#include "../src/vec/vec/impls/seq/ftn-kernels/faypx.h"
+#include <../src/vec/vec/impls/seq/ftn-kernels/faypx.h>
 #undef __FUNCT__  
 #define __FUNCT__ "VecAYPX_Seq"
 PetscErrorCode VecAYPX_Seq(Vec yin,PetscScalar alpha,Vec xin)
 {
-  Vec_Seq           *y = (Vec_Seq *)yin->data;
   PetscErrorCode    ierr;
   PetscInt          n = yin->map->n;
-  PetscScalar       *yy = y->array;
+  PetscScalar       *yy;
   const PetscScalar *xx;
 
   PetscFunctionBegin;
-  if (alpha == 0.0) {
-    ierr = VecCopy_Seq(xin,yin);CHKERRQ(ierr);
-  } else if (alpha == 1.0) {
+  if (alpha == (PetscScalar)0.0) {
+    ierr = VecCopy(xin,yin);CHKERRQ(ierr);
+  } else if (alpha == (PetscScalar)1.0) {
     ierr = VecAXPY_Seq(yin,alpha,xin);CHKERRQ(ierr);
-  } else if (alpha == -1.0) {
+  } else if (alpha == (PetscScalar)-1.0) {
     PetscInt i;
-    ierr = VecGetArray(xin,(PetscScalar**)&xx);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
+    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
     for (i=0; i<n; i++) {
       yy[i] = xx[i] - yy[i];
     }
-    ierr = VecRestoreArray(xin,(PetscScalar**)&xx);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
+    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
     ierr = PetscLogFlops(1.0*n);CHKERRQ(ierr);
   } else {
-    ierr = VecGetArray(xin,(PetscScalar**)&xx);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
+    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
 #if defined(PETSC_USE_FORTRAN_KERNEL_AYPX)
     {
       PetscScalar oalpha = alpha;
@@ -700,13 +692,14 @@ PetscErrorCode VecAYPX_Seq(Vec yin,PetscScalar alpha,Vec xin)
       }
     }
 #endif
-    ierr = VecRestoreArray(xin,(PetscScalar**)&xx);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
+    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
     ierr = PetscLogFlops(2.0*n);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#include "../src/vec/vec/impls/seq/ftn-kernels/fwaxpy.h"
+#include <../src/vec/vec/impls/seq/ftn-kernels/fwaxpy.h>
 /*
    IBM ESSL contains a routine dzaxpy() that is our WAXPY() but it appears
   to be slower than a regular C loop.  Hence,we do not include it.
@@ -714,26 +707,26 @@ PetscErrorCode VecAYPX_Seq(Vec yin,PetscScalar alpha,Vec xin)
 */
 
 #undef __FUNCT__  
-#define __FUNCT__ "VecWAXPY_Seq"
+#define __FUNCT__ "VecWAXPY_Seq" 
 PetscErrorCode VecWAXPY_Seq(Vec win, PetscScalar alpha,Vec xin,Vec yin)
 {
-  Vec_Seq            *w = (Vec_Seq *)win->data;
   PetscErrorCode     ierr;
   PetscInt           i,n = win->map->n;
-  PetscScalar        *ww = w->array;
+  PetscScalar        *ww;
   const PetscScalar  *yy,*xx;
 
   PetscFunctionBegin;
-  ierr = VecGetArray(yin,(PetscScalar**)&yy);CHKERRQ(ierr);
-  ierr = VecGetArray(xin,(PetscScalar**)&xx);CHKERRQ(ierr);
-  if (alpha == 1.0) {
+  ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(yin,&yy);CHKERRQ(ierr);
+  ierr = VecGetArray(win,&ww);CHKERRQ(ierr);
+  if (alpha == (PetscScalar)1.0) {
     ierr = PetscLogFlops(n);CHKERRQ(ierr);
     /* could call BLAS axpy after call to memcopy, but may be slower */
     for (i=0; i<n; i++) ww[i] = yy[i] + xx[i];
-  } else if (alpha == -1.0) {
+  } else if (alpha == (PetscScalar)-1.0) {
     ierr = PetscLogFlops(n);CHKERRQ(ierr);
     for (i=0; i<n; i++) ww[i] = yy[i] - xx[i];
-  } else if (alpha == 0.0) {
+  } else if (alpha == (PetscScalar)0.0) {
     ierr = PetscMemcpy(ww,yy,n*sizeof(PetscScalar));CHKERRQ(ierr);
   } else {
     PetscScalar oalpha = alpha;
@@ -744,237 +737,39 @@ PetscErrorCode VecWAXPY_Seq(Vec win, PetscScalar alpha,Vec xin,Vec yin)
 #endif
     ierr = PetscLogFlops(2.0*n);CHKERRQ(ierr);
   }
-  ierr = VecRestoreArray(yin,(PetscScalar**)&yy);CHKERRQ(ierr);
-  ierr = VecRestoreArray(xin,(PetscScalar**)&xx);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(yin,&yy);CHKERRQ(ierr);
+  ierr = VecRestoreArray(win,&ww);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "VecPointwiseMax_Seq"
-PetscErrorCode VecPointwiseMax_Seq(Vec win,Vec xin,Vec yin)
-{
-  Vec_Seq        *w = (Vec_Seq *)win->data;
-  PetscErrorCode ierr;
-  PetscInt       n = win->map->n,i;
-  PetscScalar    *ww = w->array,*xx,*yy;
-
-  PetscFunctionBegin;
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
-  } else {
-    yy = xx;
-  }
-  for (i=0; i<n; i++) {
-    ww[i] = PetscMax(PetscRealPart(xx[i]),PetscRealPart(yy[i]));
-  }
-  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
-  }
-  ierr = PetscLogFlops(n);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecPointwiseMin_Seq"
-PetscErrorCode VecPointwiseMin_Seq(Vec win,Vec xin,Vec yin)
-{
-  Vec_Seq        *w = (Vec_Seq *)win->data;
-  PetscErrorCode ierr;
-  PetscInt       n = win->map->n,i;
-  PetscScalar    *ww = w->array,*xx,*yy;
-
-  PetscFunctionBegin;
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
-  } else {
-    yy = xx;
-  }
-  for (i=0; i<n; i++) {
-    ww[i] = PetscMin(PetscRealPart(xx[i]),PetscRealPart(yy[i]));
-  }
-  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
-  }
-  ierr = PetscLogFlops(n);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecPointwiseMaxAbs_Seq"
-PetscErrorCode VecPointwiseMaxAbs_Seq(Vec win,Vec xin,Vec yin)
-{
-  Vec_Seq        *w = (Vec_Seq *)win->data;
-  PetscErrorCode ierr;
-  PetscInt       n = win->map->n,i;
-  PetscScalar    *ww = w->array,*xx,*yy;
-
-  PetscFunctionBegin;
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
-  } else {
-    yy = xx;
-  }
-  for (i=0; i<n; i++) {
-    ww[i] = PetscMax(PetscAbsScalar(xx[i]),PetscAbsScalar(yy[i]));
-  }
-  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
-  }
-  ierr = PetscLogFlops(n);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#include "../src/vec/vec/impls/seq/ftn-kernels/fxtimesy.h"
-#undef __FUNCT__  
-#define __FUNCT__ "VecPointwiseMult_Seq"
-PetscErrorCode VecPointwiseMult_Seq(Vec win,Vec xin,Vec yin)
-{
-  Vec_Seq        *w = (Vec_Seq *)win->data;
-  PetscErrorCode ierr;
-  PetscInt       n = win->map->n,i;
-  PetscScalar    *ww = w->array,*xx,*yy;
-
-  PetscFunctionBegin;
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
-  } else {
-    yy = xx;
-  }
-
-  if (ww == xx) {
-    for (i=0; i<n; i++) ww[i] *= yy[i];
-  } else if (ww == yy) {
-    for (i=0; i<n; i++) ww[i] *= xx[i];
-  } else {
-    /*  This was suppose to help on SGI but didn't really seem to
-          PetscReal * PETSC_RESTRICT www = ww;
-          PetscReal * PETSC_RESTRICT yyy = yy;
-          PetscReal * PETSC_RESTRICT xxx = xx;
-          for (i=0; i<n; i++) www[i] = xxx[i] * yyy[i];
-    */
-#if defined(PETSC_USE_FORTRAN_KERNEL_XTIMESY)
-    fortranxtimesy_(xx,yy,ww,&n);
-#else
-    for (i=0; i<n; i++) ww[i] = xx[i] * yy[i];
-#endif
-  }
-  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
-  }
-  ierr = PetscLogFlops(n);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecPointwiseDivide_Seq"
-PetscErrorCode VecPointwiseDivide_Seq(Vec win,Vec xin,Vec yin)
-{
-  Vec_Seq        *w = (Vec_Seq *)win->data;
-  PetscErrorCode ierr;
-  PetscInt       n = win->map->n,i;
-  PetscScalar    *ww = w->array,*xx,*yy;
-
-  PetscFunctionBegin;
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
-  } else {
-    yy = xx;
-  }
-  for (i=0; i<n; i++) {
-    ww[i] = xx[i] / yy[i];
-  }
-  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
-  if (xin != yin) {
-    ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
-  }
-  ierr = PetscLogFlops(n);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
 
 #undef __FUNCT__  
 #define __FUNCT__ "VecMaxPointwiseDivide_Seq"
 PetscErrorCode VecMaxPointwiseDivide_Seq(Vec xin,Vec yin,PetscReal *max)
 {
-  PetscErrorCode ierr;
-  PetscInt       n = xin->map->n,i;
-  PetscScalar    *xx,*yy;
-  PetscReal      m = 0.0;
+  PetscErrorCode    ierr;
+  PetscInt          n = xin->map->n,i;
+  const PetscScalar *xx,*yy;
+  PetscReal         m = 0.0;
 
   PetscFunctionBegin;
-  ierr = VecGetArray(yin,&yy);CHKERRQ(ierr);
-  ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(yin,&yy);CHKERRQ(ierr);
   for(i = 0; i < n; i++) {
-    if (yy[i] != 0.0) {
+    if (yy[i] != (PetscScalar)0.0) {
       m = PetscMax(PetscAbsScalar(xx[i]/yy[i]), m);
     } else {
       m = PetscMax(PetscAbsScalar(xx[i]), m);
     }
   }
-  ierr = VecRestoreArray(yin,&yy);CHKERRQ(ierr);
-  ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&m,max,1,MPIU_REAL,MPI_MAX,((PetscObject)xin)->comm);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(yin,&yy);CHKERRQ(ierr);
+  ierr = MPI_Allreduce(&m,max,1,MPIU_REAL,MPIU_MAX,((PetscObject)xin)->comm);CHKERRQ(ierr);
   ierr = PetscLogFlops(n);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "VecGetArray_Seq"
-PetscErrorCode VecGetArray_Seq(Vec vin,PetscScalar *a[])
-{
-  Vec_Seq        *v = (Vec_Seq *)vin->data;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (vin->array_gotten) {
-    SETERRQ(PETSC_ERR_ORDER,"Array has already been gotten for this vector,you may\n\
-    have forgotten a call to VecRestoreArray()");
-  }
-  vin->array_gotten = PETSC_TRUE;
-
-  *a =  v->array;
-  ierr = PetscObjectTakeAccess(vin);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecRestoreArray_Seq"
-PetscErrorCode VecRestoreArray_Seq(Vec vin,PetscScalar *a[])
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (!vin->array_gotten) {
-    SETERRQ(PETSC_ERR_ORDER,"Array has not been gotten for this vector, you may\n\
-    have forgotten a call to VecGetArray()");
-  }
-  vin->array_gotten = PETSC_FALSE;
-  if (a) *a         = 0; /* now user cannot accidently use it again */
-
-  ierr = PetscObjectGrantAccess(vin);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "VecResetArray_Seq"
-PetscErrorCode VecResetArray_Seq(Vec vin)
-{
-  Vec_Seq *v = (Vec_Seq *)vin->data;
-
-  PetscFunctionBegin;
-  v->array         = v->unplacedarray;
-  v->unplacedarray = 0;
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "VecPlaceArray_Seq"
 PetscErrorCode VecPlaceArray_Seq(Vec vin,const PetscScalar *a)
@@ -982,7 +777,7 @@ PetscErrorCode VecPlaceArray_Seq(Vec vin,const PetscScalar *a)
   Vec_Seq *v = (Vec_Seq *)vin->data;
 
   PetscFunctionBegin;
-  if (v->unplacedarray) SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"VecPlaceArray() was already called on this vector, without a call to VecResetArray()");
+  if (v->unplacedarray) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"VecPlaceArray() was already called on this vector, without a call to VecResetArray()");
   v->unplacedarray = v->array;  /* save previous array so reset can bring it back */
   v->array = (PetscScalar *)a;
   PetscFunctionReturn(0);
@@ -1001,29 +796,6 @@ PetscErrorCode VecReplaceArray_Seq(Vec vin,const PetscScalar *a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "VecGetSize_Seq"
-PetscErrorCode VecGetSize_Seq(Vec vin,PetscInt *size)
-{
-  PetscFunctionBegin;
-  *size = vin->map->n;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "VecConjugate_Seq"
-PetscErrorCode VecConjugate_Seq(Vec xin)
-{
-  PetscScalar *x = ((Vec_Seq *)xin->data)->array;
-  PetscInt    n = xin->map->n;
-
-  PetscFunctionBegin;
-  while (n-->0) {
-    *x = PetscConj(*x);
-    x++;
-  }
-  PetscFunctionReturn(0);
-}
  
 
 
