@@ -1,6 +1,6 @@
 
 #if !defined(__FNORM_H)
-#include "petscsys.h"
+#include <petscsys.h>
 #if defined(PETSC_USE_FORTRAN_KERNEL_NORM)
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define fortrannormsqr_    FORTRANNORMSQR
@@ -8,7 +8,7 @@
 #define fortrannormsqr_    fortrannormsqr
 #endif
 EXTERN_C_BEGIN
-EXTERN void fortrannormsqr_(void*,PetscInt*,void*);
+extern void fortrannormsqr_(void*,PetscInt*,void*);
 EXTERN_C_END
 #endif
 #endif
