@@ -1,9 +1,8 @@
-#define PETSCVEC_DLL
 
-#include "petscvec.h"
-#include "petscpf.h"
+#include <petscvec.h>
+#include <petscpf.h>
 
-static PetscTruth ISPackageInitialized = PETSC_FALSE;
+static PetscBool  ISPackageInitialized = PETSC_FALSE;
 #undef __FUNCT__  
 #define __FUNCT__ "ISFinalizePackage"
 /*@C
@@ -15,10 +14,12 @@ static PetscTruth ISPackageInitialized = PETSC_FALSE;
 .keywords: Petsc, destroy, package
 .seealso: PetscFinalize()
 @*/
-PetscErrorCode PETSC_DLLEXPORT ISFinalizePackage(void) 
+PetscErrorCode  ISFinalizePackage(void)
 {
   PetscFunctionBegin;
   ISPackageInitialized = PETSC_FALSE;
+  ISList               = PETSC_NULL;
+  ISRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
@@ -37,27 +38,29 @@ PetscErrorCode PETSC_DLLEXPORT ISFinalizePackage(void)
 .keywords: Vec, initialize, package
 .seealso: PetscInitialize()
 @*/
-PetscErrorCode PETSCVEC_DLLEXPORT ISInitializePackage(const char path[]) 
+PetscErrorCode  ISInitializePackage(const char path[]) 
 {
   char              logList[256];
   char              *className;
-  PetscTruth        opt;
+  PetscBool         opt;
   PetscErrorCode    ierr;
 
   PetscFunctionBegin;
   if (ISPackageInitialized) PetscFunctionReturn(0);
   ISPackageInitialized = PETSC_TRUE;
+  /* Register Constructors */
+  ierr = ISRegisterAll(path);CHKERRQ(ierr);
   /* Register Classes */
-  ierr = PetscCookieRegister("Index Set",&IS_COOKIE);CHKERRQ(ierr);
-  ierr = PetscCookieRegister("IS L to G Mapping",&IS_LTOGM_COOKIE);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Index Set",&IS_CLASSID);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("IS L to G Mapping",&IS_LTOGM_CLASSID);CHKERRQ(ierr);
 
   /* Process info exclusions */
   ierr = PetscOptionsGetString(PETSC_NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList, "is", &className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscInfoDeactivateClass(IS_COOKIE);CHKERRQ(ierr);
-      ierr = PetscInfoDeactivateClass(IS_LTOGM_COOKIE);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(IS_CLASSID);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(IS_LTOGM_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
@@ -65,8 +68,8 @@ PetscErrorCode PETSCVEC_DLLEXPORT ISInitializePackage(const char path[])
   if (opt) {
     ierr = PetscStrstr(logList, "is", &className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscLogEventDeactivateClass(IS_COOKIE);CHKERRQ(ierr);
-      ierr = PetscLogEventDeactivateClass(IS_LTOGM_COOKIE);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(IS_CLASSID);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(IS_LTOGM_CLASSID);CHKERRQ(ierr);
     }
   }
   ierr = PetscRegisterFinalize(ISFinalizePackage);CHKERRQ(ierr);
@@ -78,15 +81,15 @@ extern MPI_Op VecMax_Local_Op;
 extern MPI_Op VecMin_Local_Op;
 
 EXTERN_C_BEGIN
-extern void PETSCVEC_DLLEXPORT MPIAPI VecMax_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
-extern void PETSCVEC_DLLEXPORT MPIAPI VecMin_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
-extern void PETSCVEC_DLLEXPORT MPIAPI PetscSplitReduction_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
+extern void  MPIAPI VecMax_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
+extern void  MPIAPI VecMin_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
+extern void  MPIAPI PetscSplitReduction_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 EXTERN_C_END
 
 const char *NormTypes[] = {"1","2","FROBENIUS","INFINITY","1_AND_2","NormType","NORM_",0};
 PetscInt   NormIds[7];  /* map from NormType to IDs used to cache Normvalues */
 
-static PetscTruth VecPackageInitialized = PETSC_FALSE;
+static PetscBool  VecPackageInitialized = PETSC_FALSE;
 
 #undef __FUNCT__  
 #define __FUNCT__ "VecInitializePackage"
@@ -103,11 +106,11 @@ static PetscTruth VecPackageInitialized = PETSC_FALSE;
 .keywords: Vec, initialize, package
 .seealso: PetscInitialize()
 @*/
-PetscErrorCode PETSCVEC_DLLEXPORT VecInitializePackage(const char path[]) 
+PetscErrorCode  VecInitializePackage(const char path[]) 
 {
   char              logList[256];
   char              *className;
-  PetscTruth        opt;
+  PetscBool         opt;
   PetscErrorCode    ierr;
   PetscInt          i;
 
@@ -115,47 +118,53 @@ PetscErrorCode PETSCVEC_DLLEXPORT VecInitializePackage(const char path[])
   if (VecPackageInitialized) PetscFunctionReturn(0);
   VecPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscCookieRegister("Vec",&VEC_COOKIE);CHKERRQ(ierr);
-  ierr = PetscCookieRegister("Vec Scatter",&VEC_SCATTER_COOKIE);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Vector",&VEC_CLASSID);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Vector Scatter",&VEC_SCATTER_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = VecRegisterAll(path);CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("VecView",          VEC_COOKIE,&VEC_View);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMax",           VEC_COOKIE,&VEC_Max);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMin",           VEC_COOKIE,&VEC_Min);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecDotBarrier",    VEC_COOKIE,&VEC_DotBarrier);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecDot",           VEC_COOKIE,&VEC_Dot);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecDotNormBarr",   VEC_COOKIE,&VEC_DotNormBarrier);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecDotNorm2",      VEC_COOKIE,&VEC_DotNorm);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMDotBarrier",   VEC_COOKIE,&VEC_MDotBarrier);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMDot",          VEC_COOKIE,&VEC_MDot);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecTDot",          VEC_COOKIE,&VEC_TDot);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMTDot",         VEC_COOKIE,&VEC_MTDot);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecNormBarrier",   VEC_COOKIE,&VEC_NormBarrier);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecNorm",          VEC_COOKIE,&VEC_Norm);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecScale",         VEC_COOKIE,&VEC_Scale);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecCopy",          VEC_COOKIE,&VEC_Copy);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecSet",           VEC_COOKIE,&VEC_Set);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecAXPY",          VEC_COOKIE,&VEC_AXPY);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecAYPX",          VEC_COOKIE,&VEC_AYPX);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecAXPBYCZ",       VEC_COOKIE,&VEC_AXPBYPCZ);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecWAXPY",         VEC_COOKIE,&VEC_WAXPY);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecMAXPY",         VEC_COOKIE,&VEC_MAXPY);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecSwap",          VEC_COOKIE,&VEC_Swap);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecOps",           VEC_COOKIE,&VEC_Ops);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecAssemblyBegin", VEC_COOKIE,&VEC_AssemblyBegin);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecAssemblyEnd",   VEC_COOKIE,&VEC_AssemblyEnd);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecPointwiseMult", VEC_COOKIE,&VEC_PointwiseMult);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecSetValues",     VEC_COOKIE,&VEC_SetValues);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecLoad",          VEC_COOKIE,&VEC_Load);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecScatterBarrie", VEC_COOKIE,&VEC_ScatterBarrier);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecScatterBegin",  VEC_COOKIE,&VEC_ScatterBegin);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecScatterEnd",    VEC_COOKIE,&VEC_ScatterEnd);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecSetRandom",     VEC_COOKIE,&VEC_SetRandom);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecReduceArith",   VEC_COOKIE,&VEC_ReduceArithmetic);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecReduceBarrier", VEC_COOKIE,&VEC_ReduceBarrier);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecReduceComm",    VEC_COOKIE,&VEC_ReduceCommunication);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("VecNormalize",     VEC_COOKIE,&VEC_Normalize);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecView",          VEC_CLASSID,&VEC_View);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMax",           VEC_CLASSID,&VEC_Max);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMin",           VEC_CLASSID,&VEC_Min);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecDotBarrier",    VEC_CLASSID,&VEC_DotBarrier);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecDot",           VEC_CLASSID,&VEC_Dot);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecDotNormBarr",   VEC_CLASSID,&VEC_DotNormBarrier);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecDotNorm2",      VEC_CLASSID,&VEC_DotNorm);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMDotBarrier",   VEC_CLASSID,&VEC_MDotBarrier);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMDot",          VEC_CLASSID,&VEC_MDot);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecTDot",          VEC_CLASSID,&VEC_TDot);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMTDot",         VEC_CLASSID,&VEC_MTDot);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecNormBarrier",   VEC_CLASSID,&VEC_NormBarrier);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecNorm",          VEC_CLASSID,&VEC_Norm);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecScale",         VEC_CLASSID,&VEC_Scale);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecCopy",          VEC_CLASSID,&VEC_Copy);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecSet",           VEC_CLASSID,&VEC_Set);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecAXPY",          VEC_CLASSID,&VEC_AXPY);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecAYPX",          VEC_CLASSID,&VEC_AYPX);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecAXPBYCZ",       VEC_CLASSID,&VEC_AXPBYPCZ);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecWAXPY",         VEC_CLASSID,&VEC_WAXPY);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecMAXPY",         VEC_CLASSID,&VEC_MAXPY);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecSwap",          VEC_CLASSID,&VEC_Swap);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecOps",           VEC_CLASSID,&VEC_Ops);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecAssemblyBegin", VEC_CLASSID,&VEC_AssemblyBegin);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecAssemblyEnd",   VEC_CLASSID,&VEC_AssemblyEnd);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecPointwiseMult", VEC_CLASSID,&VEC_PointwiseMult);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecSetValues",     VEC_CLASSID,&VEC_SetValues);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecLoad",          VEC_CLASSID,&VEC_Load);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecScatterBarrie", VEC_CLASSID,&VEC_ScatterBarrier);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecScatterBegin",  VEC_CLASSID,&VEC_ScatterBegin);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecScatterEnd",    VEC_CLASSID,&VEC_ScatterEnd);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecSetRandom",     VEC_CLASSID,&VEC_SetRandom);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecReduceArith",   VEC_CLASSID,&VEC_ReduceArithmetic);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecReduceBarrier", VEC_CLASSID,&VEC_ReduceBarrier);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecReduceComm",    VEC_CLASSID,&VEC_ReduceCommunication);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecNormalize",     VEC_CLASSID,&VEC_Normalize);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUSP)
+  ierr = PetscLogEventRegister("VecCUSPCopyTo",     VEC_CLASSID,&VEC_CUSPCopyToGPU);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecCUSPCopyFrom",   VEC_CLASSID,&VEC_CUSPCopyFromGPU);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecCopyToSome",     VEC_CLASSID,&VEC_CUSPCopyToGPUSome);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("VecCopyFromSome",   VEC_CLASSID,&VEC_CUSPCopyFromGPUSome);CHKERRQ(ierr);
+#endif
   /* Turn off high traffic events by default */
   ierr = PetscLogEventSetActiveAll(VEC_DotBarrier, PETSC_FALSE);CHKERRQ(ierr);
   ierr = PetscLogEventSetActiveAll(VEC_DotNormBarrier, PETSC_FALSE);CHKERRQ(ierr);
@@ -169,7 +178,7 @@ PetscErrorCode PETSCVEC_DLLEXPORT VecInitializePackage(const char path[])
   if (opt) {
     ierr = PetscStrstr(logList, "vec", &className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscInfoDeactivateClass(VEC_COOKIE);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(VEC_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
@@ -177,12 +186,12 @@ PetscErrorCode PETSCVEC_DLLEXPORT VecInitializePackage(const char path[])
   if (opt) {
     ierr = PetscStrstr(logList, "vec", &className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscLogEventDeactivateClass(VEC_COOKIE);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(VEC_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Special processing */
   opt = PETSC_FALSE;
-  ierr = PetscOptionsGetTruth(PETSC_NULL, "-log_sync", &opt,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(PETSC_NULL, "-log_sync", &opt,PETSC_NULL);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscLogEventSetActiveAll(VEC_ScatterBarrier, PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscLogEventSetActiveAll(VEC_NormBarrier, PETSC_TRUE);CHKERRQ(ierr);
@@ -220,7 +229,7 @@ PetscErrorCode PETSCVEC_DLLEXPORT VecInitializePackage(const char path[])
 .keywords: Vec, initialize, package
 .seealso: PetscInitialize()
 @*/
-PetscErrorCode PETSCVEC_DLLEXPORT VecFinalizePackage(void) {
+PetscErrorCode  VecFinalizePackage(void) {
   PetscErrorCode ierr;
   PetscFunctionBegin;
   ierr = MPI_Op_free(&PetscSplitReduction_Op);CHKERRQ(ierr);
@@ -244,16 +253,11 @@ EXTERN_C_BEGIN
   Input Parameter:
   path - library path
  */
-PetscErrorCode PETSCVEC_DLLEXPORT PetscDLLibraryRegister_petscvec(const char path[])
+PetscErrorCode  PetscDLLibraryRegister_petscvec(const char path[])
 {
   PetscErrorCode ierr;
 
-  ierr = PetscInitializeNoArguments(); if (ierr) return 1;
-
   PetscFunctionBegin;
-  /*
-      If we got here then PETSc was properly loaded
-  */
   ierr = ISInitializePackage(path);CHKERRQ(ierr);
   ierr = VecInitializePackage(path);CHKERRQ(ierr);
   ierr = PFInitializePackage(path);CHKERRQ(ierr);
