@@ -1,6 +1,6 @@
 
-#include "petscvec.h"
-#include "../src/sys/f90-src/f90impl.h"
+#include <petscvec.h>
+#include <../src/sys/f90-src/f90impl.h>
 
 #ifdef PETSC_HAVE_FORTRAN_CAPS
 #define vecgetarrayf90_            VECGETARRAYF90
@@ -20,6 +20,10 @@ void PETSC_STDCALL vecgetarrayf90_(Vec *x,F90Array1d *ptr,int *__ierr PETSC_F90_
 {
   PetscScalar *fa;
   PetscInt     len;
+  if (!ptr) {
+    *__ierr = PetscError(((PetscObject)*x)->comm,__LINE__,PETSC_FUNCTION_NAME,__FILE__,__SDIR__,PETSC_ERR_ARG_BADPTR,PETSC_ERROR_INITIAL,"ptr==NULL, maybe #include <finclude/petscvec.h90> is missing?");
+    return;
+  }
   *__ierr = VecGetArray(*x,&fa);      if (*__ierr) return;
   *__ierr = VecGetLocalSize(*x,&len); if (*__ierr) return;
   *__ierr = F90Array1dCreate(fa,PETSC_SCALAR,1,len,ptr PETSC_F90_2PTR_PARAM(ptrd));
@@ -34,9 +38,9 @@ void PETSC_STDCALL vecrestorearrayf90_(Vec *x,F90Array1d *ptr,int *__ierr PETSC_
 
 void PETSC_STDCALL vecduplicatevecsf90_(Vec *v,int *m,F90Array1d *ptr,int *__ierr PETSC_F90_2PTR_PROTO(ptrd))
 {
-  Vec *lV;
+  Vec              *lV;
   PetscFortranAddr *newvint;
-  int i;
+  int              i;
   *__ierr = VecDuplicateVecs(*v,*m,&lV); if (*__ierr) return;
   *__ierr = PetscMalloc((*m)*sizeof(PetscFortranAddr),&newvint);  if (*__ierr) return;
 
@@ -47,14 +51,14 @@ void PETSC_STDCALL vecduplicatevecsf90_(Vec *v,int *m,F90Array1d *ptr,int *__ier
   *__ierr = F90Array1dCreate(newvint,PETSC_FORTRANADDR,1,*m,ptr PETSC_F90_2PTR_PARAM(ptrd));
 }
 
-void PETSC_STDCALL vecdestroyvecsf90_(F90Array1d *ptr,int *m,int *__ierr PETSC_F90_2PTR_PROTO(ptrd))
+void PETSC_STDCALL vecdestroyvecsf90_(int *m,F90Array1d *ptr,int *__ierr PETSC_F90_2PTR_PROTO(ptrd))
 {
-  PetscFortranAddr *vecs;
-  int       i;
+  Vec *vecs;
+  int i;
 
   *__ierr = F90Array1dAccess(ptr,PETSC_FORTRANADDR,(void**)&vecs PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
   for (i=0; i<*m; i++) {
-    *__ierr = VecDestroy((Vec)vecs[i]);
+    *__ierr = VecDestroy(&vecs[i]);
     if (*__ierr) return;
   }
   *__ierr = F90Array1dDestroy(ptr,PETSC_FORTRANADDR PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
