@@ -1,11 +1,13 @@
-#include "private/fortranimpl.h"
-#include "petscvec.h"
+#include <private/fortranimpl.h>
+#include <petscvec.h>
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define vecscattercreate_         VECSCATTERCREATE
-#define vecscatterremap_ VECSCATTERREMAP
+#define vecscatterremap_          VECSCATTERREMAP
+#define vecscatterdestroy_        VECSCATTERDESTROY
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define vecscattercreate_         vecscattercreate
-#define vecscatterremap_ vecscatterremap
+#define vecscatterremap_          vecscatterremap
+#define vecscatterdestroy_        vecscatterdestroy
 #endif
 
 
@@ -25,4 +27,8 @@ void PETSC_STDCALL vecscatterremap_(VecScatter *scat,PetscInt *rto,PetscInt *rfr
   *ierr = VecScatterRemap(*scat,rto,rfrom);
 }
 
+void PETSC_STDCALL  vecscatterdestroy_(VecScatter *ctx, int *__ierr )
+{
+  *__ierr = VecScatterDestroy(ctx);
+}
 EXTERN_C_END
